// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_ASSOCIATED_INTERFACES_MOJOM_H_
#define CONTENT_COMMON_ASSOCIATED_INTERFACES_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "content/common/associated_interfaces.mojom-shared.h"
#include "third_party/blink/public/mojom/associated_interfaces/associated_interfaces.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "content/common/content_export.h"


namespace content {
namespace mojom {
class RouteProvider;
using RouteProviderPtr = mojo::InterfacePtr<RouteProvider>;
using RouteProviderPtrInfo = mojo::InterfacePtrInfo<RouteProvider>;
using ThreadSafeRouteProviderPtr =
    mojo::ThreadSafeInterfacePtr<RouteProvider>;
using RouteProviderRequest = mojo::InterfaceRequest<RouteProvider>;
using RouteProviderAssociatedPtr =
    mojo::AssociatedInterfacePtr<RouteProvider>;
using ThreadSafeRouteProviderAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<RouteProvider>;
using RouteProviderAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<RouteProvider>;
using RouteProviderAssociatedRequest =
    mojo::AssociatedInterfaceRequest<RouteProvider>;


class RouteProviderProxy;

template <typename ImplRefTraits>
class RouteProviderStub;

class RouteProviderRequestValidator;

class CONTENT_EXPORT RouteProvider
    : public RouteProviderInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = true;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = RouteProviderProxy;

  template <typename ImplRefTraits>
  using Stub_ = RouteProviderStub<ImplRefTraits>;

  using RequestValidator_ = RouteProviderRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kGetRouteMinVersion = 0,
  };
  virtual ~RouteProvider() {}

  virtual void GetRoute(int32_t routing_id, ::blink::mojom::AssociatedInterfaceProviderAssociatedRequest request) = 0;
};

class CONTENT_EXPORT RouteProviderProxy
    : public RouteProvider {
 public:
  explicit RouteProviderProxy(mojo::MessageReceiverWithResponder* receiver);
  void GetRoute(int32_t routing_id, ::blink::mojom::AssociatedInterfaceProviderAssociatedRequest request) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class CONTENT_EXPORT RouteProviderStubDispatch {
 public:
  static bool Accept(RouteProvider* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      RouteProvider* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<RouteProvider>>
class RouteProviderStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  RouteProviderStub() {}
  ~RouteProviderStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return RouteProviderStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return RouteProviderStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT RouteProviderRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace content

namespace mojo {

}  // namespace mojo

#endif  // CONTENT_COMMON_ASSOCIATED_INTERFACES_MOJOM_H_