// Copyright 2019 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_FRAME_MOJOM_TEST_UTILS_H_
#define CONTENT_COMMON_FRAME_MOJOM_TEST_UTILS_H_

#include "content/common/frame.mojom.h"
#include "content/common/content_export.h"


namespace content {
namespace mojom {


class CONTENT_EXPORT FrameInterceptorForTesting : public Frame {
  virtual Frame* GetForwardingInterface() = 0;
  void GetInterfaceProvider(::service_manager::mojom::InterfaceProviderRequest interfaces) override;
  void GetCanonicalUrlForSharing(GetCanonicalUrlForSharingCallback callback) override;
  void BlockRequests() override;
  void ResumeBlockedRequests() override;
  void CancelBlockedRequests() override;
  void OnPortalActivated() override;
};
class CONTENT_EXPORT FrameAsyncWaiter {
 public:
  explicit FrameAsyncWaiter(Frame* proxy);
  ~FrameAsyncWaiter();
  void GetCanonicalUrlForSharing(
      base::Optional<GURL>* out_canonical_url);

 private:
  Frame* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(FrameAsyncWaiter);
};


class CONTENT_EXPORT FrameNavigationControlInterceptorForTesting : public FrameNavigationControl {
  virtual FrameNavigationControl* GetForwardingInterface() = 0;
  void CommitNavigation(const network::ResourceResponseHead& head, const content::CommonNavigationParams& common_params, const content::CommitNavigationParams& request_params, ::network::mojom::URLLoaderClientEndpointsPtr url_loader_client_endpoints, std::unique_ptr<blink::URLLoaderFactoryBundleInfo> subresource_loader_factories, base::Optional<std::vector<::content::mojom::TransferrableURLLoaderPtr>> subresource_overrides, ::blink::mojom::ControllerServiceWorkerInfoPtr controller_service_worker_info, ::network::mojom::URLLoaderFactoryPtr prefetch_loader_factory, const base::UnguessableToken& devtools_navigation_token, CommitNavigationCallback callback) override;
  void CommitFailedNavigation(const content::CommonNavigationParams& common_params, const content::CommitNavigationParams& request_params, bool has_stale_copy_in_cache, int32_t error_code, const base::Optional<std::string>& error_page_content, std::unique_ptr<blink::URLLoaderFactoryBundleInfo> subresource_loader_factories, CommitFailedNavigationCallback callback) override;
  void CommitSameDocumentNavigation(const content::CommonNavigationParams& common_params, const content::CommitNavigationParams& request_params, CommitSameDocumentNavigationCallback callback) override;
  void HandleRendererDebugURL(const GURL& url) override;
  void UpdateSubresourceLoaderFactories(std::unique_ptr<blink::URLLoaderFactoryBundleInfo> subresource_loader_factories) override;
  void BindDevToolsAgent(::blink::mojom::DevToolsAgentHostAssociatedPtrInfo agent_host, ::blink::mojom::DevToolsAgentAssociatedRequest agent) override;
};
class CONTENT_EXPORT FrameNavigationControlAsyncWaiter {
 public:
  explicit FrameNavigationControlAsyncWaiter(FrameNavigationControl* proxy);
  ~FrameNavigationControlAsyncWaiter();
  void CommitNavigation(
      const network::ResourceResponseHead& head, const content::CommonNavigationParams& common_params, const content::CommitNavigationParams& request_params, ::network::mojom::URLLoaderClientEndpointsPtr url_loader_client_endpoints, std::unique_ptr<blink::URLLoaderFactoryBundleInfo> subresource_loader_factories, base::Optional<std::vector<::content::mojom::TransferrableURLLoaderPtr>> subresource_overrides, ::blink::mojom::ControllerServiceWorkerInfoPtr controller_service_worker_info, ::network::mojom::URLLoaderFactoryPtr prefetch_loader_factory, const base::UnguessableToken& devtools_navigation_token, ::blink::mojom::CommitResult* out_commit_result);
  void CommitFailedNavigation(
      const content::CommonNavigationParams& common_params, const content::CommitNavigationParams& request_params, bool has_stale_copy_in_cache, int32_t error_code, const base::Optional<std::string>& error_page_content, std::unique_ptr<blink::URLLoaderFactoryBundleInfo> subresource_loader_factories, ::blink::mojom::CommitResult* out_commit_result);
  void CommitSameDocumentNavigation(
      const content::CommonNavigationParams& common_params, const content::CommitNavigationParams& request_params, ::blink::mojom::CommitResult* out_commit_result);

 private:
  FrameNavigationControl* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(FrameNavigationControlAsyncWaiter);
};


class CONTENT_EXPORT FrameBindingsControlInterceptorForTesting : public FrameBindingsControl {
  virtual FrameBindingsControl* GetForwardingInterface() = 0;
  void AllowBindings(int32_t enabled_bindings_flags) override;
};
class CONTENT_EXPORT FrameBindingsControlAsyncWaiter {
 public:
  explicit FrameBindingsControlAsyncWaiter(FrameBindingsControl* proxy);
  ~FrameBindingsControlAsyncWaiter();

 private:
  FrameBindingsControl* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(FrameBindingsControlAsyncWaiter);
};


class CONTENT_EXPORT FrameFactoryInterceptorForTesting : public FrameFactory {
  virtual FrameFactory* GetForwardingInterface() = 0;
  void CreateFrame(int32_t frame_routing_id, FrameRequest frame) override;
};
class CONTENT_EXPORT FrameFactoryAsyncWaiter {
 public:
  explicit FrameFactoryAsyncWaiter(FrameFactory* proxy);
  ~FrameFactoryAsyncWaiter();

 private:
  FrameFactory* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(FrameFactoryAsyncWaiter);
};


class CONTENT_EXPORT KeepAliveHandleInterceptorForTesting : public KeepAliveHandle {
  virtual KeepAliveHandle* GetForwardingInterface() = 0;
};
class CONTENT_EXPORT KeepAliveHandleAsyncWaiter {
 public:
  explicit KeepAliveHandleAsyncWaiter(KeepAliveHandle* proxy);
  ~KeepAliveHandleAsyncWaiter();

 private:
  KeepAliveHandle* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(KeepAliveHandleAsyncWaiter);
};


class CONTENT_EXPORT FrameHostInterceptorForTesting : public FrameHost {
  virtual FrameHost* GetForwardingInterface() = 0;
  void CreateNewWindow(CreateNewWindowParamsPtr params, CreateNewWindowCallback callback) override;
  void CreatePortal(::blink::mojom::PortalRequest portal, CreatePortalCallback callback) override;
  void IssueKeepAliveHandle(KeepAliveHandleRequest keep_alive_handle) override;
  void DidCommitProvisionalLoad(std::unique_ptr<::FrameHostMsg_DidCommitProvisionalLoad_Params> params, DidCommitProvisionalLoadInterfaceParamsPtr interface_params) override;
  void DidCommitSameDocumentNavigation(std::unique_ptr<::FrameHostMsg_DidCommitProvisionalLoad_Params> params) override;
  void BeginNavigation(const content::CommonNavigationParams& common_params, ::content::mojom::BeginNavigationParamsPtr begin_params, ::blink::mojom::BlobURLTokenPtr blob_url_token, ::content::mojom::NavigationClientAssociatedPtrInfo navigation_client, ::blink::mojom::NavigationInitiatorPtr navigation_initiator) override;
  void SubresourceResponseStarted(const GURL& url, uint32_t cert_status) override;
  void ResourceLoadComplete(::content::mojom::ResourceLoadInfoPtr url_load_info) override;
  void DidChangeName(const std::string& name, const std::string& unique_name) override;
  void EnforceInsecureRequestPolicy(uint8_t policy_bitmap) override;
  void EnforceInsecureNavigationsSet(const std::vector<uint32_t>& set) override;
  void DidSetFramePolicyHeaders(blink::WebSandboxFlags sandbox_flags, const std::vector<blink::ParsedFeaturePolicyDeclaration>& parsed_header) override;
  void CancelInitialHistoryLoad() override;
  void UpdateEncoding(const std::string& encoding_name) override;
  void FrameSizeChanged(const gfx::Size& size) override;
  void FullscreenStateChanged(bool is_fullscreen) override;
};
class CONTENT_EXPORT FrameHostAsyncWaiter {
 public:
  explicit FrameHostAsyncWaiter(FrameHost* proxy);
  ~FrameHostAsyncWaiter();
  void CreateNewWindow(
      CreateNewWindowParamsPtr params, CreateNewWindowStatus* out_status, CreateNewWindowReplyPtr* out_reply);
  void CreatePortal(
      ::blink::mojom::PortalRequest portal, int32_t* out_proxy_routing_id, base::UnguessableToken* out_portal_token);

 private:
  FrameHost* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(FrameHostAsyncWaiter);
};




}  // namespace mojom
}  // namespace content

#endif  // CONTENT_COMMON_FRAME_MOJOM_TEST_UTILS_H_