// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_NAVIGATION_CLIENT_MOJOM_SHARED_H_
#define CONTENT_COMMON_NAVIGATION_CLIENT_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "content/common/navigation_client.mojom-shared-internal.h"
#include "services/network/public/mojom/url_loader.mojom-shared.h"
#include "services/network/public/mojom/url_loader_factory.mojom-shared.h"
#include "content/public/common/transferrable_url_loader.mojom-shared.h"
#include "mojo/public/mojom/base/unguessable_token.mojom-shared.h"
#include "url/mojom/url.mojom-shared.h"
#include "third_party/blink/public/mojom/loader/url_loader_factory_bundle.mojom-shared.h"
#include "third_party/blink/public/mojom/service_worker/controller_service_worker.mojom-shared.h"
#include "third_party/blink/public/web/commit_result.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "base/component_export.h"


namespace content {
namespace mojom {
using CommonNavigationParamsDataView = mojo::native::NativeStructDataView;

using CommitNavigationParamsDataView = mojo::native::NativeStructDataView;



}  // namespace mojom
}  // namespace content

namespace mojo {
namespace internal {

}  // namespace internal
}  // namespace mojo


namespace content {
namespace mojom {
// Interface base classes. They are used for type safety check.
class NavigationClientInterfaceBase {};

using NavigationClientPtrDataView =
    mojo::InterfacePtrDataView<NavigationClientInterfaceBase>;
using NavigationClientRequestDataView =
    mojo::InterfaceRequestDataView<NavigationClientInterfaceBase>;
using NavigationClientAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<NavigationClientInterfaceBase>;
using NavigationClientAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<NavigationClientInterfaceBase>;
class NavigationClient_CommitNavigation_ParamsDataView {
 public:
  NavigationClient_CommitNavigation_ParamsDataView() {}

  NavigationClient_CommitNavigation_ParamsDataView(
      internal::NavigationClient_CommitNavigation_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetHeadDataView(
      ::network::mojom::URLResponseHeadDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadHead(UserType* output) {
    auto* pointer = data_->head.Get();
    return mojo::internal::Deserialize<::network::mojom::URLResponseHeadDataView>(
        pointer, output, context_);
  }
  inline void GetCommonParamsDataView(
      CommonNavigationParamsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCommonParams(UserType* output) {
    auto* pointer = data_->common_params.Get();
    return mojo::internal::Deserialize<::content::mojom::CommonNavigationParamsDataView>(
        pointer, output, context_);
  }
  inline void GetRequestParamsDataView(
      CommitNavigationParamsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRequestParams(UserType* output) {
    auto* pointer = data_->request_params.Get();
    return mojo::internal::Deserialize<::content::mojom::CommitNavigationParamsDataView>(
        pointer, output, context_);
  }
  inline void GetUrlLoaderClientEndpointsDataView(
      ::network::mojom::URLLoaderClientEndpointsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUrlLoaderClientEndpoints(UserType* output) {
    auto* pointer = data_->url_loader_client_endpoints.Get();
    return mojo::internal::Deserialize<::network::mojom::URLLoaderClientEndpointsDataView>(
        pointer, output, context_);
  }
  inline void GetSubresourceLoaderFactoriesDataView(
      ::blink::mojom::URLLoaderFactoryBundleDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSubresourceLoaderFactories(UserType* output) {
    auto* pointer = data_->subresource_loader_factories.Get();
    return mojo::internal::Deserialize<::blink::mojom::URLLoaderFactoryBundleDataView>(
        pointer, output, context_);
  }
  inline void GetSubresourceOverridesDataView(
      mojo::ArrayDataView<::content::mojom::TransferrableURLLoaderDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSubresourceOverrides(UserType* output) {
    auto* pointer = data_->subresource_overrides.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::content::mojom::TransferrableURLLoaderDataView>>(
        pointer, output, context_);
  }
  inline void GetControllerServiceWorkerInfoDataView(
      ::blink::mojom::ControllerServiceWorkerInfoDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadControllerServiceWorkerInfo(UserType* output) {
    auto* pointer = data_->controller_service_worker_info.Get();
    return mojo::internal::Deserialize<::blink::mojom::ControllerServiceWorkerInfoDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  UserType TakePrefetchLoaderFactory() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::URLLoaderFactoryPtrDataView>(
            &data_->prefetch_loader_factory, &result, context_);
    DCHECK(ret);
    return result;
  }
  inline void GetDevtoolsNavigationTokenDataView(
      ::mojo_base::mojom::UnguessableTokenDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDevtoolsNavigationToken(UserType* output) {
    auto* pointer = data_->devtools_navigation_token.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::UnguessableTokenDataView>(
        pointer, output, context_);
  }
 private:
  internal::NavigationClient_CommitNavigation_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NavigationClient_CommitNavigation_ResponseParamsDataView {
 public:
  NavigationClient_CommitNavigation_ResponseParamsDataView() {}

  NavigationClient_CommitNavigation_ResponseParamsDataView(
      internal::NavigationClient_CommitNavigation_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCommitResult(UserType* output) const {
    auto data_value = data_->commit_result;
    return mojo::internal::Deserialize<::blink::mojom::CommitResult>(
        data_value, output);
  }

  ::blink::mojom::CommitResult commit_result() const {
    return static_cast<::blink::mojom::CommitResult>(data_->commit_result);
  }
 private:
  internal::NavigationClient_CommitNavigation_ResponseParams_Data* data_ = nullptr;
};

class NavigationClient_CommitFailedNavigation_ParamsDataView {
 public:
  NavigationClient_CommitFailedNavigation_ParamsDataView() {}

  NavigationClient_CommitFailedNavigation_ParamsDataView(
      internal::NavigationClient_CommitFailedNavigation_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetCommonParamsDataView(
      CommonNavigationParamsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCommonParams(UserType* output) {
    auto* pointer = data_->common_params.Get();
    return mojo::internal::Deserialize<::content::mojom::CommonNavigationParamsDataView>(
        pointer, output, context_);
  }
  inline void GetRequestParamsDataView(
      CommitNavigationParamsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRequestParams(UserType* output) {
    auto* pointer = data_->request_params.Get();
    return mojo::internal::Deserialize<::content::mojom::CommitNavigationParamsDataView>(
        pointer, output, context_);
  }
  bool has_stale_copy_in_cache() const {
    return data_->has_stale_copy_in_cache;
  }
  int32_t error_code() const {
    return data_->error_code;
  }
  inline void GetErrorPageContentDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadErrorPageContent(UserType* output) {
    auto* pointer = data_->error_page_content.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetSubresourceLoaderFactoriesDataView(
      ::blink::mojom::URLLoaderFactoryBundleDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSubresourceLoaderFactories(UserType* output) {
    auto* pointer = data_->subresource_loader_factories.Get();
    return mojo::internal::Deserialize<::blink::mojom::URLLoaderFactoryBundleDataView>(
        pointer, output, context_);
  }
 private:
  internal::NavigationClient_CommitFailedNavigation_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NavigationClient_CommitFailedNavigation_ResponseParamsDataView {
 public:
  NavigationClient_CommitFailedNavigation_ResponseParamsDataView() {}

  NavigationClient_CommitFailedNavigation_ResponseParamsDataView(
      internal::NavigationClient_CommitFailedNavigation_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCommitResult(UserType* output) const {
    auto data_value = data_->commit_result;
    return mojo::internal::Deserialize<::blink::mojom::CommitResult>(
        data_value, output);
  }

  ::blink::mojom::CommitResult commit_result() const {
    return static_cast<::blink::mojom::CommitResult>(data_->commit_result);
  }
 private:
  internal::NavigationClient_CommitFailedNavigation_ResponseParams_Data* data_ = nullptr;
};



}  // namespace mojom
}  // namespace content

namespace std {

}  // namespace std

namespace mojo {

}  // namespace mojo


namespace content {
namespace mojom {

inline void NavigationClient_CommitNavigation_ParamsDataView::GetHeadDataView(
    ::network::mojom::URLResponseHeadDataView* output) {
  auto pointer = data_->head.Get();
  *output = ::network::mojom::URLResponseHeadDataView(pointer, context_);
}
inline void NavigationClient_CommitNavigation_ParamsDataView::GetCommonParamsDataView(
    CommonNavigationParamsDataView* output) {
  auto pointer = data_->common_params.Get();
  *output = CommonNavigationParamsDataView(pointer, context_);
}
inline void NavigationClient_CommitNavigation_ParamsDataView::GetRequestParamsDataView(
    CommitNavigationParamsDataView* output) {
  auto pointer = data_->request_params.Get();
  *output = CommitNavigationParamsDataView(pointer, context_);
}
inline void NavigationClient_CommitNavigation_ParamsDataView::GetUrlLoaderClientEndpointsDataView(
    ::network::mojom::URLLoaderClientEndpointsDataView* output) {
  auto pointer = data_->url_loader_client_endpoints.Get();
  *output = ::network::mojom::URLLoaderClientEndpointsDataView(pointer, context_);
}
inline void NavigationClient_CommitNavigation_ParamsDataView::GetSubresourceLoaderFactoriesDataView(
    ::blink::mojom::URLLoaderFactoryBundleDataView* output) {
  auto pointer = data_->subresource_loader_factories.Get();
  *output = ::blink::mojom::URLLoaderFactoryBundleDataView(pointer, context_);
}
inline void NavigationClient_CommitNavigation_ParamsDataView::GetSubresourceOverridesDataView(
    mojo::ArrayDataView<::content::mojom::TransferrableURLLoaderDataView>* output) {
  auto pointer = data_->subresource_overrides.Get();
  *output = mojo::ArrayDataView<::content::mojom::TransferrableURLLoaderDataView>(pointer, context_);
}
inline void NavigationClient_CommitNavigation_ParamsDataView::GetControllerServiceWorkerInfoDataView(
    ::blink::mojom::ControllerServiceWorkerInfoDataView* output) {
  auto pointer = data_->controller_service_worker_info.Get();
  *output = ::blink::mojom::ControllerServiceWorkerInfoDataView(pointer, context_);
}
inline void NavigationClient_CommitNavigation_ParamsDataView::GetDevtoolsNavigationTokenDataView(
    ::mojo_base::mojom::UnguessableTokenDataView* output) {
  auto pointer = data_->devtools_navigation_token.Get();
  *output = ::mojo_base::mojom::UnguessableTokenDataView(pointer, context_);
}




inline void NavigationClient_CommitFailedNavigation_ParamsDataView::GetCommonParamsDataView(
    CommonNavigationParamsDataView* output) {
  auto pointer = data_->common_params.Get();
  *output = CommonNavigationParamsDataView(pointer, context_);
}
inline void NavigationClient_CommitFailedNavigation_ParamsDataView::GetRequestParamsDataView(
    CommitNavigationParamsDataView* output) {
  auto pointer = data_->request_params.Get();
  *output = CommitNavigationParamsDataView(pointer, context_);
}
inline void NavigationClient_CommitFailedNavigation_ParamsDataView::GetErrorPageContentDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->error_page_content.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void NavigationClient_CommitFailedNavigation_ParamsDataView::GetSubresourceLoaderFactoriesDataView(
    ::blink::mojom::URLLoaderFactoryBundleDataView* output) {
  auto pointer = data_->subresource_loader_factories.Get();
  *output = ::blink::mojom::URLLoaderFactoryBundleDataView(pointer, context_);
}





}  // namespace mojom
}  // namespace content

#endif  // CONTENT_COMMON_NAVIGATION_CLIENT_MOJOM_SHARED_H_