// Copyright 2019 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_PUSH_MESSAGING_MOJOM_TEST_UTILS_H_
#define CONTENT_COMMON_PUSH_MESSAGING_MOJOM_TEST_UTILS_H_

#include "content/common/push_messaging.mojom.h"
#include "content/common/content_export.h"


namespace content {
namespace mojom {


class CONTENT_EXPORT PushMessagingInterceptorForTesting : public PushMessaging {
  virtual PushMessaging* GetForwardingInterface() = 0;
  void Subscribe(int32_t render_frame_id, int64_t service_worker_registration_id, const content::PushSubscriptionOptions& options, bool user_gesture, SubscribeCallback callback) override;
  void Unsubscribe(int64_t service_worker_registration_id, UnsubscribeCallback callback) override;
  void GetSubscription(int64_t service_worker_registration_id, GetSubscriptionCallback callback) override;
};
class CONTENT_EXPORT PushMessagingAsyncWaiter {
 public:
  explicit PushMessagingAsyncWaiter(PushMessaging* proxy);
  ~PushMessagingAsyncWaiter();
  void Subscribe(
      int32_t render_frame_id, int64_t service_worker_registration_id, const content::PushSubscriptionOptions& options, bool user_gesture, ::content::mojom::PushRegistrationStatus* out_status, base::Optional<GURL>* out_endpoint, base::Optional<content::PushSubscriptionOptions>* out_options, base::Optional<std::vector<uint8_t>>* out_p256dh, base::Optional<std::vector<uint8_t>>* out_auth);
  void Unsubscribe(
      int64_t service_worker_registration_id, blink::WebPushError::ErrorType* out_error_type, bool* out_did_unsubscribe, base::Optional<std::string>* out_error_message);
  void GetSubscription(
      int64_t service_worker_registration_id, ::content::mojom::PushGetRegistrationStatus* out_status, base::Optional<GURL>* out_endpoint, base::Optional<content::PushSubscriptionOptions>* out_options, base::Optional<std::vector<uint8_t>>* out_p256dh, base::Optional<std::vector<uint8_t>>* out_auth);

 private:
  PushMessaging* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(PushMessagingAsyncWaiter);
};




}  // namespace mojom
}  // namespace content

#endif  // CONTENT_COMMON_PUSH_MESSAGING_MOJOM_TEST_UTILS_H_