// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_PUBLIC_COMMON_RENDERER_PREFERENCE_WATCHER_MOJOM_H_
#define CONTENT_PUBLIC_COMMON_RENDERER_PREFERENCE_WATCHER_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "content/public/common/renderer_preference_watcher.mojom-shared.h"
#include "content/public/common/renderer_preferences.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "content/common/content_export.h"


namespace content {
namespace mojom {
class RendererPreferenceWatcher;
using RendererPreferenceWatcherPtr = mojo::InterfacePtr<RendererPreferenceWatcher>;
using RendererPreferenceWatcherPtrInfo = mojo::InterfacePtrInfo<RendererPreferenceWatcher>;
using ThreadSafeRendererPreferenceWatcherPtr =
    mojo::ThreadSafeInterfacePtr<RendererPreferenceWatcher>;
using RendererPreferenceWatcherRequest = mojo::InterfaceRequest<RendererPreferenceWatcher>;
using RendererPreferenceWatcherAssociatedPtr =
    mojo::AssociatedInterfacePtr<RendererPreferenceWatcher>;
using ThreadSafeRendererPreferenceWatcherAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<RendererPreferenceWatcher>;
using RendererPreferenceWatcherAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<RendererPreferenceWatcher>;
using RendererPreferenceWatcherAssociatedRequest =
    mojo::AssociatedInterfaceRequest<RendererPreferenceWatcher>;


class RendererPreferenceWatcherProxy;

template <typename ImplRefTraits>
class RendererPreferenceWatcherStub;

class RendererPreferenceWatcherRequestValidator;

class CONTENT_EXPORT RendererPreferenceWatcher
    : public RendererPreferenceWatcherInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = RendererPreferenceWatcherProxy;

  template <typename ImplRefTraits>
  using Stub_ = RendererPreferenceWatcherStub<ImplRefTraits>;

  using RequestValidator_ = RendererPreferenceWatcherRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kNotifyUpdateMinVersion = 0,
  };
  virtual ~RendererPreferenceWatcher() {}

  virtual void NotifyUpdate(const content::RendererPreferences& new_prefs) = 0;
};

class CONTENT_EXPORT RendererPreferenceWatcherProxy
    : public RendererPreferenceWatcher {
 public:
  explicit RendererPreferenceWatcherProxy(mojo::MessageReceiverWithResponder* receiver);
  void NotifyUpdate(const content::RendererPreferences& new_prefs) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class CONTENT_EXPORT RendererPreferenceWatcherStubDispatch {
 public:
  static bool Accept(RendererPreferenceWatcher* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      RendererPreferenceWatcher* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<RendererPreferenceWatcher>>
class RendererPreferenceWatcherStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  RendererPreferenceWatcherStub() {}
  ~RendererPreferenceWatcherStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return RendererPreferenceWatcherStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return RendererPreferenceWatcherStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT RendererPreferenceWatcherRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace content

namespace mojo {

}  // namespace mojo

#endif  // CONTENT_PUBLIC_COMMON_RENDERER_PREFERENCE_WATCHER_MOJOM_H_