// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4065)
#endif

#include "extensions/common/mojo/guest_view.mojom-shared.h"

#include <utility>

#include "base/logging.h"
#include "base/stl_util.h"  // for base::size()
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/cpp/bindings/lib/validation_util.h"
namespace extensions {
namespace mojom {

namespace internal {


// static
bool GuestView_CreateEmbeddedMimeHandlerViewGuest_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const GuestView_CreateEmbeddedMimeHandlerViewGuest_Params_Data* object = static_cast<const GuestView_CreateEmbeddedMimeHandlerViewGuest_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 48 }};

  if (object->header_.version <=
          kVersionSizes[base::size(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = base::size(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[base::size(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->original_url, 3, validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->original_url, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->element_size, 5, validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->element_size, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->transferrable_url_loader, 6, validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->transferrable_url_loader, validation_context))
    return false;

  return true;
}

GuestView_CreateEmbeddedMimeHandlerViewGuest_Params_Data::GuestView_CreateEmbeddedMimeHandlerViewGuest_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
bool GuestView_CreateMimeHandlerViewGuest_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const GuestView_CreateMimeHandlerViewGuest_Params_Data* object = static_cast<const GuestView_CreateMimeHandlerViewGuest_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 48 }};

  if (object->header_.version <=
          kVersionSizes[base::size(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = base::size(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[base::size(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->view_id, 2, validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams view_id_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->view_id, validation_context,
                                         &view_id_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->element_size, 4, validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->element_size, validation_context))
    return false;

  if (!mojo::internal::ValidateHandleOrInterface(object->before_unload_control,
                                                 validation_context)) {
    return false;
  }

  return true;
}

GuestView_CreateMimeHandlerViewGuest_Params_Data::GuestView_CreateMimeHandlerViewGuest_Params_Data()
    : header_({sizeof(*this), 0}) {}

}  // namespace internal
}  // namespace mojom
}  // namespace extensions

#if defined(_MSC_VER)
#pragma warning(pop)
#endif