// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_AUDIO_PUBLIC_MOJOM_LOG_FACTORY_MANAGER_MOJOM_H_
#define SERVICES_AUDIO_PUBLIC_MOJOM_LOG_FACTORY_MANAGER_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/audio/public/mojom/log_factory_manager.mojom-shared.h"
#include "media/mojo/interfaces/audio_logging.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace audio {
namespace mojom {
class LogFactoryManager;
using LogFactoryManagerPtr = mojo::InterfacePtr<LogFactoryManager>;
using LogFactoryManagerPtrInfo = mojo::InterfacePtrInfo<LogFactoryManager>;
using ThreadSafeLogFactoryManagerPtr =
    mojo::ThreadSafeInterfacePtr<LogFactoryManager>;
using LogFactoryManagerRequest = mojo::InterfaceRequest<LogFactoryManager>;
using LogFactoryManagerAssociatedPtr =
    mojo::AssociatedInterfacePtr<LogFactoryManager>;
using ThreadSafeLogFactoryManagerAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<LogFactoryManager>;
using LogFactoryManagerAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<LogFactoryManager>;
using LogFactoryManagerAssociatedRequest =
    mojo::AssociatedInterfaceRequest<LogFactoryManager>;


class LogFactoryManagerProxy;

template <typename ImplRefTraits>
class LogFactoryManagerStub;

class LogFactoryManagerRequestValidator;

class  LogFactoryManager
    : public LogFactoryManagerInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = LogFactoryManagerProxy;

  template <typename ImplRefTraits>
  using Stub_ = LogFactoryManagerStub<ImplRefTraits>;

  using RequestValidator_ = LogFactoryManagerRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kSetLogFactoryMinVersion = 0,
  };
  virtual ~LogFactoryManager() {}

  virtual void SetLogFactory(::media::mojom::AudioLogFactoryPtr factory) = 0;
};

class  LogFactoryManagerProxy
    : public LogFactoryManager {
 public:
  explicit LogFactoryManagerProxy(mojo::MessageReceiverWithResponder* receiver);
  void SetLogFactory(::media::mojom::AudioLogFactoryPtr factory) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  LogFactoryManagerStubDispatch {
 public:
  static bool Accept(LogFactoryManager* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      LogFactoryManager* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<LogFactoryManager>>
class LogFactoryManagerStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  LogFactoryManagerStub() {}
  ~LogFactoryManagerStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return LogFactoryManagerStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return LogFactoryManagerStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  LogFactoryManagerRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace audio

namespace mojo {

}  // namespace mojo

#endif  // SERVICES_AUDIO_PUBLIC_MOJOM_LOG_FACTORY_MANAGER_MOJOM_H_