// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_CONTENT_PUBLIC_MOJOM_NAVIGABLE_CONTENTS_MOJOM_SHARED_H_
#define SERVICES_CONTENT_PUBLIC_MOJOM_NAVIGABLE_CONTENTS_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "services/content/public/mojom/navigable_contents.mojom-shared-internal.h"
#include "mojo/public/mojom/base/unguessable_token.mojom-shared.h"
#include "services/network/public/mojom/network_param.mojom-shared.h"
#include "ui/accessibility/mojom/ax_tree_id.mojom-shared.h"
#include "ui/base/mojo/window_open_disposition.mojom-shared.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-shared.h"
#include "url/mojom/url.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "base/component_export.h"


namespace content {
namespace mojom {
class NavigateParamsDataView;



}  // namespace mojom
}  // namespace content

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::content::mojom::NavigateParamsDataView> {
  using Data = ::content::mojom::internal::NavigateParams_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace content {
namespace mojom {
// Interface base classes. They are used for type safety check.
class NavigableContentsInterfaceBase {};

using NavigableContentsPtrDataView =
    mojo::InterfacePtrDataView<NavigableContentsInterfaceBase>;
using NavigableContentsRequestDataView =
    mojo::InterfaceRequestDataView<NavigableContentsInterfaceBase>;
using NavigableContentsAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<NavigableContentsInterfaceBase>;
using NavigableContentsAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<NavigableContentsInterfaceBase>;
class NavigableContentsClientInterfaceBase {};

using NavigableContentsClientPtrDataView =
    mojo::InterfacePtrDataView<NavigableContentsClientInterfaceBase>;
using NavigableContentsClientRequestDataView =
    mojo::InterfaceRequestDataView<NavigableContentsClientInterfaceBase>;
using NavigableContentsClientAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<NavigableContentsClientInterfaceBase>;
using NavigableContentsClientAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<NavigableContentsClientInterfaceBase>;
class NavigateParamsDataView {
 public:
  NavigateParamsDataView() {}

  NavigateParamsDataView(
      internal::NavigateParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool should_clear_session_history() const {
    return data_->should_clear_session_history;
  }
 private:
  internal::NavigateParams_Data* data_ = nullptr;
};

class NavigableContents_Navigate_ParamsDataView {
 public:
  NavigableContents_Navigate_ParamsDataView() {}

  NavigableContents_Navigate_ParamsDataView(
      internal::NavigableContents_Navigate_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUrl(UserType* output) {
    auto* pointer = data_->url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  inline void GetParamsDataView(
      NavigateParamsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadParams(UserType* output) {
    auto* pointer = data_->params.Get();
    return mojo::internal::Deserialize<::content::mojom::NavigateParamsDataView>(
        pointer, output, context_);
  }
 private:
  internal::NavigableContents_Navigate_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NavigableContents_GoBack_ParamsDataView {
 public:
  NavigableContents_GoBack_ParamsDataView() {}

  NavigableContents_GoBack_ParamsDataView(
      internal::NavigableContents_GoBack_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NavigableContents_GoBack_Params_Data* data_ = nullptr;
};

class NavigableContents_GoBack_ResponseParamsDataView {
 public:
  NavigableContents_GoBack_ResponseParamsDataView() {}

  NavigableContents_GoBack_ResponseParamsDataView(
      internal::NavigableContents_GoBack_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool success() const {
    return data_->success;
  }
 private:
  internal::NavigableContents_GoBack_ResponseParams_Data* data_ = nullptr;
};

class NavigableContents_CreateView_ParamsDataView {
 public:
  NavigableContents_CreateView_ParamsDataView() {}

  NavigableContents_CreateView_ParamsDataView(
      internal::NavigableContents_CreateView_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool in_service_process() const {
    return data_->in_service_process;
  }
 private:
  internal::NavigableContents_CreateView_Params_Data* data_ = nullptr;
};

class NavigableContents_CreateView_ResponseParamsDataView {
 public:
  NavigableContents_CreateView_ResponseParamsDataView() {}

  NavigableContents_CreateView_ResponseParamsDataView(
      internal::NavigableContents_CreateView_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetEmbedTokenDataView(
      ::mojo_base::mojom::UnguessableTokenDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadEmbedToken(UserType* output) {
    auto* pointer = data_->embed_token.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::UnguessableTokenDataView>(
        pointer, output, context_);
  }
 private:
  internal::NavigableContents_CreateView_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NavigableContents_Focus_ParamsDataView {
 public:
  NavigableContents_Focus_ParamsDataView() {}

  NavigableContents_Focus_ParamsDataView(
      internal::NavigableContents_Focus_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NavigableContents_Focus_Params_Data* data_ = nullptr;
};

class NavigableContents_FocusThroughTabTraversal_ParamsDataView {
 public:
  NavigableContents_FocusThroughTabTraversal_ParamsDataView() {}

  NavigableContents_FocusThroughTabTraversal_ParamsDataView(
      internal::NavigableContents_FocusThroughTabTraversal_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool reverse() const {
    return data_->reverse;
  }
 private:
  internal::NavigableContents_FocusThroughTabTraversal_Params_Data* data_ = nullptr;
};

class NavigableContentsClient_ClearViewFocus_ParamsDataView {
 public:
  NavigableContentsClient_ClearViewFocus_ParamsDataView() {}

  NavigableContentsClient_ClearViewFocus_ParamsDataView(
      internal::NavigableContentsClient_ClearViewFocus_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NavigableContentsClient_ClearViewFocus_Params_Data* data_ = nullptr;
};

class NavigableContentsClient_DidFinishNavigation_ParamsDataView {
 public:
  NavigableContentsClient_DidFinishNavigation_ParamsDataView() {}

  NavigableContentsClient_DidFinishNavigation_ParamsDataView(
      internal::NavigableContentsClient_DidFinishNavigation_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUrl(UserType* output) {
    auto* pointer = data_->url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  bool is_main_frame() const {
    return data_->is_main_frame;
  }
  bool is_error_page() const {
    return data_->is_error_page;
  }
  inline void GetResponseHeadersDataView(
      ::network::mojom::HttpResponseHeadersDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadResponseHeaders(UserType* output) {
    auto* pointer = data_->response_headers.Get();
    return mojo::internal::Deserialize<::network::mojom::HttpResponseHeadersDataView>(
        pointer, output, context_);
  }
 private:
  internal::NavigableContentsClient_DidFinishNavigation_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NavigableContentsClient_DidStopLoading_ParamsDataView {
 public:
  NavigableContentsClient_DidStopLoading_ParamsDataView() {}

  NavigableContentsClient_DidStopLoading_ParamsDataView(
      internal::NavigableContentsClient_DidStopLoading_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NavigableContentsClient_DidStopLoading_Params_Data* data_ = nullptr;
};

class NavigableContentsClient_DidAutoResizeView_ParamsDataView {
 public:
  NavigableContentsClient_DidAutoResizeView_ParamsDataView() {}

  NavigableContentsClient_DidAutoResizeView_ParamsDataView(
      internal::NavigableContentsClient_DidAutoResizeView_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetNewSizeDataView(
      ::gfx::mojom::SizeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadNewSize(UserType* output) {
    auto* pointer = data_->new_size.Get();
    return mojo::internal::Deserialize<::gfx::mojom::SizeDataView>(
        pointer, output, context_);
  }
 private:
  internal::NavigableContentsClient_DidAutoResizeView_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NavigableContentsClient_DidSuppressNavigation_ParamsDataView {
 public:
  NavigableContentsClient_DidSuppressNavigation_ParamsDataView() {}

  NavigableContentsClient_DidSuppressNavigation_ParamsDataView(
      internal::NavigableContentsClient_DidSuppressNavigation_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUrl(UserType* output) {
    auto* pointer = data_->url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDisposition(UserType* output) const {
    auto data_value = data_->disposition;
    return mojo::internal::Deserialize<::ui::mojom::WindowOpenDisposition>(
        data_value, output);
  }

  ::ui::mojom::WindowOpenDisposition disposition() const {
    return static_cast<::ui::mojom::WindowOpenDisposition>(data_->disposition);
  }
  bool from_user_gesture() const {
    return data_->from_user_gesture;
  }
 private:
  internal::NavigableContentsClient_DidSuppressNavigation_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NavigableContentsClient_UpdateContentAXTree_ParamsDataView {
 public:
  NavigableContentsClient_UpdateContentAXTree_ParamsDataView() {}

  NavigableContentsClient_UpdateContentAXTree_ParamsDataView(
      internal::NavigableContentsClient_UpdateContentAXTree_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetIdDataView(
      ::ax::mojom::AXTreeIDDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadId(UserType* output) {
    auto* pointer = !data_->id.is_null() ? &data_->id : nullptr;
    return mojo::internal::Deserialize<::ax::mojom::AXTreeIDDataView>(
        pointer, output, context_);
  }
 private:
  internal::NavigableContentsClient_UpdateContentAXTree_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace content

namespace std {

}  // namespace std

namespace mojo {


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::content::mojom::NavigateParamsDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::content::mojom::NavigateParamsDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::content::mojom::internal::NavigateParams_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    (*output)->should_clear_session_history = CallWithContext(Traits::should_clear_session_history, input, custom_context);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::content::mojom::internal::NavigateParams_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::content::mojom::NavigateParamsDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace content {
namespace mojom {



inline void NavigableContents_Navigate_ParamsDataView::GetUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}
inline void NavigableContents_Navigate_ParamsDataView::GetParamsDataView(
    NavigateParamsDataView* output) {
  auto pointer = data_->params.Get();
  *output = NavigateParamsDataView(pointer, context_);
}








inline void NavigableContents_CreateView_ResponseParamsDataView::GetEmbedTokenDataView(
    ::mojo_base::mojom::UnguessableTokenDataView* output) {
  auto pointer = data_->embed_token.Get();
  *output = ::mojo_base::mojom::UnguessableTokenDataView(pointer, context_);
}








inline void NavigableContentsClient_DidFinishNavigation_ParamsDataView::GetUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}
inline void NavigableContentsClient_DidFinishNavigation_ParamsDataView::GetResponseHeadersDataView(
    ::network::mojom::HttpResponseHeadersDataView* output) {
  auto pointer = data_->response_headers.Get();
  *output = ::network::mojom::HttpResponseHeadersDataView(pointer, context_);
}




inline void NavigableContentsClient_DidAutoResizeView_ParamsDataView::GetNewSizeDataView(
    ::gfx::mojom::SizeDataView* output) {
  auto pointer = data_->new_size.Get();
  *output = ::gfx::mojom::SizeDataView(pointer, context_);
}


inline void NavigableContentsClient_DidSuppressNavigation_ParamsDataView::GetUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}


inline void NavigableContentsClient_UpdateContentAXTree_ParamsDataView::GetIdDataView(
    ::ax::mojom::AXTreeIDDataView* output) {
  auto pointer = &data_->id;
  *output = ::ax::mojom::AXTreeIDDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace content

#endif  // SERVICES_CONTENT_PUBLIC_MOJOM_NAVIGABLE_CONTENTS_MOJOM_SHARED_H_