// Copyright 2019 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_DEVICE_PUBLIC_MOJOM_SERIAL_MOJOM_TEST_UTILS_H_
#define SERVICES_DEVICE_PUBLIC_MOJOM_SERIAL_MOJOM_TEST_UTILS_H_

#include "services/device/public/mojom/serial.mojom.h"


namespace device {
namespace mojom {


class  SerialPortManagerInterceptorForTesting : public SerialPortManager {
  virtual SerialPortManager* GetForwardingInterface() = 0;
  void GetDevices(GetDevicesCallback callback) override;
  void GetPort(const base::UnguessableToken& token, SerialPortRequest port_request) override;
};
class  SerialPortManagerAsyncWaiter {
 public:
  explicit SerialPortManagerAsyncWaiter(SerialPortManager* proxy);
  ~SerialPortManagerAsyncWaiter();
  void GetDevices(
      std::vector<SerialPortInfoPtr>* out_devices);

 private:
  SerialPortManager* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(SerialPortManagerAsyncWaiter);
};


class  SerialPortInterceptorForTesting : public SerialPort {
  virtual SerialPort* GetForwardingInterface() = 0;
  void Open(SerialConnectionOptionsPtr options, OpenCallback callback) override;
  void Read(uint32_t bytes, ReadCallback callback) override;
  void Write(const std::vector<uint8_t>& data, WriteCallback callback) override;
  void CancelRead(SerialReceiveError reason) override;
  void CancelWrite(SerialSendError reason) override;
  void Flush(FlushCallback callback) override;
  void GetControlSignals(GetControlSignalsCallback callback) override;
  void SetControlSignals(SerialHostControlSignalsPtr signals, SetControlSignalsCallback callback) override;
  void ConfigurePort(SerialConnectionOptionsPtr options, ConfigurePortCallback callback) override;
  void GetPortInfo(GetPortInfoCallback callback) override;
  void SetBreak(SetBreakCallback callback) override;
  void ClearBreak(ClearBreakCallback callback) override;
};
class  SerialPortAsyncWaiter {
 public:
  explicit SerialPortAsyncWaiter(SerialPort* proxy);
  ~SerialPortAsyncWaiter();
  void Open(
      SerialConnectionOptionsPtr options, bool* out_success);
  void Read(
      uint32_t bytes, std::vector<uint8_t>* out_data, SerialReceiveError* out_error);
  void Write(
      const std::vector<uint8_t>& data, uint32_t* out_bytes_written, SerialSendError* out_error);
  void Flush(
      bool* out_success);
  void GetControlSignals(
      SerialPortControlSignalsPtr* out_signals);
  void SetControlSignals(
      SerialHostControlSignalsPtr signals, bool* out_success);
  void ConfigurePort(
      SerialConnectionOptionsPtr options, bool* out_success);
  void GetPortInfo(
      SerialConnectionInfoPtr* out_info);
  void SetBreak(
      bool* out_success);
  void ClearBreak(
      bool* out_success);

 private:
  SerialPort* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(SerialPortAsyncWaiter);
};




}  // namespace mojom
}  // namespace device

#endif  // SERVICES_DEVICE_PUBLIC_MOJOM_SERIAL_MOJOM_TEST_UTILS_H_