// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_MEDIA_SESSION_PUBLIC_MOJOM_AUDIO_FOCUS_MOJOM_SHARED_H_
#define SERVICES_MEDIA_SESSION_PUBLIC_MOJOM_AUDIO_FOCUS_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "services/media_session/public/mojom/audio_focus.mojom-shared-internal.h"
#include "mojo/public/mojom/base/unguessable_token.mojom-shared.h"
#include "services/media_session/public/mojom/media_session.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace media_session {
namespace mojom {
class AudioFocusRequestStateDataView;



}  // namespace mojom
}  // namespace media_session

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::media_session::mojom::AudioFocusRequestStateDataView> {
  using Data = ::media_session::mojom::internal::AudioFocusRequestState_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace media_session {
namespace mojom {

enum class EnforcementMode : int32_t {
  kDefault,
  kNone,
  kSingleGroup,
  kSingleSession,
  kMinValue = 0,
  kMaxValue = 3,
};

 std::ostream& operator<<(std::ostream& os, EnforcementMode value);
inline bool IsKnownEnumValue(EnforcementMode value) {
  return internal::EnforcementMode_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class AudioFocusType : int32_t {
  kGain,
  kGainTransientMayDuck,
  kGainTransient,
  kMinValue = 0,
  kMaxValue = 2,
};

 std::ostream& operator<<(std::ostream& os, AudioFocusType value);
inline bool IsKnownEnumValue(AudioFocusType value) {
  return internal::AudioFocusType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class AudioFocusObserverInterfaceBase {};

using AudioFocusObserverPtrDataView =
    mojo::InterfacePtrDataView<AudioFocusObserverInterfaceBase>;
using AudioFocusObserverRequestDataView =
    mojo::InterfaceRequestDataView<AudioFocusObserverInterfaceBase>;
using AudioFocusObserverAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<AudioFocusObserverInterfaceBase>;
using AudioFocusObserverAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<AudioFocusObserverInterfaceBase>;
class AudioFocusRequestClientInterfaceBase {};

using AudioFocusRequestClientPtrDataView =
    mojo::InterfacePtrDataView<AudioFocusRequestClientInterfaceBase>;
using AudioFocusRequestClientRequestDataView =
    mojo::InterfaceRequestDataView<AudioFocusRequestClientInterfaceBase>;
using AudioFocusRequestClientAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<AudioFocusRequestClientInterfaceBase>;
using AudioFocusRequestClientAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<AudioFocusRequestClientInterfaceBase>;
class AudioFocusManagerInterfaceBase {};

using AudioFocusManagerPtrDataView =
    mojo::InterfacePtrDataView<AudioFocusManagerInterfaceBase>;
using AudioFocusManagerRequestDataView =
    mojo::InterfaceRequestDataView<AudioFocusManagerInterfaceBase>;
using AudioFocusManagerAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<AudioFocusManagerInterfaceBase>;
using AudioFocusManagerAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<AudioFocusManagerInterfaceBase>;
class AudioFocusManagerDebugInterfaceBase {};

using AudioFocusManagerDebugPtrDataView =
    mojo::InterfacePtrDataView<AudioFocusManagerDebugInterfaceBase>;
using AudioFocusManagerDebugRequestDataView =
    mojo::InterfaceRequestDataView<AudioFocusManagerDebugInterfaceBase>;
using AudioFocusManagerDebugAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<AudioFocusManagerDebugInterfaceBase>;
using AudioFocusManagerDebugAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<AudioFocusManagerDebugInterfaceBase>;
class AudioFocusRequestStateDataView {
 public:
  AudioFocusRequestStateDataView() {}

  AudioFocusRequestStateDataView(
      internal::AudioFocusRequestState_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetSessionInfoDataView(
      ::media_session::mojom::MediaSessionInfoDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSessionInfo(UserType* output) {
    auto* pointer = data_->session_info.Get();
    return mojo::internal::Deserialize<::media_session::mojom::MediaSessionInfoDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAudioFocusType(UserType* output) const {
    auto data_value = data_->audio_focus_type;
    return mojo::internal::Deserialize<::media_session::mojom::AudioFocusType>(
        data_value, output);
  }

  AudioFocusType audio_focus_type() const {
    return static_cast<AudioFocusType>(data_->audio_focus_type);
  }
  inline void GetSourceNameDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSourceName(UserType* output) {
    auto* pointer = data_->header_.version >= 2
                    ? data_->source_name.Get() : nullptr;
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetRequestIdDataView(
      ::mojo_base::mojom::UnguessableTokenDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRequestId(UserType* output) {
    auto* pointer = data_->header_.version >= 3
                    ? data_->request_id.Get() : nullptr;
    return mojo::internal::Deserialize<::mojo_base::mojom::UnguessableTokenDataView>(
        pointer, output, context_);
  }
 private:
  internal::AudioFocusRequestState_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AudioFocusObserver_OnFocusGained_ParamsDataView {
 public:
  AudioFocusObserver_OnFocusGained_ParamsDataView() {}

  AudioFocusObserver_OnFocusGained_ParamsDataView(
      internal::AudioFocusObserver_OnFocusGained_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetStateDataView(
      AudioFocusRequestStateDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadState(UserType* output) {
    auto* pointer = data_->state.Get();
    return mojo::internal::Deserialize<::media_session::mojom::AudioFocusRequestStateDataView>(
        pointer, output, context_);
  }
 private:
  internal::AudioFocusObserver_OnFocusGained_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AudioFocusObserver_OnFocusLost_ParamsDataView {
 public:
  AudioFocusObserver_OnFocusLost_ParamsDataView() {}

  AudioFocusObserver_OnFocusLost_ParamsDataView(
      internal::AudioFocusObserver_OnFocusLost_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetStateDataView(
      AudioFocusRequestStateDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadState(UserType* output) {
    auto* pointer = data_->state.Get();
    return mojo::internal::Deserialize<::media_session::mojom::AudioFocusRequestStateDataView>(
        pointer, output, context_);
  }
 private:
  internal::AudioFocusObserver_OnFocusLost_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AudioFocusObserver_OnActiveSessionChanged_ParamsDataView {
 public:
  AudioFocusObserver_OnActiveSessionChanged_ParamsDataView() {}

  AudioFocusObserver_OnActiveSessionChanged_ParamsDataView(
      internal::AudioFocusObserver_OnActiveSessionChanged_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetSessionDataView(
      AudioFocusRequestStateDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSession(UserType* output) {
    auto* pointer = data_->session.Get();
    return mojo::internal::Deserialize<::media_session::mojom::AudioFocusRequestStateDataView>(
        pointer, output, context_);
  }
 private:
  internal::AudioFocusObserver_OnActiveSessionChanged_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AudioFocusRequestClient_RequestAudioFocus_ParamsDataView {
 public:
  AudioFocusRequestClient_RequestAudioFocus_ParamsDataView() {}

  AudioFocusRequestClient_RequestAudioFocus_ParamsDataView(
      internal::AudioFocusRequestClient_RequestAudioFocus_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetSessionInfoDataView(
      ::media_session::mojom::MediaSessionInfoDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSessionInfo(UserType* output) {
    auto* pointer = data_->session_info.Get();
    return mojo::internal::Deserialize<::media_session::mojom::MediaSessionInfoDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::media_session::mojom::AudioFocusType>(
        data_value, output);
  }

  AudioFocusType type() const {
    return static_cast<AudioFocusType>(data_->type);
  }
 private:
  internal::AudioFocusRequestClient_RequestAudioFocus_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AudioFocusRequestClient_RequestAudioFocus_ResponseParamsDataView {
 public:
  AudioFocusRequestClient_RequestAudioFocus_ResponseParamsDataView() {}

  AudioFocusRequestClient_RequestAudioFocus_ResponseParamsDataView(
      internal::AudioFocusRequestClient_RequestAudioFocus_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::AudioFocusRequestClient_RequestAudioFocus_ResponseParams_Data* data_ = nullptr;
};

class AudioFocusRequestClient_AbandonAudioFocus_ParamsDataView {
 public:
  AudioFocusRequestClient_AbandonAudioFocus_ParamsDataView() {}

  AudioFocusRequestClient_AbandonAudioFocus_ParamsDataView(
      internal::AudioFocusRequestClient_AbandonAudioFocus_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::AudioFocusRequestClient_AbandonAudioFocus_Params_Data* data_ = nullptr;
};

class AudioFocusRequestClient_MediaSessionInfoChanged_ParamsDataView {
 public:
  AudioFocusRequestClient_MediaSessionInfoChanged_ParamsDataView() {}

  AudioFocusRequestClient_MediaSessionInfoChanged_ParamsDataView(
      internal::AudioFocusRequestClient_MediaSessionInfoChanged_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetSessionInfoDataView(
      ::media_session::mojom::MediaSessionInfoDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSessionInfo(UserType* output) {
    auto* pointer = data_->session_info.Get();
    return mojo::internal::Deserialize<::media_session::mojom::MediaSessionInfoDataView>(
        pointer, output, context_);
  }
 private:
  internal::AudioFocusRequestClient_MediaSessionInfoChanged_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AudioFocusRequestClient_GetRequestId_ParamsDataView {
 public:
  AudioFocusRequestClient_GetRequestId_ParamsDataView() {}

  AudioFocusRequestClient_GetRequestId_ParamsDataView(
      internal::AudioFocusRequestClient_GetRequestId_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::AudioFocusRequestClient_GetRequestId_Params_Data* data_ = nullptr;
};

class AudioFocusRequestClient_GetRequestId_ResponseParamsDataView {
 public:
  AudioFocusRequestClient_GetRequestId_ResponseParamsDataView() {}

  AudioFocusRequestClient_GetRequestId_ResponseParamsDataView(
      internal::AudioFocusRequestClient_GetRequestId_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetRequestIdDataView(
      ::mojo_base::mojom::UnguessableTokenDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRequestId(UserType* output) {
    auto* pointer = data_->request_id.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::UnguessableTokenDataView>(
        pointer, output, context_);
  }
 private:
  internal::AudioFocusRequestClient_GetRequestId_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AudioFocusManager_RequestAudioFocus_ParamsDataView {
 public:
  AudioFocusManager_RequestAudioFocus_ParamsDataView() {}

  AudioFocusManager_RequestAudioFocus_ParamsDataView(
      internal::AudioFocusManager_RequestAudioFocus_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeClient() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::media_session::mojom::AudioFocusRequestClientRequestDataView>(
            &data_->client, &result, context_);
    DCHECK(ret);
    return result;
  }
  template <typename UserType>
  UserType TakeMediaSession() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::media_session::mojom::MediaSessionPtrDataView>(
            &data_->media_session, &result, context_);
    DCHECK(ret);
    return result;
  }
  inline void GetSessionInfoDataView(
      ::media_session::mojom::MediaSessionInfoDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSessionInfo(UserType* output) {
    auto* pointer = data_->session_info.Get();
    return mojo::internal::Deserialize<::media_session::mojom::MediaSessionInfoDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::media_session::mojom::AudioFocusType>(
        data_value, output);
  }

  AudioFocusType type() const {
    return static_cast<AudioFocusType>(data_->type);
  }
 private:
  internal::AudioFocusManager_RequestAudioFocus_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AudioFocusManager_RequestAudioFocus_ResponseParamsDataView {
 public:
  AudioFocusManager_RequestAudioFocus_ResponseParamsDataView() {}

  AudioFocusManager_RequestAudioFocus_ResponseParamsDataView(
      internal::AudioFocusManager_RequestAudioFocus_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::AudioFocusManager_RequestAudioFocus_ResponseParams_Data* data_ = nullptr;
};

class AudioFocusManager_RequestGroupedAudioFocus_ParamsDataView {
 public:
  AudioFocusManager_RequestGroupedAudioFocus_ParamsDataView() {}

  AudioFocusManager_RequestGroupedAudioFocus_ParamsDataView(
      internal::AudioFocusManager_RequestGroupedAudioFocus_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeClient() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::media_session::mojom::AudioFocusRequestClientRequestDataView>(
            &data_->client, &result, context_);
    DCHECK(ret);
    return result;
  }
  template <typename UserType>
  UserType TakeMediaSession() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::media_session::mojom::MediaSessionPtrDataView>(
            &data_->media_session, &result, context_);
    DCHECK(ret);
    return result;
  }
  inline void GetSessionInfoDataView(
      ::media_session::mojom::MediaSessionInfoDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSessionInfo(UserType* output) {
    auto* pointer = data_->session_info.Get();
    return mojo::internal::Deserialize<::media_session::mojom::MediaSessionInfoDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::media_session::mojom::AudioFocusType>(
        data_value, output);
  }

  AudioFocusType type() const {
    return static_cast<AudioFocusType>(data_->type);
  }
  inline void GetGroupIdDataView(
      ::mojo_base::mojom::UnguessableTokenDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadGroupId(UserType* output) {
    auto* pointer = data_->group_id.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::UnguessableTokenDataView>(
        pointer, output, context_);
  }
 private:
  internal::AudioFocusManager_RequestGroupedAudioFocus_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AudioFocusManager_RequestGroupedAudioFocus_ResponseParamsDataView {
 public:
  AudioFocusManager_RequestGroupedAudioFocus_ResponseParamsDataView() {}

  AudioFocusManager_RequestGroupedAudioFocus_ResponseParamsDataView(
      internal::AudioFocusManager_RequestGroupedAudioFocus_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::AudioFocusManager_RequestGroupedAudioFocus_ResponseParams_Data* data_ = nullptr;
};

class AudioFocusManager_GetFocusRequests_ParamsDataView {
 public:
  AudioFocusManager_GetFocusRequests_ParamsDataView() {}

  AudioFocusManager_GetFocusRequests_ParamsDataView(
      internal::AudioFocusManager_GetFocusRequests_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::AudioFocusManager_GetFocusRequests_Params_Data* data_ = nullptr;
};

class AudioFocusManager_GetFocusRequests_ResponseParamsDataView {
 public:
  AudioFocusManager_GetFocusRequests_ResponseParamsDataView() {}

  AudioFocusManager_GetFocusRequests_ResponseParamsDataView(
      internal::AudioFocusManager_GetFocusRequests_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetRequestsDataView(
      mojo::ArrayDataView<AudioFocusRequestStateDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRequests(UserType* output) {
    auto* pointer = data_->requests.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::media_session::mojom::AudioFocusRequestStateDataView>>(
        pointer, output, context_);
  }
 private:
  internal::AudioFocusManager_GetFocusRequests_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AudioFocusManager_AddObserver_ParamsDataView {
 public:
  AudioFocusManager_AddObserver_ParamsDataView() {}

  AudioFocusManager_AddObserver_ParamsDataView(
      internal::AudioFocusManager_AddObserver_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeObserver() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::media_session::mojom::AudioFocusObserverPtrDataView>(
            &data_->observer, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::AudioFocusManager_AddObserver_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AudioFocusManager_SetSourceName_ParamsDataView {
 public:
  AudioFocusManager_SetSourceName_ParamsDataView() {}

  AudioFocusManager_SetSourceName_ParamsDataView(
      internal::AudioFocusManager_SetSourceName_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetNameDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadName(UserType* output) {
    auto* pointer = data_->name.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::AudioFocusManager_SetSourceName_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AudioFocusManager_SetEnforcementMode_ParamsDataView {
 public:
  AudioFocusManager_SetEnforcementMode_ParamsDataView() {}

  AudioFocusManager_SetEnforcementMode_ParamsDataView(
      internal::AudioFocusManager_SetEnforcementMode_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadMode(UserType* output) const {
    auto data_value = data_->mode;
    return mojo::internal::Deserialize<::media_session::mojom::EnforcementMode>(
        data_value, output);
  }

  EnforcementMode mode() const {
    return static_cast<EnforcementMode>(data_->mode);
  }
 private:
  internal::AudioFocusManager_SetEnforcementMode_Params_Data* data_ = nullptr;
};

class AudioFocusManagerDebug_GetDebugInfoForRequest_ParamsDataView {
 public:
  AudioFocusManagerDebug_GetDebugInfoForRequest_ParamsDataView() {}

  AudioFocusManagerDebug_GetDebugInfoForRequest_ParamsDataView(
      internal::AudioFocusManagerDebug_GetDebugInfoForRequest_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetRequestIdDataView(
      ::mojo_base::mojom::UnguessableTokenDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRequestId(UserType* output) {
    auto* pointer = data_->request_id.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::UnguessableTokenDataView>(
        pointer, output, context_);
  }
 private:
  internal::AudioFocusManagerDebug_GetDebugInfoForRequest_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AudioFocusManagerDebug_GetDebugInfoForRequest_ResponseParamsDataView {
 public:
  AudioFocusManagerDebug_GetDebugInfoForRequest_ResponseParamsDataView() {}

  AudioFocusManagerDebug_GetDebugInfoForRequest_ResponseParamsDataView(
      internal::AudioFocusManagerDebug_GetDebugInfoForRequest_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetDebugInfoDataView(
      ::media_session::mojom::MediaSessionDebugInfoDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDebugInfo(UserType* output) {
    auto* pointer = data_->debug_info.Get();
    return mojo::internal::Deserialize<::media_session::mojom::MediaSessionDebugInfoDataView>(
        pointer, output, context_);
  }
 private:
  internal::AudioFocusManagerDebug_GetDebugInfoForRequest_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace media_session

namespace std {

template <>
struct hash<::media_session::mojom::EnforcementMode>
    : public mojo::internal::EnumHashImpl<::media_session::mojom::EnforcementMode> {};

template <>
struct hash<::media_session::mojom::AudioFocusType>
    : public mojo::internal::EnumHashImpl<::media_session::mojom::AudioFocusType> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::media_session::mojom::EnforcementMode, ::media_session::mojom::EnforcementMode> {
  static ::media_session::mojom::EnforcementMode ToMojom(::media_session::mojom::EnforcementMode input) { return input; }
  static bool FromMojom(::media_session::mojom::EnforcementMode input, ::media_session::mojom::EnforcementMode* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::media_session::mojom::EnforcementMode, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::media_session::mojom::EnforcementMode, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::media_session::mojom::EnforcementMode>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::media_session::mojom::AudioFocusType, ::media_session::mojom::AudioFocusType> {
  static ::media_session::mojom::AudioFocusType ToMojom(::media_session::mojom::AudioFocusType input) { return input; }
  static bool FromMojom(::media_session::mojom::AudioFocusType input, ::media_session::mojom::AudioFocusType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::media_session::mojom::AudioFocusType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::media_session::mojom::AudioFocusType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::media_session::mojom::AudioFocusType>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::media_session::mojom::AudioFocusRequestStateDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::media_session::mojom::AudioFocusRequestStateDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::media_session::mojom::internal::AudioFocusRequestState_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    decltype(CallWithContext(Traits::session_info, input, custom_context)) in_session_info = CallWithContext(Traits::session_info, input, custom_context);
    typename decltype((*output)->session_info)::BaseType::BufferWriter
        session_info_writer;
    mojo::internal::Serialize<::media_session::mojom::MediaSessionInfoDataView>(
        in_session_info, buffer, &session_info_writer, context);
    (*output)->session_info.Set(
        session_info_writer.is_null() ? nullptr : session_info_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->session_info.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null session_info in AudioFocusRequestState struct");
    mojo::internal::Serialize<::media_session::mojom::AudioFocusType>(
        CallWithContext(Traits::audio_focus_type, input, custom_context), &(*output)->audio_focus_type);
    decltype(CallWithContext(Traits::source_name, input, custom_context)) in_source_name = CallWithContext(Traits::source_name, input, custom_context);
    typename decltype((*output)->source_name)::BaseType::BufferWriter
        source_name_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_source_name, buffer, &source_name_writer, context);
    (*output)->source_name.Set(
        source_name_writer.is_null() ? nullptr : source_name_writer.data());
    decltype(CallWithContext(Traits::request_id, input, custom_context)) in_request_id = CallWithContext(Traits::request_id, input, custom_context);
    typename decltype((*output)->request_id)::BaseType::BufferWriter
        request_id_writer;
    mojo::internal::Serialize<::mojo_base::mojom::UnguessableTokenDataView>(
        in_request_id, buffer, &request_id_writer, context);
    (*output)->request_id.Set(
        request_id_writer.is_null() ? nullptr : request_id_writer.data());
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::media_session::mojom::internal::AudioFocusRequestState_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::media_session::mojom::AudioFocusRequestStateDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace media_session {
namespace mojom {

inline void AudioFocusRequestStateDataView::GetSessionInfoDataView(
    ::media_session::mojom::MediaSessionInfoDataView* output) {
  auto pointer = data_->session_info.Get();
  *output = ::media_session::mojom::MediaSessionInfoDataView(pointer, context_);
}
inline void AudioFocusRequestStateDataView::GetSourceNameDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->header_.version >= 2
                 ? data_->source_name.Get() : nullptr;
  *output = mojo::StringDataView(pointer, context_);
}
inline void AudioFocusRequestStateDataView::GetRequestIdDataView(
    ::mojo_base::mojom::UnguessableTokenDataView* output) {
  auto pointer = data_->header_.version >= 3
                 ? data_->request_id.Get() : nullptr;
  *output = ::mojo_base::mojom::UnguessableTokenDataView(pointer, context_);
}


inline void AudioFocusObserver_OnFocusGained_ParamsDataView::GetStateDataView(
    AudioFocusRequestStateDataView* output) {
  auto pointer = data_->state.Get();
  *output = AudioFocusRequestStateDataView(pointer, context_);
}


inline void AudioFocusObserver_OnFocusLost_ParamsDataView::GetStateDataView(
    AudioFocusRequestStateDataView* output) {
  auto pointer = data_->state.Get();
  *output = AudioFocusRequestStateDataView(pointer, context_);
}


inline void AudioFocusObserver_OnActiveSessionChanged_ParamsDataView::GetSessionDataView(
    AudioFocusRequestStateDataView* output) {
  auto pointer = data_->session.Get();
  *output = AudioFocusRequestStateDataView(pointer, context_);
}


inline void AudioFocusRequestClient_RequestAudioFocus_ParamsDataView::GetSessionInfoDataView(
    ::media_session::mojom::MediaSessionInfoDataView* output) {
  auto pointer = data_->session_info.Get();
  *output = ::media_session::mojom::MediaSessionInfoDataView(pointer, context_);
}






inline void AudioFocusRequestClient_MediaSessionInfoChanged_ParamsDataView::GetSessionInfoDataView(
    ::media_session::mojom::MediaSessionInfoDataView* output) {
  auto pointer = data_->session_info.Get();
  *output = ::media_session::mojom::MediaSessionInfoDataView(pointer, context_);
}




inline void AudioFocusRequestClient_GetRequestId_ResponseParamsDataView::GetRequestIdDataView(
    ::mojo_base::mojom::UnguessableTokenDataView* output) {
  auto pointer = data_->request_id.Get();
  *output = ::mojo_base::mojom::UnguessableTokenDataView(pointer, context_);
}


inline void AudioFocusManager_RequestAudioFocus_ParamsDataView::GetSessionInfoDataView(
    ::media_session::mojom::MediaSessionInfoDataView* output) {
  auto pointer = data_->session_info.Get();
  *output = ::media_session::mojom::MediaSessionInfoDataView(pointer, context_);
}




inline void AudioFocusManager_RequestGroupedAudioFocus_ParamsDataView::GetSessionInfoDataView(
    ::media_session::mojom::MediaSessionInfoDataView* output) {
  auto pointer = data_->session_info.Get();
  *output = ::media_session::mojom::MediaSessionInfoDataView(pointer, context_);
}
inline void AudioFocusManager_RequestGroupedAudioFocus_ParamsDataView::GetGroupIdDataView(
    ::mojo_base::mojom::UnguessableTokenDataView* output) {
  auto pointer = data_->group_id.Get();
  *output = ::mojo_base::mojom::UnguessableTokenDataView(pointer, context_);
}






inline void AudioFocusManager_GetFocusRequests_ResponseParamsDataView::GetRequestsDataView(
    mojo::ArrayDataView<AudioFocusRequestStateDataView>* output) {
  auto pointer = data_->requests.Get();
  *output = mojo::ArrayDataView<AudioFocusRequestStateDataView>(pointer, context_);
}




inline void AudioFocusManager_SetSourceName_ParamsDataView::GetNameDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->name.Get();
  *output = mojo::StringDataView(pointer, context_);
}




inline void AudioFocusManagerDebug_GetDebugInfoForRequest_ParamsDataView::GetRequestIdDataView(
    ::mojo_base::mojom::UnguessableTokenDataView* output) {
  auto pointer = data_->request_id.Get();
  *output = ::mojo_base::mojom::UnguessableTokenDataView(pointer, context_);
}


inline void AudioFocusManagerDebug_GetDebugInfoForRequest_ResponseParamsDataView::GetDebugInfoDataView(
    ::media_session::mojom::MediaSessionDebugInfoDataView* output) {
  auto pointer = data_->debug_info.Get();
  *output = ::media_session::mojom::MediaSessionDebugInfoDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace media_session

#endif  // SERVICES_MEDIA_SESSION_PUBLIC_MOJOM_AUDIO_FOCUS_MOJOM_SHARED_H_