// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_MEDIA_SESSION_PUBLIC_MOJOM_MEDIA_CONTROLLER_MOJOM_H_
#define SERVICES_MEDIA_SESSION_PUBLIC_MOJOM_MEDIA_CONTROLLER_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/media_session/public/mojom/media_controller.mojom-shared.h"
#include "mojo/public/mojom/base/time.mojom.h"
#include "mojo/public/mojom/base/unguessable_token.mojom.h"
#include "services/media_session/public/mojom/media_session.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace media_session {
namespace mojom {
class MediaControllerManager;
using MediaControllerManagerPtr = mojo::InterfacePtr<MediaControllerManager>;
using MediaControllerManagerPtrInfo = mojo::InterfacePtrInfo<MediaControllerManager>;
using ThreadSafeMediaControllerManagerPtr =
    mojo::ThreadSafeInterfacePtr<MediaControllerManager>;
using MediaControllerManagerRequest = mojo::InterfaceRequest<MediaControllerManager>;
using MediaControllerManagerAssociatedPtr =
    mojo::AssociatedInterfacePtr<MediaControllerManager>;
using ThreadSafeMediaControllerManagerAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<MediaControllerManager>;
using MediaControllerManagerAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<MediaControllerManager>;
using MediaControllerManagerAssociatedRequest =
    mojo::AssociatedInterfaceRequest<MediaControllerManager>;

class MediaController;
using MediaControllerPtr = mojo::InterfacePtr<MediaController>;
using MediaControllerPtrInfo = mojo::InterfacePtrInfo<MediaController>;
using ThreadSafeMediaControllerPtr =
    mojo::ThreadSafeInterfacePtr<MediaController>;
using MediaControllerRequest = mojo::InterfaceRequest<MediaController>;
using MediaControllerAssociatedPtr =
    mojo::AssociatedInterfacePtr<MediaController>;
using ThreadSafeMediaControllerAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<MediaController>;
using MediaControllerAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<MediaController>;
using MediaControllerAssociatedRequest =
    mojo::AssociatedInterfaceRequest<MediaController>;

class MediaControllerObserver;
using MediaControllerObserverPtr = mojo::InterfacePtr<MediaControllerObserver>;
using MediaControllerObserverPtrInfo = mojo::InterfacePtrInfo<MediaControllerObserver>;
using ThreadSafeMediaControllerObserverPtr =
    mojo::ThreadSafeInterfacePtr<MediaControllerObserver>;
using MediaControllerObserverRequest = mojo::InterfaceRequest<MediaControllerObserver>;
using MediaControllerObserverAssociatedPtr =
    mojo::AssociatedInterfacePtr<MediaControllerObserver>;
using ThreadSafeMediaControllerObserverAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<MediaControllerObserver>;
using MediaControllerObserverAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<MediaControllerObserver>;
using MediaControllerObserverAssociatedRequest =
    mojo::AssociatedInterfaceRequest<MediaControllerObserver>;


class MediaControllerManagerProxy;

template <typename ImplRefTraits>
class MediaControllerManagerStub;

class MediaControllerManagerRequestValidator;

class  MediaControllerManager
    : public MediaControllerManagerInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = MediaControllerManagerProxy;

  template <typename ImplRefTraits>
  using Stub_ = MediaControllerManagerStub<ImplRefTraits>;

  using RequestValidator_ = MediaControllerManagerRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kCreateMediaControllerForSessionMinVersion = 0,
    kCreateActiveMediaControllerMinVersion = 0,
  };
  virtual ~MediaControllerManager() {}

  virtual void CreateMediaControllerForSession(MediaControllerRequest request, const base::UnguessableToken& request_id) = 0;

  virtual void CreateActiveMediaController(MediaControllerRequest request) = 0;
};

class MediaControllerProxy;

template <typename ImplRefTraits>
class MediaControllerStub;

class MediaControllerRequestValidator;

class  MediaController
    : public MediaControllerInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = MediaControllerProxy;

  template <typename ImplRefTraits>
  using Stub_ = MediaControllerStub<ImplRefTraits>;

  using RequestValidator_ = MediaControllerRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kSuspendMinVersion = 0,
    kResumeMinVersion = 0,
    kStopMinVersion = 0,
    kToggleSuspendResumeMinVersion = 0,
    kAddObserverMinVersion = 0,
    kPreviousTrackMinVersion = 0,
    kNextTrackMinVersion = 0,
    kSeekMinVersion = 0,
  };
  virtual ~MediaController() {}

  virtual void Suspend() = 0;

  virtual void Resume() = 0;

  virtual void Stop() = 0;

  virtual void ToggleSuspendResume() = 0;

  virtual void AddObserver(MediaControllerObserverPtr observer) = 0;

  virtual void PreviousTrack() = 0;

  virtual void NextTrack() = 0;

  virtual void Seek(base::TimeDelta seek_time) = 0;
};

class MediaControllerObserverProxy;

template <typename ImplRefTraits>
class MediaControllerObserverStub;

class MediaControllerObserverRequestValidator;

class  MediaControllerObserver
    : public MediaControllerObserverInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = MediaControllerObserverProxy;

  template <typename ImplRefTraits>
  using Stub_ = MediaControllerObserverStub<ImplRefTraits>;

  using RequestValidator_ = MediaControllerObserverRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kMediaSessionInfoChangedMinVersion = 0,
    kMediaSessionMetadataChangedMinVersion = 0,
    kMediaSessionActionsChangedMinVersion = 0,
  };
  virtual ~MediaControllerObserver() {}

  virtual void MediaSessionInfoChanged(::media_session::mojom::MediaSessionInfoPtr info) = 0;

  virtual void MediaSessionMetadataChanged(const base::Optional<media_session::MediaMetadata>& metadata) = 0;

  virtual void MediaSessionActionsChanged(const std::vector<::media_session::mojom::MediaSessionAction>& action) = 0;
};

class  MediaControllerManagerProxy
    : public MediaControllerManager {
 public:
  explicit MediaControllerManagerProxy(mojo::MessageReceiverWithResponder* receiver);
  void CreateMediaControllerForSession(MediaControllerRequest request, const base::UnguessableToken& request_id) final;
  void CreateActiveMediaController(MediaControllerRequest request) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  MediaControllerProxy
    : public MediaController {
 public:
  explicit MediaControllerProxy(mojo::MessageReceiverWithResponder* receiver);
  void Suspend() final;
  void Resume() final;
  void Stop() final;
  void ToggleSuspendResume() final;
  void AddObserver(MediaControllerObserverPtr observer) final;
  void PreviousTrack() final;
  void NextTrack() final;
  void Seek(base::TimeDelta seek_time) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  MediaControllerObserverProxy
    : public MediaControllerObserver {
 public:
  explicit MediaControllerObserverProxy(mojo::MessageReceiverWithResponder* receiver);
  void MediaSessionInfoChanged(::media_session::mojom::MediaSessionInfoPtr info) final;
  void MediaSessionMetadataChanged(const base::Optional<media_session::MediaMetadata>& metadata) final;
  void MediaSessionActionsChanged(const std::vector<::media_session::mojom::MediaSessionAction>& action) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  MediaControllerManagerStubDispatch {
 public:
  static bool Accept(MediaControllerManager* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      MediaControllerManager* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<MediaControllerManager>>
class MediaControllerManagerStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  MediaControllerManagerStub() {}
  ~MediaControllerManagerStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return MediaControllerManagerStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return MediaControllerManagerStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  MediaControllerStubDispatch {
 public:
  static bool Accept(MediaController* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      MediaController* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<MediaController>>
class MediaControllerStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  MediaControllerStub() {}
  ~MediaControllerStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return MediaControllerStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return MediaControllerStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  MediaControllerObserverStubDispatch {
 public:
  static bool Accept(MediaControllerObserver* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      MediaControllerObserver* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<MediaControllerObserver>>
class MediaControllerObserverStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  MediaControllerObserverStub() {}
  ~MediaControllerObserverStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return MediaControllerObserverStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return MediaControllerObserverStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  MediaControllerManagerRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  MediaControllerRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  MediaControllerObserverRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace media_session

namespace mojo {

}  // namespace mojo

#endif  // SERVICES_MEDIA_SESSION_PUBLIC_MOJOM_MEDIA_CONTROLLER_MOJOM_H_