// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_MEDIA_SESSION_PUBLIC_MOJOM_MEDIA_SESSION_MOJOM_SHARED_INTERNAL_H_
#define SERVICES_MEDIA_SESSION_PUBLIC_MOJOM_MEDIA_SESSION_MOJOM_SHARED_INTERNAL_H_

#include "mojo/public/cpp/bindings/lib/array_internal.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/map_data_internal.h"
#include "mojo/public/cpp/bindings/lib/buffer.h"
#include "mojo/public/mojom/base/string16.mojom-shared-internal.h"
#include "mojo/public/mojom/base/time.mojom-shared-internal.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-shared-internal.h"
#include "url/mojom/url.mojom-shared-internal.h"
#include "mojo/public/cpp/bindings/lib/native_enum_data.h"
#include "mojo/public/interfaces/bindings/native_struct.mojom-shared-internal.h"

namespace mojo {
namespace internal {
class ValidationContext;
}
}
namespace media_session {
namespace mojom {
namespace internal {
class MediaImage_Data;
class MediaMetadata_Data;
class MediaSessionInfo_Data;
class MediaSessionDebugInfo_Data;

struct MediaPlaybackState_Data {
 public:
  static bool constexpr kIsExtensible = true;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

struct MediaSessionAction_Data {
 public:
  static bool constexpr kIsExtensible = true;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
      case 2:
      case 3:
      case 4:
      case 5:
      case 6:
      case 7:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

struct MediaSessionInfo_SessionState_Data {
 public:
  static bool constexpr kIsExtensible = true;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
      case 2:
      case 3:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

struct MediaSession_SuspendType_Data {
 public:
  static bool constexpr kIsExtensible = true;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
      case 2:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

#pragma pack(push, 1)
class  MediaImage_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaImage_Data));
      new (data()) MediaImage_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaImage_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaImage_Data>(index_);
    }
    MediaImage_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> src;
  mojo::internal::Pointer<::mojo_base::mojom::internal::String16_Data> type;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<::gfx::mojom::internal::Size_Data>>> sizes;

 private:
  MediaImage_Data();
  ~MediaImage_Data() = delete;
};
static_assert(sizeof(MediaImage_Data) == 32,
              "Bad sizeof(MediaImage_Data)");
// Used by MediaImage::WrapAsMessage to lazily serialize the struct.
template <typename UserType, typename DataView>
struct MediaImage_UnserializedMessageContext
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  MediaImage_UnserializedMessageContext(
    uint32_t message_name,
    uint32_t message_flags,
    UserType input)
      : mojo::internal::UnserializedMessageContext(&kMessageTag, message_name, message_flags)
      , user_data_(std::move(input)) {}
  ~MediaImage_UnserializedMessageContext() override = default;

  UserType TakeData() {
    return std::move(user_data_);
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* context,
                 mojo::internal::Buffer* buffer) override {
    MediaImage_Data::BufferWriter writer;
    mojo::internal::Serialize<DataView>(user_data_, buffer, &writer, context);
  }

  UserType user_data_;
};

template <typename UserType, typename DataView>
const mojo::internal::UnserializedMessageContext::Tag
    MediaImage_UnserializedMessageContext<UserType, DataView>::kMessageTag = {};
class  MediaMetadata_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaMetadata_Data));
      new (data()) MediaMetadata_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaMetadata_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaMetadata_Data>(index_);
    }
    MediaMetadata_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::mojo_base::mojom::internal::String16_Data> title;
  mojo::internal::Pointer<::mojo_base::mojom::internal::String16_Data> artist;
  mojo::internal::Pointer<::mojo_base::mojom::internal::String16_Data> album;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<internal::MediaImage_Data>>> artwork;
  mojo::internal::Pointer<::mojo_base::mojom::internal::String16_Data> source_title;

 private:
  MediaMetadata_Data();
  ~MediaMetadata_Data() = delete;
};
static_assert(sizeof(MediaMetadata_Data) == 48,
              "Bad sizeof(MediaMetadata_Data)");
// Used by MediaMetadata::WrapAsMessage to lazily serialize the struct.
template <typename UserType, typename DataView>
struct MediaMetadata_UnserializedMessageContext
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  MediaMetadata_UnserializedMessageContext(
    uint32_t message_name,
    uint32_t message_flags,
    UserType input)
      : mojo::internal::UnserializedMessageContext(&kMessageTag, message_name, message_flags)
      , user_data_(std::move(input)) {}
  ~MediaMetadata_UnserializedMessageContext() override = default;

  UserType TakeData() {
    return std::move(user_data_);
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* context,
                 mojo::internal::Buffer* buffer) override {
    MediaMetadata_Data::BufferWriter writer;
    mojo::internal::Serialize<DataView>(user_data_, buffer, &writer, context);
  }

  UserType user_data_;
};

template <typename UserType, typename DataView>
const mojo::internal::UnserializedMessageContext::Tag
    MediaMetadata_UnserializedMessageContext<UserType, DataView>::kMessageTag = {};
class  MediaSessionInfo_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSessionInfo_Data));
      new (data()) MediaSessionInfo_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSessionInfo_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSessionInfo_Data>(index_);
    }
    MediaSessionInfo_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t state;
  uint8_t force_duck : 1;
  uint8_t is_controllable : 1;
  uint8_t prefer_stop_for_gain_focus_loss : 1;
  uint8_t pad3_[3];
  int32_t playback_state;
  uint8_t padfinal_[4];

 private:
  MediaSessionInfo_Data();
  ~MediaSessionInfo_Data() = delete;
};
static_assert(sizeof(MediaSessionInfo_Data) == 24,
              "Bad sizeof(MediaSessionInfo_Data)");
// Used by MediaSessionInfo::WrapAsMessage to lazily serialize the struct.
template <typename UserType, typename DataView>
struct MediaSessionInfo_UnserializedMessageContext
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  MediaSessionInfo_UnserializedMessageContext(
    uint32_t message_name,
    uint32_t message_flags,
    UserType input)
      : mojo::internal::UnserializedMessageContext(&kMessageTag, message_name, message_flags)
      , user_data_(std::move(input)) {}
  ~MediaSessionInfo_UnserializedMessageContext() override = default;

  UserType TakeData() {
    return std::move(user_data_);
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* context,
                 mojo::internal::Buffer* buffer) override {
    MediaSessionInfo_Data::BufferWriter writer;
    mojo::internal::Serialize<DataView>(user_data_, buffer, &writer, context);
  }

  UserType user_data_;
};

template <typename UserType, typename DataView>
const mojo::internal::UnserializedMessageContext::Tag
    MediaSessionInfo_UnserializedMessageContext<UserType, DataView>::kMessageTag = {};
class  MediaSessionDebugInfo_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSessionDebugInfo_Data));
      new (data()) MediaSessionDebugInfo_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSessionDebugInfo_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSessionDebugInfo_Data>(index_);
    }
    MediaSessionDebugInfo_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::String_Data> name;
  mojo::internal::Pointer<mojo::internal::String_Data> owner;
  mojo::internal::Pointer<mojo::internal::String_Data> state;

 private:
  MediaSessionDebugInfo_Data();
  ~MediaSessionDebugInfo_Data() = delete;
};
static_assert(sizeof(MediaSessionDebugInfo_Data) == 32,
              "Bad sizeof(MediaSessionDebugInfo_Data)");
// Used by MediaSessionDebugInfo::WrapAsMessage to lazily serialize the struct.
template <typename UserType, typename DataView>
struct MediaSessionDebugInfo_UnserializedMessageContext
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  MediaSessionDebugInfo_UnserializedMessageContext(
    uint32_t message_name,
    uint32_t message_flags,
    UserType input)
      : mojo::internal::UnserializedMessageContext(&kMessageTag, message_name, message_flags)
      , user_data_(std::move(input)) {}
  ~MediaSessionDebugInfo_UnserializedMessageContext() override = default;

  UserType TakeData() {
    return std::move(user_data_);
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* context,
                 mojo::internal::Buffer* buffer) override {
    MediaSessionDebugInfo_Data::BufferWriter writer;
    mojo::internal::Serialize<DataView>(user_data_, buffer, &writer, context);
  }

  UserType user_data_;
};

template <typename UserType, typename DataView>
const mojo::internal::UnserializedMessageContext::Tag
    MediaSessionDebugInfo_UnserializedMessageContext<UserType, DataView>::kMessageTag = {};
class  MediaSessionObserver_MediaSessionInfoChanged_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSessionObserver_MediaSessionInfoChanged_Params_Data));
      new (data()) MediaSessionObserver_MediaSessionInfoChanged_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSessionObserver_MediaSessionInfoChanged_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSessionObserver_MediaSessionInfoChanged_Params_Data>(index_);
    }
    MediaSessionObserver_MediaSessionInfoChanged_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<internal::MediaSessionInfo_Data> info;

 private:
  MediaSessionObserver_MediaSessionInfoChanged_Params_Data();
  ~MediaSessionObserver_MediaSessionInfoChanged_Params_Data() = delete;
};
static_assert(sizeof(MediaSessionObserver_MediaSessionInfoChanged_Params_Data) == 16,
              "Bad sizeof(MediaSessionObserver_MediaSessionInfoChanged_Params_Data)");
class  MediaSessionObserver_MediaSessionMetadataChanged_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSessionObserver_MediaSessionMetadataChanged_Params_Data));
      new (data()) MediaSessionObserver_MediaSessionMetadataChanged_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSessionObserver_MediaSessionMetadataChanged_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSessionObserver_MediaSessionMetadataChanged_Params_Data>(index_);
    }
    MediaSessionObserver_MediaSessionMetadataChanged_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<internal::MediaMetadata_Data> metadata;

 private:
  MediaSessionObserver_MediaSessionMetadataChanged_Params_Data();
  ~MediaSessionObserver_MediaSessionMetadataChanged_Params_Data() = delete;
};
static_assert(sizeof(MediaSessionObserver_MediaSessionMetadataChanged_Params_Data) == 16,
              "Bad sizeof(MediaSessionObserver_MediaSessionMetadataChanged_Params_Data)");
class  MediaSessionObserver_MediaSessionActionsChanged_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSessionObserver_MediaSessionActionsChanged_Params_Data));
      new (data()) MediaSessionObserver_MediaSessionActionsChanged_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSessionObserver_MediaSessionActionsChanged_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSessionObserver_MediaSessionActionsChanged_Params_Data>(index_);
    }
    MediaSessionObserver_MediaSessionActionsChanged_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::Array_Data<int32_t>> action;

 private:
  MediaSessionObserver_MediaSessionActionsChanged_Params_Data();
  ~MediaSessionObserver_MediaSessionActionsChanged_Params_Data() = delete;
};
static_assert(sizeof(MediaSessionObserver_MediaSessionActionsChanged_Params_Data) == 16,
              "Bad sizeof(MediaSessionObserver_MediaSessionActionsChanged_Params_Data)");
class  MediaSession_GetMediaSessionInfo_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSession_GetMediaSessionInfo_Params_Data));
      new (data()) MediaSession_GetMediaSessionInfo_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSession_GetMediaSessionInfo_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSession_GetMediaSessionInfo_Params_Data>(index_);
    }
    MediaSession_GetMediaSessionInfo_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  MediaSession_GetMediaSessionInfo_Params_Data();
  ~MediaSession_GetMediaSessionInfo_Params_Data() = delete;
};
static_assert(sizeof(MediaSession_GetMediaSessionInfo_Params_Data) == 8,
              "Bad sizeof(MediaSession_GetMediaSessionInfo_Params_Data)");
class  MediaSession_GetMediaSessionInfo_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSession_GetMediaSessionInfo_ResponseParams_Data));
      new (data()) MediaSession_GetMediaSessionInfo_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSession_GetMediaSessionInfo_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSession_GetMediaSessionInfo_ResponseParams_Data>(index_);
    }
    MediaSession_GetMediaSessionInfo_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<internal::MediaSessionInfo_Data> info;

 private:
  MediaSession_GetMediaSessionInfo_ResponseParams_Data();
  ~MediaSession_GetMediaSessionInfo_ResponseParams_Data() = delete;
};
static_assert(sizeof(MediaSession_GetMediaSessionInfo_ResponseParams_Data) == 16,
              "Bad sizeof(MediaSession_GetMediaSessionInfo_ResponseParams_Data)");
class  MediaSession_GetDebugInfo_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSession_GetDebugInfo_Params_Data));
      new (data()) MediaSession_GetDebugInfo_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSession_GetDebugInfo_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSession_GetDebugInfo_Params_Data>(index_);
    }
    MediaSession_GetDebugInfo_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  MediaSession_GetDebugInfo_Params_Data();
  ~MediaSession_GetDebugInfo_Params_Data() = delete;
};
static_assert(sizeof(MediaSession_GetDebugInfo_Params_Data) == 8,
              "Bad sizeof(MediaSession_GetDebugInfo_Params_Data)");
class  MediaSession_GetDebugInfo_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSession_GetDebugInfo_ResponseParams_Data));
      new (data()) MediaSession_GetDebugInfo_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSession_GetDebugInfo_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSession_GetDebugInfo_ResponseParams_Data>(index_);
    }
    MediaSession_GetDebugInfo_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<internal::MediaSessionDebugInfo_Data> info;

 private:
  MediaSession_GetDebugInfo_ResponseParams_Data();
  ~MediaSession_GetDebugInfo_ResponseParams_Data() = delete;
};
static_assert(sizeof(MediaSession_GetDebugInfo_ResponseParams_Data) == 16,
              "Bad sizeof(MediaSession_GetDebugInfo_ResponseParams_Data)");
class  MediaSession_StartDucking_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSession_StartDucking_Params_Data));
      new (data()) MediaSession_StartDucking_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSession_StartDucking_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSession_StartDucking_Params_Data>(index_);
    }
    MediaSession_StartDucking_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  MediaSession_StartDucking_Params_Data();
  ~MediaSession_StartDucking_Params_Data() = delete;
};
static_assert(sizeof(MediaSession_StartDucking_Params_Data) == 8,
              "Bad sizeof(MediaSession_StartDucking_Params_Data)");
class  MediaSession_StopDucking_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSession_StopDucking_Params_Data));
      new (data()) MediaSession_StopDucking_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSession_StopDucking_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSession_StopDucking_Params_Data>(index_);
    }
    MediaSession_StopDucking_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  MediaSession_StopDucking_Params_Data();
  ~MediaSession_StopDucking_Params_Data() = delete;
};
static_assert(sizeof(MediaSession_StopDucking_Params_Data) == 8,
              "Bad sizeof(MediaSession_StopDucking_Params_Data)");
class  MediaSession_Suspend_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSession_Suspend_Params_Data));
      new (data()) MediaSession_Suspend_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSession_Suspend_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSession_Suspend_Params_Data>(index_);
    }
    MediaSession_Suspend_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t suspend_type;
  uint8_t padfinal_[4];

 private:
  MediaSession_Suspend_Params_Data();
  ~MediaSession_Suspend_Params_Data() = delete;
};
static_assert(sizeof(MediaSession_Suspend_Params_Data) == 16,
              "Bad sizeof(MediaSession_Suspend_Params_Data)");
class  MediaSession_Resume_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSession_Resume_Params_Data));
      new (data()) MediaSession_Resume_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSession_Resume_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSession_Resume_Params_Data>(index_);
    }
    MediaSession_Resume_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t suspend_type;
  uint8_t padfinal_[4];

 private:
  MediaSession_Resume_Params_Data();
  ~MediaSession_Resume_Params_Data() = delete;
};
static_assert(sizeof(MediaSession_Resume_Params_Data) == 16,
              "Bad sizeof(MediaSession_Resume_Params_Data)");
class  MediaSession_AddObserver_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSession_AddObserver_Params_Data));
      new (data()) MediaSession_AddObserver_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSession_AddObserver_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSession_AddObserver_Params_Data>(index_);
    }
    MediaSession_AddObserver_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Interface_Data observer;

 private:
  MediaSession_AddObserver_Params_Data();
  ~MediaSession_AddObserver_Params_Data() = delete;
};
static_assert(sizeof(MediaSession_AddObserver_Params_Data) == 16,
              "Bad sizeof(MediaSession_AddObserver_Params_Data)");
class  MediaSession_PreviousTrack_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSession_PreviousTrack_Params_Data));
      new (data()) MediaSession_PreviousTrack_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSession_PreviousTrack_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSession_PreviousTrack_Params_Data>(index_);
    }
    MediaSession_PreviousTrack_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  MediaSession_PreviousTrack_Params_Data();
  ~MediaSession_PreviousTrack_Params_Data() = delete;
};
static_assert(sizeof(MediaSession_PreviousTrack_Params_Data) == 8,
              "Bad sizeof(MediaSession_PreviousTrack_Params_Data)");
class  MediaSession_NextTrack_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSession_NextTrack_Params_Data));
      new (data()) MediaSession_NextTrack_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSession_NextTrack_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSession_NextTrack_Params_Data>(index_);
    }
    MediaSession_NextTrack_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  MediaSession_NextTrack_Params_Data();
  ~MediaSession_NextTrack_Params_Data() = delete;
};
static_assert(sizeof(MediaSession_NextTrack_Params_Data) == 8,
              "Bad sizeof(MediaSession_NextTrack_Params_Data)");
class  MediaSession_Seek_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSession_Seek_Params_Data));
      new (data()) MediaSession_Seek_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSession_Seek_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSession_Seek_Params_Data>(index_);
    }
    MediaSession_Seek_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::mojo_base::mojom::internal::TimeDelta_Data> seek_time;

 private:
  MediaSession_Seek_Params_Data();
  ~MediaSession_Seek_Params_Data() = delete;
};
static_assert(sizeof(MediaSession_Seek_Params_Data) == 16,
              "Bad sizeof(MediaSession_Seek_Params_Data)");
class  MediaSession_Stop_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSession_Stop_Params_Data));
      new (data()) MediaSession_Stop_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSession_Stop_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSession_Stop_Params_Data>(index_);
    }
    MediaSession_Stop_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t suspend_type;
  uint8_t padfinal_[4];

 private:
  MediaSession_Stop_Params_Data();
  ~MediaSession_Stop_Params_Data() = delete;
};
static_assert(sizeof(MediaSession_Stop_Params_Data) == 16,
              "Bad sizeof(MediaSession_Stop_Params_Data)");
class  MediaSession_SkipAd_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSession_SkipAd_Params_Data));
      new (data()) MediaSession_SkipAd_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSession_SkipAd_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSession_SkipAd_Params_Data>(index_);
    }
    MediaSession_SkipAd_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  MediaSession_SkipAd_Params_Data();
  ~MediaSession_SkipAd_Params_Data() = delete;
};
static_assert(sizeof(MediaSession_SkipAd_Params_Data) == 8,
              "Bad sizeof(MediaSession_SkipAd_Params_Data)");

#pragma pack(pop)

}  // namespace internal
}  // namespace mojom
}  // namespace media_session

#endif  // SERVICES_MEDIA_SESSION_PUBLIC_MOJOM_MEDIA_SESSION_MOJOM_SHARED_INTERNAL_H_