// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_NETWORK_PUBLIC_MOJOM_NETWORK_CONTEXT_MOJOM_SHARED_H_
#define SERVICES_NETWORK_PUBLIC_MOJOM_NETWORK_CONTEXT_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "services/network/public/mojom/network_context.mojom-shared-internal.h"
#include "mojo/public/mojom/base/file_path.mojom-shared.h"
#include "mojo/public/mojom/base/time.mojom-shared.h"
#include "mojo/public/mojom/base/unguessable_token.mojom-shared.h"
#include "mojo/public/mojom/base/values.mojom-shared.h"
#include "services/network/public/mojom/address_list.mojom-shared.h"
#include "services/network/public/mojom/cookie_manager.mojom-shared.h"
#include "services/network/public/mojom/cors_origin_pattern.mojom-shared.h"
#include "services/network/public/mojom/host_resolver.mojom-shared.h"
#include "services/network/public/mojom/http_request_headers.mojom-shared.h"
#include "services/network/public/mojom/ip_endpoint.mojom-shared.h"
#include "services/network/public/mojom/mdns_responder.mojom-shared.h"
#include "services/network/public/mojom/mutable_network_traffic_annotation_tag.mojom-shared.h"
#include "services/network/public/mojom/net_log.mojom-shared.h"
#include "services/network/public/mojom/network_param.mojom-shared.h"
#include "services/network/public/mojom/p2p.mojom-shared.h"
#include "services/network/public/mojom/p2p_trusted.mojom-shared.h"
#include "services/network/public/mojom/proxy_config.mojom-shared.h"
#include "services/network/public/mojom/proxy_config_with_annotation.mojom-shared.h"
#include "services/network/public/mojom/proxy_lookup_client.mojom-shared.h"
#include "services/network/public/mojom/proxy_resolving_socket.mojom-shared.h"
#include "services/network/public/mojom/restricted_cookie_manager.mojom-shared.h"
#include "services/network/public/mojom/ssl_config.mojom-shared.h"
#include "services/network/public/mojom/tcp_socket.mojom-shared.h"
#include "services/network/public/mojom/udp_socket.mojom-shared.h"
#include "services/network/public/mojom/url_loader.mojom-shared.h"
#include "services/network/public/mojom/url_loader_factory.mojom-shared.h"
#include "services/network/public/mojom/websocket.mojom-shared.h"
#include "services/proxy_resolver/public/mojom/proxy_resolver.mojom-shared.h"
#include "url/mojom/origin.mojom-shared.h"
#include "url/mojom/url.mojom-shared.h"
#include "services/network/public/mojom/ct_log_info.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace network {
namespace mojom {
class CustomProxyConfigDataView;

class NetworkContextParamsDataView;

class NetworkConditionsDataView;

class ClearDataFilterDataView;

class NetworkUsageDataView;

class URLLoaderFactoryParamsDataView;



}  // namespace mojom
}  // namespace network

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::network::mojom::CustomProxyConfigDataView> {
  using Data = ::network::mojom::internal::CustomProxyConfig_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::network::mojom::NetworkContextParamsDataView> {
  using Data = ::network::mojom::internal::NetworkContextParams_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::network::mojom::NetworkConditionsDataView> {
  using Data = ::network::mojom::internal::NetworkConditions_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::network::mojom::ClearDataFilterDataView> {
  using Data = ::network::mojom::internal::ClearDataFilter_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::network::mojom::NetworkUsageDataView> {
  using Data = ::network::mojom::internal::NetworkUsage_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::network::mojom::URLLoaderFactoryParamsDataView> {
  using Data = ::network::mojom::internal::URLLoaderFactoryParams_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace network {
namespace mojom {

enum class ClearDataFilter_Type : int32_t {
  DELETE_MATCHES,
  KEEP_MATCHES,
  kMinValue = 0,
  kMaxValue = 1,
};

 std::ostream& operator<<(std::ostream& os, ClearDataFilter_Type value);
inline bool IsKnownEnumValue(ClearDataFilter_Type value) {
  return internal::ClearDataFilter_Type_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class NetworkContext_DomainReliabilityClearMode : int32_t {
  CLEAR_CONTEXTS,
  CLEAR_BEACONS,
  kMinValue = 0,
  kMaxValue = 1,
};

 std::ostream& operator<<(std::ostream& os, NetworkContext_DomainReliabilityClearMode value);
inline bool IsKnownEnumValue(NetworkContext_DomainReliabilityClearMode value) {
  return internal::NetworkContext_DomainReliabilityClearMode_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class CustomProxyConfigClientInterfaceBase {};

using CustomProxyConfigClientPtrDataView =
    mojo::InterfacePtrDataView<CustomProxyConfigClientInterfaceBase>;
using CustomProxyConfigClientRequestDataView =
    mojo::InterfaceRequestDataView<CustomProxyConfigClientInterfaceBase>;
using CustomProxyConfigClientAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<CustomProxyConfigClientInterfaceBase>;
using CustomProxyConfigClientAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<CustomProxyConfigClientInterfaceBase>;
class TrustedHeaderClientInterfaceBase {};

using TrustedHeaderClientPtrDataView =
    mojo::InterfacePtrDataView<TrustedHeaderClientInterfaceBase>;
using TrustedHeaderClientRequestDataView =
    mojo::InterfaceRequestDataView<TrustedHeaderClientInterfaceBase>;
using TrustedHeaderClientAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<TrustedHeaderClientInterfaceBase>;
using TrustedHeaderClientAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<TrustedHeaderClientInterfaceBase>;
class TrustedURLLoaderHeaderClientInterfaceBase {};

using TrustedURLLoaderHeaderClientPtrDataView =
    mojo::InterfacePtrDataView<TrustedURLLoaderHeaderClientInterfaceBase>;
using TrustedURLLoaderHeaderClientRequestDataView =
    mojo::InterfaceRequestDataView<TrustedURLLoaderHeaderClientInterfaceBase>;
using TrustedURLLoaderHeaderClientAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<TrustedURLLoaderHeaderClientInterfaceBase>;
using TrustedURLLoaderHeaderClientAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<TrustedURLLoaderHeaderClientInterfaceBase>;
class NetworkContextClientInterfaceBase {};

using NetworkContextClientPtrDataView =
    mojo::InterfacePtrDataView<NetworkContextClientInterfaceBase>;
using NetworkContextClientRequestDataView =
    mojo::InterfaceRequestDataView<NetworkContextClientInterfaceBase>;
using NetworkContextClientAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<NetworkContextClientInterfaceBase>;
using NetworkContextClientAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<NetworkContextClientInterfaceBase>;
class NetworkContextInterfaceBase {};

using NetworkContextPtrDataView =
    mojo::InterfacePtrDataView<NetworkContextInterfaceBase>;
using NetworkContextRequestDataView =
    mojo::InterfaceRequestDataView<NetworkContextInterfaceBase>;
using NetworkContextAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<NetworkContextInterfaceBase>;
using NetworkContextAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<NetworkContextInterfaceBase>;
class CustomProxyConfigDataView {
 public:
  CustomProxyConfigDataView() {}

  CustomProxyConfigDataView(
      internal::CustomProxyConfig_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetRulesDataView(
      ::network::mojom::ProxyRulesDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRules(UserType* output) {
    auto* pointer = data_->rules.Get();
    return mojo::internal::Deserialize<::network::mojom::ProxyRulesDataView>(
        pointer, output, context_);
  }
  inline void GetAlternateProxyListDataView(
      ::network::mojom::ProxyListDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAlternateProxyList(UserType* output) {
    auto* pointer = data_->alternate_proxy_list.Get();
    return mojo::internal::Deserialize<::network::mojom::ProxyListDataView>(
        pointer, output, context_);
  }
  inline void GetPreCacheHeadersDataView(
      ::network::mojom::HttpRequestHeadersDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPreCacheHeaders(UserType* output) {
    auto* pointer = data_->pre_cache_headers.Get();
    return mojo::internal::Deserialize<::network::mojom::HttpRequestHeadersDataView>(
        pointer, output, context_);
  }
  inline void GetPostCacheHeadersDataView(
      ::network::mojom::HttpRequestHeadersDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPostCacheHeaders(UserType* output) {
    auto* pointer = data_->post_cache_headers.Get();
    return mojo::internal::Deserialize<::network::mojom::HttpRequestHeadersDataView>(
        pointer, output, context_);
  }
 private:
  internal::CustomProxyConfig_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContextParamsDataView {
 public:
  NetworkContextParamsDataView() {}

  NetworkContextParamsDataView(
      internal::NetworkContextParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetContextNameDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadContextName(UserType* output) {
    auto* pointer = data_->context_name.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetUserAgentDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUserAgent(UserType* output) {
    auto* pointer = data_->user_agent.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetAcceptLanguageDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAcceptLanguage(UserType* output) {
    auto* pointer = data_->accept_language.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  bool enable_brotli() const {
    return data_->enable_brotli;
  }
  inline void GetQuicUserAgentIdDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadQuicUserAgentId(UserType* output) {
    auto* pointer = data_->quic_user_agent_id.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  bool enable_referrers() const {
    return data_->enable_referrers;
  }
  bool validate_referrer_policy_on_initial_request() const {
    return data_->validate_referrer_policy_on_initial_request;
  }
  template <typename UserType>
  UserType TakeProxyResolverFactory() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::proxy_resolver::mojom::ProxyResolverFactoryPtrDataView>(
            &data_->proxy_resolver_factory, &result, context_);
    DCHECK(ret);
    return result;
  }
  inline void GetCookiePathDataView(
      ::mojo_base::mojom::FilePathDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCookiePath(UserType* output) {
    auto* pointer = data_->cookie_path.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::FilePathDataView>(
        pointer, output, context_);
  }
  bool enable_encrypted_cookies() const {
    return data_->enable_encrypted_cookies;
  }
  bool restore_old_session_cookies() const {
    return data_->restore_old_session_cookies;
  }
  bool persist_session_cookies() const {
    return data_->persist_session_cookies;
  }
  inline void GetChannelIdPathDataView(
      ::mojo_base::mojom::FilePathDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadChannelIdPath(UserType* output) {
    auto* pointer = data_->channel_id_path.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::FilePathDataView>(
        pointer, output, context_);
  }
  bool http_cache_enabled() const {
    return data_->http_cache_enabled;
  }
  int32_t http_cache_max_size() const {
    return data_->http_cache_max_size;
  }
  inline void GetHttpCachePathDataView(
      ::mojo_base::mojom::FilePathDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadHttpCachePath(UserType* output) {
    auto* pointer = data_->http_cache_path.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::FilePathDataView>(
        pointer, output, context_);
  }
  inline void GetHttpServerPropertiesPathDataView(
      ::mojo_base::mojom::FilePathDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadHttpServerPropertiesPath(UserType* output) {
    auto* pointer = data_->http_server_properties_path.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::FilePathDataView>(
        pointer, output, context_);
  }
  inline void GetTransportSecurityPersisterPathDataView(
      ::mojo_base::mojom::FilePathDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTransportSecurityPersisterPath(UserType* output) {
    auto* pointer = data_->transport_security_persister_path.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::FilePathDataView>(
        pointer, output, context_);
  }
  bool enable_data_url_support() const {
    return data_->enable_data_url_support;
  }
  bool enable_file_url_support() const {
    return data_->enable_file_url_support;
  }
  bool enable_ftp_url_support() const {
    return data_->enable_ftp_url_support;
  }
  bool enforce_chrome_ct_policy() const {
    return data_->enforce_chrome_ct_policy;
  }
  bool http_09_on_non_default_ports_enabled() const {
    return data_->http_09_on_non_default_ports_enabled;
  }
  inline void GetInitialSslConfigDataView(
      ::network::mojom::SSLConfigDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadInitialSslConfig(UserType* output) {
    auto* pointer = data_->initial_ssl_config.Get();
    return mojo::internal::Deserialize<::network::mojom::SSLConfigDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  UserType TakeSslConfigClientRequest() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::SSLConfigClientRequestDataView>(
            &data_->ssl_config_client_request, &result, context_);
    DCHECK(ret);
    return result;
  }
  inline void GetInitialProxyConfigDataView(
      ::network::mojom::ProxyConfigWithAnnotationDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadInitialProxyConfig(UserType* output) {
    auto* pointer = data_->initial_proxy_config.Get();
    return mojo::internal::Deserialize<::network::mojom::ProxyConfigWithAnnotationDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  UserType TakeProxyConfigClientRequest() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::ProxyConfigClientRequestDataView>(
            &data_->proxy_config_client_request, &result, context_);
    DCHECK(ret);
    return result;
  }
  inline void GetInitialCustomProxyConfigDataView(
      CustomProxyConfigDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadInitialCustomProxyConfig(UserType* output) {
    auto* pointer = data_->initial_custom_proxy_config.Get();
    return mojo::internal::Deserialize<::network::mojom::CustomProxyConfigDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  UserType TakeCustomProxyConfigClientRequest() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::CustomProxyConfigClientRequestDataView>(
            &data_->custom_proxy_config_client_request, &result, context_);
    DCHECK(ret);
    return result;
  }
  template <typename UserType>
  UserType TakeProxyConfigPollerClient() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::ProxyConfigPollerClientPtrDataView>(
            &data_->proxy_config_poller_client, &result, context_);
    DCHECK(ret);
    return result;
  }
  template <typename UserType>
  UserType TakeProxyErrorClient() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::ProxyErrorClientPtrDataView>(
            &data_->proxy_error_client, &result, context_);
    DCHECK(ret);
    return result;
  }
  bool pac_quick_check_enabled() const {
    return data_->pac_quick_check_enabled;
  }
  bool dangerously_allow_pac_access_to_secure_urls() const {
    return data_->dangerously_allow_pac_access_to_secure_urls;
  }
  bool enable_certificate_reporting() const {
    return data_->enable_certificate_reporting;
  }
  bool enable_expect_ct_reporting() const {
    return data_->enable_expect_ct_reporting;
  }
  inline void GetCtLogsDataView(
      mojo::ArrayDataView<::network::mojom::CTLogInfoDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCtLogs(UserType* output) {
    auto* pointer = data_->ct_logs.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::network::mojom::CTLogInfoDataView>>(
        pointer, output, context_);
  }
  inline void GetCookieManagerParamsDataView(
      ::network::mojom::CookieManagerParamsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCookieManagerParams(UserType* output) {
    auto* pointer = data_->cookie_manager_params.Get();
    return mojo::internal::Deserialize<::network::mojom::CookieManagerParamsDataView>(
        pointer, output, context_);
  }
  bool enable_domain_reliability() const {
    return data_->enable_domain_reliability;
  }
  inline void GetDomainReliabilityUploadReporterDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDomainReliabilityUploadReporter(UserType* output) {
    auto* pointer = data_->domain_reliability_upload_reporter.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  bool discard_domain_reliablity_uploads() const {
    return data_->discard_domain_reliablity_uploads;
  }
  inline void GetReportingDeliveryIntervalDataView(
      ::mojo_base::mojom::TimeDeltaDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadReportingDeliveryInterval(UserType* output) {
    auto* pointer = data_->reporting_delivery_interval.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::TimeDeltaDataView>(
        pointer, output, context_);
  }
  bool skip_reporting_send_permission_check() const {
    return data_->skip_reporting_send_permission_check;
  }
  bool primary_network_context() const {
    return data_->primary_network_context;
  }
  inline void GetCorsOriginAccessListDataView(
      mojo::ArrayDataView<::network::mojom::CorsOriginAccessPatternsDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCorsOriginAccessList(UserType* output) {
    auto* pointer = data_->cors_origin_access_list.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::network::mojom::CorsOriginAccessPatternsDataView>>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContextParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkConditionsDataView {
 public:
  NetworkConditionsDataView() {}

  NetworkConditionsDataView(
      internal::NetworkConditions_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  bool offline() const {
    return data_->offline;
  }
  inline void GetLatencyDataView(
      ::mojo_base::mojom::TimeDeltaDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadLatency(UserType* output) {
    auto* pointer = data_->latency.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::TimeDeltaDataView>(
        pointer, output, context_);
  }
  double download_throughput() const {
    return data_->download_throughput;
  }
  double upload_throughput() const {
    return data_->upload_throughput;
  }
 private:
  internal::NetworkConditions_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ClearDataFilterDataView {
 public:
  ClearDataFilterDataView() {}

  ClearDataFilterDataView(
      internal::ClearDataFilter_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::network::mojom::ClearDataFilter_Type>(
        data_value, output);
  }

  ClearDataFilter_Type type() const {
    return static_cast<ClearDataFilter_Type>(data_->type);
  }
  inline void GetDomainsDataView(
      mojo::ArrayDataView<mojo::StringDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDomains(UserType* output) {
    auto* pointer = data_->domains.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<mojo::StringDataView>>(
        pointer, output, context_);
  }
  inline void GetOriginsDataView(
      mojo::ArrayDataView<::url::mojom::OriginDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOrigins(UserType* output) {
    auto* pointer = data_->origins.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::url::mojom::OriginDataView>>(
        pointer, output, context_);
  }
 private:
  internal::ClearDataFilter_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkUsageDataView {
 public:
  NetworkUsageDataView() {}

  NetworkUsageDataView(
      internal::NetworkUsage_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  uint32_t process_id() const {
    return data_->process_id;
  }
  uint32_t routing_id() const {
    return data_->routing_id;
  }
  int64_t total_bytes_received() const {
    return data_->total_bytes_received;
  }
  int64_t total_bytes_sent() const {
    return data_->total_bytes_sent;
  }
 private:
  internal::NetworkUsage_Data* data_ = nullptr;
};

class URLLoaderFactoryParamsDataView {
 public:
  URLLoaderFactoryParamsDataView() {}

  URLLoaderFactoryParamsDataView(
      internal::URLLoaderFactoryParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  uint32_t process_id() const {
    return data_->process_id;
  }
  inline void GetRequestInitiatorSiteLockDataView(
      ::url::mojom::OriginDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRequestInitiatorSiteLock(UserType* output) {
    auto* pointer = data_->request_initiator_site_lock.Get();
    return mojo::internal::Deserialize<::url::mojom::OriginDataView>(
        pointer, output, context_);
  }
  bool is_corb_enabled() const {
    return data_->is_corb_enabled;
  }
  int32_t corb_detachable_resource_type() const {
    return data_->corb_detachable_resource_type;
  }
  int32_t corb_excluded_resource_type() const {
    return data_->corb_excluded_resource_type;
  }
  bool disable_web_security() const {
    return data_->disable_web_security;
  }
  template <typename UserType>
  UserType TakeHeaderClient() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::TrustedURLLoaderHeaderClientPtrDataView>(
            &data_->header_client, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::URLLoaderFactoryParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class CustomProxyConfigClient_OnCustomProxyConfigUpdated_ParamsDataView {
 public:
  CustomProxyConfigClient_OnCustomProxyConfigUpdated_ParamsDataView() {}

  CustomProxyConfigClient_OnCustomProxyConfigUpdated_ParamsDataView(
      internal::CustomProxyConfigClient_OnCustomProxyConfigUpdated_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetProxyConfigDataView(
      CustomProxyConfigDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadProxyConfig(UserType* output) {
    auto* pointer = data_->proxy_config.Get();
    return mojo::internal::Deserialize<::network::mojom::CustomProxyConfigDataView>(
        pointer, output, context_);
  }
 private:
  internal::CustomProxyConfigClient_OnCustomProxyConfigUpdated_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class CustomProxyConfigClient_MarkProxiesAsBad_ParamsDataView {
 public:
  CustomProxyConfigClient_MarkProxiesAsBad_ParamsDataView() {}

  CustomProxyConfigClient_MarkProxiesAsBad_ParamsDataView(
      internal::CustomProxyConfigClient_MarkProxiesAsBad_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetBypassDurationDataView(
      ::mojo_base::mojom::TimeDeltaDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadBypassDuration(UserType* output) {
    auto* pointer = data_->bypass_duration.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::TimeDeltaDataView>(
        pointer, output, context_);
  }
  inline void GetBadProxiesDataView(
      ::network::mojom::ProxyListDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadBadProxies(UserType* output) {
    auto* pointer = data_->bad_proxies.Get();
    return mojo::internal::Deserialize<::network::mojom::ProxyListDataView>(
        pointer, output, context_);
  }
 private:
  internal::CustomProxyConfigClient_MarkProxiesAsBad_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class CustomProxyConfigClient_MarkProxiesAsBad_ResponseParamsDataView {
 public:
  CustomProxyConfigClient_MarkProxiesAsBad_ResponseParamsDataView() {}

  CustomProxyConfigClient_MarkProxiesAsBad_ResponseParamsDataView(
      internal::CustomProxyConfigClient_MarkProxiesAsBad_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::CustomProxyConfigClient_MarkProxiesAsBad_ResponseParams_Data* data_ = nullptr;
};

class CustomProxyConfigClient_ClearBadProxiesCache_ParamsDataView {
 public:
  CustomProxyConfigClient_ClearBadProxiesCache_ParamsDataView() {}

  CustomProxyConfigClient_ClearBadProxiesCache_ParamsDataView(
      internal::CustomProxyConfigClient_ClearBadProxiesCache_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::CustomProxyConfigClient_ClearBadProxiesCache_Params_Data* data_ = nullptr;
};

class TrustedHeaderClient_OnBeforeSendHeaders_ParamsDataView {
 public:
  TrustedHeaderClient_OnBeforeSendHeaders_ParamsDataView() {}

  TrustedHeaderClient_OnBeforeSendHeaders_ParamsDataView(
      internal::TrustedHeaderClient_OnBeforeSendHeaders_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetHeadersDataView(
      ::network::mojom::HttpRequestHeadersDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadHeaders(UserType* output) {
    auto* pointer = data_->headers.Get();
    return mojo::internal::Deserialize<::network::mojom::HttpRequestHeadersDataView>(
        pointer, output, context_);
  }
 private:
  internal::TrustedHeaderClient_OnBeforeSendHeaders_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class TrustedHeaderClient_OnBeforeSendHeaders_ResponseParamsDataView {
 public:
  TrustedHeaderClient_OnBeforeSendHeaders_ResponseParamsDataView() {}

  TrustedHeaderClient_OnBeforeSendHeaders_ResponseParamsDataView(
      internal::TrustedHeaderClient_OnBeforeSendHeaders_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int32_t result() const {
    return data_->result;
  }
  inline void GetHeadersDataView(
      ::network::mojom::HttpRequestHeadersDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadHeaders(UserType* output) {
    auto* pointer = data_->headers.Get();
    return mojo::internal::Deserialize<::network::mojom::HttpRequestHeadersDataView>(
        pointer, output, context_);
  }
 private:
  internal::TrustedHeaderClient_OnBeforeSendHeaders_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class TrustedHeaderClient_OnHeadersReceived_ParamsDataView {
 public:
  TrustedHeaderClient_OnHeadersReceived_ParamsDataView() {}

  TrustedHeaderClient_OnHeadersReceived_ParamsDataView(
      internal::TrustedHeaderClient_OnHeadersReceived_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetHeadersDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadHeaders(UserType* output) {
    auto* pointer = data_->headers.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::TrustedHeaderClient_OnHeadersReceived_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class TrustedHeaderClient_OnHeadersReceived_ResponseParamsDataView {
 public:
  TrustedHeaderClient_OnHeadersReceived_ResponseParamsDataView() {}

  TrustedHeaderClient_OnHeadersReceived_ResponseParamsDataView(
      internal::TrustedHeaderClient_OnHeadersReceived_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int32_t result() const {
    return data_->result;
  }
  inline void GetHeadersDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadHeaders(UserType* output) {
    auto* pointer = data_->headers.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetAllowedUnsafeRedirectUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAllowedUnsafeRedirectUrl(UserType* output) {
    auto* pointer = data_->allowed_unsafe_redirect_url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
 private:
  internal::TrustedHeaderClient_OnHeadersReceived_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class TrustedURLLoaderHeaderClient_OnLoaderCreated_ParamsDataView {
 public:
  TrustedURLLoaderHeaderClient_OnLoaderCreated_ParamsDataView() {}

  TrustedURLLoaderHeaderClient_OnLoaderCreated_ParamsDataView(
      internal::TrustedURLLoaderHeaderClient_OnLoaderCreated_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int32_t request_id() const {
    return data_->request_id;
  }
  template <typename UserType>
  UserType TakeHeaderClient() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::TrustedHeaderClientRequestDataView>(
            &data_->header_client, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::TrustedURLLoaderHeaderClient_OnLoaderCreated_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContextClient_OnCanSendReportingReports_ParamsDataView {
 public:
  NetworkContextClient_OnCanSendReportingReports_ParamsDataView() {}

  NetworkContextClient_OnCanSendReportingReports_ParamsDataView(
      internal::NetworkContextClient_OnCanSendReportingReports_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetOriginsDataView(
      mojo::ArrayDataView<::url::mojom::OriginDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOrigins(UserType* output) {
    auto* pointer = data_->origins.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::url::mojom::OriginDataView>>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContextClient_OnCanSendReportingReports_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContextClient_OnCanSendReportingReports_ResponseParamsDataView {
 public:
  NetworkContextClient_OnCanSendReportingReports_ResponseParamsDataView() {}

  NetworkContextClient_OnCanSendReportingReports_ResponseParamsDataView(
      internal::NetworkContextClient_OnCanSendReportingReports_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetOriginsDataView(
      mojo::ArrayDataView<::url::mojom::OriginDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOrigins(UserType* output) {
    auto* pointer = data_->origins.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::url::mojom::OriginDataView>>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContextClient_OnCanSendReportingReports_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContextClient_OnCanSendDomainReliabilityUpload_ParamsDataView {
 public:
  NetworkContextClient_OnCanSendDomainReliabilityUpload_ParamsDataView() {}

  NetworkContextClient_OnCanSendDomainReliabilityUpload_ParamsDataView(
      internal::NetworkContextClient_OnCanSendDomainReliabilityUpload_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetOriginDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOrigin(UserType* output) {
    auto* pointer = data_->origin.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContextClient_OnCanSendDomainReliabilityUpload_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContextClient_OnCanSendDomainReliabilityUpload_ResponseParamsDataView {
 public:
  NetworkContextClient_OnCanSendDomainReliabilityUpload_ResponseParamsDataView() {}

  NetworkContextClient_OnCanSendDomainReliabilityUpload_ResponseParamsDataView(
      internal::NetworkContextClient_OnCanSendDomainReliabilityUpload_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool allowed() const {
    return data_->allowed;
  }
 private:
  internal::NetworkContextClient_OnCanSendDomainReliabilityUpload_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_SetClient_ParamsDataView {
 public:
  NetworkContext_SetClient_ParamsDataView() {}

  NetworkContext_SetClient_ParamsDataView(
      internal::NetworkContext_SetClient_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeClient() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::NetworkContextClientPtrDataView>(
            &data_->client, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::NetworkContext_SetClient_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_CreateURLLoaderFactory_ParamsDataView {
 public:
  NetworkContext_CreateURLLoaderFactory_ParamsDataView() {}

  NetworkContext_CreateURLLoaderFactory_ParamsDataView(
      internal::NetworkContext_CreateURLLoaderFactory_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeUrlLoaderFactory() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::URLLoaderFactoryRequestDataView>(
            &data_->url_loader_factory, &result, context_);
    DCHECK(ret);
    return result;
  }
  inline void GetParamsDataView(
      URLLoaderFactoryParamsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadParams(UserType* output) {
    auto* pointer = data_->params.Get();
    return mojo::internal::Deserialize<::network::mojom::URLLoaderFactoryParamsDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_CreateURLLoaderFactory_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_ResetURLLoaderFactories_ParamsDataView {
 public:
  NetworkContext_ResetURLLoaderFactories_ParamsDataView() {}

  NetworkContext_ResetURLLoaderFactories_ParamsDataView(
      internal::NetworkContext_ResetURLLoaderFactories_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_ResetURLLoaderFactories_Params_Data* data_ = nullptr;
};

class NetworkContext_GetCookieManager_ParamsDataView {
 public:
  NetworkContext_GetCookieManager_ParamsDataView() {}

  NetworkContext_GetCookieManager_ParamsDataView(
      internal::NetworkContext_GetCookieManager_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeCookieManager() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::CookieManagerRequestDataView>(
            &data_->cookie_manager, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::NetworkContext_GetCookieManager_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_GetRestrictedCookieManager_ParamsDataView {
 public:
  NetworkContext_GetRestrictedCookieManager_ParamsDataView() {}

  NetworkContext_GetRestrictedCookieManager_ParamsDataView(
      internal::NetworkContext_GetRestrictedCookieManager_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeRestrictedCookieManager() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::RestrictedCookieManagerRequestDataView>(
            &data_->restricted_cookie_manager, &result, context_);
    DCHECK(ret);
    return result;
  }
  inline void GetOriginDataView(
      ::url::mojom::OriginDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOrigin(UserType* output) {
    auto* pointer = data_->origin.Get();
    return mojo::internal::Deserialize<::url::mojom::OriginDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_GetRestrictedCookieManager_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_ClearNetworkingHistorySince_ParamsDataView {
 public:
  NetworkContext_ClearNetworkingHistorySince_ParamsDataView() {}

  NetworkContext_ClearNetworkingHistorySince_ParamsDataView(
      internal::NetworkContext_ClearNetworkingHistorySince_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetStartTimeDataView(
      ::mojo_base::mojom::TimeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadStartTime(UserType* output) {
    auto* pointer = data_->start_time.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::TimeDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_ClearNetworkingHistorySince_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_ClearNetworkingHistorySince_ResponseParamsDataView {
 public:
  NetworkContext_ClearNetworkingHistorySince_ResponseParamsDataView() {}

  NetworkContext_ClearNetworkingHistorySince_ResponseParamsDataView(
      internal::NetworkContext_ClearNetworkingHistorySince_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_ClearNetworkingHistorySince_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_ClearHttpCache_ParamsDataView {
 public:
  NetworkContext_ClearHttpCache_ParamsDataView() {}

  NetworkContext_ClearHttpCache_ParamsDataView(
      internal::NetworkContext_ClearHttpCache_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetStartTimeDataView(
      ::mojo_base::mojom::TimeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadStartTime(UserType* output) {
    auto* pointer = data_->start_time.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::TimeDataView>(
        pointer, output, context_);
  }
  inline void GetEndTimeDataView(
      ::mojo_base::mojom::TimeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadEndTime(UserType* output) {
    auto* pointer = data_->end_time.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::TimeDataView>(
        pointer, output, context_);
  }
  inline void GetFilterDataView(
      ClearDataFilterDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFilter(UserType* output) {
    auto* pointer = data_->filter.Get();
    return mojo::internal::Deserialize<::network::mojom::ClearDataFilterDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_ClearHttpCache_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_ClearHttpCache_ResponseParamsDataView {
 public:
  NetworkContext_ClearHttpCache_ResponseParamsDataView() {}

  NetworkContext_ClearHttpCache_ResponseParamsDataView(
      internal::NetworkContext_ClearHttpCache_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_ClearHttpCache_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_ComputeHttpCacheSize_ParamsDataView {
 public:
  NetworkContext_ComputeHttpCacheSize_ParamsDataView() {}

  NetworkContext_ComputeHttpCacheSize_ParamsDataView(
      internal::NetworkContext_ComputeHttpCacheSize_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetStartTimeDataView(
      ::mojo_base::mojom::TimeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadStartTime(UserType* output) {
    auto* pointer = data_->start_time.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::TimeDataView>(
        pointer, output, context_);
  }
  inline void GetEndTimeDataView(
      ::mojo_base::mojom::TimeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadEndTime(UserType* output) {
    auto* pointer = data_->end_time.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::TimeDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_ComputeHttpCacheSize_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_ComputeHttpCacheSize_ResponseParamsDataView {
 public:
  NetworkContext_ComputeHttpCacheSize_ResponseParamsDataView() {}

  NetworkContext_ComputeHttpCacheSize_ResponseParamsDataView(
      internal::NetworkContext_ComputeHttpCacheSize_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool is_upper_bound() const {
    return data_->is_upper_bound;
  }
  int64_t size_or_error() const {
    return data_->size_or_error;
  }
 private:
  internal::NetworkContext_ComputeHttpCacheSize_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_NotifyExternalCacheHit_ParamsDataView {
 public:
  NetworkContext_NotifyExternalCacheHit_ParamsDataView() {}

  NetworkContext_NotifyExternalCacheHit_ParamsDataView(
      internal::NetworkContext_NotifyExternalCacheHit_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUrl(UserType* output) {
    auto* pointer = data_->url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  inline void GetHttpMethodDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadHttpMethod(UserType* output) {
    auto* pointer = data_->http_method.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetTopFrameOriginDataView(
      ::url::mojom::OriginDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTopFrameOrigin(UserType* output) {
    auto* pointer = data_->top_frame_origin.Get();
    return mojo::internal::Deserialize<::url::mojom::OriginDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_NotifyExternalCacheHit_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_WriteCacheMetadata_ParamsDataView {
 public:
  NetworkContext_WriteCacheMetadata_ParamsDataView() {}

  NetworkContext_WriteCacheMetadata_ParamsDataView(
      internal::NetworkContext_WriteCacheMetadata_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUrl(UserType* output) {
    auto* pointer = data_->url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPriority(UserType* output) const {
    auto data_value = data_->priority;
    return mojo::internal::Deserialize<::network::mojom::RequestPriority>(
        data_value, output);
  }

  ::network::mojom::RequestPriority priority() const {
    return static_cast<::network::mojom::RequestPriority>(data_->priority);
  }
  inline void GetExpectedResponseTimeDataView(
      ::mojo_base::mojom::TimeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadExpectedResponseTime(UserType* output) {
    auto* pointer = data_->expected_response_time.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::TimeDataView>(
        pointer, output, context_);
  }
  inline void GetDataDataView(
      mojo::ArrayDataView<uint8_t>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadData(UserType* output) {
    auto* pointer = data_->data.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<uint8_t>>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_WriteCacheMetadata_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_ClearChannelIds_ParamsDataView {
 public:
  NetworkContext_ClearChannelIds_ParamsDataView() {}

  NetworkContext_ClearChannelIds_ParamsDataView(
      internal::NetworkContext_ClearChannelIds_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetStartTimeDataView(
      ::mojo_base::mojom::TimeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadStartTime(UserType* output) {
    auto* pointer = data_->start_time.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::TimeDataView>(
        pointer, output, context_);
  }
  inline void GetEndTimeDataView(
      ::mojo_base::mojom::TimeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadEndTime(UserType* output) {
    auto* pointer = data_->end_time.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::TimeDataView>(
        pointer, output, context_);
  }
  inline void GetFilterDataView(
      ClearDataFilterDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFilter(UserType* output) {
    auto* pointer = data_->filter.Get();
    return mojo::internal::Deserialize<::network::mojom::ClearDataFilterDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_ClearChannelIds_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_ClearChannelIds_ResponseParamsDataView {
 public:
  NetworkContext_ClearChannelIds_ResponseParamsDataView() {}

  NetworkContext_ClearChannelIds_ResponseParamsDataView(
      internal::NetworkContext_ClearChannelIds_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_ClearChannelIds_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_ClearHostCache_ParamsDataView {
 public:
  NetworkContext_ClearHostCache_ParamsDataView() {}

  NetworkContext_ClearHostCache_ParamsDataView(
      internal::NetworkContext_ClearHostCache_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetFilterDataView(
      ClearDataFilterDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFilter(UserType* output) {
    auto* pointer = data_->filter.Get();
    return mojo::internal::Deserialize<::network::mojom::ClearDataFilterDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_ClearHostCache_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_ClearHostCache_ResponseParamsDataView {
 public:
  NetworkContext_ClearHostCache_ResponseParamsDataView() {}

  NetworkContext_ClearHostCache_ResponseParamsDataView(
      internal::NetworkContext_ClearHostCache_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_ClearHostCache_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_ClearHttpAuthCache_ParamsDataView {
 public:
  NetworkContext_ClearHttpAuthCache_ParamsDataView() {}

  NetworkContext_ClearHttpAuthCache_ParamsDataView(
      internal::NetworkContext_ClearHttpAuthCache_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetStartTimeDataView(
      ::mojo_base::mojom::TimeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadStartTime(UserType* output) {
    auto* pointer = data_->start_time.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::TimeDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_ClearHttpAuthCache_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_ClearHttpAuthCache_ResponseParamsDataView {
 public:
  NetworkContext_ClearHttpAuthCache_ResponseParamsDataView() {}

  NetworkContext_ClearHttpAuthCache_ResponseParamsDataView(
      internal::NetworkContext_ClearHttpAuthCache_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_ClearHttpAuthCache_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_ClearReportingCacheReports_ParamsDataView {
 public:
  NetworkContext_ClearReportingCacheReports_ParamsDataView() {}

  NetworkContext_ClearReportingCacheReports_ParamsDataView(
      internal::NetworkContext_ClearReportingCacheReports_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetFilterDataView(
      ClearDataFilterDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFilter(UserType* output) {
    auto* pointer = data_->filter.Get();
    return mojo::internal::Deserialize<::network::mojom::ClearDataFilterDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_ClearReportingCacheReports_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_ClearReportingCacheReports_ResponseParamsDataView {
 public:
  NetworkContext_ClearReportingCacheReports_ResponseParamsDataView() {}

  NetworkContext_ClearReportingCacheReports_ResponseParamsDataView(
      internal::NetworkContext_ClearReportingCacheReports_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_ClearReportingCacheReports_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_ClearReportingCacheClients_ParamsDataView {
 public:
  NetworkContext_ClearReportingCacheClients_ParamsDataView() {}

  NetworkContext_ClearReportingCacheClients_ParamsDataView(
      internal::NetworkContext_ClearReportingCacheClients_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetFilterDataView(
      ClearDataFilterDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFilter(UserType* output) {
    auto* pointer = data_->filter.Get();
    return mojo::internal::Deserialize<::network::mojom::ClearDataFilterDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_ClearReportingCacheClients_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_ClearReportingCacheClients_ResponseParamsDataView {
 public:
  NetworkContext_ClearReportingCacheClients_ResponseParamsDataView() {}

  NetworkContext_ClearReportingCacheClients_ResponseParamsDataView(
      internal::NetworkContext_ClearReportingCacheClients_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_ClearReportingCacheClients_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_ClearNetworkErrorLogging_ParamsDataView {
 public:
  NetworkContext_ClearNetworkErrorLogging_ParamsDataView() {}

  NetworkContext_ClearNetworkErrorLogging_ParamsDataView(
      internal::NetworkContext_ClearNetworkErrorLogging_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetFilterDataView(
      ClearDataFilterDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFilter(UserType* output) {
    auto* pointer = data_->filter.Get();
    return mojo::internal::Deserialize<::network::mojom::ClearDataFilterDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_ClearNetworkErrorLogging_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_ClearNetworkErrorLogging_ResponseParamsDataView {
 public:
  NetworkContext_ClearNetworkErrorLogging_ResponseParamsDataView() {}

  NetworkContext_ClearNetworkErrorLogging_ResponseParamsDataView(
      internal::NetworkContext_ClearNetworkErrorLogging_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_ClearNetworkErrorLogging_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_ClearDomainReliability_ParamsDataView {
 public:
  NetworkContext_ClearDomainReliability_ParamsDataView() {}

  NetworkContext_ClearDomainReliability_ParamsDataView(
      internal::NetworkContext_ClearDomainReliability_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetFilterDataView(
      ClearDataFilterDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFilter(UserType* output) {
    auto* pointer = data_->filter.Get();
    return mojo::internal::Deserialize<::network::mojom::ClearDataFilterDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadMode(UserType* output) const {
    auto data_value = data_->mode;
    return mojo::internal::Deserialize<::network::mojom::NetworkContext_DomainReliabilityClearMode>(
        data_value, output);
  }

  NetworkContext_DomainReliabilityClearMode mode() const {
    return static_cast<NetworkContext_DomainReliabilityClearMode>(data_->mode);
  }
 private:
  internal::NetworkContext_ClearDomainReliability_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_ClearDomainReliability_ResponseParamsDataView {
 public:
  NetworkContext_ClearDomainReliability_ResponseParamsDataView() {}

  NetworkContext_ClearDomainReliability_ResponseParamsDataView(
      internal::NetworkContext_ClearDomainReliability_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_ClearDomainReliability_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_GetDomainReliabilityJSON_ParamsDataView {
 public:
  NetworkContext_GetDomainReliabilityJSON_ParamsDataView() {}

  NetworkContext_GetDomainReliabilityJSON_ParamsDataView(
      internal::NetworkContext_GetDomainReliabilityJSON_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_GetDomainReliabilityJSON_Params_Data* data_ = nullptr;
};

class NetworkContext_GetDomainReliabilityJSON_ResponseParamsDataView {
 public:
  NetworkContext_GetDomainReliabilityJSON_ResponseParamsDataView() {}

  NetworkContext_GetDomainReliabilityJSON_ResponseParamsDataView(
      internal::NetworkContext_GetDomainReliabilityJSON_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetDataDataView(
      ::mojo_base::mojom::ValueDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadData(UserType* output) {
    auto* pointer = !data_->data.is_null() ? &data_->data : nullptr;
    return mojo::internal::Deserialize<::mojo_base::mojom::ValueDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_GetDomainReliabilityJSON_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_QueueReport_ParamsDataView {
 public:
  NetworkContext_QueueReport_ParamsDataView() {}

  NetworkContext_QueueReport_ParamsDataView(
      internal::NetworkContext_QueueReport_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetTypeDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) {
    auto* pointer = data_->type.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetGroupDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadGroup(UserType* output) {
    auto* pointer = data_->group.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUrl(UserType* output) {
    auto* pointer = data_->url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  inline void GetUserAgentDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUserAgent(UserType* output) {
    auto* pointer = data_->user_agent.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetBodyDataView(
      ::mojo_base::mojom::DictionaryValueDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadBody(UserType* output) {
    auto* pointer = data_->body.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::DictionaryValueDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_QueueReport_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_CloseAllConnections_ParamsDataView {
 public:
  NetworkContext_CloseAllConnections_ParamsDataView() {}

  NetworkContext_CloseAllConnections_ParamsDataView(
      internal::NetworkContext_CloseAllConnections_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_CloseAllConnections_Params_Data* data_ = nullptr;
};

class NetworkContext_CloseAllConnections_ResponseParamsDataView {
 public:
  NetworkContext_CloseAllConnections_ResponseParamsDataView() {}

  NetworkContext_CloseAllConnections_ResponseParamsDataView(
      internal::NetworkContext_CloseAllConnections_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_CloseAllConnections_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_CloseIdleConnections_ParamsDataView {
 public:
  NetworkContext_CloseIdleConnections_ParamsDataView() {}

  NetworkContext_CloseIdleConnections_ParamsDataView(
      internal::NetworkContext_CloseIdleConnections_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_CloseIdleConnections_Params_Data* data_ = nullptr;
};

class NetworkContext_CloseIdleConnections_ResponseParamsDataView {
 public:
  NetworkContext_CloseIdleConnections_ResponseParamsDataView() {}

  NetworkContext_CloseIdleConnections_ResponseParamsDataView(
      internal::NetworkContext_CloseIdleConnections_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_CloseIdleConnections_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_SetNetworkConditions_ParamsDataView {
 public:
  NetworkContext_SetNetworkConditions_ParamsDataView() {}

  NetworkContext_SetNetworkConditions_ParamsDataView(
      internal::NetworkContext_SetNetworkConditions_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetThrottlingProfileIdDataView(
      ::mojo_base::mojom::UnguessableTokenDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadThrottlingProfileId(UserType* output) {
    auto* pointer = data_->throttling_profile_id.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::UnguessableTokenDataView>(
        pointer, output, context_);
  }
  inline void GetConditionsDataView(
      NetworkConditionsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadConditions(UserType* output) {
    auto* pointer = data_->conditions.Get();
    return mojo::internal::Deserialize<::network::mojom::NetworkConditionsDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_SetNetworkConditions_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_SetAcceptLanguage_ParamsDataView {
 public:
  NetworkContext_SetAcceptLanguage_ParamsDataView() {}

  NetworkContext_SetAcceptLanguage_ParamsDataView(
      internal::NetworkContext_SetAcceptLanguage_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetNewAcceptLanguageDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadNewAcceptLanguage(UserType* output) {
    auto* pointer = data_->new_accept_language.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_SetAcceptLanguage_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_SetEnableReferrers_ParamsDataView {
 public:
  NetworkContext_SetEnableReferrers_ParamsDataView() {}

  NetworkContext_SetEnableReferrers_ParamsDataView(
      internal::NetworkContext_SetEnableReferrers_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool enable_referrers() const {
    return data_->enable_referrers;
  }
 private:
  internal::NetworkContext_SetEnableReferrers_Params_Data* data_ = nullptr;
};

class NetworkContext_SetCTPolicy_ParamsDataView {
 public:
  NetworkContext_SetCTPolicy_ParamsDataView() {}

  NetworkContext_SetCTPolicy_ParamsDataView(
      internal::NetworkContext_SetCTPolicy_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetRequiredHostsDataView(
      mojo::ArrayDataView<mojo::StringDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRequiredHosts(UserType* output) {
    auto* pointer = data_->required_hosts.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<mojo::StringDataView>>(
        pointer, output, context_);
  }
  inline void GetExcludedHostsDataView(
      mojo::ArrayDataView<mojo::StringDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadExcludedHosts(UserType* output) {
    auto* pointer = data_->excluded_hosts.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<mojo::StringDataView>>(
        pointer, output, context_);
  }
  inline void GetExcludedSpkisDataView(
      mojo::ArrayDataView<mojo::StringDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadExcludedSpkis(UserType* output) {
    auto* pointer = data_->excluded_spkis.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<mojo::StringDataView>>(
        pointer, output, context_);
  }
  inline void GetExcludedLegacySpkisDataView(
      mojo::ArrayDataView<mojo::StringDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadExcludedLegacySpkis(UserType* output) {
    auto* pointer = data_->excluded_legacy_spkis.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<mojo::StringDataView>>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_SetCTPolicy_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_AddExpectCT_ParamsDataView {
 public:
  NetworkContext_AddExpectCT_ParamsDataView() {}

  NetworkContext_AddExpectCT_ParamsDataView(
      internal::NetworkContext_AddExpectCT_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetHostDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadHost(UserType* output) {
    auto* pointer = data_->host.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetExpiryDataView(
      ::mojo_base::mojom::TimeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadExpiry(UserType* output) {
    auto* pointer = data_->expiry.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::TimeDataView>(
        pointer, output, context_);
  }
  bool enforce() const {
    return data_->enforce;
  }
  inline void GetReportUriDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadReportUri(UserType* output) {
    auto* pointer = data_->report_uri.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_AddExpectCT_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_AddExpectCT_ResponseParamsDataView {
 public:
  NetworkContext_AddExpectCT_ResponseParamsDataView() {}

  NetworkContext_AddExpectCT_ResponseParamsDataView(
      internal::NetworkContext_AddExpectCT_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool success() const {
    return data_->success;
  }
 private:
  internal::NetworkContext_AddExpectCT_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_SetExpectCTTestReport_ParamsDataView {
 public:
  NetworkContext_SetExpectCTTestReport_ParamsDataView() {}

  NetworkContext_SetExpectCTTestReport_ParamsDataView(
      internal::NetworkContext_SetExpectCTTestReport_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetReportUriDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadReportUri(UserType* output) {
    auto* pointer = data_->report_uri.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_SetExpectCTTestReport_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_SetExpectCTTestReport_ResponseParamsDataView {
 public:
  NetworkContext_SetExpectCTTestReport_ResponseParamsDataView() {}

  NetworkContext_SetExpectCTTestReport_ResponseParamsDataView(
      internal::NetworkContext_SetExpectCTTestReport_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool success() const {
    return data_->success;
  }
 private:
  internal::NetworkContext_SetExpectCTTestReport_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_GetExpectCTState_ParamsDataView {
 public:
  NetworkContext_GetExpectCTState_ParamsDataView() {}

  NetworkContext_GetExpectCTState_ParamsDataView(
      internal::NetworkContext_GetExpectCTState_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetDomainDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDomain(UserType* output) {
    auto* pointer = data_->domain.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_GetExpectCTState_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_GetExpectCTState_ResponseParamsDataView {
 public:
  NetworkContext_GetExpectCTState_ResponseParamsDataView() {}

  NetworkContext_GetExpectCTState_ResponseParamsDataView(
      internal::NetworkContext_GetExpectCTState_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetStateDataView(
      ::mojo_base::mojom::DictionaryValueDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadState(UserType* output) {
    auto* pointer = data_->state.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::DictionaryValueDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_GetExpectCTState_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_CreateUDPSocket_ParamsDataView {
 public:
  NetworkContext_CreateUDPSocket_ParamsDataView() {}

  NetworkContext_CreateUDPSocket_ParamsDataView(
      internal::NetworkContext_CreateUDPSocket_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeRequest() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::UDPSocketRequestDataView>(
            &data_->request, &result, context_);
    DCHECK(ret);
    return result;
  }
  template <typename UserType>
  UserType TakeReceiver() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::UDPSocketReceiverPtrDataView>(
            &data_->receiver, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::NetworkContext_CreateUDPSocket_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_CreateTCPServerSocket_ParamsDataView {
 public:
  NetworkContext_CreateTCPServerSocket_ParamsDataView() {}

  NetworkContext_CreateTCPServerSocket_ParamsDataView(
      internal::NetworkContext_CreateTCPServerSocket_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetLocalAddrDataView(
      ::network::mojom::IPEndPointDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadLocalAddr(UserType* output) {
    auto* pointer = data_->local_addr.Get();
    return mojo::internal::Deserialize<::network::mojom::IPEndPointDataView>(
        pointer, output, context_);
  }
  uint32_t backlog() const {
    return data_->backlog;
  }
  inline void GetTrafficAnnotationDataView(
      ::network::mojom::MutableNetworkTrafficAnnotationTagDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTrafficAnnotation(UserType* output) {
    auto* pointer = data_->traffic_annotation.Get();
    return mojo::internal::Deserialize<::network::mojom::MutableNetworkTrafficAnnotationTagDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  UserType TakeSocket() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::TCPServerSocketRequestDataView>(
            &data_->socket, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::NetworkContext_CreateTCPServerSocket_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_CreateTCPServerSocket_ResponseParamsDataView {
 public:
  NetworkContext_CreateTCPServerSocket_ResponseParamsDataView() {}

  NetworkContext_CreateTCPServerSocket_ResponseParamsDataView(
      internal::NetworkContext_CreateTCPServerSocket_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int32_t result() const {
    return data_->result;
  }
  inline void GetLocalAddrOutDataView(
      ::network::mojom::IPEndPointDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadLocalAddrOut(UserType* output) {
    auto* pointer = data_->local_addr_out.Get();
    return mojo::internal::Deserialize<::network::mojom::IPEndPointDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_CreateTCPServerSocket_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_CreateTCPConnectedSocket_ParamsDataView {
 public:
  NetworkContext_CreateTCPConnectedSocket_ParamsDataView() {}

  NetworkContext_CreateTCPConnectedSocket_ParamsDataView(
      internal::NetworkContext_CreateTCPConnectedSocket_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetLocalAddrDataView(
      ::network::mojom::IPEndPointDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadLocalAddr(UserType* output) {
    auto* pointer = data_->local_addr.Get();
    return mojo::internal::Deserialize<::network::mojom::IPEndPointDataView>(
        pointer, output, context_);
  }
  inline void GetRemoteAddrListDataView(
      ::network::mojom::AddressListDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRemoteAddrList(UserType* output) {
    auto* pointer = data_->remote_addr_list.Get();
    return mojo::internal::Deserialize<::network::mojom::AddressListDataView>(
        pointer, output, context_);
  }
  inline void GetTcpConnectedSocketOptionsDataView(
      ::network::mojom::TCPConnectedSocketOptionsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTcpConnectedSocketOptions(UserType* output) {
    auto* pointer = data_->tcp_connected_socket_options.Get();
    return mojo::internal::Deserialize<::network::mojom::TCPConnectedSocketOptionsDataView>(
        pointer, output, context_);
  }
  inline void GetTrafficAnnotationDataView(
      ::network::mojom::MutableNetworkTrafficAnnotationTagDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTrafficAnnotation(UserType* output) {
    auto* pointer = data_->traffic_annotation.Get();
    return mojo::internal::Deserialize<::network::mojom::MutableNetworkTrafficAnnotationTagDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  UserType TakeSocket() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::TCPConnectedSocketRequestDataView>(
            &data_->socket, &result, context_);
    DCHECK(ret);
    return result;
  }
  template <typename UserType>
  UserType TakeObserver() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::SocketObserverPtrDataView>(
            &data_->observer, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::NetworkContext_CreateTCPConnectedSocket_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_CreateTCPConnectedSocket_ResponseParamsDataView {
 public:
  NetworkContext_CreateTCPConnectedSocket_ResponseParamsDataView() {}

  NetworkContext_CreateTCPConnectedSocket_ResponseParamsDataView(
      internal::NetworkContext_CreateTCPConnectedSocket_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int32_t result() const {
    return data_->result;
  }
  inline void GetLocalAddrDataView(
      ::network::mojom::IPEndPointDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadLocalAddr(UserType* output) {
    auto* pointer = data_->local_addr.Get();
    return mojo::internal::Deserialize<::network::mojom::IPEndPointDataView>(
        pointer, output, context_);
  }
  inline void GetPeerAddrDataView(
      ::network::mojom::IPEndPointDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPeerAddr(UserType* output) {
    auto* pointer = data_->peer_addr.Get();
    return mojo::internal::Deserialize<::network::mojom::IPEndPointDataView>(
        pointer, output, context_);
  }
  mojo::ScopedDataPipeConsumerHandle TakeReceiveStream() {
    mojo::ScopedDataPipeConsumerHandle result;
    bool ret =
        mojo::internal::Deserialize<mojo::ScopedDataPipeConsumerHandle>(
            &data_->receive_stream, &result, context_);
    DCHECK(ret);
    return result;
  }
  mojo::ScopedDataPipeProducerHandle TakeSendStream() {
    mojo::ScopedDataPipeProducerHandle result;
    bool ret =
        mojo::internal::Deserialize<mojo::ScopedDataPipeProducerHandle>(
            &data_->send_stream, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::NetworkContext_CreateTCPConnectedSocket_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_CreateTCPBoundSocket_ParamsDataView {
 public:
  NetworkContext_CreateTCPBoundSocket_ParamsDataView() {}

  NetworkContext_CreateTCPBoundSocket_ParamsDataView(
      internal::NetworkContext_CreateTCPBoundSocket_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetLocalAddrDataView(
      ::network::mojom::IPEndPointDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadLocalAddr(UserType* output) {
    auto* pointer = data_->local_addr.Get();
    return mojo::internal::Deserialize<::network::mojom::IPEndPointDataView>(
        pointer, output, context_);
  }
  inline void GetTrafficAnnotationDataView(
      ::network::mojom::MutableNetworkTrafficAnnotationTagDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTrafficAnnotation(UserType* output) {
    auto* pointer = data_->traffic_annotation.Get();
    return mojo::internal::Deserialize<::network::mojom::MutableNetworkTrafficAnnotationTagDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  UserType TakeSocket() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::TCPBoundSocketRequestDataView>(
            &data_->socket, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::NetworkContext_CreateTCPBoundSocket_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_CreateTCPBoundSocket_ResponseParamsDataView {
 public:
  NetworkContext_CreateTCPBoundSocket_ResponseParamsDataView() {}

  NetworkContext_CreateTCPBoundSocket_ResponseParamsDataView(
      internal::NetworkContext_CreateTCPBoundSocket_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int32_t result() const {
    return data_->result;
  }
  inline void GetLocalAddrDataView(
      ::network::mojom::IPEndPointDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadLocalAddr(UserType* output) {
    auto* pointer = data_->local_addr.Get();
    return mojo::internal::Deserialize<::network::mojom::IPEndPointDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_CreateTCPBoundSocket_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_CreateProxyResolvingSocketFactory_ParamsDataView {
 public:
  NetworkContext_CreateProxyResolvingSocketFactory_ParamsDataView() {}

  NetworkContext_CreateProxyResolvingSocketFactory_ParamsDataView(
      internal::NetworkContext_CreateProxyResolvingSocketFactory_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeFactory() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::ProxyResolvingSocketFactoryRequestDataView>(
            &data_->factory, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::NetworkContext_CreateProxyResolvingSocketFactory_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_LookUpProxyForURL_ParamsDataView {
 public:
  NetworkContext_LookUpProxyForURL_ParamsDataView() {}

  NetworkContext_LookUpProxyForURL_ParamsDataView(
      internal::NetworkContext_LookUpProxyForURL_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUrl(UserType* output) {
    auto* pointer = data_->url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  UserType TakeProxyLookupClient() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::ProxyLookupClientPtrDataView>(
            &data_->proxy_lookup_client, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::NetworkContext_LookUpProxyForURL_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_ForceReloadProxyConfig_ParamsDataView {
 public:
  NetworkContext_ForceReloadProxyConfig_ParamsDataView() {}

  NetworkContext_ForceReloadProxyConfig_ParamsDataView(
      internal::NetworkContext_ForceReloadProxyConfig_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_ForceReloadProxyConfig_Params_Data* data_ = nullptr;
};

class NetworkContext_ForceReloadProxyConfig_ResponseParamsDataView {
 public:
  NetworkContext_ForceReloadProxyConfig_ResponseParamsDataView() {}

  NetworkContext_ForceReloadProxyConfig_ResponseParamsDataView(
      internal::NetworkContext_ForceReloadProxyConfig_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_ForceReloadProxyConfig_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_ClearBadProxiesCache_ParamsDataView {
 public:
  NetworkContext_ClearBadProxiesCache_ParamsDataView() {}

  NetworkContext_ClearBadProxiesCache_ParamsDataView(
      internal::NetworkContext_ClearBadProxiesCache_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_ClearBadProxiesCache_Params_Data* data_ = nullptr;
};

class NetworkContext_ClearBadProxiesCache_ResponseParamsDataView {
 public:
  NetworkContext_ClearBadProxiesCache_ResponseParamsDataView() {}

  NetworkContext_ClearBadProxiesCache_ResponseParamsDataView(
      internal::NetworkContext_ClearBadProxiesCache_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_ClearBadProxiesCache_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_CreateWebSocket_ParamsDataView {
 public:
  NetworkContext_CreateWebSocket_ParamsDataView() {}

  NetworkContext_CreateWebSocket_ParamsDataView(
      internal::NetworkContext_CreateWebSocket_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeRequest() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::WebSocketRequestDataView>(
            &data_->request, &result, context_);
    DCHECK(ret);
    return result;
  }
  int32_t process_id() const {
    return data_->process_id;
  }
  int32_t render_frame_id() const {
    return data_->render_frame_id;
  }
  inline void GetOriginDataView(
      ::url::mojom::OriginDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOrigin(UserType* output) {
    auto* pointer = data_->origin.Get();
    return mojo::internal::Deserialize<::url::mojom::OriginDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  UserType TakeAuthHandler() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::AuthenticationHandlerPtrDataView>(
            &data_->auth_handler, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::NetworkContext_CreateWebSocket_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_CreateNetLogExporter_ParamsDataView {
 public:
  NetworkContext_CreateNetLogExporter_ParamsDataView() {}

  NetworkContext_CreateNetLogExporter_ParamsDataView(
      internal::NetworkContext_CreateNetLogExporter_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeExporter() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::NetLogExporterRequestDataView>(
            &data_->exporter, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::NetworkContext_CreateNetLogExporter_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_PreconnectSockets_ParamsDataView {
 public:
  NetworkContext_PreconnectSockets_ParamsDataView() {}

  NetworkContext_PreconnectSockets_ParamsDataView(
      internal::NetworkContext_PreconnectSockets_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  uint32_t num_streams() const {
    return data_->num_streams;
  }
  inline void GetUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUrl(UserType* output) {
    auto* pointer = data_->url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  int32_t load_flags() const {
    return data_->load_flags;
  }
  bool privacy_mode_enabled() const {
    return data_->privacy_mode_enabled;
  }
 private:
  internal::NetworkContext_PreconnectSockets_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_CreateP2PSocketManager_ParamsDataView {
 public:
  NetworkContext_CreateP2PSocketManager_ParamsDataView() {}

  NetworkContext_CreateP2PSocketManager_ParamsDataView(
      internal::NetworkContext_CreateP2PSocketManager_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeClient() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::P2PTrustedSocketManagerClientPtrDataView>(
            &data_->client, &result, context_);
    DCHECK(ret);
    return result;
  }
  template <typename UserType>
  UserType TakeTrustedSocketManager() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::P2PTrustedSocketManagerRequestDataView>(
            &data_->trusted_socket_manager, &result, context_);
    DCHECK(ret);
    return result;
  }
  template <typename UserType>
  UserType TakeSocketManager() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::P2PSocketManagerRequestDataView>(
            &data_->socket_manager, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::NetworkContext_CreateP2PSocketManager_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_CreateMdnsResponder_ParamsDataView {
 public:
  NetworkContext_CreateMdnsResponder_ParamsDataView() {}

  NetworkContext_CreateMdnsResponder_ParamsDataView(
      internal::NetworkContext_CreateMdnsResponder_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeResponderRequest() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::MdnsResponderRequestDataView>(
            &data_->responder_request, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::NetworkContext_CreateMdnsResponder_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_ResolveHost_ParamsDataView {
 public:
  NetworkContext_ResolveHost_ParamsDataView() {}

  NetworkContext_ResolveHost_ParamsDataView(
      internal::NetworkContext_ResolveHost_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetHostDataView(
      ::network::mojom::HostPortPairDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadHost(UserType* output) {
    auto* pointer = data_->host.Get();
    return mojo::internal::Deserialize<::network::mojom::HostPortPairDataView>(
        pointer, output, context_);
  }
  inline void GetOptionalParametersDataView(
      ::network::mojom::ResolveHostParametersDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOptionalParameters(UserType* output) {
    auto* pointer = data_->optional_parameters.Get();
    return mojo::internal::Deserialize<::network::mojom::ResolveHostParametersDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  UserType TakeResponseClient() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::ResolveHostClientPtrDataView>(
            &data_->response_client, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::NetworkContext_ResolveHost_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_CreateHostResolver_ParamsDataView {
 public:
  NetworkContext_CreateHostResolver_ParamsDataView() {}

  NetworkContext_CreateHostResolver_ParamsDataView(
      internal::NetworkContext_CreateHostResolver_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetConfigOverridesDataView(
      ::network::mojom::DnsConfigOverridesDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadConfigOverrides(UserType* output) {
    auto* pointer = data_->config_overrides.Get();
    return mojo::internal::Deserialize<::network::mojom::DnsConfigOverridesDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  UserType TakeHostResolver() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::HostResolverRequestDataView>(
            &data_->host_resolver, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::NetworkContext_CreateHostResolver_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_VerifyCertForSignedExchange_ParamsDataView {
 public:
  NetworkContext_VerifyCertForSignedExchange_ParamsDataView() {}

  NetworkContext_VerifyCertForSignedExchange_ParamsDataView(
      internal::NetworkContext_VerifyCertForSignedExchange_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetCertificateDataView(
      ::network::mojom::X509CertificateDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCertificate(UserType* output) {
    auto* pointer = data_->certificate.Get();
    return mojo::internal::Deserialize<::network::mojom::X509CertificateDataView>(
        pointer, output, context_);
  }
  inline void GetUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUrl(UserType* output) {
    auto* pointer = data_->url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  inline void GetOcspResponseDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOcspResponse(UserType* output) {
    auto* pointer = data_->ocsp_response.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetSctListDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSctList(UserType* output) {
    auto* pointer = data_->sct_list.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_VerifyCertForSignedExchange_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_VerifyCertForSignedExchange_ResponseParamsDataView {
 public:
  NetworkContext_VerifyCertForSignedExchange_ResponseParamsDataView() {}

  NetworkContext_VerifyCertForSignedExchange_ResponseParamsDataView(
      internal::NetworkContext_VerifyCertForSignedExchange_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int32_t error_code() const {
    return data_->error_code;
  }
  inline void GetCvResultDataView(
      ::network::mojom::CertVerifyResultDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCvResult(UserType* output) {
    auto* pointer = data_->cv_result.Get();
    return mojo::internal::Deserialize<::network::mojom::CertVerifyResultDataView>(
        pointer, output, context_);
  }
  inline void GetCtResultDataView(
      ::network::mojom::CTVerifyResultDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCtResult(UserType* output) {
    auto* pointer = data_->ct_result.Get();
    return mojo::internal::Deserialize<::network::mojom::CTVerifyResultDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_VerifyCertForSignedExchange_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_AddHSTS_ParamsDataView {
 public:
  NetworkContext_AddHSTS_ParamsDataView() {}

  NetworkContext_AddHSTS_ParamsDataView(
      internal::NetworkContext_AddHSTS_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetHostDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadHost(UserType* output) {
    auto* pointer = data_->host.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetExpiryDataView(
      ::mojo_base::mojom::TimeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadExpiry(UserType* output) {
    auto* pointer = data_->expiry.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::TimeDataView>(
        pointer, output, context_);
  }
  bool include_subdomains() const {
    return data_->include_subdomains;
  }
 private:
  internal::NetworkContext_AddHSTS_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_AddHSTS_ResponseParamsDataView {
 public:
  NetworkContext_AddHSTS_ResponseParamsDataView() {}

  NetworkContext_AddHSTS_ResponseParamsDataView(
      internal::NetworkContext_AddHSTS_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_AddHSTS_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_IsHSTSActiveForHost_ParamsDataView {
 public:
  NetworkContext_IsHSTSActiveForHost_ParamsDataView() {}

  NetworkContext_IsHSTSActiveForHost_ParamsDataView(
      internal::NetworkContext_IsHSTSActiveForHost_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetHostDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadHost(UserType* output) {
    auto* pointer = data_->host.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_IsHSTSActiveForHost_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_IsHSTSActiveForHost_ResponseParamsDataView {
 public:
  NetworkContext_IsHSTSActiveForHost_ResponseParamsDataView() {}

  NetworkContext_IsHSTSActiveForHost_ResponseParamsDataView(
      internal::NetworkContext_IsHSTSActiveForHost_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool result() const {
    return data_->result;
  }
 private:
  internal::NetworkContext_IsHSTSActiveForHost_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_GetHSTSState_ParamsDataView {
 public:
  NetworkContext_GetHSTSState_ParamsDataView() {}

  NetworkContext_GetHSTSState_ParamsDataView(
      internal::NetworkContext_GetHSTSState_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetDomainDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDomain(UserType* output) {
    auto* pointer = data_->domain.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_GetHSTSState_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_GetHSTSState_ResponseParamsDataView {
 public:
  NetworkContext_GetHSTSState_ResponseParamsDataView() {}

  NetworkContext_GetHSTSState_ResponseParamsDataView(
      internal::NetworkContext_GetHSTSState_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetStateDataView(
      ::mojo_base::mojom::DictionaryValueDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadState(UserType* output) {
    auto* pointer = data_->state.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::DictionaryValueDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_GetHSTSState_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_SetCorsOriginAccessListsForOrigin_ParamsDataView {
 public:
  NetworkContext_SetCorsOriginAccessListsForOrigin_ParamsDataView() {}

  NetworkContext_SetCorsOriginAccessListsForOrigin_ParamsDataView(
      internal::NetworkContext_SetCorsOriginAccessListsForOrigin_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetSourceOriginDataView(
      ::url::mojom::OriginDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSourceOrigin(UserType* output) {
    auto* pointer = data_->source_origin.Get();
    return mojo::internal::Deserialize<::url::mojom::OriginDataView>(
        pointer, output, context_);
  }
  inline void GetAllowPatternsDataView(
      mojo::ArrayDataView<::network::mojom::CorsOriginPatternDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAllowPatterns(UserType* output) {
    auto* pointer = data_->allow_patterns.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::network::mojom::CorsOriginPatternDataView>>(
        pointer, output, context_);
  }
  inline void GetBlockPatternsDataView(
      mojo::ArrayDataView<::network::mojom::CorsOriginPatternDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadBlockPatterns(UserType* output) {
    auto* pointer = data_->block_patterns.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::network::mojom::CorsOriginPatternDataView>>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_SetCorsOriginAccessListsForOrigin_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_SetCorsOriginAccessListsForOrigin_ResponseParamsDataView {
 public:
  NetworkContext_SetCorsOriginAccessListsForOrigin_ResponseParamsDataView() {}

  NetworkContext_SetCorsOriginAccessListsForOrigin_ResponseParamsDataView(
      internal::NetworkContext_SetCorsOriginAccessListsForOrigin_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_SetCorsOriginAccessListsForOrigin_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_DeleteDynamicDataForHost_ParamsDataView {
 public:
  NetworkContext_DeleteDynamicDataForHost_ParamsDataView() {}

  NetworkContext_DeleteDynamicDataForHost_ParamsDataView(
      internal::NetworkContext_DeleteDynamicDataForHost_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetHostDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadHost(UserType* output) {
    auto* pointer = data_->host.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_DeleteDynamicDataForHost_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_DeleteDynamicDataForHost_ResponseParamsDataView {
 public:
  NetworkContext_DeleteDynamicDataForHost_ResponseParamsDataView() {}

  NetworkContext_DeleteDynamicDataForHost_ResponseParamsDataView(
      internal::NetworkContext_DeleteDynamicDataForHost_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool result() const {
    return data_->result;
  }
 private:
  internal::NetworkContext_DeleteDynamicDataForHost_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_SaveHttpAuthCache_ParamsDataView {
 public:
  NetworkContext_SaveHttpAuthCache_ParamsDataView() {}

  NetworkContext_SaveHttpAuthCache_ParamsDataView(
      internal::NetworkContext_SaveHttpAuthCache_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_SaveHttpAuthCache_Params_Data* data_ = nullptr;
};

class NetworkContext_SaveHttpAuthCache_ResponseParamsDataView {
 public:
  NetworkContext_SaveHttpAuthCache_ResponseParamsDataView() {}

  NetworkContext_SaveHttpAuthCache_ResponseParamsDataView(
      internal::NetworkContext_SaveHttpAuthCache_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetCacheKeyDataView(
      ::mojo_base::mojom::UnguessableTokenDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCacheKey(UserType* output) {
    auto* pointer = data_->cache_key.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::UnguessableTokenDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_SaveHttpAuthCache_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_LoadHttpAuthCache_ParamsDataView {
 public:
  NetworkContext_LoadHttpAuthCache_ParamsDataView() {}

  NetworkContext_LoadHttpAuthCache_ParamsDataView(
      internal::NetworkContext_LoadHttpAuthCache_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetCacheKeyDataView(
      ::mojo_base::mojom::UnguessableTokenDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCacheKey(UserType* output) {
    auto* pointer = data_->cache_key.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::UnguessableTokenDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_LoadHttpAuthCache_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_LoadHttpAuthCache_ResponseParamsDataView {
 public:
  NetworkContext_LoadHttpAuthCache_ResponseParamsDataView() {}

  NetworkContext_LoadHttpAuthCache_ResponseParamsDataView(
      internal::NetworkContext_LoadHttpAuthCache_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_LoadHttpAuthCache_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_LookupBasicAuthCredentials_ParamsDataView {
 public:
  NetworkContext_LookupBasicAuthCredentials_ParamsDataView() {}

  NetworkContext_LookupBasicAuthCredentials_ParamsDataView(
      internal::NetworkContext_LookupBasicAuthCredentials_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUrl(UserType* output) {
    auto* pointer = data_->url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_LookupBasicAuthCredentials_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_LookupBasicAuthCredentials_ResponseParamsDataView {
 public:
  NetworkContext_LookupBasicAuthCredentials_ResponseParamsDataView() {}

  NetworkContext_LookupBasicAuthCredentials_ResponseParamsDataView(
      internal::NetworkContext_LookupBasicAuthCredentials_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetCredentialsDataView(
      ::network::mojom::AuthCredentialsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCredentials(UserType* output) {
    auto* pointer = data_->credentials.Get();
    return mojo::internal::Deserialize<::network::mojom::AuthCredentialsDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_LookupBasicAuthCredentials_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_EnableStaticKeyPinningForTesting_ParamsDataView {
 public:
  NetworkContext_EnableStaticKeyPinningForTesting_ParamsDataView() {}

  NetworkContext_EnableStaticKeyPinningForTesting_ParamsDataView(
      internal::NetworkContext_EnableStaticKeyPinningForTesting_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_EnableStaticKeyPinningForTesting_Params_Data* data_ = nullptr;
};

class NetworkContext_EnableStaticKeyPinningForTesting_ResponseParamsDataView {
 public:
  NetworkContext_EnableStaticKeyPinningForTesting_ResponseParamsDataView() {}

  NetworkContext_EnableStaticKeyPinningForTesting_ResponseParamsDataView(
      internal::NetworkContext_EnableStaticKeyPinningForTesting_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_EnableStaticKeyPinningForTesting_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_SetFailingHttpTransactionForTesting_ParamsDataView {
 public:
  NetworkContext_SetFailingHttpTransactionForTesting_ParamsDataView() {}

  NetworkContext_SetFailingHttpTransactionForTesting_ParamsDataView(
      internal::NetworkContext_SetFailingHttpTransactionForTesting_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  int32_t rv() const {
    return data_->rv;
  }
 private:
  internal::NetworkContext_SetFailingHttpTransactionForTesting_Params_Data* data_ = nullptr;
};

class NetworkContext_SetFailingHttpTransactionForTesting_ResponseParamsDataView {
 public:
  NetworkContext_SetFailingHttpTransactionForTesting_ResponseParamsDataView() {}

  NetworkContext_SetFailingHttpTransactionForTesting_ResponseParamsDataView(
      internal::NetworkContext_SetFailingHttpTransactionForTesting_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_SetFailingHttpTransactionForTesting_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_VerifyCertificateForTesting_ParamsDataView {
 public:
  NetworkContext_VerifyCertificateForTesting_ParamsDataView() {}

  NetworkContext_VerifyCertificateForTesting_ParamsDataView(
      internal::NetworkContext_VerifyCertificateForTesting_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetCertificateDataView(
      ::network::mojom::X509CertificateDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCertificate(UserType* output) {
    auto* pointer = data_->certificate.Get();
    return mojo::internal::Deserialize<::network::mojom::X509CertificateDataView>(
        pointer, output, context_);
  }
  inline void GetHostnameDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadHostname(UserType* output) {
    auto* pointer = data_->hostname.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetOcspResponseDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOcspResponse(UserType* output) {
    auto* pointer = data_->ocsp_response.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_VerifyCertificateForTesting_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_VerifyCertificateForTesting_ResponseParamsDataView {
 public:
  NetworkContext_VerifyCertificateForTesting_ResponseParamsDataView() {}

  NetworkContext_VerifyCertificateForTesting_ResponseParamsDataView(
      internal::NetworkContext_VerifyCertificateForTesting_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  int32_t error_code() const {
    return data_->error_code;
  }
 private:
  internal::NetworkContext_VerifyCertificateForTesting_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_AddDomainReliabilityContextForTesting_ParamsDataView {
 public:
  NetworkContext_AddDomainReliabilityContextForTesting_ParamsDataView() {}

  NetworkContext_AddDomainReliabilityContextForTesting_ParamsDataView(
      internal::NetworkContext_AddDomainReliabilityContextForTesting_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetOriginDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOrigin(UserType* output) {
    auto* pointer = data_->origin.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  inline void GetUploadUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUploadUrl(UserType* output) {
    auto* pointer = data_->upload_url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
 private:
  internal::NetworkContext_AddDomainReliabilityContextForTesting_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class NetworkContext_AddDomainReliabilityContextForTesting_ResponseParamsDataView {
 public:
  NetworkContext_AddDomainReliabilityContextForTesting_ResponseParamsDataView() {}

  NetworkContext_AddDomainReliabilityContextForTesting_ResponseParamsDataView(
      internal::NetworkContext_AddDomainReliabilityContextForTesting_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_AddDomainReliabilityContextForTesting_ResponseParams_Data* data_ = nullptr;
};

class NetworkContext_ForceDomainReliabilityUploadsForTesting_ParamsDataView {
 public:
  NetworkContext_ForceDomainReliabilityUploadsForTesting_ParamsDataView() {}

  NetworkContext_ForceDomainReliabilityUploadsForTesting_ParamsDataView(
      internal::NetworkContext_ForceDomainReliabilityUploadsForTesting_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_ForceDomainReliabilityUploadsForTesting_Params_Data* data_ = nullptr;
};

class NetworkContext_ForceDomainReliabilityUploadsForTesting_ResponseParamsDataView {
 public:
  NetworkContext_ForceDomainReliabilityUploadsForTesting_ResponseParamsDataView() {}

  NetworkContext_ForceDomainReliabilityUploadsForTesting_ResponseParamsDataView(
      internal::NetworkContext_ForceDomainReliabilityUploadsForTesting_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::NetworkContext_ForceDomainReliabilityUploadsForTesting_ResponseParams_Data* data_ = nullptr;
};



}  // namespace mojom
}  // namespace network

namespace std {

template <>
struct hash<::network::mojom::ClearDataFilter_Type>
    : public mojo::internal::EnumHashImpl<::network::mojom::ClearDataFilter_Type> {};

template <>
struct hash<::network::mojom::NetworkContext_DomainReliabilityClearMode>
    : public mojo::internal::EnumHashImpl<::network::mojom::NetworkContext_DomainReliabilityClearMode> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::network::mojom::ClearDataFilter_Type, ::network::mojom::ClearDataFilter_Type> {
  static ::network::mojom::ClearDataFilter_Type ToMojom(::network::mojom::ClearDataFilter_Type input) { return input; }
  static bool FromMojom(::network::mojom::ClearDataFilter_Type input, ::network::mojom::ClearDataFilter_Type* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::network::mojom::ClearDataFilter_Type, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::network::mojom::ClearDataFilter_Type, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::network::mojom::ClearDataFilter_Type>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::network::mojom::NetworkContext_DomainReliabilityClearMode, ::network::mojom::NetworkContext_DomainReliabilityClearMode> {
  static ::network::mojom::NetworkContext_DomainReliabilityClearMode ToMojom(::network::mojom::NetworkContext_DomainReliabilityClearMode input) { return input; }
  static bool FromMojom(::network::mojom::NetworkContext_DomainReliabilityClearMode input, ::network::mojom::NetworkContext_DomainReliabilityClearMode* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::network::mojom::NetworkContext_DomainReliabilityClearMode, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::network::mojom::NetworkContext_DomainReliabilityClearMode, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::network::mojom::NetworkContext_DomainReliabilityClearMode>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::network::mojom::CustomProxyConfigDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::network::mojom::CustomProxyConfigDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::network::mojom::internal::CustomProxyConfig_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    decltype(CallWithContext(Traits::rules, input, custom_context)) in_rules = CallWithContext(Traits::rules, input, custom_context);
    typename decltype((*output)->rules)::BaseType::BufferWriter
        rules_writer;
    mojo::internal::Serialize<::network::mojom::ProxyRulesDataView>(
        in_rules, buffer, &rules_writer, context);
    (*output)->rules.Set(
        rules_writer.is_null() ? nullptr : rules_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->rules.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null rules in CustomProxyConfig struct");
    decltype(CallWithContext(Traits::alternate_proxy_list, input, custom_context)) in_alternate_proxy_list = CallWithContext(Traits::alternate_proxy_list, input, custom_context);
    typename decltype((*output)->alternate_proxy_list)::BaseType::BufferWriter
        alternate_proxy_list_writer;
    mojo::internal::Serialize<::network::mojom::ProxyListDataView>(
        in_alternate_proxy_list, buffer, &alternate_proxy_list_writer, context);
    (*output)->alternate_proxy_list.Set(
        alternate_proxy_list_writer.is_null() ? nullptr : alternate_proxy_list_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->alternate_proxy_list.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null alternate_proxy_list in CustomProxyConfig struct");
    decltype(CallWithContext(Traits::pre_cache_headers, input, custom_context)) in_pre_cache_headers = CallWithContext(Traits::pre_cache_headers, input, custom_context);
    typename decltype((*output)->pre_cache_headers)::BaseType::BufferWriter
        pre_cache_headers_writer;
    mojo::internal::Serialize<::network::mojom::HttpRequestHeadersDataView>(
        in_pre_cache_headers, buffer, &pre_cache_headers_writer, context);
    (*output)->pre_cache_headers.Set(
        pre_cache_headers_writer.is_null() ? nullptr : pre_cache_headers_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->pre_cache_headers.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null pre_cache_headers in CustomProxyConfig struct");
    decltype(CallWithContext(Traits::post_cache_headers, input, custom_context)) in_post_cache_headers = CallWithContext(Traits::post_cache_headers, input, custom_context);
    typename decltype((*output)->post_cache_headers)::BaseType::BufferWriter
        post_cache_headers_writer;
    mojo::internal::Serialize<::network::mojom::HttpRequestHeadersDataView>(
        in_post_cache_headers, buffer, &post_cache_headers_writer, context);
    (*output)->post_cache_headers.Set(
        post_cache_headers_writer.is_null() ? nullptr : post_cache_headers_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->post_cache_headers.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null post_cache_headers in CustomProxyConfig struct");
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::network::mojom::internal::CustomProxyConfig_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::network::mojom::CustomProxyConfigDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::network::mojom::NetworkContextParamsDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::network::mojom::NetworkContextParamsDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::network::mojom::internal::NetworkContextParams_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    decltype(CallWithContext(Traits::context_name, input, custom_context)) in_context_name = CallWithContext(Traits::context_name, input, custom_context);
    typename decltype((*output)->context_name)::BaseType::BufferWriter
        context_name_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_context_name, buffer, &context_name_writer, context);
    (*output)->context_name.Set(
        context_name_writer.is_null() ? nullptr : context_name_writer.data());
    decltype(CallWithContext(Traits::user_agent, input, custom_context)) in_user_agent = CallWithContext(Traits::user_agent, input, custom_context);
    typename decltype((*output)->user_agent)::BaseType::BufferWriter
        user_agent_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_user_agent, buffer, &user_agent_writer, context);
    (*output)->user_agent.Set(
        user_agent_writer.is_null() ? nullptr : user_agent_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->user_agent.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null user_agent in NetworkContextParams struct");
    decltype(CallWithContext(Traits::accept_language, input, custom_context)) in_accept_language = CallWithContext(Traits::accept_language, input, custom_context);
    typename decltype((*output)->accept_language)::BaseType::BufferWriter
        accept_language_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_accept_language, buffer, &accept_language_writer, context);
    (*output)->accept_language.Set(
        accept_language_writer.is_null() ? nullptr : accept_language_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->accept_language.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null accept_language in NetworkContextParams struct");
    (*output)->enable_brotli = CallWithContext(Traits::enable_brotli, input, custom_context);
    decltype(CallWithContext(Traits::quic_user_agent_id, input, custom_context)) in_quic_user_agent_id = CallWithContext(Traits::quic_user_agent_id, input, custom_context);
    typename decltype((*output)->quic_user_agent_id)::BaseType::BufferWriter
        quic_user_agent_id_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_quic_user_agent_id, buffer, &quic_user_agent_id_writer, context);
    (*output)->quic_user_agent_id.Set(
        quic_user_agent_id_writer.is_null() ? nullptr : quic_user_agent_id_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->quic_user_agent_id.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null quic_user_agent_id in NetworkContextParams struct");
    (*output)->enable_referrers = CallWithContext(Traits::enable_referrers, input, custom_context);
    (*output)->validate_referrer_policy_on_initial_request = CallWithContext(Traits::validate_referrer_policy_on_initial_request, input, custom_context);
    decltype(CallWithContext(Traits::proxy_resolver_factory, input, custom_context)) in_proxy_resolver_factory = CallWithContext(Traits::proxy_resolver_factory, input, custom_context);
    mojo::internal::Serialize<::proxy_resolver::mojom::ProxyResolverFactoryPtrDataView>(
        in_proxy_resolver_factory, &(*output)->proxy_resolver_factory, context);
    decltype(CallWithContext(Traits::cookie_path, input, custom_context)) in_cookie_path = CallWithContext(Traits::cookie_path, input, custom_context);
    typename decltype((*output)->cookie_path)::BaseType::BufferWriter
        cookie_path_writer;
    mojo::internal::Serialize<::mojo_base::mojom::FilePathDataView>(
        in_cookie_path, buffer, &cookie_path_writer, context);
    (*output)->cookie_path.Set(
        cookie_path_writer.is_null() ? nullptr : cookie_path_writer.data());
    (*output)->enable_encrypted_cookies = CallWithContext(Traits::enable_encrypted_cookies, input, custom_context);
    (*output)->restore_old_session_cookies = CallWithContext(Traits::restore_old_session_cookies, input, custom_context);
    (*output)->persist_session_cookies = CallWithContext(Traits::persist_session_cookies, input, custom_context);
    decltype(CallWithContext(Traits::channel_id_path, input, custom_context)) in_channel_id_path = CallWithContext(Traits::channel_id_path, input, custom_context);
    typename decltype((*output)->channel_id_path)::BaseType::BufferWriter
        channel_id_path_writer;
    mojo::internal::Serialize<::mojo_base::mojom::FilePathDataView>(
        in_channel_id_path, buffer, &channel_id_path_writer, context);
    (*output)->channel_id_path.Set(
        channel_id_path_writer.is_null() ? nullptr : channel_id_path_writer.data());
    (*output)->http_cache_enabled = CallWithContext(Traits::http_cache_enabled, input, custom_context);
    (*output)->http_cache_max_size = CallWithContext(Traits::http_cache_max_size, input, custom_context);
    decltype(CallWithContext(Traits::http_cache_path, input, custom_context)) in_http_cache_path = CallWithContext(Traits::http_cache_path, input, custom_context);
    typename decltype((*output)->http_cache_path)::BaseType::BufferWriter
        http_cache_path_writer;
    mojo::internal::Serialize<::mojo_base::mojom::FilePathDataView>(
        in_http_cache_path, buffer, &http_cache_path_writer, context);
    (*output)->http_cache_path.Set(
        http_cache_path_writer.is_null() ? nullptr : http_cache_path_writer.data());
    decltype(CallWithContext(Traits::http_server_properties_path, input, custom_context)) in_http_server_properties_path = CallWithContext(Traits::http_server_properties_path, input, custom_context);
    typename decltype((*output)->http_server_properties_path)::BaseType::BufferWriter
        http_server_properties_path_writer;
    mojo::internal::Serialize<::mojo_base::mojom::FilePathDataView>(
        in_http_server_properties_path, buffer, &http_server_properties_path_writer, context);
    (*output)->http_server_properties_path.Set(
        http_server_properties_path_writer.is_null() ? nullptr : http_server_properties_path_writer.data());
    decltype(CallWithContext(Traits::transport_security_persister_path, input, custom_context)) in_transport_security_persister_path = CallWithContext(Traits::transport_security_persister_path, input, custom_context);
    typename decltype((*output)->transport_security_persister_path)::BaseType::BufferWriter
        transport_security_persister_path_writer;
    mojo::internal::Serialize<::mojo_base::mojom::FilePathDataView>(
        in_transport_security_persister_path, buffer, &transport_security_persister_path_writer, context);
    (*output)->transport_security_persister_path.Set(
        transport_security_persister_path_writer.is_null() ? nullptr : transport_security_persister_path_writer.data());
    (*output)->enable_data_url_support = CallWithContext(Traits::enable_data_url_support, input, custom_context);
    (*output)->enable_file_url_support = CallWithContext(Traits::enable_file_url_support, input, custom_context);
    (*output)->enable_ftp_url_support = CallWithContext(Traits::enable_ftp_url_support, input, custom_context);
    (*output)->enforce_chrome_ct_policy = CallWithContext(Traits::enforce_chrome_ct_policy, input, custom_context);
    (*output)->http_09_on_non_default_ports_enabled = CallWithContext(Traits::http_09_on_non_default_ports_enabled, input, custom_context);
    decltype(CallWithContext(Traits::initial_ssl_config, input, custom_context)) in_initial_ssl_config = CallWithContext(Traits::initial_ssl_config, input, custom_context);
    typename decltype((*output)->initial_ssl_config)::BaseType::BufferWriter
        initial_ssl_config_writer;
    mojo::internal::Serialize<::network::mojom::SSLConfigDataView>(
        in_initial_ssl_config, buffer, &initial_ssl_config_writer, context);
    (*output)->initial_ssl_config.Set(
        initial_ssl_config_writer.is_null() ? nullptr : initial_ssl_config_writer.data());
    decltype(CallWithContext(Traits::ssl_config_client_request, input, custom_context)) in_ssl_config_client_request = CallWithContext(Traits::ssl_config_client_request, input, custom_context);
    mojo::internal::Serialize<::network::mojom::SSLConfigClientRequestDataView>(
        in_ssl_config_client_request, &(*output)->ssl_config_client_request, context);
    decltype(CallWithContext(Traits::initial_proxy_config, input, custom_context)) in_initial_proxy_config = CallWithContext(Traits::initial_proxy_config, input, custom_context);
    typename decltype((*output)->initial_proxy_config)::BaseType::BufferWriter
        initial_proxy_config_writer;
    mojo::internal::Serialize<::network::mojom::ProxyConfigWithAnnotationDataView>(
        in_initial_proxy_config, buffer, &initial_proxy_config_writer, context);
    (*output)->initial_proxy_config.Set(
        initial_proxy_config_writer.is_null() ? nullptr : initial_proxy_config_writer.data());
    decltype(CallWithContext(Traits::proxy_config_client_request, input, custom_context)) in_proxy_config_client_request = CallWithContext(Traits::proxy_config_client_request, input, custom_context);
    mojo::internal::Serialize<::network::mojom::ProxyConfigClientRequestDataView>(
        in_proxy_config_client_request, &(*output)->proxy_config_client_request, context);
    decltype(CallWithContext(Traits::initial_custom_proxy_config, input, custom_context)) in_initial_custom_proxy_config = CallWithContext(Traits::initial_custom_proxy_config, input, custom_context);
    typename decltype((*output)->initial_custom_proxy_config)::BaseType::BufferWriter
        initial_custom_proxy_config_writer;
    mojo::internal::Serialize<::network::mojom::CustomProxyConfigDataView>(
        in_initial_custom_proxy_config, buffer, &initial_custom_proxy_config_writer, context);
    (*output)->initial_custom_proxy_config.Set(
        initial_custom_proxy_config_writer.is_null() ? nullptr : initial_custom_proxy_config_writer.data());
    decltype(CallWithContext(Traits::custom_proxy_config_client_request, input, custom_context)) in_custom_proxy_config_client_request = CallWithContext(Traits::custom_proxy_config_client_request, input, custom_context);
    mojo::internal::Serialize<::network::mojom::CustomProxyConfigClientRequestDataView>(
        in_custom_proxy_config_client_request, &(*output)->custom_proxy_config_client_request, context);
    decltype(CallWithContext(Traits::proxy_config_poller_client, input, custom_context)) in_proxy_config_poller_client = CallWithContext(Traits::proxy_config_poller_client, input, custom_context);
    mojo::internal::Serialize<::network::mojom::ProxyConfigPollerClientPtrDataView>(
        in_proxy_config_poller_client, &(*output)->proxy_config_poller_client, context);
    decltype(CallWithContext(Traits::proxy_error_client, input, custom_context)) in_proxy_error_client = CallWithContext(Traits::proxy_error_client, input, custom_context);
    mojo::internal::Serialize<::network::mojom::ProxyErrorClientPtrDataView>(
        in_proxy_error_client, &(*output)->proxy_error_client, context);
    (*output)->pac_quick_check_enabled = CallWithContext(Traits::pac_quick_check_enabled, input, custom_context);
    (*output)->dangerously_allow_pac_access_to_secure_urls = CallWithContext(Traits::dangerously_allow_pac_access_to_secure_urls, input, custom_context);
    (*output)->enable_certificate_reporting = CallWithContext(Traits::enable_certificate_reporting, input, custom_context);
    (*output)->enable_expect_ct_reporting = CallWithContext(Traits::enable_expect_ct_reporting, input, custom_context);
    decltype(CallWithContext(Traits::ct_logs, input, custom_context)) in_ct_logs = CallWithContext(Traits::ct_logs, input, custom_context);
    typename decltype((*output)->ct_logs)::BaseType::BufferWriter
        ct_logs_writer;
    const mojo::internal::ContainerValidateParams ct_logs_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::network::mojom::CTLogInfoDataView>>(
        in_ct_logs, buffer, &ct_logs_writer, &ct_logs_validate_params,
        context);
    (*output)->ct_logs.Set(
        ct_logs_writer.is_null() ? nullptr : ct_logs_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->ct_logs.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null ct_logs in NetworkContextParams struct");
    decltype(CallWithContext(Traits::cookie_manager_params, input, custom_context)) in_cookie_manager_params = CallWithContext(Traits::cookie_manager_params, input, custom_context);
    typename decltype((*output)->cookie_manager_params)::BaseType::BufferWriter
        cookie_manager_params_writer;
    mojo::internal::Serialize<::network::mojom::CookieManagerParamsDataView>(
        in_cookie_manager_params, buffer, &cookie_manager_params_writer, context);
    (*output)->cookie_manager_params.Set(
        cookie_manager_params_writer.is_null() ? nullptr : cookie_manager_params_writer.data());
    (*output)->enable_domain_reliability = CallWithContext(Traits::enable_domain_reliability, input, custom_context);
    decltype(CallWithContext(Traits::domain_reliability_upload_reporter, input, custom_context)) in_domain_reliability_upload_reporter = CallWithContext(Traits::domain_reliability_upload_reporter, input, custom_context);
    typename decltype((*output)->domain_reliability_upload_reporter)::BaseType::BufferWriter
        domain_reliability_upload_reporter_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_domain_reliability_upload_reporter, buffer, &domain_reliability_upload_reporter_writer, context);
    (*output)->domain_reliability_upload_reporter.Set(
        domain_reliability_upload_reporter_writer.is_null() ? nullptr : domain_reliability_upload_reporter_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->domain_reliability_upload_reporter.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null domain_reliability_upload_reporter in NetworkContextParams struct");
    (*output)->discard_domain_reliablity_uploads = CallWithContext(Traits::discard_domain_reliablity_uploads, input, custom_context);
    decltype(CallWithContext(Traits::reporting_delivery_interval, input, custom_context)) in_reporting_delivery_interval = CallWithContext(Traits::reporting_delivery_interval, input, custom_context);
    typename decltype((*output)->reporting_delivery_interval)::BaseType::BufferWriter
        reporting_delivery_interval_writer;
    mojo::internal::Serialize<::mojo_base::mojom::TimeDeltaDataView>(
        in_reporting_delivery_interval, buffer, &reporting_delivery_interval_writer, context);
    (*output)->reporting_delivery_interval.Set(
        reporting_delivery_interval_writer.is_null() ? nullptr : reporting_delivery_interval_writer.data());
    (*output)->skip_reporting_send_permission_check = CallWithContext(Traits::skip_reporting_send_permission_check, input, custom_context);
    (*output)->primary_network_context = CallWithContext(Traits::primary_network_context, input, custom_context);
    decltype(CallWithContext(Traits::cors_origin_access_list, input, custom_context)) in_cors_origin_access_list = CallWithContext(Traits::cors_origin_access_list, input, custom_context);
    typename decltype((*output)->cors_origin_access_list)::BaseType::BufferWriter
        cors_origin_access_list_writer;
    const mojo::internal::ContainerValidateParams cors_origin_access_list_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::network::mojom::CorsOriginAccessPatternsDataView>>(
        in_cors_origin_access_list, buffer, &cors_origin_access_list_writer, &cors_origin_access_list_validate_params,
        context);
    (*output)->cors_origin_access_list.Set(
        cors_origin_access_list_writer.is_null() ? nullptr : cors_origin_access_list_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->cors_origin_access_list.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null cors_origin_access_list in NetworkContextParams struct");
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::network::mojom::internal::NetworkContextParams_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::network::mojom::NetworkContextParamsDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::network::mojom::NetworkConditionsDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::network::mojom::NetworkConditionsDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::network::mojom::internal::NetworkConditions_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    (*output)->offline = CallWithContext(Traits::offline, input, custom_context);
    decltype(CallWithContext(Traits::latency, input, custom_context)) in_latency = CallWithContext(Traits::latency, input, custom_context);
    typename decltype((*output)->latency)::BaseType::BufferWriter
        latency_writer;
    mojo::internal::Serialize<::mojo_base::mojom::TimeDeltaDataView>(
        in_latency, buffer, &latency_writer, context);
    (*output)->latency.Set(
        latency_writer.is_null() ? nullptr : latency_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->latency.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null latency in NetworkConditions struct");
    (*output)->download_throughput = CallWithContext(Traits::download_throughput, input, custom_context);
    (*output)->upload_throughput = CallWithContext(Traits::upload_throughput, input, custom_context);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::network::mojom::internal::NetworkConditions_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::network::mojom::NetworkConditionsDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::network::mojom::ClearDataFilterDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::network::mojom::ClearDataFilterDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::network::mojom::internal::ClearDataFilter_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    mojo::internal::Serialize<::network::mojom::ClearDataFilter_Type>(
        CallWithContext(Traits::type, input, custom_context), &(*output)->type);
    decltype(CallWithContext(Traits::domains, input, custom_context)) in_domains = CallWithContext(Traits::domains, input, custom_context);
    typename decltype((*output)->domains)::BaseType::BufferWriter
        domains_writer;
    const mojo::internal::ContainerValidateParams domains_validate_params(
        0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
    mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
        in_domains, buffer, &domains_writer, &domains_validate_params,
        context);
    (*output)->domains.Set(
        domains_writer.is_null() ? nullptr : domains_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->domains.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null domains in ClearDataFilter struct");
    decltype(CallWithContext(Traits::origins, input, custom_context)) in_origins = CallWithContext(Traits::origins, input, custom_context);
    typename decltype((*output)->origins)::BaseType::BufferWriter
        origins_writer;
    const mojo::internal::ContainerValidateParams origins_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::url::mojom::OriginDataView>>(
        in_origins, buffer, &origins_writer, &origins_validate_params,
        context);
    (*output)->origins.Set(
        origins_writer.is_null() ? nullptr : origins_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->origins.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null origins in ClearDataFilter struct");
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::network::mojom::internal::ClearDataFilter_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::network::mojom::ClearDataFilterDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::network::mojom::NetworkUsageDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::network::mojom::NetworkUsageDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::network::mojom::internal::NetworkUsage_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    (*output)->process_id = CallWithContext(Traits::process_id, input, custom_context);
    (*output)->routing_id = CallWithContext(Traits::routing_id, input, custom_context);
    (*output)->total_bytes_received = CallWithContext(Traits::total_bytes_received, input, custom_context);
    (*output)->total_bytes_sent = CallWithContext(Traits::total_bytes_sent, input, custom_context);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::network::mojom::internal::NetworkUsage_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::network::mojom::NetworkUsageDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::network::mojom::URLLoaderFactoryParamsDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::network::mojom::URLLoaderFactoryParamsDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::network::mojom::internal::URLLoaderFactoryParams_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    (*output)->process_id = CallWithContext(Traits::process_id, input, custom_context);
    decltype(CallWithContext(Traits::request_initiator_site_lock, input, custom_context)) in_request_initiator_site_lock = CallWithContext(Traits::request_initiator_site_lock, input, custom_context);
    typename decltype((*output)->request_initiator_site_lock)::BaseType::BufferWriter
        request_initiator_site_lock_writer;
    mojo::internal::Serialize<::url::mojom::OriginDataView>(
        in_request_initiator_site_lock, buffer, &request_initiator_site_lock_writer, context);
    (*output)->request_initiator_site_lock.Set(
        request_initiator_site_lock_writer.is_null() ? nullptr : request_initiator_site_lock_writer.data());
    (*output)->is_corb_enabled = CallWithContext(Traits::is_corb_enabled, input, custom_context);
    (*output)->corb_detachable_resource_type = CallWithContext(Traits::corb_detachable_resource_type, input, custom_context);
    (*output)->corb_excluded_resource_type = CallWithContext(Traits::corb_excluded_resource_type, input, custom_context);
    (*output)->disable_web_security = CallWithContext(Traits::disable_web_security, input, custom_context);
    decltype(CallWithContext(Traits::header_client, input, custom_context)) in_header_client = CallWithContext(Traits::header_client, input, custom_context);
    mojo::internal::Serialize<::network::mojom::TrustedURLLoaderHeaderClientPtrDataView>(
        in_header_client, &(*output)->header_client, context);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::network::mojom::internal::URLLoaderFactoryParams_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::network::mojom::URLLoaderFactoryParamsDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace network {
namespace mojom {

inline void CustomProxyConfigDataView::GetRulesDataView(
    ::network::mojom::ProxyRulesDataView* output) {
  auto pointer = data_->rules.Get();
  *output = ::network::mojom::ProxyRulesDataView(pointer, context_);
}
inline void CustomProxyConfigDataView::GetAlternateProxyListDataView(
    ::network::mojom::ProxyListDataView* output) {
  auto pointer = data_->alternate_proxy_list.Get();
  *output = ::network::mojom::ProxyListDataView(pointer, context_);
}
inline void CustomProxyConfigDataView::GetPreCacheHeadersDataView(
    ::network::mojom::HttpRequestHeadersDataView* output) {
  auto pointer = data_->pre_cache_headers.Get();
  *output = ::network::mojom::HttpRequestHeadersDataView(pointer, context_);
}
inline void CustomProxyConfigDataView::GetPostCacheHeadersDataView(
    ::network::mojom::HttpRequestHeadersDataView* output) {
  auto pointer = data_->post_cache_headers.Get();
  *output = ::network::mojom::HttpRequestHeadersDataView(pointer, context_);
}


inline void NetworkContextParamsDataView::GetContextNameDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->context_name.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void NetworkContextParamsDataView::GetUserAgentDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->user_agent.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void NetworkContextParamsDataView::GetAcceptLanguageDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->accept_language.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void NetworkContextParamsDataView::GetQuicUserAgentIdDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->quic_user_agent_id.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void NetworkContextParamsDataView::GetCookiePathDataView(
    ::mojo_base::mojom::FilePathDataView* output) {
  auto pointer = data_->cookie_path.Get();
  *output = ::mojo_base::mojom::FilePathDataView(pointer, context_);
}
inline void NetworkContextParamsDataView::GetChannelIdPathDataView(
    ::mojo_base::mojom::FilePathDataView* output) {
  auto pointer = data_->channel_id_path.Get();
  *output = ::mojo_base::mojom::FilePathDataView(pointer, context_);
}
inline void NetworkContextParamsDataView::GetHttpCachePathDataView(
    ::mojo_base::mojom::FilePathDataView* output) {
  auto pointer = data_->http_cache_path.Get();
  *output = ::mojo_base::mojom::FilePathDataView(pointer, context_);
}
inline void NetworkContextParamsDataView::GetHttpServerPropertiesPathDataView(
    ::mojo_base::mojom::FilePathDataView* output) {
  auto pointer = data_->http_server_properties_path.Get();
  *output = ::mojo_base::mojom::FilePathDataView(pointer, context_);
}
inline void NetworkContextParamsDataView::GetTransportSecurityPersisterPathDataView(
    ::mojo_base::mojom::FilePathDataView* output) {
  auto pointer = data_->transport_security_persister_path.Get();
  *output = ::mojo_base::mojom::FilePathDataView(pointer, context_);
}
inline void NetworkContextParamsDataView::GetInitialSslConfigDataView(
    ::network::mojom::SSLConfigDataView* output) {
  auto pointer = data_->initial_ssl_config.Get();
  *output = ::network::mojom::SSLConfigDataView(pointer, context_);
}
inline void NetworkContextParamsDataView::GetInitialProxyConfigDataView(
    ::network::mojom::ProxyConfigWithAnnotationDataView* output) {
  auto pointer = data_->initial_proxy_config.Get();
  *output = ::network::mojom::ProxyConfigWithAnnotationDataView(pointer, context_);
}
inline void NetworkContextParamsDataView::GetInitialCustomProxyConfigDataView(
    CustomProxyConfigDataView* output) {
  auto pointer = data_->initial_custom_proxy_config.Get();
  *output = CustomProxyConfigDataView(pointer, context_);
}
inline void NetworkContextParamsDataView::GetCtLogsDataView(
    mojo::ArrayDataView<::network::mojom::CTLogInfoDataView>* output) {
  auto pointer = data_->ct_logs.Get();
  *output = mojo::ArrayDataView<::network::mojom::CTLogInfoDataView>(pointer, context_);
}
inline void NetworkContextParamsDataView::GetCookieManagerParamsDataView(
    ::network::mojom::CookieManagerParamsDataView* output) {
  auto pointer = data_->cookie_manager_params.Get();
  *output = ::network::mojom::CookieManagerParamsDataView(pointer, context_);
}
inline void NetworkContextParamsDataView::GetDomainReliabilityUploadReporterDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->domain_reliability_upload_reporter.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void NetworkContextParamsDataView::GetReportingDeliveryIntervalDataView(
    ::mojo_base::mojom::TimeDeltaDataView* output) {
  auto pointer = data_->reporting_delivery_interval.Get();
  *output = ::mojo_base::mojom::TimeDeltaDataView(pointer, context_);
}
inline void NetworkContextParamsDataView::GetCorsOriginAccessListDataView(
    mojo::ArrayDataView<::network::mojom::CorsOriginAccessPatternsDataView>* output) {
  auto pointer = data_->cors_origin_access_list.Get();
  *output = mojo::ArrayDataView<::network::mojom::CorsOriginAccessPatternsDataView>(pointer, context_);
}


inline void NetworkConditionsDataView::GetLatencyDataView(
    ::mojo_base::mojom::TimeDeltaDataView* output) {
  auto pointer = data_->latency.Get();
  *output = ::mojo_base::mojom::TimeDeltaDataView(pointer, context_);
}


inline void ClearDataFilterDataView::GetDomainsDataView(
    mojo::ArrayDataView<mojo::StringDataView>* output) {
  auto pointer = data_->domains.Get();
  *output = mojo::ArrayDataView<mojo::StringDataView>(pointer, context_);
}
inline void ClearDataFilterDataView::GetOriginsDataView(
    mojo::ArrayDataView<::url::mojom::OriginDataView>* output) {
  auto pointer = data_->origins.Get();
  *output = mojo::ArrayDataView<::url::mojom::OriginDataView>(pointer, context_);
}




inline void URLLoaderFactoryParamsDataView::GetRequestInitiatorSiteLockDataView(
    ::url::mojom::OriginDataView* output) {
  auto pointer = data_->request_initiator_site_lock.Get();
  *output = ::url::mojom::OriginDataView(pointer, context_);
}


inline void CustomProxyConfigClient_OnCustomProxyConfigUpdated_ParamsDataView::GetProxyConfigDataView(
    CustomProxyConfigDataView* output) {
  auto pointer = data_->proxy_config.Get();
  *output = CustomProxyConfigDataView(pointer, context_);
}


inline void CustomProxyConfigClient_MarkProxiesAsBad_ParamsDataView::GetBypassDurationDataView(
    ::mojo_base::mojom::TimeDeltaDataView* output) {
  auto pointer = data_->bypass_duration.Get();
  *output = ::mojo_base::mojom::TimeDeltaDataView(pointer, context_);
}
inline void CustomProxyConfigClient_MarkProxiesAsBad_ParamsDataView::GetBadProxiesDataView(
    ::network::mojom::ProxyListDataView* output) {
  auto pointer = data_->bad_proxies.Get();
  *output = ::network::mojom::ProxyListDataView(pointer, context_);
}






inline void TrustedHeaderClient_OnBeforeSendHeaders_ParamsDataView::GetHeadersDataView(
    ::network::mojom::HttpRequestHeadersDataView* output) {
  auto pointer = data_->headers.Get();
  *output = ::network::mojom::HttpRequestHeadersDataView(pointer, context_);
}


inline void TrustedHeaderClient_OnBeforeSendHeaders_ResponseParamsDataView::GetHeadersDataView(
    ::network::mojom::HttpRequestHeadersDataView* output) {
  auto pointer = data_->headers.Get();
  *output = ::network::mojom::HttpRequestHeadersDataView(pointer, context_);
}


inline void TrustedHeaderClient_OnHeadersReceived_ParamsDataView::GetHeadersDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->headers.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void TrustedHeaderClient_OnHeadersReceived_ResponseParamsDataView::GetHeadersDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->headers.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void TrustedHeaderClient_OnHeadersReceived_ResponseParamsDataView::GetAllowedUnsafeRedirectUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->allowed_unsafe_redirect_url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}




inline void NetworkContextClient_OnCanSendReportingReports_ParamsDataView::GetOriginsDataView(
    mojo::ArrayDataView<::url::mojom::OriginDataView>* output) {
  auto pointer = data_->origins.Get();
  *output = mojo::ArrayDataView<::url::mojom::OriginDataView>(pointer, context_);
}


inline void NetworkContextClient_OnCanSendReportingReports_ResponseParamsDataView::GetOriginsDataView(
    mojo::ArrayDataView<::url::mojom::OriginDataView>* output) {
  auto pointer = data_->origins.Get();
  *output = mojo::ArrayDataView<::url::mojom::OriginDataView>(pointer, context_);
}


inline void NetworkContextClient_OnCanSendDomainReliabilityUpload_ParamsDataView::GetOriginDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->origin.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}






inline void NetworkContext_CreateURLLoaderFactory_ParamsDataView::GetParamsDataView(
    URLLoaderFactoryParamsDataView* output) {
  auto pointer = data_->params.Get();
  *output = URLLoaderFactoryParamsDataView(pointer, context_);
}






inline void NetworkContext_GetRestrictedCookieManager_ParamsDataView::GetOriginDataView(
    ::url::mojom::OriginDataView* output) {
  auto pointer = data_->origin.Get();
  *output = ::url::mojom::OriginDataView(pointer, context_);
}


inline void NetworkContext_ClearNetworkingHistorySince_ParamsDataView::GetStartTimeDataView(
    ::mojo_base::mojom::TimeDataView* output) {
  auto pointer = data_->start_time.Get();
  *output = ::mojo_base::mojom::TimeDataView(pointer, context_);
}




inline void NetworkContext_ClearHttpCache_ParamsDataView::GetStartTimeDataView(
    ::mojo_base::mojom::TimeDataView* output) {
  auto pointer = data_->start_time.Get();
  *output = ::mojo_base::mojom::TimeDataView(pointer, context_);
}
inline void NetworkContext_ClearHttpCache_ParamsDataView::GetEndTimeDataView(
    ::mojo_base::mojom::TimeDataView* output) {
  auto pointer = data_->end_time.Get();
  *output = ::mojo_base::mojom::TimeDataView(pointer, context_);
}
inline void NetworkContext_ClearHttpCache_ParamsDataView::GetFilterDataView(
    ClearDataFilterDataView* output) {
  auto pointer = data_->filter.Get();
  *output = ClearDataFilterDataView(pointer, context_);
}




inline void NetworkContext_ComputeHttpCacheSize_ParamsDataView::GetStartTimeDataView(
    ::mojo_base::mojom::TimeDataView* output) {
  auto pointer = data_->start_time.Get();
  *output = ::mojo_base::mojom::TimeDataView(pointer, context_);
}
inline void NetworkContext_ComputeHttpCacheSize_ParamsDataView::GetEndTimeDataView(
    ::mojo_base::mojom::TimeDataView* output) {
  auto pointer = data_->end_time.Get();
  *output = ::mojo_base::mojom::TimeDataView(pointer, context_);
}




inline void NetworkContext_NotifyExternalCacheHit_ParamsDataView::GetUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}
inline void NetworkContext_NotifyExternalCacheHit_ParamsDataView::GetHttpMethodDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->http_method.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void NetworkContext_NotifyExternalCacheHit_ParamsDataView::GetTopFrameOriginDataView(
    ::url::mojom::OriginDataView* output) {
  auto pointer = data_->top_frame_origin.Get();
  *output = ::url::mojom::OriginDataView(pointer, context_);
}


inline void NetworkContext_WriteCacheMetadata_ParamsDataView::GetUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}
inline void NetworkContext_WriteCacheMetadata_ParamsDataView::GetExpectedResponseTimeDataView(
    ::mojo_base::mojom::TimeDataView* output) {
  auto pointer = data_->expected_response_time.Get();
  *output = ::mojo_base::mojom::TimeDataView(pointer, context_);
}
inline void NetworkContext_WriteCacheMetadata_ParamsDataView::GetDataDataView(
    mojo::ArrayDataView<uint8_t>* output) {
  auto pointer = data_->data.Get();
  *output = mojo::ArrayDataView<uint8_t>(pointer, context_);
}


inline void NetworkContext_ClearChannelIds_ParamsDataView::GetStartTimeDataView(
    ::mojo_base::mojom::TimeDataView* output) {
  auto pointer = data_->start_time.Get();
  *output = ::mojo_base::mojom::TimeDataView(pointer, context_);
}
inline void NetworkContext_ClearChannelIds_ParamsDataView::GetEndTimeDataView(
    ::mojo_base::mojom::TimeDataView* output) {
  auto pointer = data_->end_time.Get();
  *output = ::mojo_base::mojom::TimeDataView(pointer, context_);
}
inline void NetworkContext_ClearChannelIds_ParamsDataView::GetFilterDataView(
    ClearDataFilterDataView* output) {
  auto pointer = data_->filter.Get();
  *output = ClearDataFilterDataView(pointer, context_);
}




inline void NetworkContext_ClearHostCache_ParamsDataView::GetFilterDataView(
    ClearDataFilterDataView* output) {
  auto pointer = data_->filter.Get();
  *output = ClearDataFilterDataView(pointer, context_);
}




inline void NetworkContext_ClearHttpAuthCache_ParamsDataView::GetStartTimeDataView(
    ::mojo_base::mojom::TimeDataView* output) {
  auto pointer = data_->start_time.Get();
  *output = ::mojo_base::mojom::TimeDataView(pointer, context_);
}




inline void NetworkContext_ClearReportingCacheReports_ParamsDataView::GetFilterDataView(
    ClearDataFilterDataView* output) {
  auto pointer = data_->filter.Get();
  *output = ClearDataFilterDataView(pointer, context_);
}




inline void NetworkContext_ClearReportingCacheClients_ParamsDataView::GetFilterDataView(
    ClearDataFilterDataView* output) {
  auto pointer = data_->filter.Get();
  *output = ClearDataFilterDataView(pointer, context_);
}




inline void NetworkContext_ClearNetworkErrorLogging_ParamsDataView::GetFilterDataView(
    ClearDataFilterDataView* output) {
  auto pointer = data_->filter.Get();
  *output = ClearDataFilterDataView(pointer, context_);
}




inline void NetworkContext_ClearDomainReliability_ParamsDataView::GetFilterDataView(
    ClearDataFilterDataView* output) {
  auto pointer = data_->filter.Get();
  *output = ClearDataFilterDataView(pointer, context_);
}






inline void NetworkContext_GetDomainReliabilityJSON_ResponseParamsDataView::GetDataDataView(
    ::mojo_base::mojom::ValueDataView* output) {
  auto pointer = &data_->data;
  *output = ::mojo_base::mojom::ValueDataView(pointer, context_);
}


inline void NetworkContext_QueueReport_ParamsDataView::GetTypeDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->type.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void NetworkContext_QueueReport_ParamsDataView::GetGroupDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->group.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void NetworkContext_QueueReport_ParamsDataView::GetUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}
inline void NetworkContext_QueueReport_ParamsDataView::GetUserAgentDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->user_agent.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void NetworkContext_QueueReport_ParamsDataView::GetBodyDataView(
    ::mojo_base::mojom::DictionaryValueDataView* output) {
  auto pointer = data_->body.Get();
  *output = ::mojo_base::mojom::DictionaryValueDataView(pointer, context_);
}










inline void NetworkContext_SetNetworkConditions_ParamsDataView::GetThrottlingProfileIdDataView(
    ::mojo_base::mojom::UnguessableTokenDataView* output) {
  auto pointer = data_->throttling_profile_id.Get();
  *output = ::mojo_base::mojom::UnguessableTokenDataView(pointer, context_);
}
inline void NetworkContext_SetNetworkConditions_ParamsDataView::GetConditionsDataView(
    NetworkConditionsDataView* output) {
  auto pointer = data_->conditions.Get();
  *output = NetworkConditionsDataView(pointer, context_);
}


inline void NetworkContext_SetAcceptLanguage_ParamsDataView::GetNewAcceptLanguageDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->new_accept_language.Get();
  *output = mojo::StringDataView(pointer, context_);
}




inline void NetworkContext_SetCTPolicy_ParamsDataView::GetRequiredHostsDataView(
    mojo::ArrayDataView<mojo::StringDataView>* output) {
  auto pointer = data_->required_hosts.Get();
  *output = mojo::ArrayDataView<mojo::StringDataView>(pointer, context_);
}
inline void NetworkContext_SetCTPolicy_ParamsDataView::GetExcludedHostsDataView(
    mojo::ArrayDataView<mojo::StringDataView>* output) {
  auto pointer = data_->excluded_hosts.Get();
  *output = mojo::ArrayDataView<mojo::StringDataView>(pointer, context_);
}
inline void NetworkContext_SetCTPolicy_ParamsDataView::GetExcludedSpkisDataView(
    mojo::ArrayDataView<mojo::StringDataView>* output) {
  auto pointer = data_->excluded_spkis.Get();
  *output = mojo::ArrayDataView<mojo::StringDataView>(pointer, context_);
}
inline void NetworkContext_SetCTPolicy_ParamsDataView::GetExcludedLegacySpkisDataView(
    mojo::ArrayDataView<mojo::StringDataView>* output) {
  auto pointer = data_->excluded_legacy_spkis.Get();
  *output = mojo::ArrayDataView<mojo::StringDataView>(pointer, context_);
}


inline void NetworkContext_AddExpectCT_ParamsDataView::GetHostDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->host.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void NetworkContext_AddExpectCT_ParamsDataView::GetExpiryDataView(
    ::mojo_base::mojom::TimeDataView* output) {
  auto pointer = data_->expiry.Get();
  *output = ::mojo_base::mojom::TimeDataView(pointer, context_);
}
inline void NetworkContext_AddExpectCT_ParamsDataView::GetReportUriDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->report_uri.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}




inline void NetworkContext_SetExpectCTTestReport_ParamsDataView::GetReportUriDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->report_uri.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}




inline void NetworkContext_GetExpectCTState_ParamsDataView::GetDomainDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->domain.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void NetworkContext_GetExpectCTState_ResponseParamsDataView::GetStateDataView(
    ::mojo_base::mojom::DictionaryValueDataView* output) {
  auto pointer = data_->state.Get();
  *output = ::mojo_base::mojom::DictionaryValueDataView(pointer, context_);
}




inline void NetworkContext_CreateTCPServerSocket_ParamsDataView::GetLocalAddrDataView(
    ::network::mojom::IPEndPointDataView* output) {
  auto pointer = data_->local_addr.Get();
  *output = ::network::mojom::IPEndPointDataView(pointer, context_);
}
inline void NetworkContext_CreateTCPServerSocket_ParamsDataView::GetTrafficAnnotationDataView(
    ::network::mojom::MutableNetworkTrafficAnnotationTagDataView* output) {
  auto pointer = data_->traffic_annotation.Get();
  *output = ::network::mojom::MutableNetworkTrafficAnnotationTagDataView(pointer, context_);
}


inline void NetworkContext_CreateTCPServerSocket_ResponseParamsDataView::GetLocalAddrOutDataView(
    ::network::mojom::IPEndPointDataView* output) {
  auto pointer = data_->local_addr_out.Get();
  *output = ::network::mojom::IPEndPointDataView(pointer, context_);
}


inline void NetworkContext_CreateTCPConnectedSocket_ParamsDataView::GetLocalAddrDataView(
    ::network::mojom::IPEndPointDataView* output) {
  auto pointer = data_->local_addr.Get();
  *output = ::network::mojom::IPEndPointDataView(pointer, context_);
}
inline void NetworkContext_CreateTCPConnectedSocket_ParamsDataView::GetRemoteAddrListDataView(
    ::network::mojom::AddressListDataView* output) {
  auto pointer = data_->remote_addr_list.Get();
  *output = ::network::mojom::AddressListDataView(pointer, context_);
}
inline void NetworkContext_CreateTCPConnectedSocket_ParamsDataView::GetTcpConnectedSocketOptionsDataView(
    ::network::mojom::TCPConnectedSocketOptionsDataView* output) {
  auto pointer = data_->tcp_connected_socket_options.Get();
  *output = ::network::mojom::TCPConnectedSocketOptionsDataView(pointer, context_);
}
inline void NetworkContext_CreateTCPConnectedSocket_ParamsDataView::GetTrafficAnnotationDataView(
    ::network::mojom::MutableNetworkTrafficAnnotationTagDataView* output) {
  auto pointer = data_->traffic_annotation.Get();
  *output = ::network::mojom::MutableNetworkTrafficAnnotationTagDataView(pointer, context_);
}


inline void NetworkContext_CreateTCPConnectedSocket_ResponseParamsDataView::GetLocalAddrDataView(
    ::network::mojom::IPEndPointDataView* output) {
  auto pointer = data_->local_addr.Get();
  *output = ::network::mojom::IPEndPointDataView(pointer, context_);
}
inline void NetworkContext_CreateTCPConnectedSocket_ResponseParamsDataView::GetPeerAddrDataView(
    ::network::mojom::IPEndPointDataView* output) {
  auto pointer = data_->peer_addr.Get();
  *output = ::network::mojom::IPEndPointDataView(pointer, context_);
}


inline void NetworkContext_CreateTCPBoundSocket_ParamsDataView::GetLocalAddrDataView(
    ::network::mojom::IPEndPointDataView* output) {
  auto pointer = data_->local_addr.Get();
  *output = ::network::mojom::IPEndPointDataView(pointer, context_);
}
inline void NetworkContext_CreateTCPBoundSocket_ParamsDataView::GetTrafficAnnotationDataView(
    ::network::mojom::MutableNetworkTrafficAnnotationTagDataView* output) {
  auto pointer = data_->traffic_annotation.Get();
  *output = ::network::mojom::MutableNetworkTrafficAnnotationTagDataView(pointer, context_);
}


inline void NetworkContext_CreateTCPBoundSocket_ResponseParamsDataView::GetLocalAddrDataView(
    ::network::mojom::IPEndPointDataView* output) {
  auto pointer = data_->local_addr.Get();
  *output = ::network::mojom::IPEndPointDataView(pointer, context_);
}




inline void NetworkContext_LookUpProxyForURL_ParamsDataView::GetUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}










inline void NetworkContext_CreateWebSocket_ParamsDataView::GetOriginDataView(
    ::url::mojom::OriginDataView* output) {
  auto pointer = data_->origin.Get();
  *output = ::url::mojom::OriginDataView(pointer, context_);
}




inline void NetworkContext_PreconnectSockets_ParamsDataView::GetUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}






inline void NetworkContext_ResolveHost_ParamsDataView::GetHostDataView(
    ::network::mojom::HostPortPairDataView* output) {
  auto pointer = data_->host.Get();
  *output = ::network::mojom::HostPortPairDataView(pointer, context_);
}
inline void NetworkContext_ResolveHost_ParamsDataView::GetOptionalParametersDataView(
    ::network::mojom::ResolveHostParametersDataView* output) {
  auto pointer = data_->optional_parameters.Get();
  *output = ::network::mojom::ResolveHostParametersDataView(pointer, context_);
}


inline void NetworkContext_CreateHostResolver_ParamsDataView::GetConfigOverridesDataView(
    ::network::mojom::DnsConfigOverridesDataView* output) {
  auto pointer = data_->config_overrides.Get();
  *output = ::network::mojom::DnsConfigOverridesDataView(pointer, context_);
}


inline void NetworkContext_VerifyCertForSignedExchange_ParamsDataView::GetCertificateDataView(
    ::network::mojom::X509CertificateDataView* output) {
  auto pointer = data_->certificate.Get();
  *output = ::network::mojom::X509CertificateDataView(pointer, context_);
}
inline void NetworkContext_VerifyCertForSignedExchange_ParamsDataView::GetUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}
inline void NetworkContext_VerifyCertForSignedExchange_ParamsDataView::GetOcspResponseDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->ocsp_response.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void NetworkContext_VerifyCertForSignedExchange_ParamsDataView::GetSctListDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->sct_list.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void NetworkContext_VerifyCertForSignedExchange_ResponseParamsDataView::GetCvResultDataView(
    ::network::mojom::CertVerifyResultDataView* output) {
  auto pointer = data_->cv_result.Get();
  *output = ::network::mojom::CertVerifyResultDataView(pointer, context_);
}
inline void NetworkContext_VerifyCertForSignedExchange_ResponseParamsDataView::GetCtResultDataView(
    ::network::mojom::CTVerifyResultDataView* output) {
  auto pointer = data_->ct_result.Get();
  *output = ::network::mojom::CTVerifyResultDataView(pointer, context_);
}


inline void NetworkContext_AddHSTS_ParamsDataView::GetHostDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->host.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void NetworkContext_AddHSTS_ParamsDataView::GetExpiryDataView(
    ::mojo_base::mojom::TimeDataView* output) {
  auto pointer = data_->expiry.Get();
  *output = ::mojo_base::mojom::TimeDataView(pointer, context_);
}




inline void NetworkContext_IsHSTSActiveForHost_ParamsDataView::GetHostDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->host.Get();
  *output = mojo::StringDataView(pointer, context_);
}




inline void NetworkContext_GetHSTSState_ParamsDataView::GetDomainDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->domain.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void NetworkContext_GetHSTSState_ResponseParamsDataView::GetStateDataView(
    ::mojo_base::mojom::DictionaryValueDataView* output) {
  auto pointer = data_->state.Get();
  *output = ::mojo_base::mojom::DictionaryValueDataView(pointer, context_);
}


inline void NetworkContext_SetCorsOriginAccessListsForOrigin_ParamsDataView::GetSourceOriginDataView(
    ::url::mojom::OriginDataView* output) {
  auto pointer = data_->source_origin.Get();
  *output = ::url::mojom::OriginDataView(pointer, context_);
}
inline void NetworkContext_SetCorsOriginAccessListsForOrigin_ParamsDataView::GetAllowPatternsDataView(
    mojo::ArrayDataView<::network::mojom::CorsOriginPatternDataView>* output) {
  auto pointer = data_->allow_patterns.Get();
  *output = mojo::ArrayDataView<::network::mojom::CorsOriginPatternDataView>(pointer, context_);
}
inline void NetworkContext_SetCorsOriginAccessListsForOrigin_ParamsDataView::GetBlockPatternsDataView(
    mojo::ArrayDataView<::network::mojom::CorsOriginPatternDataView>* output) {
  auto pointer = data_->block_patterns.Get();
  *output = mojo::ArrayDataView<::network::mojom::CorsOriginPatternDataView>(pointer, context_);
}




inline void NetworkContext_DeleteDynamicDataForHost_ParamsDataView::GetHostDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->host.Get();
  *output = mojo::StringDataView(pointer, context_);
}






inline void NetworkContext_SaveHttpAuthCache_ResponseParamsDataView::GetCacheKeyDataView(
    ::mojo_base::mojom::UnguessableTokenDataView* output) {
  auto pointer = data_->cache_key.Get();
  *output = ::mojo_base::mojom::UnguessableTokenDataView(pointer, context_);
}


inline void NetworkContext_LoadHttpAuthCache_ParamsDataView::GetCacheKeyDataView(
    ::mojo_base::mojom::UnguessableTokenDataView* output) {
  auto pointer = data_->cache_key.Get();
  *output = ::mojo_base::mojom::UnguessableTokenDataView(pointer, context_);
}




inline void NetworkContext_LookupBasicAuthCredentials_ParamsDataView::GetUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}


inline void NetworkContext_LookupBasicAuthCredentials_ResponseParamsDataView::GetCredentialsDataView(
    ::network::mojom::AuthCredentialsDataView* output) {
  auto pointer = data_->credentials.Get();
  *output = ::network::mojom::AuthCredentialsDataView(pointer, context_);
}










inline void NetworkContext_VerifyCertificateForTesting_ParamsDataView::GetCertificateDataView(
    ::network::mojom::X509CertificateDataView* output) {
  auto pointer = data_->certificate.Get();
  *output = ::network::mojom::X509CertificateDataView(pointer, context_);
}
inline void NetworkContext_VerifyCertificateForTesting_ParamsDataView::GetHostnameDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->hostname.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void NetworkContext_VerifyCertificateForTesting_ParamsDataView::GetOcspResponseDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->ocsp_response.Get();
  *output = mojo::StringDataView(pointer, context_);
}




inline void NetworkContext_AddDomainReliabilityContextForTesting_ParamsDataView::GetOriginDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->origin.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}
inline void NetworkContext_AddDomainReliabilityContextForTesting_ParamsDataView::GetUploadUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->upload_url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}









}  // namespace mojom
}  // namespace network

#endif  // SERVICES_NETWORK_PUBLIC_MOJOM_NETWORK_CONTEXT_MOJOM_SHARED_H_