// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_NETWORK_PUBLIC_MOJOM_TCP_SOCKET_MOJOM_SHARED_H_
#define SERVICES_NETWORK_PUBLIC_MOJOM_TCP_SOCKET_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "services/network/public/mojom/tcp_socket.mojom-shared-internal.h"
#include "services/network/public/mojom/address_list.mojom-shared.h"
#include "services/network/public/mojom/ip_endpoint.mojom-shared.h"
#include "services/network/public/mojom/ssl_config.mojom-shared.h"
#include "services/network/public/mojom/tls_socket.mojom-shared.h"
#include "services/network/public/mojom/network_param.mojom-shared.h"
#include "services/network/public/mojom/mutable_network_traffic_annotation_tag.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace network {
namespace mojom {
class TCPConnectedSocketOptionsDataView;



}  // namespace mojom
}  // namespace network

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::network::mojom::TCPConnectedSocketOptionsDataView> {
  using Data = ::network::mojom::internal::TCPConnectedSocketOptions_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace network {
namespace mojom {
// Interface base classes. They are used for type safety check.
class TCPBoundSocketInterfaceBase {};

using TCPBoundSocketPtrDataView =
    mojo::InterfacePtrDataView<TCPBoundSocketInterfaceBase>;
using TCPBoundSocketRequestDataView =
    mojo::InterfaceRequestDataView<TCPBoundSocketInterfaceBase>;
using TCPBoundSocketAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<TCPBoundSocketInterfaceBase>;
using TCPBoundSocketAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<TCPBoundSocketInterfaceBase>;
class TCPConnectedSocketInterfaceBase {};

using TCPConnectedSocketPtrDataView =
    mojo::InterfacePtrDataView<TCPConnectedSocketInterfaceBase>;
using TCPConnectedSocketRequestDataView =
    mojo::InterfaceRequestDataView<TCPConnectedSocketInterfaceBase>;
using TCPConnectedSocketAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<TCPConnectedSocketInterfaceBase>;
using TCPConnectedSocketAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<TCPConnectedSocketInterfaceBase>;
class SocketObserverInterfaceBase {};

using SocketObserverPtrDataView =
    mojo::InterfacePtrDataView<SocketObserverInterfaceBase>;
using SocketObserverRequestDataView =
    mojo::InterfaceRequestDataView<SocketObserverInterfaceBase>;
using SocketObserverAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<SocketObserverInterfaceBase>;
using SocketObserverAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<SocketObserverInterfaceBase>;
class TCPServerSocketInterfaceBase {};

using TCPServerSocketPtrDataView =
    mojo::InterfacePtrDataView<TCPServerSocketInterfaceBase>;
using TCPServerSocketRequestDataView =
    mojo::InterfaceRequestDataView<TCPServerSocketInterfaceBase>;
using TCPServerSocketAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<TCPServerSocketInterfaceBase>;
using TCPServerSocketAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<TCPServerSocketInterfaceBase>;
class TCPConnectedSocketOptionsDataView {
 public:
  TCPConnectedSocketOptionsDataView() {}

  TCPConnectedSocketOptionsDataView(
      internal::TCPConnectedSocketOptions_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  int32_t send_buffer_size() const {
    return data_->send_buffer_size;
  }
  int32_t receive_buffer_size() const {
    return data_->receive_buffer_size;
  }
  bool no_delay() const {
    return data_->no_delay;
  }
 private:
  internal::TCPConnectedSocketOptions_Data* data_ = nullptr;
};

class TCPBoundSocket_Listen_ParamsDataView {
 public:
  TCPBoundSocket_Listen_ParamsDataView() {}

  TCPBoundSocket_Listen_ParamsDataView(
      internal::TCPBoundSocket_Listen_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  uint32_t backlog() const {
    return data_->backlog;
  }
  template <typename UserType>
  UserType TakeSocket() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::TCPServerSocketRequestDataView>(
            &data_->socket, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::TCPBoundSocket_Listen_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class TCPBoundSocket_Listen_ResponseParamsDataView {
 public:
  TCPBoundSocket_Listen_ResponseParamsDataView() {}

  TCPBoundSocket_Listen_ResponseParamsDataView(
      internal::TCPBoundSocket_Listen_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  int32_t net_error() const {
    return data_->net_error;
  }
 private:
  internal::TCPBoundSocket_Listen_ResponseParams_Data* data_ = nullptr;
};

class TCPBoundSocket_Connect_ParamsDataView {
 public:
  TCPBoundSocket_Connect_ParamsDataView() {}

  TCPBoundSocket_Connect_ParamsDataView(
      internal::TCPBoundSocket_Connect_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetRemoteAddrListDataView(
      ::network::mojom::AddressListDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRemoteAddrList(UserType* output) {
    auto* pointer = data_->remote_addr_list.Get();
    return mojo::internal::Deserialize<::network::mojom::AddressListDataView>(
        pointer, output, context_);
  }
  inline void GetTcpConnectedSocketOptionsDataView(
      TCPConnectedSocketOptionsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTcpConnectedSocketOptions(UserType* output) {
    auto* pointer = data_->tcp_connected_socket_options.Get();
    return mojo::internal::Deserialize<::network::mojom::TCPConnectedSocketOptionsDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  UserType TakeSocket() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::TCPConnectedSocketRequestDataView>(
            &data_->socket, &result, context_);
    DCHECK(ret);
    return result;
  }
  template <typename UserType>
  UserType TakeObserver() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::SocketObserverPtrDataView>(
            &data_->observer, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::TCPBoundSocket_Connect_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class TCPBoundSocket_Connect_ResponseParamsDataView {
 public:
  TCPBoundSocket_Connect_ResponseParamsDataView() {}

  TCPBoundSocket_Connect_ResponseParamsDataView(
      internal::TCPBoundSocket_Connect_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int32_t net_error() const {
    return data_->net_error;
  }
  inline void GetLocalAddrDataView(
      ::network::mojom::IPEndPointDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadLocalAddr(UserType* output) {
    auto* pointer = data_->local_addr.Get();
    return mojo::internal::Deserialize<::network::mojom::IPEndPointDataView>(
        pointer, output, context_);
  }
  inline void GetPeerAddrDataView(
      ::network::mojom::IPEndPointDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPeerAddr(UserType* output) {
    auto* pointer = data_->peer_addr.Get();
    return mojo::internal::Deserialize<::network::mojom::IPEndPointDataView>(
        pointer, output, context_);
  }
  mojo::ScopedDataPipeConsumerHandle TakeReceiveStream() {
    mojo::ScopedDataPipeConsumerHandle result;
    bool ret =
        mojo::internal::Deserialize<mojo::ScopedDataPipeConsumerHandle>(
            &data_->receive_stream, &result, context_);
    DCHECK(ret);
    return result;
  }
  mojo::ScopedDataPipeProducerHandle TakeSendStream() {
    mojo::ScopedDataPipeProducerHandle result;
    bool ret =
        mojo::internal::Deserialize<mojo::ScopedDataPipeProducerHandle>(
            &data_->send_stream, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::TCPBoundSocket_Connect_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class TCPConnectedSocket_UpgradeToTLS_ParamsDataView {
 public:
  TCPConnectedSocket_UpgradeToTLS_ParamsDataView() {}

  TCPConnectedSocket_UpgradeToTLS_ParamsDataView(
      internal::TCPConnectedSocket_UpgradeToTLS_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetHostPortPairDataView(
      ::network::mojom::HostPortPairDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadHostPortPair(UserType* output) {
    auto* pointer = data_->host_port_pair.Get();
    return mojo::internal::Deserialize<::network::mojom::HostPortPairDataView>(
        pointer, output, context_);
  }
  inline void GetOptionsDataView(
      ::network::mojom::TLSClientSocketOptionsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOptions(UserType* output) {
    auto* pointer = data_->options.Get();
    return mojo::internal::Deserialize<::network::mojom::TLSClientSocketOptionsDataView>(
        pointer, output, context_);
  }
  inline void GetTrafficAnnotationDataView(
      ::network::mojom::MutableNetworkTrafficAnnotationTagDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTrafficAnnotation(UserType* output) {
    auto* pointer = data_->traffic_annotation.Get();
    return mojo::internal::Deserialize<::network::mojom::MutableNetworkTrafficAnnotationTagDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  UserType TakeRequest() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::TLSClientSocketRequestDataView>(
            &data_->request, &result, context_);
    DCHECK(ret);
    return result;
  }
  template <typename UserType>
  UserType TakeObserver() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::SocketObserverPtrDataView>(
            &data_->observer, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::TCPConnectedSocket_UpgradeToTLS_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class TCPConnectedSocket_UpgradeToTLS_ResponseParamsDataView {
 public:
  TCPConnectedSocket_UpgradeToTLS_ResponseParamsDataView() {}

  TCPConnectedSocket_UpgradeToTLS_ResponseParamsDataView(
      internal::TCPConnectedSocket_UpgradeToTLS_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int32_t net_error() const {
    return data_->net_error;
  }
  mojo::ScopedDataPipeConsumerHandle TakeReceiveStream() {
    mojo::ScopedDataPipeConsumerHandle result;
    bool ret =
        mojo::internal::Deserialize<mojo::ScopedDataPipeConsumerHandle>(
            &data_->receive_stream, &result, context_);
    DCHECK(ret);
    return result;
  }
  mojo::ScopedDataPipeProducerHandle TakeSendStream() {
    mojo::ScopedDataPipeProducerHandle result;
    bool ret =
        mojo::internal::Deserialize<mojo::ScopedDataPipeProducerHandle>(
            &data_->send_stream, &result, context_);
    DCHECK(ret);
    return result;
  }
  inline void GetSslInfoDataView(
      ::network::mojom::SSLInfoDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSslInfo(UserType* output) {
    auto* pointer = data_->ssl_info.Get();
    return mojo::internal::Deserialize<::network::mojom::SSLInfoDataView>(
        pointer, output, context_);
  }
 private:
  internal::TCPConnectedSocket_UpgradeToTLS_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class TCPConnectedSocket_SetSendBufferSize_ParamsDataView {
 public:
  TCPConnectedSocket_SetSendBufferSize_ParamsDataView() {}

  TCPConnectedSocket_SetSendBufferSize_ParamsDataView(
      internal::TCPConnectedSocket_SetSendBufferSize_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  int32_t send_buffer_size() const {
    return data_->send_buffer_size;
  }
 private:
  internal::TCPConnectedSocket_SetSendBufferSize_Params_Data* data_ = nullptr;
};

class TCPConnectedSocket_SetSendBufferSize_ResponseParamsDataView {
 public:
  TCPConnectedSocket_SetSendBufferSize_ResponseParamsDataView() {}

  TCPConnectedSocket_SetSendBufferSize_ResponseParamsDataView(
      internal::TCPConnectedSocket_SetSendBufferSize_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  int32_t net_error() const {
    return data_->net_error;
  }
 private:
  internal::TCPConnectedSocket_SetSendBufferSize_ResponseParams_Data* data_ = nullptr;
};

class TCPConnectedSocket_SetReceiveBufferSize_ParamsDataView {
 public:
  TCPConnectedSocket_SetReceiveBufferSize_ParamsDataView() {}

  TCPConnectedSocket_SetReceiveBufferSize_ParamsDataView(
      internal::TCPConnectedSocket_SetReceiveBufferSize_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  int32_t receive_buffer_size() const {
    return data_->receive_buffer_size;
  }
 private:
  internal::TCPConnectedSocket_SetReceiveBufferSize_Params_Data* data_ = nullptr;
};

class TCPConnectedSocket_SetReceiveBufferSize_ResponseParamsDataView {
 public:
  TCPConnectedSocket_SetReceiveBufferSize_ResponseParamsDataView() {}

  TCPConnectedSocket_SetReceiveBufferSize_ResponseParamsDataView(
      internal::TCPConnectedSocket_SetReceiveBufferSize_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  int32_t net_error() const {
    return data_->net_error;
  }
 private:
  internal::TCPConnectedSocket_SetReceiveBufferSize_ResponseParams_Data* data_ = nullptr;
};

class TCPConnectedSocket_SetNoDelay_ParamsDataView {
 public:
  TCPConnectedSocket_SetNoDelay_ParamsDataView() {}

  TCPConnectedSocket_SetNoDelay_ParamsDataView(
      internal::TCPConnectedSocket_SetNoDelay_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool no_delay() const {
    return data_->no_delay;
  }
 private:
  internal::TCPConnectedSocket_SetNoDelay_Params_Data* data_ = nullptr;
};

class TCPConnectedSocket_SetNoDelay_ResponseParamsDataView {
 public:
  TCPConnectedSocket_SetNoDelay_ResponseParamsDataView() {}

  TCPConnectedSocket_SetNoDelay_ResponseParamsDataView(
      internal::TCPConnectedSocket_SetNoDelay_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool success() const {
    return data_->success;
  }
 private:
  internal::TCPConnectedSocket_SetNoDelay_ResponseParams_Data* data_ = nullptr;
};

class TCPConnectedSocket_SetKeepAlive_ParamsDataView {
 public:
  TCPConnectedSocket_SetKeepAlive_ParamsDataView() {}

  TCPConnectedSocket_SetKeepAlive_ParamsDataView(
      internal::TCPConnectedSocket_SetKeepAlive_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool enable() const {
    return data_->enable;
  }
  int32_t delay_secs() const {
    return data_->delay_secs;
  }
 private:
  internal::TCPConnectedSocket_SetKeepAlive_Params_Data* data_ = nullptr;
};

class TCPConnectedSocket_SetKeepAlive_ResponseParamsDataView {
 public:
  TCPConnectedSocket_SetKeepAlive_ResponseParamsDataView() {}

  TCPConnectedSocket_SetKeepAlive_ResponseParamsDataView(
      internal::TCPConnectedSocket_SetKeepAlive_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool success() const {
    return data_->success;
  }
 private:
  internal::TCPConnectedSocket_SetKeepAlive_ResponseParams_Data* data_ = nullptr;
};

class SocketObserver_OnReadError_ParamsDataView {
 public:
  SocketObserver_OnReadError_ParamsDataView() {}

  SocketObserver_OnReadError_ParamsDataView(
      internal::SocketObserver_OnReadError_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  int32_t net_error() const {
    return data_->net_error;
  }
 private:
  internal::SocketObserver_OnReadError_Params_Data* data_ = nullptr;
};

class SocketObserver_OnWriteError_ParamsDataView {
 public:
  SocketObserver_OnWriteError_ParamsDataView() {}

  SocketObserver_OnWriteError_ParamsDataView(
      internal::SocketObserver_OnWriteError_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  int32_t net_error() const {
    return data_->net_error;
  }
 private:
  internal::SocketObserver_OnWriteError_Params_Data* data_ = nullptr;
};

class TCPServerSocket_Accept_ParamsDataView {
 public:
  TCPServerSocket_Accept_ParamsDataView() {}

  TCPServerSocket_Accept_ParamsDataView(
      internal::TCPServerSocket_Accept_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeObserver() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::SocketObserverPtrDataView>(
            &data_->observer, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::TCPServerSocket_Accept_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class TCPServerSocket_Accept_ResponseParamsDataView {
 public:
  TCPServerSocket_Accept_ResponseParamsDataView() {}

  TCPServerSocket_Accept_ResponseParamsDataView(
      internal::TCPServerSocket_Accept_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int32_t net_error() const {
    return data_->net_error;
  }
  inline void GetRemoteAddrDataView(
      ::network::mojom::IPEndPointDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRemoteAddr(UserType* output) {
    auto* pointer = data_->remote_addr.Get();
    return mojo::internal::Deserialize<::network::mojom::IPEndPointDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  UserType TakeConnectedSocket() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::network::mojom::TCPConnectedSocketPtrDataView>(
            &data_->connected_socket, &result, context_);
    DCHECK(ret);
    return result;
  }
  mojo::ScopedDataPipeConsumerHandle TakeSendStream() {
    mojo::ScopedDataPipeConsumerHandle result;
    bool ret =
        mojo::internal::Deserialize<mojo::ScopedDataPipeConsumerHandle>(
            &data_->send_stream, &result, context_);
    DCHECK(ret);
    return result;
  }
  mojo::ScopedDataPipeProducerHandle TakeReceiveStream() {
    mojo::ScopedDataPipeProducerHandle result;
    bool ret =
        mojo::internal::Deserialize<mojo::ScopedDataPipeProducerHandle>(
            &data_->receive_stream, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::TCPServerSocket_Accept_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace network

namespace std {

}  // namespace std

namespace mojo {


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::network::mojom::TCPConnectedSocketOptionsDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::network::mojom::TCPConnectedSocketOptionsDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::network::mojom::internal::TCPConnectedSocketOptions_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    (*output)->send_buffer_size = CallWithContext(Traits::send_buffer_size, input, custom_context);
    (*output)->receive_buffer_size = CallWithContext(Traits::receive_buffer_size, input, custom_context);
    (*output)->no_delay = CallWithContext(Traits::no_delay, input, custom_context);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::network::mojom::internal::TCPConnectedSocketOptions_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::network::mojom::TCPConnectedSocketOptionsDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace network {
namespace mojom {







inline void TCPBoundSocket_Connect_ParamsDataView::GetRemoteAddrListDataView(
    ::network::mojom::AddressListDataView* output) {
  auto pointer = data_->remote_addr_list.Get();
  *output = ::network::mojom::AddressListDataView(pointer, context_);
}
inline void TCPBoundSocket_Connect_ParamsDataView::GetTcpConnectedSocketOptionsDataView(
    TCPConnectedSocketOptionsDataView* output) {
  auto pointer = data_->tcp_connected_socket_options.Get();
  *output = TCPConnectedSocketOptionsDataView(pointer, context_);
}


inline void TCPBoundSocket_Connect_ResponseParamsDataView::GetLocalAddrDataView(
    ::network::mojom::IPEndPointDataView* output) {
  auto pointer = data_->local_addr.Get();
  *output = ::network::mojom::IPEndPointDataView(pointer, context_);
}
inline void TCPBoundSocket_Connect_ResponseParamsDataView::GetPeerAddrDataView(
    ::network::mojom::IPEndPointDataView* output) {
  auto pointer = data_->peer_addr.Get();
  *output = ::network::mojom::IPEndPointDataView(pointer, context_);
}


inline void TCPConnectedSocket_UpgradeToTLS_ParamsDataView::GetHostPortPairDataView(
    ::network::mojom::HostPortPairDataView* output) {
  auto pointer = data_->host_port_pair.Get();
  *output = ::network::mojom::HostPortPairDataView(pointer, context_);
}
inline void TCPConnectedSocket_UpgradeToTLS_ParamsDataView::GetOptionsDataView(
    ::network::mojom::TLSClientSocketOptionsDataView* output) {
  auto pointer = data_->options.Get();
  *output = ::network::mojom::TLSClientSocketOptionsDataView(pointer, context_);
}
inline void TCPConnectedSocket_UpgradeToTLS_ParamsDataView::GetTrafficAnnotationDataView(
    ::network::mojom::MutableNetworkTrafficAnnotationTagDataView* output) {
  auto pointer = data_->traffic_annotation.Get();
  *output = ::network::mojom::MutableNetworkTrafficAnnotationTagDataView(pointer, context_);
}


inline void TCPConnectedSocket_UpgradeToTLS_ResponseParamsDataView::GetSslInfoDataView(
    ::network::mojom::SSLInfoDataView* output) {
  auto pointer = data_->ssl_info.Get();
  *output = ::network::mojom::SSLInfoDataView(pointer, context_);
}
























inline void TCPServerSocket_Accept_ResponseParamsDataView::GetRemoteAddrDataView(
    ::network::mojom::IPEndPointDataView* output) {
  auto pointer = data_->remote_addr.Get();
  *output = ::network::mojom::IPEndPointDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace network

#endif  // SERVICES_NETWORK_PUBLIC_MOJOM_TCP_SOCKET_MOJOM_SHARED_H_