// Copyright 2019 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_RESOURCE_COORDINATOR_PUBLIC_MOJOM_COORDINATION_UNIT_MOJOM_TEST_UTILS_H_
#define SERVICES_RESOURCE_COORDINATOR_PUBLIC_MOJOM_COORDINATION_UNIT_MOJOM_TEST_UTILS_H_

#include "services/resource_coordinator/public/mojom/coordination_unit.mojom.h"
#include "base/component_export.h"


namespace resource_coordinator {
namespace mojom {


class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) FrameCoordinationUnitInterceptorForTesting : public FrameCoordinationUnit {
  virtual FrameCoordinationUnit* GetForwardingInterface() = 0;
  void GetID(GetIDCallback callback) override;
  void SetProcess(const ::resource_coordinator::CoordinationUnitID& id) override;
  void AddBinding(FrameCoordinationUnitRequest request) override;
  void AddChildFrame(const ::resource_coordinator::CoordinationUnitID& cu_id) override;
  void RemoveChildFrame(const ::resource_coordinator::CoordinationUnitID& cu_id) override;
  void SetNetworkAlmostIdle(bool idle) override;
  void SetLifecycleState(::resource_coordinator::mojom::LifecycleState state) override;
  void SetHasNonEmptyBeforeUnload(bool has_nonempty_beforeunload) override;
  void SetInterventionPolicy(PolicyControlledIntervention intervention, InterventionPolicy policy) override;
  void OnNonPersistentNotificationCreated() override;
};
class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) FrameCoordinationUnitAsyncWaiter {
 public:
  explicit FrameCoordinationUnitAsyncWaiter(FrameCoordinationUnit* proxy);
  ~FrameCoordinationUnitAsyncWaiter();
  void GetID(
      ::resource_coordinator::CoordinationUnitID* out_id);

 private:
  FrameCoordinationUnit* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(FrameCoordinationUnitAsyncWaiter);
};


class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) PageCoordinationUnitInterceptorForTesting : public PageCoordinationUnit {
  virtual PageCoordinationUnit* GetForwardingInterface() = 0;
  void GetID(GetIDCallback callback) override;
  void AddBinding(PageCoordinationUnitRequest request) override;
  void AddFrame(const ::resource_coordinator::CoordinationUnitID& cu_id) override;
  void RemoveFrame(const ::resource_coordinator::CoordinationUnitID& cu_id) override;
  void SetIsLoading(bool is_loading) override;
  void SetVisibility(bool visible) override;
  void SetUKMSourceId(int64_t ukm_source_id) override;
  void OnFaviconUpdated() override;
  void OnTitleUpdated() override;
  void OnMainFrameNavigationCommitted(base::TimeTicks navigation_committed_time, int64_t navigation_id, const std::string& url) override;
};
class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) PageCoordinationUnitAsyncWaiter {
 public:
  explicit PageCoordinationUnitAsyncWaiter(PageCoordinationUnit* proxy);
  ~PageCoordinationUnitAsyncWaiter();
  void GetID(
      ::resource_coordinator::CoordinationUnitID* out_id);

 private:
  PageCoordinationUnit* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(PageCoordinationUnitAsyncWaiter);
};


class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) ProcessCoordinationUnitInterceptorForTesting : public ProcessCoordinationUnit {
  virtual ProcessCoordinationUnit* GetForwardingInterface() = 0;
  void GetID(GetIDCallback callback) override;
  void AddBinding(ProcessCoordinationUnitRequest request) override;
  void SetCPUUsage(double cpu_usage) override;
  void SetExpectedTaskQueueingDuration(base::TimeDelta duration) override;
  void SetLaunchTime(base::Time launch_time) override;
  void SetMainThreadTaskLoadIsLow(bool main_thread_task_load_is_low) override;
  void SetPID(base::ProcessId pid) override;
  void SetProcessExitStatus(int32_t exit_status) override;
  void OnRendererIsBloated() override;
};
class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) ProcessCoordinationUnitAsyncWaiter {
 public:
  explicit ProcessCoordinationUnitAsyncWaiter(ProcessCoordinationUnit* proxy);
  ~ProcessCoordinationUnitAsyncWaiter();
  void GetID(
      ::resource_coordinator::CoordinationUnitID* out_id);

 private:
  ProcessCoordinationUnit* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(ProcessCoordinationUnitAsyncWaiter);
};


class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) SystemCoordinationUnitInterceptorForTesting : public SystemCoordinationUnit {
  virtual SystemCoordinationUnit* GetForwardingInterface() = 0;
  void GetID(GetIDCallback callback) override;
  void AddBinding(SystemCoordinationUnitRequest request) override;
  void OnProcessCPUUsageReady() override;
  void DistributeMeasurementBatch(ProcessResourceMeasurementBatchPtr measurement_batch) override;
};
class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) SystemCoordinationUnitAsyncWaiter {
 public:
  explicit SystemCoordinationUnitAsyncWaiter(SystemCoordinationUnit* proxy);
  ~SystemCoordinationUnitAsyncWaiter();
  void GetID(
      ::resource_coordinator::CoordinationUnitID* out_id);

 private:
  SystemCoordinationUnit* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(SystemCoordinationUnitAsyncWaiter);
};




}  // namespace mojom
}  // namespace resource_coordinator

#endif  // SERVICES_RESOURCE_COORDINATOR_PUBLIC_MOJOM_COORDINATION_UNIT_MOJOM_TEST_UTILS_H_