// Copyright 2019 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_RESOURCE_COORDINATOR_PUBLIC_MOJOM_PAGE_SIGNAL_MOJOM_TEST_UTILS_H_
#define SERVICES_RESOURCE_COORDINATOR_PUBLIC_MOJOM_PAGE_SIGNAL_MOJOM_TEST_UTILS_H_

#include "services/resource_coordinator/public/mojom/page_signal.mojom.h"
#include "base/component_export.h"


namespace resource_coordinator {
namespace mojom {


class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) PageSignalReceiverInterceptorForTesting : public PageSignalReceiver {
  virtual PageSignalReceiver* GetForwardingInterface() = 0;
  void NotifyPageAlmostIdle(const ::resource_coordinator::PageNavigationIdentity& page_navigation_id) override;
  void NotifyRendererIsBloated(const ::resource_coordinator::PageNavigationIdentity& page_navigation_id) override;
  void SetExpectedTaskQueueingDuration(const ::resource_coordinator::PageNavigationIdentity& page_navigation_id, base::TimeDelta duration) override;
  void SetLifecycleState(const ::resource_coordinator::PageNavigationIdentity& page_navigation_id, ::resource_coordinator::mojom::LifecycleState state) override;
  void NotifyNonPersistentNotificationCreated(const ::resource_coordinator::PageNavigationIdentity& page_navigation_id) override;
  void OnLoadTimePerformanceEstimate(const ::resource_coordinator::PageNavigationIdentity& page_navigation_id, base::TimeDelta load_duration, base::TimeDelta cpu_usage_estimate, uint64_t private_footprint_kb_estimate) override;
};
class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) PageSignalReceiverAsyncWaiter {
 public:
  explicit PageSignalReceiverAsyncWaiter(PageSignalReceiver* proxy);
  ~PageSignalReceiverAsyncWaiter();

 private:
  PageSignalReceiver* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(PageSignalReceiverAsyncWaiter);
};


class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) PageSignalGeneratorInterceptorForTesting : public PageSignalGenerator {
  virtual PageSignalGenerator* GetForwardingInterface() = 0;
  void AddReceiver(PageSignalReceiverPtr receiver) override;
};
class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) PageSignalGeneratorAsyncWaiter {
 public:
  explicit PageSignalGeneratorAsyncWaiter(PageSignalGenerator* proxy);
  ~PageSignalGeneratorAsyncWaiter();

 private:
  PageSignalGenerator* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(PageSignalGeneratorAsyncWaiter);
};




}  // namespace mojom
}  // namespace resource_coordinator

#endif  // SERVICES_RESOURCE_COORDINATOR_PUBLIC_MOJOM_PAGE_SIGNAL_MOJOM_TEST_UTILS_H_