// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_RESOURCE_COORDINATOR_PUBLIC_MOJOM_PAGE_SIGNAL_MOJOM_H_
#define SERVICES_RESOURCE_COORDINATOR_PUBLIC_MOJOM_PAGE_SIGNAL_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/resource_coordinator/public/mojom/page_signal.mojom-shared.h"
#include "services/resource_coordinator/public/mojom/coordination_unit.mojom.h"
#include "services/resource_coordinator/public/mojom/lifecycle.mojom.h"
#include "mojo/public/mojom/base/time.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "services/resource_coordinator/public/cpp/coordination_unit_id.h"
#include "services/resource_coordinator/public/cpp/coordination_unit_types.h"
#include "services/resource_coordinator/public/cpp/page_navigation_identity.h"
#include "base/component_export.h"


namespace resource_coordinator {
namespace mojom {
class PageSignalReceiver;
using PageSignalReceiverPtr = mojo::InterfacePtr<PageSignalReceiver>;
using PageSignalReceiverPtrInfo = mojo::InterfacePtrInfo<PageSignalReceiver>;
using ThreadSafePageSignalReceiverPtr =
    mojo::ThreadSafeInterfacePtr<PageSignalReceiver>;
using PageSignalReceiverRequest = mojo::InterfaceRequest<PageSignalReceiver>;
using PageSignalReceiverAssociatedPtr =
    mojo::AssociatedInterfacePtr<PageSignalReceiver>;
using ThreadSafePageSignalReceiverAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<PageSignalReceiver>;
using PageSignalReceiverAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<PageSignalReceiver>;
using PageSignalReceiverAssociatedRequest =
    mojo::AssociatedInterfaceRequest<PageSignalReceiver>;

class PageSignalGenerator;
using PageSignalGeneratorPtr = mojo::InterfacePtr<PageSignalGenerator>;
using PageSignalGeneratorPtrInfo = mojo::InterfacePtrInfo<PageSignalGenerator>;
using ThreadSafePageSignalGeneratorPtr =
    mojo::ThreadSafeInterfacePtr<PageSignalGenerator>;
using PageSignalGeneratorRequest = mojo::InterfaceRequest<PageSignalGenerator>;
using PageSignalGeneratorAssociatedPtr =
    mojo::AssociatedInterfacePtr<PageSignalGenerator>;
using ThreadSafePageSignalGeneratorAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<PageSignalGenerator>;
using PageSignalGeneratorAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<PageSignalGenerator>;
using PageSignalGeneratorAssociatedRequest =
    mojo::AssociatedInterfaceRequest<PageSignalGenerator>;

class PageNavigationIdentity;
using PageNavigationIdentityPtr = mojo::StructPtr<PageNavigationIdentity>;


class PageSignalReceiverProxy;

template <typename ImplRefTraits>
class PageSignalReceiverStub;

class PageSignalReceiverRequestValidator;

class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) PageSignalReceiver
    : public PageSignalReceiverInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = PageSignalReceiverProxy;

  template <typename ImplRefTraits>
  using Stub_ = PageSignalReceiverStub<ImplRefTraits>;

  using RequestValidator_ = PageSignalReceiverRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kNotifyPageAlmostIdleMinVersion = 0,
    kNotifyRendererIsBloatedMinVersion = 0,
    kSetExpectedTaskQueueingDurationMinVersion = 0,
    kSetLifecycleStateMinVersion = 0,
    kNotifyNonPersistentNotificationCreatedMinVersion = 0,
    kOnLoadTimePerformanceEstimateMinVersion = 0,
  };
  virtual ~PageSignalReceiver() {}

  virtual void NotifyPageAlmostIdle(const ::resource_coordinator::PageNavigationIdentity& page_navigation_id) = 0;

  virtual void NotifyRendererIsBloated(const ::resource_coordinator::PageNavigationIdentity& page_navigation_id) = 0;

  virtual void SetExpectedTaskQueueingDuration(const ::resource_coordinator::PageNavigationIdentity& page_navigation_id, base::TimeDelta duration) = 0;

  virtual void SetLifecycleState(const ::resource_coordinator::PageNavigationIdentity& page_navigation_id, ::resource_coordinator::mojom::LifecycleState state) = 0;

  virtual void NotifyNonPersistentNotificationCreated(const ::resource_coordinator::PageNavigationIdentity& page_navigation_id) = 0;

  virtual void OnLoadTimePerformanceEstimate(const ::resource_coordinator::PageNavigationIdentity& page_navigation_id, base::TimeDelta load_duration, base::TimeDelta cpu_usage_estimate, uint64_t private_footprint_kb_estimate) = 0;
};

class PageSignalGeneratorProxy;

template <typename ImplRefTraits>
class PageSignalGeneratorStub;

class PageSignalGeneratorRequestValidator;

class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) PageSignalGenerator
    : public PageSignalGeneratorInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = PageSignalGeneratorProxy;

  template <typename ImplRefTraits>
  using Stub_ = PageSignalGeneratorStub<ImplRefTraits>;

  using RequestValidator_ = PageSignalGeneratorRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kAddReceiverMinVersion = 0,
  };
  virtual ~PageSignalGenerator() {}

  virtual void AddReceiver(PageSignalReceiverPtr receiver) = 0;
};

class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) PageSignalReceiverProxy
    : public PageSignalReceiver {
 public:
  explicit PageSignalReceiverProxy(mojo::MessageReceiverWithResponder* receiver);
  void NotifyPageAlmostIdle(const ::resource_coordinator::PageNavigationIdentity& page_navigation_id) final;
  void NotifyRendererIsBloated(const ::resource_coordinator::PageNavigationIdentity& page_navigation_id) final;
  void SetExpectedTaskQueueingDuration(const ::resource_coordinator::PageNavigationIdentity& page_navigation_id, base::TimeDelta duration) final;
  void SetLifecycleState(const ::resource_coordinator::PageNavigationIdentity& page_navigation_id, ::resource_coordinator::mojom::LifecycleState state) final;
  void NotifyNonPersistentNotificationCreated(const ::resource_coordinator::PageNavigationIdentity& page_navigation_id) final;
  void OnLoadTimePerformanceEstimate(const ::resource_coordinator::PageNavigationIdentity& page_navigation_id, base::TimeDelta load_duration, base::TimeDelta cpu_usage_estimate, uint64_t private_footprint_kb_estimate) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) PageSignalGeneratorProxy
    : public PageSignalGenerator {
 public:
  explicit PageSignalGeneratorProxy(mojo::MessageReceiverWithResponder* receiver);
  void AddReceiver(PageSignalReceiverPtr receiver) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) PageSignalReceiverStubDispatch {
 public:
  static bool Accept(PageSignalReceiver* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      PageSignalReceiver* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<PageSignalReceiver>>
class PageSignalReceiverStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  PageSignalReceiverStub() {}
  ~PageSignalReceiverStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PageSignalReceiverStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PageSignalReceiverStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) PageSignalGeneratorStubDispatch {
 public:
  static bool Accept(PageSignalGenerator* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      PageSignalGenerator* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<PageSignalGenerator>>
class PageSignalGeneratorStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  PageSignalGeneratorStub() {}
  ~PageSignalGeneratorStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PageSignalGeneratorStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PageSignalGeneratorStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) PageSignalReceiverRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) PageSignalGeneratorRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) PageNavigationIdentity {
 public:
  using DataView = PageNavigationIdentityDataView;
  using Data_ = internal::PageNavigationIdentity_Data;

  template <typename... Args>
  static PageNavigationIdentityPtr New(Args&&... args) {
    return PageNavigationIdentityPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static PageNavigationIdentityPtr From(const U& u) {
    return mojo::TypeConverter<PageNavigationIdentityPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, PageNavigationIdentity>::Convert(*this);
  }


  PageNavigationIdentity();

  PageNavigationIdentity(
      const ::resource_coordinator::CoordinationUnitID& page_cu_id,
      int64_t navigation_id,
      const std::string& url);

  ~PageNavigationIdentity();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = PageNavigationIdentityPtr>
  PageNavigationIdentityPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, PageNavigationIdentity>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        PageNavigationIdentity::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        PageNavigationIdentity::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::PageNavigationIdentity_UnserializedMessageContext<
            UserType, PageNavigationIdentity::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<PageNavigationIdentity::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return PageNavigationIdentity::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::PageNavigationIdentity_UnserializedMessageContext<
            UserType, PageNavigationIdentity::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<PageNavigationIdentity::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  ::resource_coordinator::CoordinationUnitID page_cu_id;
  int64_t navigation_id;
  std::string url;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};

template <typename StructPtrType>
PageNavigationIdentityPtr PageNavigationIdentity::Clone() const {
  return New(
      mojo::Clone(page_cu_id),
      mojo::Clone(navigation_id),
      mojo::Clone(url)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, PageNavigationIdentity>::value>::type*>
bool PageNavigationIdentity::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->page_cu_id, other_struct.page_cu_id))
    return false;
  if (!mojo::Equals(this->navigation_id, other_struct.navigation_id))
    return false;
  if (!mojo::Equals(this->url, other_struct.url))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace resource_coordinator

namespace mojo {


template <>
struct COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) StructTraits<::resource_coordinator::mojom::PageNavigationIdentity::DataView,
                                         ::resource_coordinator::mojom::PageNavigationIdentityPtr> {
  static bool IsNull(const ::resource_coordinator::mojom::PageNavigationIdentityPtr& input) { return !input; }
  static void SetToNull(::resource_coordinator::mojom::PageNavigationIdentityPtr* output) { output->reset(); }

  static const decltype(::resource_coordinator::mojom::PageNavigationIdentity::page_cu_id)& page_cu_id(
      const ::resource_coordinator::mojom::PageNavigationIdentityPtr& input) {
    return input->page_cu_id;
  }

  static decltype(::resource_coordinator::mojom::PageNavigationIdentity::navigation_id) navigation_id(
      const ::resource_coordinator::mojom::PageNavigationIdentityPtr& input) {
    return input->navigation_id;
  }

  static const decltype(::resource_coordinator::mojom::PageNavigationIdentity::url)& url(
      const ::resource_coordinator::mojom::PageNavigationIdentityPtr& input) {
    return input->url;
  }

  static bool Read(::resource_coordinator::mojom::PageNavigationIdentity::DataView input, ::resource_coordinator::mojom::PageNavigationIdentityPtr* output);
};

}  // namespace mojo

#endif  // SERVICES_RESOURCE_COORDINATOR_PUBLIC_MOJOM_PAGE_SIGNAL_MOJOM_H_