// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_RESOURCE_COORDINATOR_PUBLIC_MOJOM_WEBUI_GRAPH_DUMP_MOJOM_SHARED_H_
#define SERVICES_RESOURCE_COORDINATOR_PUBLIC_MOJOM_WEBUI_GRAPH_DUMP_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "services/resource_coordinator/public/mojom/webui_graph_dump.mojom-shared-internal.h"
#include "mojo/public/mojom/base/process_id.mojom-shared.h"
#include "mojo/public/mojom/base/time.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "base/component_export.h"


namespace resource_coordinator {
namespace mojom {
class WebUIPageInfoDataView;

class WebUIFrameInfoDataView;

class WebUIProcessInfoDataView;

class WebUIGraphDataView;



}  // namespace mojom
}  // namespace resource_coordinator

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::resource_coordinator::mojom::WebUIPageInfoDataView> {
  using Data = ::resource_coordinator::mojom::internal::WebUIPageInfo_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::resource_coordinator::mojom::WebUIFrameInfoDataView> {
  using Data = ::resource_coordinator::mojom::internal::WebUIFrameInfo_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::resource_coordinator::mojom::WebUIProcessInfoDataView> {
  using Data = ::resource_coordinator::mojom::internal::WebUIProcessInfo_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::resource_coordinator::mojom::WebUIGraphDataView> {
  using Data = ::resource_coordinator::mojom::internal::WebUIGraph_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace resource_coordinator {
namespace mojom {
// Interface base classes. They are used for type safety check.
class WebUIGraphDumpInterfaceBase {};

using WebUIGraphDumpPtrDataView =
    mojo::InterfacePtrDataView<WebUIGraphDumpInterfaceBase>;
using WebUIGraphDumpRequestDataView =
    mojo::InterfaceRequestDataView<WebUIGraphDumpInterfaceBase>;
using WebUIGraphDumpAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<WebUIGraphDumpInterfaceBase>;
using WebUIGraphDumpAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<WebUIGraphDumpInterfaceBase>;
class WebUIPageInfoDataView {
 public:
  WebUIPageInfoDataView() {}

  WebUIPageInfoDataView(
      internal::WebUIPageInfo_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int64_t id() const {
    return data_->id;
  }
  int64_t main_frame_id() const {
    return data_->main_frame_id;
  }
  inline void GetMainFrameUrlDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadMainFrameUrl(UserType* output) {
    auto* pointer = data_->main_frame_url.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::WebUIPageInfo_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class WebUIFrameInfoDataView {
 public:
  WebUIFrameInfoDataView() {}

  WebUIFrameInfoDataView(
      internal::WebUIFrameInfo_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  int64_t id() const {
    return data_->id;
  }
  int64_t parent_frame_id() const {
    return data_->parent_frame_id;
  }
  int64_t process_id() const {
    return data_->process_id;
  }
 private:
  internal::WebUIFrameInfo_Data* data_ = nullptr;
};

class WebUIProcessInfoDataView {
 public:
  WebUIProcessInfoDataView() {}

  WebUIProcessInfoDataView(
      internal::WebUIProcessInfo_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int64_t id() const {
    return data_->id;
  }
  inline void GetPidDataView(
      ::mojo_base::mojom::ProcessIdDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPid(UserType* output) {
    auto* pointer = data_->pid.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::ProcessIdDataView>(
        pointer, output, context_);
  }
  inline void GetCumulativeCpuUsageDataView(
      ::mojo_base::mojom::TimeDeltaDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCumulativeCpuUsage(UserType* output) {
    auto* pointer = data_->cumulative_cpu_usage.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::TimeDeltaDataView>(
        pointer, output, context_);
  }
  uint64_t private_footprint_kb() const {
    return data_->private_footprint_kb;
  }
 private:
  internal::WebUIProcessInfo_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class WebUIGraphDataView {
 public:
  WebUIGraphDataView() {}

  WebUIGraphDataView(
      internal::WebUIGraph_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetPagesDataView(
      mojo::ArrayDataView<WebUIPageInfoDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPages(UserType* output) {
    auto* pointer = data_->pages.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::resource_coordinator::mojom::WebUIPageInfoDataView>>(
        pointer, output, context_);
  }
  inline void GetFramesDataView(
      mojo::ArrayDataView<WebUIFrameInfoDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFrames(UserType* output) {
    auto* pointer = data_->frames.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::resource_coordinator::mojom::WebUIFrameInfoDataView>>(
        pointer, output, context_);
  }
  inline void GetProcessesDataView(
      mojo::ArrayDataView<WebUIProcessInfoDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadProcesses(UserType* output) {
    auto* pointer = data_->processes.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::resource_coordinator::mojom::WebUIProcessInfoDataView>>(
        pointer, output, context_);
  }
 private:
  internal::WebUIGraph_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class WebUIGraphDump_GetCurrentGraph_ParamsDataView {
 public:
  WebUIGraphDump_GetCurrentGraph_ParamsDataView() {}

  WebUIGraphDump_GetCurrentGraph_ParamsDataView(
      internal::WebUIGraphDump_GetCurrentGraph_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::WebUIGraphDump_GetCurrentGraph_Params_Data* data_ = nullptr;
};

class WebUIGraphDump_GetCurrentGraph_ResponseParamsDataView {
 public:
  WebUIGraphDump_GetCurrentGraph_ResponseParamsDataView() {}

  WebUIGraphDump_GetCurrentGraph_ResponseParamsDataView(
      internal::WebUIGraphDump_GetCurrentGraph_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetGraphDataView(
      WebUIGraphDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadGraph(UserType* output) {
    auto* pointer = data_->graph.Get();
    return mojo::internal::Deserialize<::resource_coordinator::mojom::WebUIGraphDataView>(
        pointer, output, context_);
  }
 private:
  internal::WebUIGraphDump_GetCurrentGraph_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace resource_coordinator

namespace std {

}  // namespace std

namespace mojo {


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::resource_coordinator::mojom::WebUIPageInfoDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::resource_coordinator::mojom::WebUIPageInfoDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::resource_coordinator::mojom::internal::WebUIPageInfo_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    (*output)->id = CallWithContext(Traits::id, input, custom_context);
    (*output)->main_frame_id = CallWithContext(Traits::main_frame_id, input, custom_context);
    decltype(CallWithContext(Traits::main_frame_url, input, custom_context)) in_main_frame_url = CallWithContext(Traits::main_frame_url, input, custom_context);
    typename decltype((*output)->main_frame_url)::BaseType::BufferWriter
        main_frame_url_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_main_frame_url, buffer, &main_frame_url_writer, context);
    (*output)->main_frame_url.Set(
        main_frame_url_writer.is_null() ? nullptr : main_frame_url_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->main_frame_url.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null main_frame_url in WebUIPageInfo struct");
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::resource_coordinator::mojom::internal::WebUIPageInfo_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::resource_coordinator::mojom::WebUIPageInfoDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::resource_coordinator::mojom::WebUIFrameInfoDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::resource_coordinator::mojom::WebUIFrameInfoDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::resource_coordinator::mojom::internal::WebUIFrameInfo_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    (*output)->id = CallWithContext(Traits::id, input, custom_context);
    (*output)->parent_frame_id = CallWithContext(Traits::parent_frame_id, input, custom_context);
    (*output)->process_id = CallWithContext(Traits::process_id, input, custom_context);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::resource_coordinator::mojom::internal::WebUIFrameInfo_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::resource_coordinator::mojom::WebUIFrameInfoDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::resource_coordinator::mojom::WebUIProcessInfoDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::resource_coordinator::mojom::WebUIProcessInfoDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::resource_coordinator::mojom::internal::WebUIProcessInfo_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    (*output)->id = CallWithContext(Traits::id, input, custom_context);
    decltype(CallWithContext(Traits::pid, input, custom_context)) in_pid = CallWithContext(Traits::pid, input, custom_context);
    typename decltype((*output)->pid)::BaseType::BufferWriter
        pid_writer;
    mojo::internal::Serialize<::mojo_base::mojom::ProcessIdDataView>(
        in_pid, buffer, &pid_writer, context);
    (*output)->pid.Set(
        pid_writer.is_null() ? nullptr : pid_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->pid.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null pid in WebUIProcessInfo struct");
    decltype(CallWithContext(Traits::cumulative_cpu_usage, input, custom_context)) in_cumulative_cpu_usage = CallWithContext(Traits::cumulative_cpu_usage, input, custom_context);
    typename decltype((*output)->cumulative_cpu_usage)::BaseType::BufferWriter
        cumulative_cpu_usage_writer;
    mojo::internal::Serialize<::mojo_base::mojom::TimeDeltaDataView>(
        in_cumulative_cpu_usage, buffer, &cumulative_cpu_usage_writer, context);
    (*output)->cumulative_cpu_usage.Set(
        cumulative_cpu_usage_writer.is_null() ? nullptr : cumulative_cpu_usage_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->cumulative_cpu_usage.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null cumulative_cpu_usage in WebUIProcessInfo struct");
    (*output)->private_footprint_kb = CallWithContext(Traits::private_footprint_kb, input, custom_context);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::resource_coordinator::mojom::internal::WebUIProcessInfo_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::resource_coordinator::mojom::WebUIProcessInfoDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::resource_coordinator::mojom::WebUIGraphDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::resource_coordinator::mojom::WebUIGraphDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::resource_coordinator::mojom::internal::WebUIGraph_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    decltype(CallWithContext(Traits::pages, input, custom_context)) in_pages = CallWithContext(Traits::pages, input, custom_context);
    typename decltype((*output)->pages)::BaseType::BufferWriter
        pages_writer;
    const mojo::internal::ContainerValidateParams pages_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::resource_coordinator::mojom::WebUIPageInfoDataView>>(
        in_pages, buffer, &pages_writer, &pages_validate_params,
        context);
    (*output)->pages.Set(
        pages_writer.is_null() ? nullptr : pages_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->pages.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null pages in WebUIGraph struct");
    decltype(CallWithContext(Traits::frames, input, custom_context)) in_frames = CallWithContext(Traits::frames, input, custom_context);
    typename decltype((*output)->frames)::BaseType::BufferWriter
        frames_writer;
    const mojo::internal::ContainerValidateParams frames_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::resource_coordinator::mojom::WebUIFrameInfoDataView>>(
        in_frames, buffer, &frames_writer, &frames_validate_params,
        context);
    (*output)->frames.Set(
        frames_writer.is_null() ? nullptr : frames_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->frames.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null frames in WebUIGraph struct");
    decltype(CallWithContext(Traits::processes, input, custom_context)) in_processes = CallWithContext(Traits::processes, input, custom_context);
    typename decltype((*output)->processes)::BaseType::BufferWriter
        processes_writer;
    const mojo::internal::ContainerValidateParams processes_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::resource_coordinator::mojom::WebUIProcessInfoDataView>>(
        in_processes, buffer, &processes_writer, &processes_validate_params,
        context);
    (*output)->processes.Set(
        processes_writer.is_null() ? nullptr : processes_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->processes.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null processes in WebUIGraph struct");
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::resource_coordinator::mojom::internal::WebUIGraph_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::resource_coordinator::mojom::WebUIGraphDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace resource_coordinator {
namespace mojom {

inline void WebUIPageInfoDataView::GetMainFrameUrlDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->main_frame_url.Get();
  *output = mojo::StringDataView(pointer, context_);
}




inline void WebUIProcessInfoDataView::GetPidDataView(
    ::mojo_base::mojom::ProcessIdDataView* output) {
  auto pointer = data_->pid.Get();
  *output = ::mojo_base::mojom::ProcessIdDataView(pointer, context_);
}
inline void WebUIProcessInfoDataView::GetCumulativeCpuUsageDataView(
    ::mojo_base::mojom::TimeDeltaDataView* output) {
  auto pointer = data_->cumulative_cpu_usage.Get();
  *output = ::mojo_base::mojom::TimeDeltaDataView(pointer, context_);
}


inline void WebUIGraphDataView::GetPagesDataView(
    mojo::ArrayDataView<WebUIPageInfoDataView>* output) {
  auto pointer = data_->pages.Get();
  *output = mojo::ArrayDataView<WebUIPageInfoDataView>(pointer, context_);
}
inline void WebUIGraphDataView::GetFramesDataView(
    mojo::ArrayDataView<WebUIFrameInfoDataView>* output) {
  auto pointer = data_->frames.Get();
  *output = mojo::ArrayDataView<WebUIFrameInfoDataView>(pointer, context_);
}
inline void WebUIGraphDataView::GetProcessesDataView(
    mojo::ArrayDataView<WebUIProcessInfoDataView>* output) {
  auto pointer = data_->processes.Get();
  *output = mojo::ArrayDataView<WebUIProcessInfoDataView>(pointer, context_);
}




inline void WebUIGraphDump_GetCurrentGraph_ResponseParamsDataView::GetGraphDataView(
    WebUIGraphDataView* output) {
  auto pointer = data_->graph.Get();
  *output = WebUIGraphDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace resource_coordinator

#endif  // SERVICES_RESOURCE_COORDINATOR_PUBLIC_MOJOM_WEBUI_GRAPH_DUMP_MOJOM_SHARED_H_