// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "services/resource_coordinator/public/mojom/webui_graph_dump.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "base/run_loop.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"

#include "services/resource_coordinator/public/mojom/webui_graph_dump.mojom-shared-message-ids.h"


#ifndef SERVICES_RESOURCE_COORDINATOR_PUBLIC_MOJOM_WEBUI_GRAPH_DUMP_MOJOM_JUMBO_H_
#define SERVICES_RESOURCE_COORDINATOR_PUBLIC_MOJOM_WEBUI_GRAPH_DUMP_MOJOM_JUMBO_H_
#include "mojo/public/cpp/base/process_id_mojom_traits.h"
#include "mojo/public/cpp/base/time_mojom_traits.h"
#endif
namespace resource_coordinator {
namespace mojom {
WebUIPageInfo::WebUIPageInfo()
    : id(),
      main_frame_id(),
      main_frame_url() {}

WebUIPageInfo::WebUIPageInfo(
    int64_t id_in,
    int64_t main_frame_id_in,
    const std::string& main_frame_url_in)
    : id(std::move(id_in)),
      main_frame_id(std::move(main_frame_id_in)),
      main_frame_url(std::move(main_frame_url_in)) {}

WebUIPageInfo::~WebUIPageInfo() = default;
size_t WebUIPageInfo::Hash(size_t seed) const {
  seed = mojo::internal::Hash(seed, this->id);
  seed = mojo::internal::Hash(seed, this->main_frame_id);
  seed = mojo::internal::Hash(seed, this->main_frame_url);
  return seed;
}

bool WebUIPageInfo::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
WebUIFrameInfo::WebUIFrameInfo()
    : id(),
      parent_frame_id(),
      process_id() {}

WebUIFrameInfo::WebUIFrameInfo(
    int64_t id_in,
    int64_t parent_frame_id_in,
    int64_t process_id_in)
    : id(std::move(id_in)),
      parent_frame_id(std::move(parent_frame_id_in)),
      process_id(std::move(process_id_in)) {}

WebUIFrameInfo::~WebUIFrameInfo() = default;
size_t WebUIFrameInfo::Hash(size_t seed) const {
  seed = mojo::internal::Hash(seed, this->id);
  seed = mojo::internal::Hash(seed, this->parent_frame_id);
  seed = mojo::internal::Hash(seed, this->process_id);
  return seed;
}

bool WebUIFrameInfo::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
WebUIProcessInfo::WebUIProcessInfo()
    : id(),
      pid(),
      cumulative_cpu_usage(),
      private_footprint_kb() {}

WebUIProcessInfo::WebUIProcessInfo(
    int64_t id_in,
    base::ProcessId pid_in,
    base::TimeDelta cumulative_cpu_usage_in,
    uint64_t private_footprint_kb_in)
    : id(std::move(id_in)),
      pid(std::move(pid_in)),
      cumulative_cpu_usage(std::move(cumulative_cpu_usage_in)),
      private_footprint_kb(std::move(private_footprint_kb_in)) {}

WebUIProcessInfo::~WebUIProcessInfo() = default;

bool WebUIProcessInfo::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
WebUIGraph::WebUIGraph()
    : pages(),
      frames(),
      processes() {}

WebUIGraph::WebUIGraph(
    std::vector<WebUIPageInfoPtr> pages_in,
    std::vector<WebUIFrameInfoPtr> frames_in,
    std::vector<WebUIProcessInfoPtr> processes_in)
    : pages(std::move(pages_in)),
      frames(std::move(frames_in)),
      processes(std::move(processes_in)) {}

WebUIGraph::~WebUIGraph() = default;

bool WebUIGraph::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
const char WebUIGraphDump::Name_[] = "resource_coordinator.mojom.WebUIGraphDump";

class WebUIGraphDump_GetCurrentGraph_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  WebUIGraphDump_GetCurrentGraph_ForwardToCallback(
      WebUIGraphDump::GetCurrentGraphCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  WebUIGraphDump::GetCurrentGraphCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(WebUIGraphDump_GetCurrentGraph_ForwardToCallback);
};

WebUIGraphDumpProxy::WebUIGraphDumpProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void WebUIGraphDumpProxy::GetCurrentGraph(
    GetCurrentGraphCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "resource_coordinator::mojom::WebUIGraphDump::GetCurrentGraph");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kWebUIGraphDump_GetCurrentGraph_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::resource_coordinator::mojom::internal::WebUIGraphDump_GetCurrentGraph_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(WebUIGraphDump::Name_);
  message.set_method_name("GetCurrentGraph");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new WebUIGraphDump_GetCurrentGraph_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class WebUIGraphDump_GetCurrentGraph_ProxyToResponder {
 public:
  static WebUIGraphDump::GetCurrentGraphCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<WebUIGraphDump_GetCurrentGraph_ProxyToResponder> proxy(
        new WebUIGraphDump_GetCurrentGraph_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&WebUIGraphDump_GetCurrentGraph_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~WebUIGraphDump_GetCurrentGraph_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  WebUIGraphDump_GetCurrentGraph_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "WebUIGraphDump::GetCurrentGraphCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      WebUIGraphPtr in_graph);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(WebUIGraphDump_GetCurrentGraph_ProxyToResponder);
};

bool WebUIGraphDump_GetCurrentGraph_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "resource_coordinator::mojom::WebUIGraphDump::GetCurrentGraphCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::WebUIGraphDump_GetCurrentGraph_ResponseParams_Data* params =
      reinterpret_cast<
          internal::WebUIGraphDump_GetCurrentGraph_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  WebUIGraphPtr p_graph{};
  WebUIGraphDump_GetCurrentGraph_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadGraph(&p_graph))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "WebUIGraphDump::GetCurrentGraph response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_graph));
  return true;
}

void WebUIGraphDump_GetCurrentGraph_ProxyToResponder::Run(
    WebUIGraphPtr in_graph) {
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kWebUIGraphDump_GetCurrentGraph_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::resource_coordinator::mojom::internal::WebUIGraphDump_GetCurrentGraph_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->graph)::BaseType::BufferWriter
      graph_writer;
  mojo::internal::Serialize<::resource_coordinator::mojom::WebUIGraphDataView>(
      in_graph, buffer, &graph_writer, &serialization_context);
  params->graph.Set(
      graph_writer.is_null() ? nullptr : graph_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->graph.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null graph in ");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)resource_coordinator::mojom::WebUIGraphDump::GetCurrentGraphCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(WebUIGraphDump::Name_);
  message.set_method_name("GetCurrentGraph");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool WebUIGraphDumpStubDispatch::Accept(
    WebUIGraphDump* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kWebUIGraphDump_GetCurrentGraph_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)resource_coordinator::mojom::WebUIGraphDump::GetCurrentGraph",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

// static
bool WebUIGraphDumpStubDispatch::AcceptWithResponder(
    WebUIGraphDump* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kWebUIGraphDump_GetCurrentGraph_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)resource_coordinator::mojom::WebUIGraphDump::GetCurrentGraph",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      internal::WebUIGraphDump_GetCurrentGraph_Params_Data* params =
          reinterpret_cast<
              internal::WebUIGraphDump_GetCurrentGraph_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      WebUIGraphDump_GetCurrentGraph_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WebUIGraphDump::GetCurrentGraph deserializer");
        return false;
      }
      WebUIGraphDump::GetCurrentGraphCallback callback =
          WebUIGraphDump_GetCurrentGraph_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetCurrentGraph(std::move(callback));
      return true;
    }
  }
  return false;
}

bool WebUIGraphDumpRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "WebUIGraphDump RequestValidator");

  switch (message->header()->name) {
    case internal::kWebUIGraphDump_GetCurrentGraph_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WebUIGraphDump_GetCurrentGraph_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool WebUIGraphDumpResponseValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "WebUIGraphDump ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kWebUIGraphDump_GetCurrentGraph_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::WebUIGraphDump_GetCurrentGraph_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
}  // namespace mojom
}  // namespace resource_coordinator

namespace mojo {


// static
bool StructTraits<::resource_coordinator::mojom::WebUIPageInfo::DataView, ::resource_coordinator::mojom::WebUIPageInfoPtr>::Read(
    ::resource_coordinator::mojom::WebUIPageInfo::DataView input,
    ::resource_coordinator::mojom::WebUIPageInfoPtr* output) {
  bool success = true;
  ::resource_coordinator::mojom::WebUIPageInfoPtr result(::resource_coordinator::mojom::WebUIPageInfo::New());
  
      result->id = input.id();
      result->main_frame_id = input.main_frame_id();
      if (!input.ReadMainFrameUrl(&result->main_frame_url))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::resource_coordinator::mojom::WebUIFrameInfo::DataView, ::resource_coordinator::mojom::WebUIFrameInfoPtr>::Read(
    ::resource_coordinator::mojom::WebUIFrameInfo::DataView input,
    ::resource_coordinator::mojom::WebUIFrameInfoPtr* output) {
  bool success = true;
  ::resource_coordinator::mojom::WebUIFrameInfoPtr result(::resource_coordinator::mojom::WebUIFrameInfo::New());
  
      result->id = input.id();
      result->parent_frame_id = input.parent_frame_id();
      result->process_id = input.process_id();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::resource_coordinator::mojom::WebUIProcessInfo::DataView, ::resource_coordinator::mojom::WebUIProcessInfoPtr>::Read(
    ::resource_coordinator::mojom::WebUIProcessInfo::DataView input,
    ::resource_coordinator::mojom::WebUIProcessInfoPtr* output) {
  bool success = true;
  ::resource_coordinator::mojom::WebUIProcessInfoPtr result(::resource_coordinator::mojom::WebUIProcessInfo::New());
  
      result->id = input.id();
      if (!input.ReadPid(&result->pid))
        success = false;
      if (!input.ReadCumulativeCpuUsage(&result->cumulative_cpu_usage))
        success = false;
      result->private_footprint_kb = input.private_footprint_kb();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::resource_coordinator::mojom::WebUIGraph::DataView, ::resource_coordinator::mojom::WebUIGraphPtr>::Read(
    ::resource_coordinator::mojom::WebUIGraph::DataView input,
    ::resource_coordinator::mojom::WebUIGraphPtr* output) {
  bool success = true;
  ::resource_coordinator::mojom::WebUIGraphPtr result(::resource_coordinator::mojom::WebUIGraph::New());
  
      if (!input.ReadPages(&result->pages))
        success = false;
      if (!input.ReadFrames(&result->frames))
        success = false;
      if (!input.ReadProcesses(&result->processes))
        success = false;
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif