// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_RESOURCE_COORDINATOR_PUBLIC_MOJOM_WEBUI_GRAPH_DUMP_MOJOM_H_
#define SERVICES_RESOURCE_COORDINATOR_PUBLIC_MOJOM_WEBUI_GRAPH_DUMP_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/resource_coordinator/public/mojom/webui_graph_dump.mojom-shared.h"
#include "mojo/public/mojom/base/process_id.mojom.h"
#include "mojo/public/mojom/base/time.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "base/component_export.h"


namespace resource_coordinator {
namespace mojom {
class WebUIGraphDump;
using WebUIGraphDumpPtr = mojo::InterfacePtr<WebUIGraphDump>;
using WebUIGraphDumpPtrInfo = mojo::InterfacePtrInfo<WebUIGraphDump>;
using ThreadSafeWebUIGraphDumpPtr =
    mojo::ThreadSafeInterfacePtr<WebUIGraphDump>;
using WebUIGraphDumpRequest = mojo::InterfaceRequest<WebUIGraphDump>;
using WebUIGraphDumpAssociatedPtr =
    mojo::AssociatedInterfacePtr<WebUIGraphDump>;
using ThreadSafeWebUIGraphDumpAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<WebUIGraphDump>;
using WebUIGraphDumpAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<WebUIGraphDump>;
using WebUIGraphDumpAssociatedRequest =
    mojo::AssociatedInterfaceRequest<WebUIGraphDump>;

class WebUIPageInfo;
using WebUIPageInfoPtr = mojo::InlinedStructPtr<WebUIPageInfo>;

class WebUIFrameInfo;
using WebUIFrameInfoPtr = mojo::InlinedStructPtr<WebUIFrameInfo>;

class WebUIProcessInfo;
using WebUIProcessInfoPtr = mojo::StructPtr<WebUIProcessInfo>;

class WebUIGraph;
using WebUIGraphPtr = mojo::StructPtr<WebUIGraph>;


class WebUIGraphDumpProxy;

template <typename ImplRefTraits>
class WebUIGraphDumpStub;

class WebUIGraphDumpRequestValidator;
class WebUIGraphDumpResponseValidator;

class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) WebUIGraphDump
    : public WebUIGraphDumpInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = WebUIGraphDumpProxy;

  template <typename ImplRefTraits>
  using Stub_ = WebUIGraphDumpStub<ImplRefTraits>;

  using RequestValidator_ = WebUIGraphDumpRequestValidator;
  using ResponseValidator_ = WebUIGraphDumpResponseValidator;
  enum MethodMinVersions : uint32_t {
    kGetCurrentGraphMinVersion = 0,
  };
  virtual ~WebUIGraphDump() {}


  using GetCurrentGraphCallback = base::OnceCallback<void(WebUIGraphPtr)>;
  virtual void GetCurrentGraph(GetCurrentGraphCallback callback) = 0;
};

class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) WebUIGraphDumpProxy
    : public WebUIGraphDump {
 public:
  explicit WebUIGraphDumpProxy(mojo::MessageReceiverWithResponder* receiver);
  void GetCurrentGraph(GetCurrentGraphCallback callback) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) WebUIGraphDumpStubDispatch {
 public:
  static bool Accept(WebUIGraphDump* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      WebUIGraphDump* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<WebUIGraphDump>>
class WebUIGraphDumpStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  WebUIGraphDumpStub() {}
  ~WebUIGraphDumpStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return WebUIGraphDumpStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return WebUIGraphDumpStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) WebUIGraphDumpRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) WebUIGraphDumpResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};


class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) WebUIPageInfo {
 public:
  using DataView = WebUIPageInfoDataView;
  using Data_ = internal::WebUIPageInfo_Data;

  template <typename... Args>
  static WebUIPageInfoPtr New(Args&&... args) {
    return WebUIPageInfoPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static WebUIPageInfoPtr From(const U& u) {
    return mojo::TypeConverter<WebUIPageInfoPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, WebUIPageInfo>::Convert(*this);
  }


  WebUIPageInfo();

  WebUIPageInfo(
      int64_t id,
      int64_t main_frame_id,
      const std::string& main_frame_url);

  ~WebUIPageInfo();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = WebUIPageInfoPtr>
  WebUIPageInfoPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, WebUIPageInfo>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        WebUIPageInfo::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        WebUIPageInfo::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::WebUIPageInfo_UnserializedMessageContext<
            UserType, WebUIPageInfo::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<WebUIPageInfo::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return WebUIPageInfo::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::WebUIPageInfo_UnserializedMessageContext<
            UserType, WebUIPageInfo::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<WebUIPageInfo::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  int64_t id;
  int64_t main_frame_id;
  std::string main_frame_url;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) WebUIFrameInfo {
 public:
  using DataView = WebUIFrameInfoDataView;
  using Data_ = internal::WebUIFrameInfo_Data;

  template <typename... Args>
  static WebUIFrameInfoPtr New(Args&&... args) {
    return WebUIFrameInfoPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static WebUIFrameInfoPtr From(const U& u) {
    return mojo::TypeConverter<WebUIFrameInfoPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, WebUIFrameInfo>::Convert(*this);
  }


  WebUIFrameInfo();

  WebUIFrameInfo(
      int64_t id,
      int64_t parent_frame_id,
      int64_t process_id);

  ~WebUIFrameInfo();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = WebUIFrameInfoPtr>
  WebUIFrameInfoPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, WebUIFrameInfo>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        WebUIFrameInfo::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        WebUIFrameInfo::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::WebUIFrameInfo_UnserializedMessageContext<
            UserType, WebUIFrameInfo::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<WebUIFrameInfo::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return WebUIFrameInfo::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::WebUIFrameInfo_UnserializedMessageContext<
            UserType, WebUIFrameInfo::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<WebUIFrameInfo::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  int64_t id;
  int64_t parent_frame_id;
  int64_t process_id;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};








class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) WebUIProcessInfo {
 public:
  using DataView = WebUIProcessInfoDataView;
  using Data_ = internal::WebUIProcessInfo_Data;

  template <typename... Args>
  static WebUIProcessInfoPtr New(Args&&... args) {
    return WebUIProcessInfoPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static WebUIProcessInfoPtr From(const U& u) {
    return mojo::TypeConverter<WebUIProcessInfoPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, WebUIProcessInfo>::Convert(*this);
  }


  WebUIProcessInfo();

  WebUIProcessInfo(
      int64_t id,
      base::ProcessId pid,
      base::TimeDelta cumulative_cpu_usage,
      uint64_t private_footprint_kb);

  ~WebUIProcessInfo();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = WebUIProcessInfoPtr>
  WebUIProcessInfoPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, WebUIProcessInfo>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        WebUIProcessInfo::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        WebUIProcessInfo::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::WebUIProcessInfo_UnserializedMessageContext<
            UserType, WebUIProcessInfo::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<WebUIProcessInfo::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return WebUIProcessInfo::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::WebUIProcessInfo_UnserializedMessageContext<
            UserType, WebUIProcessInfo::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<WebUIProcessInfo::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  int64_t id;
  base::ProcessId pid;
  base::TimeDelta cumulative_cpu_usage;
  uint64_t private_footprint_kb;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) WebUIGraph {
 public:
  using DataView = WebUIGraphDataView;
  using Data_ = internal::WebUIGraph_Data;

  template <typename... Args>
  static WebUIGraphPtr New(Args&&... args) {
    return WebUIGraphPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static WebUIGraphPtr From(const U& u) {
    return mojo::TypeConverter<WebUIGraphPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, WebUIGraph>::Convert(*this);
  }


  WebUIGraph();

  WebUIGraph(
      std::vector<WebUIPageInfoPtr> pages,
      std::vector<WebUIFrameInfoPtr> frames,
      std::vector<WebUIProcessInfoPtr> processes);

  ~WebUIGraph();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = WebUIGraphPtr>
  WebUIGraphPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, WebUIGraph>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        WebUIGraph::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        WebUIGraph::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::WebUIGraph_UnserializedMessageContext<
            UserType, WebUIGraph::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<WebUIGraph::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return WebUIGraph::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::WebUIGraph_UnserializedMessageContext<
            UserType, WebUIGraph::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<WebUIGraph::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  std::vector<WebUIPageInfoPtr> pages;
  std::vector<WebUIFrameInfoPtr> frames;
  std::vector<WebUIProcessInfoPtr> processes;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(WebUIGraph);
};

template <typename StructPtrType>
WebUIPageInfoPtr WebUIPageInfo::Clone() const {
  return New(
      mojo::Clone(id),
      mojo::Clone(main_frame_id),
      mojo::Clone(main_frame_url)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, WebUIPageInfo>::value>::type*>
bool WebUIPageInfo::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->id, other_struct.id))
    return false;
  if (!mojo::Equals(this->main_frame_id, other_struct.main_frame_id))
    return false;
  if (!mojo::Equals(this->main_frame_url, other_struct.main_frame_url))
    return false;
  return true;
}
template <typename StructPtrType>
WebUIFrameInfoPtr WebUIFrameInfo::Clone() const {
  return New(
      mojo::Clone(id),
      mojo::Clone(parent_frame_id),
      mojo::Clone(process_id)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, WebUIFrameInfo>::value>::type*>
bool WebUIFrameInfo::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->id, other_struct.id))
    return false;
  if (!mojo::Equals(this->parent_frame_id, other_struct.parent_frame_id))
    return false;
  if (!mojo::Equals(this->process_id, other_struct.process_id))
    return false;
  return true;
}
template <typename StructPtrType>
WebUIProcessInfoPtr WebUIProcessInfo::Clone() const {
  return New(
      mojo::Clone(id),
      mojo::Clone(pid),
      mojo::Clone(cumulative_cpu_usage),
      mojo::Clone(private_footprint_kb)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, WebUIProcessInfo>::value>::type*>
bool WebUIProcessInfo::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->id, other_struct.id))
    return false;
  if (!mojo::Equals(this->pid, other_struct.pid))
    return false;
  if (!mojo::Equals(this->cumulative_cpu_usage, other_struct.cumulative_cpu_usage))
    return false;
  if (!mojo::Equals(this->private_footprint_kb, other_struct.private_footprint_kb))
    return false;
  return true;
}
template <typename StructPtrType>
WebUIGraphPtr WebUIGraph::Clone() const {
  return New(
      mojo::Clone(pages),
      mojo::Clone(frames),
      mojo::Clone(processes)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, WebUIGraph>::value>::type*>
bool WebUIGraph::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->pages, other_struct.pages))
    return false;
  if (!mojo::Equals(this->frames, other_struct.frames))
    return false;
  if (!mojo::Equals(this->processes, other_struct.processes))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace resource_coordinator

namespace mojo {


template <>
struct COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) StructTraits<::resource_coordinator::mojom::WebUIPageInfo::DataView,
                                         ::resource_coordinator::mojom::WebUIPageInfoPtr> {
  static bool IsNull(const ::resource_coordinator::mojom::WebUIPageInfoPtr& input) { return !input; }
  static void SetToNull(::resource_coordinator::mojom::WebUIPageInfoPtr* output) { output->reset(); }

  static decltype(::resource_coordinator::mojom::WebUIPageInfo::id) id(
      const ::resource_coordinator::mojom::WebUIPageInfoPtr& input) {
    return input->id;
  }

  static decltype(::resource_coordinator::mojom::WebUIPageInfo::main_frame_id) main_frame_id(
      const ::resource_coordinator::mojom::WebUIPageInfoPtr& input) {
    return input->main_frame_id;
  }

  static const decltype(::resource_coordinator::mojom::WebUIPageInfo::main_frame_url)& main_frame_url(
      const ::resource_coordinator::mojom::WebUIPageInfoPtr& input) {
    return input->main_frame_url;
  }

  static bool Read(::resource_coordinator::mojom::WebUIPageInfo::DataView input, ::resource_coordinator::mojom::WebUIPageInfoPtr* output);
};


template <>
struct COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) StructTraits<::resource_coordinator::mojom::WebUIFrameInfo::DataView,
                                         ::resource_coordinator::mojom::WebUIFrameInfoPtr> {
  static bool IsNull(const ::resource_coordinator::mojom::WebUIFrameInfoPtr& input) { return !input; }
  static void SetToNull(::resource_coordinator::mojom::WebUIFrameInfoPtr* output) { output->reset(); }

  static decltype(::resource_coordinator::mojom::WebUIFrameInfo::id) id(
      const ::resource_coordinator::mojom::WebUIFrameInfoPtr& input) {
    return input->id;
  }

  static decltype(::resource_coordinator::mojom::WebUIFrameInfo::parent_frame_id) parent_frame_id(
      const ::resource_coordinator::mojom::WebUIFrameInfoPtr& input) {
    return input->parent_frame_id;
  }

  static decltype(::resource_coordinator::mojom::WebUIFrameInfo::process_id) process_id(
      const ::resource_coordinator::mojom::WebUIFrameInfoPtr& input) {
    return input->process_id;
  }

  static bool Read(::resource_coordinator::mojom::WebUIFrameInfo::DataView input, ::resource_coordinator::mojom::WebUIFrameInfoPtr* output);
};


template <>
struct COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) StructTraits<::resource_coordinator::mojom::WebUIProcessInfo::DataView,
                                         ::resource_coordinator::mojom::WebUIProcessInfoPtr> {
  static bool IsNull(const ::resource_coordinator::mojom::WebUIProcessInfoPtr& input) { return !input; }
  static void SetToNull(::resource_coordinator::mojom::WebUIProcessInfoPtr* output) { output->reset(); }

  static decltype(::resource_coordinator::mojom::WebUIProcessInfo::id) id(
      const ::resource_coordinator::mojom::WebUIProcessInfoPtr& input) {
    return input->id;
  }

  static const decltype(::resource_coordinator::mojom::WebUIProcessInfo::pid)& pid(
      const ::resource_coordinator::mojom::WebUIProcessInfoPtr& input) {
    return input->pid;
  }

  static const decltype(::resource_coordinator::mojom::WebUIProcessInfo::cumulative_cpu_usage)& cumulative_cpu_usage(
      const ::resource_coordinator::mojom::WebUIProcessInfoPtr& input) {
    return input->cumulative_cpu_usage;
  }

  static decltype(::resource_coordinator::mojom::WebUIProcessInfo::private_footprint_kb) private_footprint_kb(
      const ::resource_coordinator::mojom::WebUIProcessInfoPtr& input) {
    return input->private_footprint_kb;
  }

  static bool Read(::resource_coordinator::mojom::WebUIProcessInfo::DataView input, ::resource_coordinator::mojom::WebUIProcessInfoPtr* output);
};


template <>
struct COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) StructTraits<::resource_coordinator::mojom::WebUIGraph::DataView,
                                         ::resource_coordinator::mojom::WebUIGraphPtr> {
  static bool IsNull(const ::resource_coordinator::mojom::WebUIGraphPtr& input) { return !input; }
  static void SetToNull(::resource_coordinator::mojom::WebUIGraphPtr* output) { output->reset(); }

  static const decltype(::resource_coordinator::mojom::WebUIGraph::pages)& pages(
      const ::resource_coordinator::mojom::WebUIGraphPtr& input) {
    return input->pages;
  }

  static const decltype(::resource_coordinator::mojom::WebUIGraph::frames)& frames(
      const ::resource_coordinator::mojom::WebUIGraphPtr& input) {
    return input->frames;
  }

  static const decltype(::resource_coordinator::mojom::WebUIGraph::processes)& processes(
      const ::resource_coordinator::mojom::WebUIGraphPtr& input) {
    return input->processes;
  }

  static bool Read(::resource_coordinator::mojom::WebUIGraph::DataView input, ::resource_coordinator::mojom::WebUIGraphPtr* output);
};

}  // namespace mojo

#endif  // SERVICES_RESOURCE_COORDINATOR_PUBLIC_MOJOM_WEBUI_GRAPH_DUMP_MOJOM_H_