// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_SHAPE_DETECTION_PUBLIC_MOJOM_BARCODEDETECTION_PROVIDER_MOJOM_SHARED_H_
#define SERVICES_SHAPE_DETECTION_PUBLIC_MOJOM_BARCODEDETECTION_PROVIDER_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "services/shape_detection/public/mojom/barcodedetection_provider.mojom-shared-internal.h"
#include "services/shape_detection/public/mojom/barcodedetection.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace shape_detection {
namespace mojom {
class BarcodeDetectorOptionsDataView;



}  // namespace mojom
}  // namespace shape_detection

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::shape_detection::mojom::BarcodeDetectorOptionsDataView> {
  using Data = ::shape_detection::mojom::internal::BarcodeDetectorOptions_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace shape_detection {
namespace mojom {

enum class BarcodeFormat : int32_t {
  AZTEC,
  CODE_128,
  CODE_39,
  CODE_93,
  CODABAR,
  DATA_MATRIX,
  EAN_13,
  EAN_8,
  ITF,
  PDF417,
  QR_CODE,
  UNKNOWN,
  UPC_A,
  UPC_E,
  kMinValue = 0,
  kMaxValue = 13,
};

 std::ostream& operator<<(std::ostream& os, BarcodeFormat value);
inline bool IsKnownEnumValue(BarcodeFormat value) {
  return internal::BarcodeFormat_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class BarcodeDetectionProviderInterfaceBase {};

using BarcodeDetectionProviderPtrDataView =
    mojo::InterfacePtrDataView<BarcodeDetectionProviderInterfaceBase>;
using BarcodeDetectionProviderRequestDataView =
    mojo::InterfaceRequestDataView<BarcodeDetectionProviderInterfaceBase>;
using BarcodeDetectionProviderAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<BarcodeDetectionProviderInterfaceBase>;
using BarcodeDetectionProviderAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<BarcodeDetectionProviderInterfaceBase>;
class BarcodeDetectorOptionsDataView {
 public:
  BarcodeDetectorOptionsDataView() {}

  BarcodeDetectorOptionsDataView(
      internal::BarcodeDetectorOptions_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetFormatsDataView(
      mojo::ArrayDataView<BarcodeFormat>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFormats(UserType* output) {
    auto* pointer = data_->formats.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::shape_detection::mojom::BarcodeFormat>>(
        pointer, output, context_);
  }
 private:
  internal::BarcodeDetectorOptions_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class BarcodeDetectionProvider_CreateBarcodeDetection_ParamsDataView {
 public:
  BarcodeDetectionProvider_CreateBarcodeDetection_ParamsDataView() {}

  BarcodeDetectionProvider_CreateBarcodeDetection_ParamsDataView(
      internal::BarcodeDetectionProvider_CreateBarcodeDetection_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeRequest() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::shape_detection::mojom::BarcodeDetectionRequestDataView>(
            &data_->request, &result, context_);
    DCHECK(ret);
    return result;
  }
  inline void GetOptionsDataView(
      BarcodeDetectorOptionsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOptions(UserType* output) {
    auto* pointer = data_->options.Get();
    return mojo::internal::Deserialize<::shape_detection::mojom::BarcodeDetectorOptionsDataView>(
        pointer, output, context_);
  }
 private:
  internal::BarcodeDetectionProvider_CreateBarcodeDetection_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class BarcodeDetectionProvider_EnumerateSupportedFormats_ParamsDataView {
 public:
  BarcodeDetectionProvider_EnumerateSupportedFormats_ParamsDataView() {}

  BarcodeDetectionProvider_EnumerateSupportedFormats_ParamsDataView(
      internal::BarcodeDetectionProvider_EnumerateSupportedFormats_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::BarcodeDetectionProvider_EnumerateSupportedFormats_Params_Data* data_ = nullptr;
};

class BarcodeDetectionProvider_EnumerateSupportedFormats_ResponseParamsDataView {
 public:
  BarcodeDetectionProvider_EnumerateSupportedFormats_ResponseParamsDataView() {}

  BarcodeDetectionProvider_EnumerateSupportedFormats_ResponseParamsDataView(
      internal::BarcodeDetectionProvider_EnumerateSupportedFormats_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetSupportedFormatsDataView(
      mojo::ArrayDataView<BarcodeFormat>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSupportedFormats(UserType* output) {
    auto* pointer = data_->supported_formats.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::shape_detection::mojom::BarcodeFormat>>(
        pointer, output, context_);
  }
 private:
  internal::BarcodeDetectionProvider_EnumerateSupportedFormats_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace shape_detection

namespace std {

template <>
struct hash<::shape_detection::mojom::BarcodeFormat>
    : public mojo::internal::EnumHashImpl<::shape_detection::mojom::BarcodeFormat> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::shape_detection::mojom::BarcodeFormat, ::shape_detection::mojom::BarcodeFormat> {
  static ::shape_detection::mojom::BarcodeFormat ToMojom(::shape_detection::mojom::BarcodeFormat input) { return input; }
  static bool FromMojom(::shape_detection::mojom::BarcodeFormat input, ::shape_detection::mojom::BarcodeFormat* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::shape_detection::mojom::BarcodeFormat, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::shape_detection::mojom::BarcodeFormat, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::shape_detection::mojom::BarcodeFormat>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::shape_detection::mojom::BarcodeDetectorOptionsDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::shape_detection::mojom::BarcodeDetectorOptionsDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::shape_detection::mojom::internal::BarcodeDetectorOptions_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    decltype(CallWithContext(Traits::formats, input, custom_context)) in_formats = CallWithContext(Traits::formats, input, custom_context);
    typename decltype((*output)->formats)::BaseType::BufferWriter
        formats_writer;
    const mojo::internal::ContainerValidateParams formats_validate_params(
        0, ::shape_detection::mojom::internal::BarcodeFormat_Data::Validate);
    mojo::internal::Serialize<mojo::ArrayDataView<::shape_detection::mojom::BarcodeFormat>>(
        in_formats, buffer, &formats_writer, &formats_validate_params,
        context);
    (*output)->formats.Set(
        formats_writer.is_null() ? nullptr : formats_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->formats.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null formats in BarcodeDetectorOptions struct");
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::shape_detection::mojom::internal::BarcodeDetectorOptions_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::shape_detection::mojom::BarcodeDetectorOptionsDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace shape_detection {
namespace mojom {

inline void BarcodeDetectorOptionsDataView::GetFormatsDataView(
    mojo::ArrayDataView<BarcodeFormat>* output) {
  auto pointer = data_->formats.Get();
  *output = mojo::ArrayDataView<BarcodeFormat>(pointer, context_);
}


inline void BarcodeDetectionProvider_CreateBarcodeDetection_ParamsDataView::GetOptionsDataView(
    BarcodeDetectorOptionsDataView* output) {
  auto pointer = data_->options.Get();
  *output = BarcodeDetectorOptionsDataView(pointer, context_);
}




inline void BarcodeDetectionProvider_EnumerateSupportedFormats_ResponseParamsDataView::GetSupportedFormatsDataView(
    mojo::ArrayDataView<BarcodeFormat>* output) {
  auto pointer = data_->supported_formats.Get();
  *output = mojo::ArrayDataView<BarcodeFormat>(pointer, context_);
}



}  // namespace mojom
}  // namespace shape_detection

#endif  // SERVICES_SHAPE_DETECTION_PUBLIC_MOJOM_BARCODEDETECTION_PROVIDER_MOJOM_SHARED_H_