// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_TRACING_PUBLIC_MOJOM_TRACING_MOJOM_SHARED_H_
#define SERVICES_TRACING_PUBLIC_MOJOM_TRACING_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "services/tracing/public/mojom/tracing.mojom-shared-internal.h"
#include "mojo/public/mojom/base/process_id.mojom-shared.h"
#include "mojo/public/mojom/base/time.mojom-shared.h"
#include "mojo/public/mojom/base/values.mojom-shared.h"
#include "services/tracing/public/mojom/perfetto_service.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "base/component_export.h"


namespace tracing {
namespace mojom {
class ConnectToTracingRequestDataView;



}  // namespace mojom
}  // namespace tracing

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::tracing::mojom::ConnectToTracingRequestDataView> {
  using Data = ::tracing::mojom::internal::ConnectToTracingRequest_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace tracing {
namespace mojom {

enum class TraceDataType : int32_t {
  ARRAY,
  OBJECT,
  STRING,
  kMinValue = 0,
  kMaxValue = 2,
};

COMPONENT_EXPORT(TRACING_MOJOM_SHARED) std::ostream& operator<<(std::ostream& os, TraceDataType value);
inline bool IsKnownEnumValue(TraceDataType value) {
  return internal::TraceDataType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class TracedProcessInterfaceBase {};

using TracedProcessPtrDataView =
    mojo::InterfacePtrDataView<TracedProcessInterfaceBase>;
using TracedProcessRequestDataView =
    mojo::InterfaceRequestDataView<TracedProcessInterfaceBase>;
using TracedProcessAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<TracedProcessInterfaceBase>;
using TracedProcessAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<TracedProcessInterfaceBase>;
class AgentRegistryInterfaceBase {};

using AgentRegistryPtrDataView =
    mojo::InterfacePtrDataView<AgentRegistryInterfaceBase>;
using AgentRegistryRequestDataView =
    mojo::InterfaceRequestDataView<AgentRegistryInterfaceBase>;
using AgentRegistryAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<AgentRegistryInterfaceBase>;
using AgentRegistryAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<AgentRegistryInterfaceBase>;
class AgentInterfaceBase {};

using AgentPtrDataView =
    mojo::InterfacePtrDataView<AgentInterfaceBase>;
using AgentRequestDataView =
    mojo::InterfaceRequestDataView<AgentInterfaceBase>;
using AgentAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<AgentInterfaceBase>;
using AgentAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<AgentInterfaceBase>;
class RecorderInterfaceBase {};

using RecorderPtrDataView =
    mojo::InterfacePtrDataView<RecorderInterfaceBase>;
using RecorderRequestDataView =
    mojo::InterfaceRequestDataView<RecorderInterfaceBase>;
using RecorderAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<RecorderInterfaceBase>;
using RecorderAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<RecorderInterfaceBase>;
class CoordinatorInterfaceBase {};

using CoordinatorPtrDataView =
    mojo::InterfacePtrDataView<CoordinatorInterfaceBase>;
using CoordinatorRequestDataView =
    mojo::InterfaceRequestDataView<CoordinatorInterfaceBase>;
using CoordinatorAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<CoordinatorInterfaceBase>;
using CoordinatorAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<CoordinatorInterfaceBase>;
class ConnectToTracingRequestDataView {
 public:
  ConnectToTracingRequestDataView() {}

  ConnectToTracingRequestDataView(
      internal::ConnectToTracingRequest_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakePerfettoService() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::tracing::mojom::PerfettoServicePtrDataView>(
            &data_->perfetto_service, &result, context_);
    DCHECK(ret);
    return result;
  }
  template <typename UserType>
  UserType TakeAgentRegistry() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::tracing::mojom::AgentRegistryPtrDataView>(
            &data_->agent_registry, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::ConnectToTracingRequest_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class TracedProcess_ConnectToTracingService_ParamsDataView {
 public:
  TracedProcess_ConnectToTracingService_ParamsDataView() {}

  TracedProcess_ConnectToTracingService_ParamsDataView(
      internal::TracedProcess_ConnectToTracingService_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetRequestDataView(
      ConnectToTracingRequestDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRequest(UserType* output) {
    auto* pointer = data_->request.Get();
    return mojo::internal::Deserialize<::tracing::mojom::ConnectToTracingRequestDataView>(
        pointer, output, context_);
  }
 private:
  internal::TracedProcess_ConnectToTracingService_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AgentRegistry_RegisterAgent_ParamsDataView {
 public:
  AgentRegistry_RegisterAgent_ParamsDataView() {}

  AgentRegistry_RegisterAgent_ParamsDataView(
      internal::AgentRegistry_RegisterAgent_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeAgent() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::tracing::mojom::AgentPtrDataView>(
            &data_->agent, &result, context_);
    DCHECK(ret);
    return result;
  }
  inline void GetLabelDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadLabel(UserType* output) {
    auto* pointer = data_->label.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::tracing::mojom::TraceDataType>(
        data_value, output);
  }

  TraceDataType type() const {
    return static_cast<TraceDataType>(data_->type);
  }
  inline void GetPidDataView(
      ::mojo_base::mojom::ProcessIdDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPid(UserType* output) {
    auto* pointer = data_->pid.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::ProcessIdDataView>(
        pointer, output, context_);
  }
 private:
  internal::AgentRegistry_RegisterAgent_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class Agent_StartTracing_ParamsDataView {
 public:
  Agent_StartTracing_ParamsDataView() {}

  Agent_StartTracing_ParamsDataView(
      internal::Agent_StartTracing_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetConfigDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadConfig(UserType* output) {
    auto* pointer = data_->config.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetCoordinatorTimeDataView(
      ::mojo_base::mojom::TimeTicksDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCoordinatorTime(UserType* output) {
    auto* pointer = data_->coordinator_time.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::TimeTicksDataView>(
        pointer, output, context_);
  }
 private:
  internal::Agent_StartTracing_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class Agent_StopAndFlush_ParamsDataView {
 public:
  Agent_StopAndFlush_ParamsDataView() {}

  Agent_StopAndFlush_ParamsDataView(
      internal::Agent_StopAndFlush_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeRecorder() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::tracing::mojom::RecorderPtrDataView>(
            &data_->recorder, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::Agent_StopAndFlush_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class Agent_RequestBufferStatus_ParamsDataView {
 public:
  Agent_RequestBufferStatus_ParamsDataView() {}

  Agent_RequestBufferStatus_ParamsDataView(
      internal::Agent_RequestBufferStatus_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::Agent_RequestBufferStatus_Params_Data* data_ = nullptr;
};

class Agent_RequestBufferStatus_ResponseParamsDataView {
 public:
  Agent_RequestBufferStatus_ResponseParamsDataView() {}

  Agent_RequestBufferStatus_ResponseParamsDataView(
      internal::Agent_RequestBufferStatus_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  uint32_t capacity() const {
    return data_->capacity;
  }
  uint32_t count() const {
    return data_->count;
  }
 private:
  internal::Agent_RequestBufferStatus_ResponseParams_Data* data_ = nullptr;
};

class Recorder_AddChunk_ParamsDataView {
 public:
  Recorder_AddChunk_ParamsDataView() {}

  Recorder_AddChunk_ParamsDataView(
      internal::Recorder_AddChunk_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetChunkDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadChunk(UserType* output) {
    auto* pointer = data_->chunk.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::Recorder_AddChunk_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class Recorder_AddMetadata_ParamsDataView {
 public:
  Recorder_AddMetadata_ParamsDataView() {}

  Recorder_AddMetadata_ParamsDataView(
      internal::Recorder_AddMetadata_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetMetadataDataView(
      ::mojo_base::mojom::DictionaryValueDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadMetadata(UserType* output) {
    auto* pointer = data_->metadata.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::DictionaryValueDataView>(
        pointer, output, context_);
  }
 private:
  internal::Recorder_AddMetadata_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class Coordinator_StartTracing_ParamsDataView {
 public:
  Coordinator_StartTracing_ParamsDataView() {}

  Coordinator_StartTracing_ParamsDataView(
      internal::Coordinator_StartTracing_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetConfigDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadConfig(UserType* output) {
    auto* pointer = data_->config.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::Coordinator_StartTracing_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class Coordinator_StopAndFlush_ParamsDataView {
 public:
  Coordinator_StopAndFlush_ParamsDataView() {}

  Coordinator_StopAndFlush_ParamsDataView(
      internal::Coordinator_StopAndFlush_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  mojo::ScopedDataPipeProducerHandle TakeStream() {
    mojo::ScopedDataPipeProducerHandle result;
    bool ret =
        mojo::internal::Deserialize<mojo::ScopedDataPipeProducerHandle>(
            &data_->stream, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::Coordinator_StopAndFlush_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class Coordinator_StopAndFlush_ResponseParamsDataView {
 public:
  Coordinator_StopAndFlush_ResponseParamsDataView() {}

  Coordinator_StopAndFlush_ResponseParamsDataView(
      internal::Coordinator_StopAndFlush_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetMetadataDataView(
      ::mojo_base::mojom::DictionaryValueDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadMetadata(UserType* output) {
    auto* pointer = data_->metadata.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::DictionaryValueDataView>(
        pointer, output, context_);
  }
 private:
  internal::Coordinator_StopAndFlush_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class Coordinator_StopAndFlushAgent_ParamsDataView {
 public:
  Coordinator_StopAndFlushAgent_ParamsDataView() {}

  Coordinator_StopAndFlushAgent_ParamsDataView(
      internal::Coordinator_StopAndFlushAgent_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  mojo::ScopedDataPipeProducerHandle TakeStream() {
    mojo::ScopedDataPipeProducerHandle result;
    bool ret =
        mojo::internal::Deserialize<mojo::ScopedDataPipeProducerHandle>(
            &data_->stream, &result, context_);
    DCHECK(ret);
    return result;
  }
  inline void GetAgentLabelDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAgentLabel(UserType* output) {
    auto* pointer = data_->agent_label.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::Coordinator_StopAndFlushAgent_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class Coordinator_StopAndFlushAgent_ResponseParamsDataView {
 public:
  Coordinator_StopAndFlushAgent_ResponseParamsDataView() {}

  Coordinator_StopAndFlushAgent_ResponseParamsDataView(
      internal::Coordinator_StopAndFlushAgent_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetMetadataDataView(
      ::mojo_base::mojom::DictionaryValueDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadMetadata(UserType* output) {
    auto* pointer = data_->metadata.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::DictionaryValueDataView>(
        pointer, output, context_);
  }
 private:
  internal::Coordinator_StopAndFlushAgent_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class Coordinator_IsTracing_ParamsDataView {
 public:
  Coordinator_IsTracing_ParamsDataView() {}

  Coordinator_IsTracing_ParamsDataView(
      internal::Coordinator_IsTracing_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::Coordinator_IsTracing_Params_Data* data_ = nullptr;
};

class Coordinator_IsTracing_ResponseParamsDataView {
 public:
  Coordinator_IsTracing_ResponseParamsDataView() {}

  Coordinator_IsTracing_ResponseParamsDataView(
      internal::Coordinator_IsTracing_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool is_tracing() const {
    return data_->is_tracing;
  }
 private:
  internal::Coordinator_IsTracing_ResponseParams_Data* data_ = nullptr;
};

class Coordinator_RequestBufferUsage_ParamsDataView {
 public:
  Coordinator_RequestBufferUsage_ParamsDataView() {}

  Coordinator_RequestBufferUsage_ParamsDataView(
      internal::Coordinator_RequestBufferUsage_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::Coordinator_RequestBufferUsage_Params_Data* data_ = nullptr;
};

class Coordinator_RequestBufferUsage_ResponseParamsDataView {
 public:
  Coordinator_RequestBufferUsage_ResponseParamsDataView() {}

  Coordinator_RequestBufferUsage_ResponseParamsDataView(
      internal::Coordinator_RequestBufferUsage_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool success() const {
    return data_->success;
  }
  float percent_full() const {
    return data_->percent_full;
  }
  uint32_t approximate_count() const {
    return data_->approximate_count;
  }
 private:
  internal::Coordinator_RequestBufferUsage_ResponseParams_Data* data_ = nullptr;
};



}  // namespace mojom
}  // namespace tracing

namespace std {

template <>
struct hash<::tracing::mojom::TraceDataType>
    : public mojo::internal::EnumHashImpl<::tracing::mojom::TraceDataType> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::tracing::mojom::TraceDataType, ::tracing::mojom::TraceDataType> {
  static ::tracing::mojom::TraceDataType ToMojom(::tracing::mojom::TraceDataType input) { return input; }
  static bool FromMojom(::tracing::mojom::TraceDataType input, ::tracing::mojom::TraceDataType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::tracing::mojom::TraceDataType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::tracing::mojom::TraceDataType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::tracing::mojom::TraceDataType>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::tracing::mojom::ConnectToTracingRequestDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::tracing::mojom::ConnectToTracingRequestDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::tracing::mojom::internal::ConnectToTracingRequest_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    decltype(CallWithContext(Traits::perfetto_service, input, custom_context)) in_perfetto_service = CallWithContext(Traits::perfetto_service, input, custom_context);
    mojo::internal::Serialize<::tracing::mojom::PerfettoServicePtrDataView>(
        in_perfetto_service, &(*output)->perfetto_service, context);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        !mojo::internal::IsHandleOrInterfaceValid((*output)->perfetto_service),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
        "invalid perfetto_service in ConnectToTracingRequest struct");
    decltype(CallWithContext(Traits::agent_registry, input, custom_context)) in_agent_registry = CallWithContext(Traits::agent_registry, input, custom_context);
    mojo::internal::Serialize<::tracing::mojom::AgentRegistryPtrDataView>(
        in_agent_registry, &(*output)->agent_registry, context);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        !mojo::internal::IsHandleOrInterfaceValid((*output)->agent_registry),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
        "invalid agent_registry in ConnectToTracingRequest struct");
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::tracing::mojom::internal::ConnectToTracingRequest_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::tracing::mojom::ConnectToTracingRequestDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace tracing {
namespace mojom {



inline void TracedProcess_ConnectToTracingService_ParamsDataView::GetRequestDataView(
    ConnectToTracingRequestDataView* output) {
  auto pointer = data_->request.Get();
  *output = ConnectToTracingRequestDataView(pointer, context_);
}


inline void AgentRegistry_RegisterAgent_ParamsDataView::GetLabelDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->label.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void AgentRegistry_RegisterAgent_ParamsDataView::GetPidDataView(
    ::mojo_base::mojom::ProcessIdDataView* output) {
  auto pointer = data_->pid.Get();
  *output = ::mojo_base::mojom::ProcessIdDataView(pointer, context_);
}


inline void Agent_StartTracing_ParamsDataView::GetConfigDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->config.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void Agent_StartTracing_ParamsDataView::GetCoordinatorTimeDataView(
    ::mojo_base::mojom::TimeTicksDataView* output) {
  auto pointer = data_->coordinator_time.Get();
  *output = ::mojo_base::mojom::TimeTicksDataView(pointer, context_);
}








inline void Recorder_AddChunk_ParamsDataView::GetChunkDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->chunk.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void Recorder_AddMetadata_ParamsDataView::GetMetadataDataView(
    ::mojo_base::mojom::DictionaryValueDataView* output) {
  auto pointer = data_->metadata.Get();
  *output = ::mojo_base::mojom::DictionaryValueDataView(pointer, context_);
}


inline void Coordinator_StartTracing_ParamsDataView::GetConfigDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->config.Get();
  *output = mojo::StringDataView(pointer, context_);
}




inline void Coordinator_StopAndFlush_ResponseParamsDataView::GetMetadataDataView(
    ::mojo_base::mojom::DictionaryValueDataView* output) {
  auto pointer = data_->metadata.Get();
  *output = ::mojo_base::mojom::DictionaryValueDataView(pointer, context_);
}


inline void Coordinator_StopAndFlushAgent_ParamsDataView::GetAgentLabelDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->agent_label.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void Coordinator_StopAndFlushAgent_ResponseParamsDataView::GetMetadataDataView(
    ::mojo_base::mojom::DictionaryValueDataView* output) {
  auto pointer = data_->metadata.Get();
  *output = ::mojo_base::mojom::DictionaryValueDataView(pointer, context_);
}











}  // namespace mojom
}  // namespace tracing

#endif  // SERVICES_TRACING_PUBLIC_MOJOM_TRACING_MOJOM_SHARED_H_