// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_TRACING_PUBLIC_MOJOM_TRACING_MOJOM_H_
#define SERVICES_TRACING_PUBLIC_MOJOM_TRACING_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/tracing/public/mojom/tracing.mojom-shared.h"
#include "mojo/public/mojom/base/process_id.mojom.h"
#include "mojo/public/mojom/base/time.mojom.h"
#include "mojo/public/mojom/base/values.mojom.h"
#include "services/tracing/public/mojom/perfetto_service.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "base/component_export.h"


namespace tracing {
namespace mojom {
class TracedProcess;
using TracedProcessPtr = mojo::InterfacePtr<TracedProcess>;
using TracedProcessPtrInfo = mojo::InterfacePtrInfo<TracedProcess>;
using ThreadSafeTracedProcessPtr =
    mojo::ThreadSafeInterfacePtr<TracedProcess>;
using TracedProcessRequest = mojo::InterfaceRequest<TracedProcess>;
using TracedProcessAssociatedPtr =
    mojo::AssociatedInterfacePtr<TracedProcess>;
using ThreadSafeTracedProcessAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<TracedProcess>;
using TracedProcessAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<TracedProcess>;
using TracedProcessAssociatedRequest =
    mojo::AssociatedInterfaceRequest<TracedProcess>;

class AgentRegistry;
using AgentRegistryPtr = mojo::InterfacePtr<AgentRegistry>;
using AgentRegistryPtrInfo = mojo::InterfacePtrInfo<AgentRegistry>;
using ThreadSafeAgentRegistryPtr =
    mojo::ThreadSafeInterfacePtr<AgentRegistry>;
using AgentRegistryRequest = mojo::InterfaceRequest<AgentRegistry>;
using AgentRegistryAssociatedPtr =
    mojo::AssociatedInterfacePtr<AgentRegistry>;
using ThreadSafeAgentRegistryAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<AgentRegistry>;
using AgentRegistryAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<AgentRegistry>;
using AgentRegistryAssociatedRequest =
    mojo::AssociatedInterfaceRequest<AgentRegistry>;

class Agent;
using AgentPtr = mojo::InterfacePtr<Agent>;
using AgentPtrInfo = mojo::InterfacePtrInfo<Agent>;
using ThreadSafeAgentPtr =
    mojo::ThreadSafeInterfacePtr<Agent>;
using AgentRequest = mojo::InterfaceRequest<Agent>;
using AgentAssociatedPtr =
    mojo::AssociatedInterfacePtr<Agent>;
using ThreadSafeAgentAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<Agent>;
using AgentAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<Agent>;
using AgentAssociatedRequest =
    mojo::AssociatedInterfaceRequest<Agent>;

class Recorder;
using RecorderPtr = mojo::InterfacePtr<Recorder>;
using RecorderPtrInfo = mojo::InterfacePtrInfo<Recorder>;
using ThreadSafeRecorderPtr =
    mojo::ThreadSafeInterfacePtr<Recorder>;
using RecorderRequest = mojo::InterfaceRequest<Recorder>;
using RecorderAssociatedPtr =
    mojo::AssociatedInterfacePtr<Recorder>;
using ThreadSafeRecorderAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<Recorder>;
using RecorderAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<Recorder>;
using RecorderAssociatedRequest =
    mojo::AssociatedInterfaceRequest<Recorder>;

class Coordinator;
using CoordinatorPtr = mojo::InterfacePtr<Coordinator>;
using CoordinatorPtrInfo = mojo::InterfacePtrInfo<Coordinator>;
using ThreadSafeCoordinatorPtr =
    mojo::ThreadSafeInterfacePtr<Coordinator>;
using CoordinatorRequest = mojo::InterfaceRequest<Coordinator>;
using CoordinatorAssociatedPtr =
    mojo::AssociatedInterfacePtr<Coordinator>;
using ThreadSafeCoordinatorAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<Coordinator>;
using CoordinatorAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<Coordinator>;
using CoordinatorAssociatedRequest =
    mojo::AssociatedInterfaceRequest<Coordinator>;

class ConnectToTracingRequest;
using ConnectToTracingRequestPtr = mojo::StructPtr<ConnectToTracingRequest>;


class TracedProcessProxy;

template <typename ImplRefTraits>
class TracedProcessStub;

class TracedProcessRequestValidator;

class COMPONENT_EXPORT(TRACING_MOJOM) TracedProcess
    : public TracedProcessInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = TracedProcessProxy;

  template <typename ImplRefTraits>
  using Stub_ = TracedProcessStub<ImplRefTraits>;

  using RequestValidator_ = TracedProcessRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kConnectToTracingServiceMinVersion = 0,
  };
  virtual ~TracedProcess() {}

  virtual void ConnectToTracingService(ConnectToTracingRequestPtr request) = 0;
};

class AgentRegistryProxy;

template <typename ImplRefTraits>
class AgentRegistryStub;

class AgentRegistryRequestValidator;

class COMPONENT_EXPORT(TRACING_MOJOM) AgentRegistry
    : public AgentRegistryInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = AgentRegistryProxy;

  template <typename ImplRefTraits>
  using Stub_ = AgentRegistryStub<ImplRefTraits>;

  using RequestValidator_ = AgentRegistryRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kRegisterAgentMinVersion = 0,
  };
  virtual ~AgentRegistry() {}

  virtual void RegisterAgent(AgentPtr agent, const std::string& label, TraceDataType type, base::ProcessId pid) = 0;
};

class AgentProxy;

template <typename ImplRefTraits>
class AgentStub;

class AgentRequestValidator;
class AgentResponseValidator;

class COMPONENT_EXPORT(TRACING_MOJOM) Agent
    : public AgentInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = AgentProxy;

  template <typename ImplRefTraits>
  using Stub_ = AgentStub<ImplRefTraits>;

  using RequestValidator_ = AgentRequestValidator;
  using ResponseValidator_ = AgentResponseValidator;
  enum MethodMinVersions : uint32_t {
    kStartTracingMinVersion = 0,
    kStopAndFlushMinVersion = 0,
    kRequestBufferStatusMinVersion = 0,
  };
  virtual ~Agent() {}

  virtual void StartTracing(const std::string& config, base::TimeTicks coordinator_time) = 0;

  virtual void StopAndFlush(RecorderPtr recorder) = 0;


  using RequestBufferStatusCallback = base::OnceCallback<void(uint32_t, uint32_t)>;
  virtual void RequestBufferStatus(RequestBufferStatusCallback callback) = 0;
};

class RecorderProxy;

template <typename ImplRefTraits>
class RecorderStub;

class RecorderRequestValidator;

class COMPONENT_EXPORT(TRACING_MOJOM) Recorder
    : public RecorderInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = RecorderProxy;

  template <typename ImplRefTraits>
  using Stub_ = RecorderStub<ImplRefTraits>;

  using RequestValidator_ = RecorderRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kAddChunkMinVersion = 0,
    kAddMetadataMinVersion = 0,
  };
  virtual ~Recorder() {}

  virtual void AddChunk(const std::string& chunk) = 0;

  virtual void AddMetadata(base::Value metadata) = 0;
};

class CoordinatorProxy;

template <typename ImplRefTraits>
class CoordinatorStub;

class CoordinatorRequestValidator;
class CoordinatorResponseValidator;

class COMPONENT_EXPORT(TRACING_MOJOM) Coordinator
    : public CoordinatorInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = CoordinatorProxy;

  template <typename ImplRefTraits>
  using Stub_ = CoordinatorStub<ImplRefTraits>;

  using RequestValidator_ = CoordinatorRequestValidator;
  using ResponseValidator_ = CoordinatorResponseValidator;
  enum MethodMinVersions : uint32_t {
    kStartTracingMinVersion = 0,
    kStopAndFlushMinVersion = 0,
    kStopAndFlushAgentMinVersion = 0,
    kIsTracingMinVersion = 0,
    kRequestBufferUsageMinVersion = 0,
  };
  virtual ~Coordinator() {}

  virtual void StartTracing(const std::string& config) = 0;


  using StopAndFlushCallback = base::OnceCallback<void(base::Value)>;
  virtual void StopAndFlush(mojo::ScopedDataPipeProducerHandle stream, StopAndFlushCallback callback) = 0;


  using StopAndFlushAgentCallback = base::OnceCallback<void(base::Value)>;
  virtual void StopAndFlushAgent(mojo::ScopedDataPipeProducerHandle stream, const std::string& agent_label, StopAndFlushAgentCallback callback) = 0;


  using IsTracingCallback = base::OnceCallback<void(bool)>;
  virtual void IsTracing(IsTracingCallback callback) = 0;


  using RequestBufferUsageCallback = base::OnceCallback<void(bool, float, uint32_t)>;
  virtual void RequestBufferUsage(RequestBufferUsageCallback callback) = 0;
};

class COMPONENT_EXPORT(TRACING_MOJOM) TracedProcessProxy
    : public TracedProcess {
 public:
  explicit TracedProcessProxy(mojo::MessageReceiverWithResponder* receiver);
  void ConnectToTracingService(ConnectToTracingRequestPtr request) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class COMPONENT_EXPORT(TRACING_MOJOM) AgentRegistryProxy
    : public AgentRegistry {
 public:
  explicit AgentRegistryProxy(mojo::MessageReceiverWithResponder* receiver);
  void RegisterAgent(AgentPtr agent, const std::string& label, TraceDataType type, base::ProcessId pid) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class COMPONENT_EXPORT(TRACING_MOJOM) AgentProxy
    : public Agent {
 public:
  explicit AgentProxy(mojo::MessageReceiverWithResponder* receiver);
  void StartTracing(const std::string& config, base::TimeTicks coordinator_time) final;
  void StopAndFlush(RecorderPtr recorder) final;
  void RequestBufferStatus(RequestBufferStatusCallback callback) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class COMPONENT_EXPORT(TRACING_MOJOM) RecorderProxy
    : public Recorder {
 public:
  explicit RecorderProxy(mojo::MessageReceiverWithResponder* receiver);
  void AddChunk(const std::string& chunk) final;
  void AddMetadata(base::Value metadata) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class COMPONENT_EXPORT(TRACING_MOJOM) CoordinatorProxy
    : public Coordinator {
 public:
  explicit CoordinatorProxy(mojo::MessageReceiverWithResponder* receiver);
  void StartTracing(const std::string& config) final;
  void StopAndFlush(mojo::ScopedDataPipeProducerHandle stream, StopAndFlushCallback callback) final;
  void StopAndFlushAgent(mojo::ScopedDataPipeProducerHandle stream, const std::string& agent_label, StopAndFlushAgentCallback callback) final;
  void IsTracing(IsTracingCallback callback) final;
  void RequestBufferUsage(RequestBufferUsageCallback callback) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class COMPONENT_EXPORT(TRACING_MOJOM) TracedProcessStubDispatch {
 public:
  static bool Accept(TracedProcess* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      TracedProcess* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<TracedProcess>>
class TracedProcessStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  TracedProcessStub() {}
  ~TracedProcessStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return TracedProcessStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return TracedProcessStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class COMPONENT_EXPORT(TRACING_MOJOM) AgentRegistryStubDispatch {
 public:
  static bool Accept(AgentRegistry* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      AgentRegistry* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<AgentRegistry>>
class AgentRegistryStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  AgentRegistryStub() {}
  ~AgentRegistryStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AgentRegistryStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AgentRegistryStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class COMPONENT_EXPORT(TRACING_MOJOM) AgentStubDispatch {
 public:
  static bool Accept(Agent* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      Agent* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<Agent>>
class AgentStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  AgentStub() {}
  ~AgentStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AgentStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AgentStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class COMPONENT_EXPORT(TRACING_MOJOM) RecorderStubDispatch {
 public:
  static bool Accept(Recorder* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      Recorder* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<Recorder>>
class RecorderStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  RecorderStub() {}
  ~RecorderStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return RecorderStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return RecorderStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class COMPONENT_EXPORT(TRACING_MOJOM) CoordinatorStubDispatch {
 public:
  static bool Accept(Coordinator* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      Coordinator* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<Coordinator>>
class CoordinatorStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  CoordinatorStub() {}
  ~CoordinatorStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CoordinatorStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CoordinatorStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class COMPONENT_EXPORT(TRACING_MOJOM) TracedProcessRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class COMPONENT_EXPORT(TRACING_MOJOM) AgentRegistryRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class COMPONENT_EXPORT(TRACING_MOJOM) AgentRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class COMPONENT_EXPORT(TRACING_MOJOM) RecorderRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class COMPONENT_EXPORT(TRACING_MOJOM) CoordinatorRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class COMPONENT_EXPORT(TRACING_MOJOM) AgentResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class COMPONENT_EXPORT(TRACING_MOJOM) CoordinatorResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





class COMPONENT_EXPORT(TRACING_MOJOM) ConnectToTracingRequest {
 public:
  using DataView = ConnectToTracingRequestDataView;
  using Data_ = internal::ConnectToTracingRequest_Data;

  template <typename... Args>
  static ConnectToTracingRequestPtr New(Args&&... args) {
    return ConnectToTracingRequestPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static ConnectToTracingRequestPtr From(const U& u) {
    return mojo::TypeConverter<ConnectToTracingRequestPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, ConnectToTracingRequest>::Convert(*this);
  }


  ConnectToTracingRequest();

  ConnectToTracingRequest(
      ::tracing::mojom::PerfettoServicePtrInfo perfetto_service,
      AgentRegistryPtrInfo agent_registry);

  ~ConnectToTracingRequest();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = ConnectToTracingRequestPtr>
  ConnectToTracingRequestPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, ConnectToTracingRequest>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        ConnectToTracingRequest::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        ConnectToTracingRequest::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::ConnectToTracingRequest_UnserializedMessageContext<
            UserType, ConnectToTracingRequest::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<ConnectToTracingRequest::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return ConnectToTracingRequest::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::ConnectToTracingRequest_UnserializedMessageContext<
            UserType, ConnectToTracingRequest::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<ConnectToTracingRequest::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  ::tracing::mojom::PerfettoServicePtrInfo perfetto_service;
  AgentRegistryPtrInfo agent_registry;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(ConnectToTracingRequest);
};

template <typename StructPtrType>
ConnectToTracingRequestPtr ConnectToTracingRequest::Clone() const {
  return New(
      mojo::Clone(perfetto_service),
      mojo::Clone(agent_registry)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, ConnectToTracingRequest>::value>::type*>
bool ConnectToTracingRequest::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->perfetto_service, other_struct.perfetto_service))
    return false;
  if (!mojo::Equals(this->agent_registry, other_struct.agent_registry))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace tracing

namespace mojo {


template <>
struct COMPONENT_EXPORT(TRACING_MOJOM) StructTraits<::tracing::mojom::ConnectToTracingRequest::DataView,
                                         ::tracing::mojom::ConnectToTracingRequestPtr> {
  static bool IsNull(const ::tracing::mojom::ConnectToTracingRequestPtr& input) { return !input; }
  static void SetToNull(::tracing::mojom::ConnectToTracingRequestPtr* output) { output->reset(); }

  static  decltype(::tracing::mojom::ConnectToTracingRequest::perfetto_service)& perfetto_service(
       ::tracing::mojom::ConnectToTracingRequestPtr& input) {
    return input->perfetto_service;
  }

  static  decltype(::tracing::mojom::ConnectToTracingRequest::agent_registry)& agent_registry(
       ::tracing::mojom::ConnectToTracingRequestPtr& input) {
    return input->agent_registry;
  }

  static bool Read(::tracing::mojom::ConnectToTracingRequest::DataView input, ::tracing::mojom::ConnectToTracingRequestPtr* output);
};

}  // namespace mojo

#endif  // SERVICES_TRACING_PUBLIC_MOJOM_TRACING_MOJOM_H_