// Copyright 2019 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_PUBLIC_MOJOM_APPCACHE_APPCACHE_MOJOM_BLINK_TEST_UTILS_H_
#define THIRD_PARTY_BLINK_PUBLIC_MOJOM_APPCACHE_APPCACHE_MOJOM_BLINK_TEST_UTILS_H_

#include "third_party/blink/public/mojom/appcache/appcache.mojom-blink.h"
#include "third_party/blink/renderer/platform/platform_export.h"


namespace blink {
namespace mojom {
namespace blink {


class PLATFORM_EXPORT AppCacheBackendInterceptorForTesting : public AppCacheBackend {
  virtual AppCacheBackend* GetForwardingInterface() = 0;
  void RegisterHost(int32_t host_id) override;
  void UnregisterHost(int32_t host_id) override;
  void SetSpawningHostId(int32_t host_id, int32_t spawning_host_id) override;
  void SelectCache(int32_t host_id, const ::blink::KURL& document_url, int64_t appcache_document_was_loaded_from, const ::blink::KURL& opt_manifest_url) override;
  void SelectCacheForSharedWorker(int32_t host_id, int64_t appcache_id) override;
  void MarkAsForeignEntry(int32_t host_id, const ::blink::KURL& document_url, int64_t appcache_document_was_loaded_from) override;
  void GetStatus(int32_t host_id, GetStatusCallback callback) override;
  void StartUpdate(int32_t host_id, StartUpdateCallback callback) override;
  void SwapCache(int32_t host_id, SwapCacheCallback callback) override;
  void GetResourceList(int32_t host_id, GetResourceListCallback callback) override;
};
class PLATFORM_EXPORT AppCacheBackendAsyncWaiter {
 public:
  explicit AppCacheBackendAsyncWaiter(AppCacheBackend* proxy);
  ~AppCacheBackendAsyncWaiter();
  void GetStatus(
      int32_t host_id, ::blink::mojom::blink::AppCacheStatus* out_status);
  void StartUpdate(
      int32_t host_id, bool* out_success);
  void SwapCache(
      int32_t host_id, bool* out_success);
  void GetResourceList(
      int32_t host_id, WTF::Vector<AppCacheResourceInfoPtr>* out_resources);

 private:
  AppCacheBackend* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(AppCacheBackendAsyncWaiter);
};


class PLATFORM_EXPORT AppCacheFrontendInterceptorForTesting : public AppCacheFrontend {
  virtual AppCacheFrontend* GetForwardingInterface() = 0;
  void CacheSelected(int32_t host_id, ::blink::mojom::blink::AppCacheInfoPtr info) override;
  void StatusChanged(const WTF::Vector<int32_t>& host_ids, ::blink::mojom::blink::AppCacheStatus status) override;
  void EventRaised(const WTF::Vector<int32_t>& host_ids, AppCacheEventID event_id) override;
  void ProgressEventRaised(const WTF::Vector<int32_t>& host_ids, const ::blink::KURL& url, int32_t total, int32_t complete) override;
  void ErrorEventRaised(const WTF::Vector<int32_t>& host_ids, AppCacheErrorDetailsPtr error_details) override;
  void LogMessage(int32_t host_id, int32_t log_level, const WTF::String& message) override;
  void ContentBlocked(int32_t host_id, const ::blink::KURL& manifest_url) override;
  void SetSubresourceFactory(int32_t host_id, ::network::mojom::blink::URLLoaderFactoryPtr url_loader_factory) override;
};
class PLATFORM_EXPORT AppCacheFrontendAsyncWaiter {
 public:
  explicit AppCacheFrontendAsyncWaiter(AppCacheFrontend* proxy);
  ~AppCacheFrontendAsyncWaiter();

 private:
  AppCacheFrontend* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(AppCacheFrontendAsyncWaiter);
};




}  // namespace blink
}  // namespace mojom
}  // namespace blink

#endif  // THIRD_PARTY_BLINK_PUBLIC_MOJOM_APPCACHE_APPCACHE_MOJOM_BLINK_TEST_UTILS_H_