// Copyright 2019 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_PUBLIC_MOJOM_CACHE_STORAGE_CACHE_STORAGE_MOJOM_TEST_UTILS_H_
#define THIRD_PARTY_BLINK_PUBLIC_MOJOM_CACHE_STORAGE_CACHE_STORAGE_MOJOM_TEST_UTILS_H_

#include "third_party/blink/public/mojom/cache_storage/cache_storage.mojom.h"
#include "third_party/blink/public/common/common_export.h"


namespace blink {
namespace mojom {


class BLINK_COMMON_EXPORT CacheStorageCacheInterceptorForTesting : public CacheStorageCache {
  virtual CacheStorageCache* GetForwardingInterface() = 0;
  void Match(::blink::mojom::FetchAPIRequestPtr request, QueryParamsPtr query_params, MatchCallback callback) override;
  void MatchAll(::blink::mojom::FetchAPIRequestPtr request, QueryParamsPtr query_params, MatchAllCallback callback) override;
  void Keys(::blink::mojom::FetchAPIRequestPtr request, QueryParamsPtr query_params, KeysCallback callback) override;
  void Batch(std::vector<BatchOperationPtr> batch_operations, bool fail_on_duplicates, BatchCallback callback) override;
  void SetSideData(const GURL& url, base::Time response_time, const std::vector<uint8_t>& side_data, SetSideDataCallback callback) override;
};
class BLINK_COMMON_EXPORT CacheStorageCacheAsyncWaiter {
 public:
  explicit CacheStorageCacheAsyncWaiter(CacheStorageCache* proxy);
  ~CacheStorageCacheAsyncWaiter();
  void Match(
      ::blink::mojom::FetchAPIRequestPtr request, QueryParamsPtr query_params, MatchResultPtr* out_result);
  void MatchAll(
      ::blink::mojom::FetchAPIRequestPtr request, QueryParamsPtr query_params, MatchAllResultPtr* out_result);
  void Keys(
      ::blink::mojom::FetchAPIRequestPtr request, QueryParamsPtr query_params, CacheKeysResultPtr* out_result);
  void Batch(
      std::vector<BatchOperationPtr> batch_operations, bool fail_on_duplicates, CacheStorageVerboseErrorPtr* out_result);
  void SetSideData(
      const GURL& url, base::Time response_time, const std::vector<uint8_t>& side_data, CacheStorageError* out_result);

 private:
  CacheStorageCache* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(CacheStorageCacheAsyncWaiter);
};


class BLINK_COMMON_EXPORT CacheStorageInterceptorForTesting : public CacheStorage {
  virtual CacheStorage* GetForwardingInterface() = 0;
  void Has(const base::string16& cache_name, HasCallback callback) override;
  void Delete(const base::string16& cache_name, DeleteCallback callback) override;
  void Keys(KeysCallback callback) override;
  void Match(::blink::mojom::FetchAPIRequestPtr request, QueryParamsPtr match_params, MatchCallback callback) override;
  void Open(const base::string16& cache_name, OpenCallback callback) override;
};
class BLINK_COMMON_EXPORT CacheStorageAsyncWaiter {
 public:
  explicit CacheStorageAsyncWaiter(CacheStorage* proxy);
  ~CacheStorageAsyncWaiter();
  void Has(
      const base::string16& cache_name, CacheStorageError* out_result);
  void Delete(
      const base::string16& cache_name, CacheStorageError* out_result);
  void Keys(
      std::vector<base::string16>* out_keys);
  void Match(
      ::blink::mojom::FetchAPIRequestPtr request, QueryParamsPtr match_params, MatchResultPtr* out_result);
  void Open(
      const base::string16& cache_name, OpenResultPtr* out_result);

 private:
  CacheStorage* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(CacheStorageAsyncWaiter);
};




}  // namespace mojom
}  // namespace blink

#endif  // THIRD_PARTY_BLINK_PUBLIC_MOJOM_CACHE_STORAGE_CACHE_STORAGE_MOJOM_TEST_UTILS_H_