// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_PUBLIC_MOJOM_FILESYSTEM_FILE_SYSTEM_MOJOM_SHARED_INTERNAL_H_
#define THIRD_PARTY_BLINK_PUBLIC_MOJOM_FILESYSTEM_FILE_SYSTEM_MOJOM_SHARED_INTERNAL_H_

#include "mojo/public/cpp/bindings/lib/array_internal.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/map_data_internal.h"
#include "mojo/public/cpp/bindings/lib/buffer.h"
#include "components/services/filesystem/public/interfaces/types.mojom-shared-internal.h"
#include "url/mojom/origin.mojom-shared-internal.h"
#include "url/mojom/url.mojom-shared-internal.h"
#include "mojo/public/mojom/base/file_error.mojom-shared-internal.h"
#include "mojo/public/mojom/base/file_path.mojom-shared-internal.h"
#include "mojo/public/mojom/base/file_info.mojom-shared-internal.h"
#include "mojo/public/mojom/base/time.mojom-shared-internal.h"
#include "mojo/public/mojom/base/string16.mojom-shared-internal.h"
#include "third_party/blink/public/mojom/filesystem/file_writer.mojom-shared-internal.h"
#include "mojo/public/cpp/bindings/lib/native_enum_data.h"
#include "mojo/public/interfaces/bindings/native_struct.mojom-shared-internal.h"
#include "base/component_export.h"

namespace mojo {
namespace internal {
class ValidationContext;
}
}
namespace blink {
namespace mojom {
namespace internal {
class FileSystemInfo_Data;
class FileSystemEntry_Data;
class ChooseFileSystemEntryAcceptsOption_Data;

struct FileSystemType_Data {
 public:
  static bool constexpr kIsExtensible = false;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
      case 2:
      case 3:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

struct ChooseFileSystemEntryType_Data {
 public:
  static bool constexpr kIsExtensible = false;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
      case 2:
      case 3:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

#pragma pack(push, 1)
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemInfo_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemInfo_Data));
      new (data()) FileSystemInfo_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemInfo_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemInfo_Data>(index_);
    }
    FileSystemInfo_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::String_Data> name;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> root_url;
  int32_t mount_type;
  uint8_t padfinal_[4];

 private:
  FileSystemInfo_Data();
  ~FileSystemInfo_Data() = delete;
};
static_assert(sizeof(FileSystemInfo_Data) == 32,
              "Bad sizeof(FileSystemInfo_Data)");
// Used by FileSystemInfo::WrapAsMessage to lazily serialize the struct.
template <typename UserType, typename DataView>
struct FileSystemInfo_UnserializedMessageContext
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  FileSystemInfo_UnserializedMessageContext(
    uint32_t message_name,
    uint32_t message_flags,
    UserType input)
      : mojo::internal::UnserializedMessageContext(&kMessageTag, message_name, message_flags)
      , user_data_(std::move(input)) {}
  ~FileSystemInfo_UnserializedMessageContext() override = default;

  UserType TakeData() {
    return std::move(user_data_);
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* context,
                 mojo::internal::Buffer* buffer) override {
    FileSystemInfo_Data::BufferWriter writer;
    mojo::internal::Serialize<DataView>(user_data_, buffer, &writer, context);
  }

  UserType user_data_;
};

template <typename UserType, typename DataView>
const mojo::internal::UnserializedMessageContext::Tag
    FileSystemInfo_UnserializedMessageContext<UserType, DataView>::kMessageTag = {};
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemEntry_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemEntry_Data));
      new (data()) FileSystemEntry_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemEntry_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemEntry_Data>(index_);
    }
    FileSystemEntry_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::String_Data> file_system_id;
  mojo::internal::Pointer<mojo::internal::String_Data> base_name;

 private:
  FileSystemEntry_Data();
  ~FileSystemEntry_Data() = delete;
};
static_assert(sizeof(FileSystemEntry_Data) == 24,
              "Bad sizeof(FileSystemEntry_Data)");
// Used by FileSystemEntry::WrapAsMessage to lazily serialize the struct.
template <typename UserType, typename DataView>
struct FileSystemEntry_UnserializedMessageContext
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  FileSystemEntry_UnserializedMessageContext(
    uint32_t message_name,
    uint32_t message_flags,
    UserType input)
      : mojo::internal::UnserializedMessageContext(&kMessageTag, message_name, message_flags)
      , user_data_(std::move(input)) {}
  ~FileSystemEntry_UnserializedMessageContext() override = default;

  UserType TakeData() {
    return std::move(user_data_);
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* context,
                 mojo::internal::Buffer* buffer) override {
    FileSystemEntry_Data::BufferWriter writer;
    mojo::internal::Serialize<DataView>(user_data_, buffer, &writer, context);
  }

  UserType user_data_;
};

template <typename UserType, typename DataView>
const mojo::internal::UnserializedMessageContext::Tag
    FileSystemEntry_UnserializedMessageContext<UserType, DataView>::kMessageTag = {};
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) ChooseFileSystemEntryAcceptsOption_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(ChooseFileSystemEntryAcceptsOption_Data));
      new (data()) ChooseFileSystemEntryAcceptsOption_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    ChooseFileSystemEntryAcceptsOption_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<ChooseFileSystemEntryAcceptsOption_Data>(index_);
    }
    ChooseFileSystemEntryAcceptsOption_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::mojo_base::mojom::internal::String16_Data> description;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<mojo::internal::String_Data>>> mime_types;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<mojo::internal::String_Data>>> extensions;

 private:
  ChooseFileSystemEntryAcceptsOption_Data();
  ~ChooseFileSystemEntryAcceptsOption_Data() = delete;
};
static_assert(sizeof(ChooseFileSystemEntryAcceptsOption_Data) == 32,
              "Bad sizeof(ChooseFileSystemEntryAcceptsOption_Data)");
// Used by ChooseFileSystemEntryAcceptsOption::WrapAsMessage to lazily serialize the struct.
template <typename UserType, typename DataView>
struct ChooseFileSystemEntryAcceptsOption_UnserializedMessageContext
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  ChooseFileSystemEntryAcceptsOption_UnserializedMessageContext(
    uint32_t message_name,
    uint32_t message_flags,
    UserType input)
      : mojo::internal::UnserializedMessageContext(&kMessageTag, message_name, message_flags)
      , user_data_(std::move(input)) {}
  ~ChooseFileSystemEntryAcceptsOption_UnserializedMessageContext() override = default;

  UserType TakeData() {
    return std::move(user_data_);
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* context,
                 mojo::internal::Buffer* buffer) override {
    ChooseFileSystemEntryAcceptsOption_Data::BufferWriter writer;
    mojo::internal::Serialize<DataView>(user_data_, buffer, &writer, context);
  }

  UserType user_data_;
};

template <typename UserType, typename DataView>
const mojo::internal::UnserializedMessageContext::Tag
    ChooseFileSystemEntryAcceptsOption_UnserializedMessageContext<UserType, DataView>::kMessageTag = {};
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemCancellableOperation_Cancel_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemCancellableOperation_Cancel_Params_Data));
      new (data()) FileSystemCancellableOperation_Cancel_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemCancellableOperation_Cancel_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemCancellableOperation_Cancel_Params_Data>(index_);
    }
    FileSystemCancellableOperation_Cancel_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  FileSystemCancellableOperation_Cancel_Params_Data();
  ~FileSystemCancellableOperation_Cancel_Params_Data() = delete;
};
static_assert(sizeof(FileSystemCancellableOperation_Cancel_Params_Data) == 8,
              "Bad sizeof(FileSystemCancellableOperation_Cancel_Params_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemCancellableOperation_Cancel_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemCancellableOperation_Cancel_ResponseParams_Data));
      new (data()) FileSystemCancellableOperation_Cancel_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemCancellableOperation_Cancel_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemCancellableOperation_Cancel_ResponseParams_Data>(index_);
    }
    FileSystemCancellableOperation_Cancel_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t error_code;
  uint8_t padfinal_[4];

 private:
  FileSystemCancellableOperation_Cancel_ResponseParams_Data();
  ~FileSystemCancellableOperation_Cancel_ResponseParams_Data() = delete;
};
static_assert(sizeof(FileSystemCancellableOperation_Cancel_ResponseParams_Data) == 16,
              "Bad sizeof(FileSystemCancellableOperation_Cancel_ResponseParams_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemOperationListener_ResultsRetrieved_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemOperationListener_ResultsRetrieved_Params_Data));
      new (data()) FileSystemOperationListener_ResultsRetrieved_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemOperationListener_ResultsRetrieved_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemOperationListener_ResultsRetrieved_Params_Data>(index_);
    }
    FileSystemOperationListener_ResultsRetrieved_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<::filesystem::mojom::internal::DirectoryEntry_Data>>> entries;
  uint8_t has_more : 1;
  uint8_t padfinal_[7];

 private:
  FileSystemOperationListener_ResultsRetrieved_Params_Data();
  ~FileSystemOperationListener_ResultsRetrieved_Params_Data() = delete;
};
static_assert(sizeof(FileSystemOperationListener_ResultsRetrieved_Params_Data) == 24,
              "Bad sizeof(FileSystemOperationListener_ResultsRetrieved_Params_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemOperationListener_DidWrite_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemOperationListener_DidWrite_Params_Data));
      new (data()) FileSystemOperationListener_DidWrite_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemOperationListener_DidWrite_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemOperationListener_DidWrite_Params_Data>(index_);
    }
    FileSystemOperationListener_DidWrite_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t byte_count;
  uint8_t complete : 1;
  uint8_t padfinal_[7];

 private:
  FileSystemOperationListener_DidWrite_Params_Data();
  ~FileSystemOperationListener_DidWrite_Params_Data() = delete;
};
static_assert(sizeof(FileSystemOperationListener_DidWrite_Params_Data) == 24,
              "Bad sizeof(FileSystemOperationListener_DidWrite_Params_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemOperationListener_ErrorOccurred_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemOperationListener_ErrorOccurred_Params_Data));
      new (data()) FileSystemOperationListener_ErrorOccurred_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemOperationListener_ErrorOccurred_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemOperationListener_ErrorOccurred_Params_Data>(index_);
    }
    FileSystemOperationListener_ErrorOccurred_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t error_code;
  uint8_t padfinal_[4];

 private:
  FileSystemOperationListener_ErrorOccurred_Params_Data();
  ~FileSystemOperationListener_ErrorOccurred_Params_Data() = delete;
};
static_assert(sizeof(FileSystemOperationListener_ErrorOccurred_Params_Data) == 16,
              "Bad sizeof(FileSystemOperationListener_ErrorOccurred_Params_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) ReceivedSnapshotListener_DidReceiveSnapshotFile_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(ReceivedSnapshotListener_DidReceiveSnapshotFile_Params_Data));
      new (data()) ReceivedSnapshotListener_DidReceiveSnapshotFile_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    ReceivedSnapshotListener_DidReceiveSnapshotFile_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<ReceivedSnapshotListener_DidReceiveSnapshotFile_Params_Data>(index_);
    }
    ReceivedSnapshotListener_DidReceiveSnapshotFile_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  ReceivedSnapshotListener_DidReceiveSnapshotFile_Params_Data();
  ~ReceivedSnapshotListener_DidReceiveSnapshotFile_Params_Data() = delete;
};
static_assert(sizeof(ReceivedSnapshotListener_DidReceiveSnapshotFile_Params_Data) == 8,
              "Bad sizeof(ReceivedSnapshotListener_DidReceiveSnapshotFile_Params_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_Open_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_Open_Params_Data));
      new (data()) FileSystemManager_Open_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_Open_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_Open_Params_Data>(index_);
    }
    FileSystemManager_Open_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::url::mojom::internal::Origin_Data> origin;
  int32_t file_system_type;
  uint8_t padfinal_[4];

 private:
  FileSystemManager_Open_Params_Data();
  ~FileSystemManager_Open_Params_Data() = delete;
};
static_assert(sizeof(FileSystemManager_Open_Params_Data) == 24,
              "Bad sizeof(FileSystemManager_Open_Params_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_Open_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_Open_ResponseParams_Data));
      new (data()) FileSystemManager_Open_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_Open_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_Open_ResponseParams_Data>(index_);
    }
    FileSystemManager_Open_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::String_Data> name;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> root_url;
  int32_t error_code;
  uint8_t padfinal_[4];

 private:
  FileSystemManager_Open_ResponseParams_Data();
  ~FileSystemManager_Open_ResponseParams_Data() = delete;
};
static_assert(sizeof(FileSystemManager_Open_ResponseParams_Data) == 32,
              "Bad sizeof(FileSystemManager_Open_ResponseParams_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_ResolveURL_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_ResolveURL_Params_Data));
      new (data()) FileSystemManager_ResolveURL_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_ResolveURL_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_ResolveURL_Params_Data>(index_);
    }
    FileSystemManager_ResolveURL_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> filesystem_url;

 private:
  FileSystemManager_ResolveURL_Params_Data();
  ~FileSystemManager_ResolveURL_Params_Data() = delete;
};
static_assert(sizeof(FileSystemManager_ResolveURL_Params_Data) == 16,
              "Bad sizeof(FileSystemManager_ResolveURL_Params_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_ResolveURL_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_ResolveURL_ResponseParams_Data));
      new (data()) FileSystemManager_ResolveURL_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_ResolveURL_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_ResolveURL_ResponseParams_Data>(index_);
    }
    FileSystemManager_ResolveURL_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<internal::FileSystemInfo_Data> info;
  mojo::internal::Pointer<::mojo_base::mojom::internal::FilePath_Data> file_path;
  uint8_t is_directory : 1;
  uint8_t pad2_[3];
  int32_t error_code;

 private:
  FileSystemManager_ResolveURL_ResponseParams_Data();
  ~FileSystemManager_ResolveURL_ResponseParams_Data() = delete;
};
static_assert(sizeof(FileSystemManager_ResolveURL_ResponseParams_Data) == 32,
              "Bad sizeof(FileSystemManager_ResolveURL_ResponseParams_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_Move_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_Move_Params_Data));
      new (data()) FileSystemManager_Move_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_Move_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_Move_Params_Data>(index_);
    }
    FileSystemManager_Move_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> src_path;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> dest_path;

 private:
  FileSystemManager_Move_Params_Data();
  ~FileSystemManager_Move_Params_Data() = delete;
};
static_assert(sizeof(FileSystemManager_Move_Params_Data) == 24,
              "Bad sizeof(FileSystemManager_Move_Params_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_Move_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_Move_ResponseParams_Data));
      new (data()) FileSystemManager_Move_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_Move_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_Move_ResponseParams_Data>(index_);
    }
    FileSystemManager_Move_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t error_code;
  uint8_t padfinal_[4];

 private:
  FileSystemManager_Move_ResponseParams_Data();
  ~FileSystemManager_Move_ResponseParams_Data() = delete;
};
static_assert(sizeof(FileSystemManager_Move_ResponseParams_Data) == 16,
              "Bad sizeof(FileSystemManager_Move_ResponseParams_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_Copy_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_Copy_Params_Data));
      new (data()) FileSystemManager_Copy_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_Copy_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_Copy_Params_Data>(index_);
    }
    FileSystemManager_Copy_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> src_path;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> dest_path;

 private:
  FileSystemManager_Copy_Params_Data();
  ~FileSystemManager_Copy_Params_Data() = delete;
};
static_assert(sizeof(FileSystemManager_Copy_Params_Data) == 24,
              "Bad sizeof(FileSystemManager_Copy_Params_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_Copy_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_Copy_ResponseParams_Data));
      new (data()) FileSystemManager_Copy_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_Copy_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_Copy_ResponseParams_Data>(index_);
    }
    FileSystemManager_Copy_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t error_code;
  uint8_t padfinal_[4];

 private:
  FileSystemManager_Copy_ResponseParams_Data();
  ~FileSystemManager_Copy_ResponseParams_Data() = delete;
};
static_assert(sizeof(FileSystemManager_Copy_ResponseParams_Data) == 16,
              "Bad sizeof(FileSystemManager_Copy_ResponseParams_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_Remove_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_Remove_Params_Data));
      new (data()) FileSystemManager_Remove_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_Remove_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_Remove_Params_Data>(index_);
    }
    FileSystemManager_Remove_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> path;
  uint8_t recursive : 1;
  uint8_t padfinal_[7];

 private:
  FileSystemManager_Remove_Params_Data();
  ~FileSystemManager_Remove_Params_Data() = delete;
};
static_assert(sizeof(FileSystemManager_Remove_Params_Data) == 24,
              "Bad sizeof(FileSystemManager_Remove_Params_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_Remove_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_Remove_ResponseParams_Data));
      new (data()) FileSystemManager_Remove_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_Remove_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_Remove_ResponseParams_Data>(index_);
    }
    FileSystemManager_Remove_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t error_code;
  uint8_t padfinal_[4];

 private:
  FileSystemManager_Remove_ResponseParams_Data();
  ~FileSystemManager_Remove_ResponseParams_Data() = delete;
};
static_assert(sizeof(FileSystemManager_Remove_ResponseParams_Data) == 16,
              "Bad sizeof(FileSystemManager_Remove_ResponseParams_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_ReadMetadata_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_ReadMetadata_Params_Data));
      new (data()) FileSystemManager_ReadMetadata_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_ReadMetadata_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_ReadMetadata_Params_Data>(index_);
    }
    FileSystemManager_ReadMetadata_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> path;

 private:
  FileSystemManager_ReadMetadata_Params_Data();
  ~FileSystemManager_ReadMetadata_Params_Data() = delete;
};
static_assert(sizeof(FileSystemManager_ReadMetadata_Params_Data) == 16,
              "Bad sizeof(FileSystemManager_ReadMetadata_Params_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_ReadMetadata_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_ReadMetadata_ResponseParams_Data));
      new (data()) FileSystemManager_ReadMetadata_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_ReadMetadata_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_ReadMetadata_ResponseParams_Data>(index_);
    }
    FileSystemManager_ReadMetadata_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::mojo_base::mojom::internal::FileInfo_Data> file_info;
  int32_t error_code;
  uint8_t padfinal_[4];

 private:
  FileSystemManager_ReadMetadata_ResponseParams_Data();
  ~FileSystemManager_ReadMetadata_ResponseParams_Data() = delete;
};
static_assert(sizeof(FileSystemManager_ReadMetadata_ResponseParams_Data) == 24,
              "Bad sizeof(FileSystemManager_ReadMetadata_ResponseParams_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_Create_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_Create_Params_Data));
      new (data()) FileSystemManager_Create_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_Create_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_Create_Params_Data>(index_);
    }
    FileSystemManager_Create_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> path;
  uint8_t exclusive : 1;
  uint8_t is_directory : 1;
  uint8_t recursive : 1;
  uint8_t padfinal_[7];

 private:
  FileSystemManager_Create_Params_Data();
  ~FileSystemManager_Create_Params_Data() = delete;
};
static_assert(sizeof(FileSystemManager_Create_Params_Data) == 24,
              "Bad sizeof(FileSystemManager_Create_Params_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_Create_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_Create_ResponseParams_Data));
      new (data()) FileSystemManager_Create_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_Create_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_Create_ResponseParams_Data>(index_);
    }
    FileSystemManager_Create_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t error_code;
  uint8_t padfinal_[4];

 private:
  FileSystemManager_Create_ResponseParams_Data();
  ~FileSystemManager_Create_ResponseParams_Data() = delete;
};
static_assert(sizeof(FileSystemManager_Create_ResponseParams_Data) == 16,
              "Bad sizeof(FileSystemManager_Create_ResponseParams_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_Exists_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_Exists_Params_Data));
      new (data()) FileSystemManager_Exists_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_Exists_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_Exists_Params_Data>(index_);
    }
    FileSystemManager_Exists_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> path;
  uint8_t is_directory : 1;
  uint8_t padfinal_[7];

 private:
  FileSystemManager_Exists_Params_Data();
  ~FileSystemManager_Exists_Params_Data() = delete;
};
static_assert(sizeof(FileSystemManager_Exists_Params_Data) == 24,
              "Bad sizeof(FileSystemManager_Exists_Params_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_Exists_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_Exists_ResponseParams_Data));
      new (data()) FileSystemManager_Exists_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_Exists_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_Exists_ResponseParams_Data>(index_);
    }
    FileSystemManager_Exists_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t error_code;
  uint8_t padfinal_[4];

 private:
  FileSystemManager_Exists_ResponseParams_Data();
  ~FileSystemManager_Exists_ResponseParams_Data() = delete;
};
static_assert(sizeof(FileSystemManager_Exists_ResponseParams_Data) == 16,
              "Bad sizeof(FileSystemManager_Exists_ResponseParams_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_ReadDirectory_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_ReadDirectory_Params_Data));
      new (data()) FileSystemManager_ReadDirectory_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_ReadDirectory_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_ReadDirectory_Params_Data>(index_);
    }
    FileSystemManager_ReadDirectory_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> path;
  mojo::internal::Interface_Data listener;

 private:
  FileSystemManager_ReadDirectory_Params_Data();
  ~FileSystemManager_ReadDirectory_Params_Data() = delete;
};
static_assert(sizeof(FileSystemManager_ReadDirectory_Params_Data) == 24,
              "Bad sizeof(FileSystemManager_ReadDirectory_Params_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_ReadDirectorySync_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_ReadDirectorySync_Params_Data));
      new (data()) FileSystemManager_ReadDirectorySync_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_ReadDirectorySync_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_ReadDirectorySync_Params_Data>(index_);
    }
    FileSystemManager_ReadDirectorySync_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> path;

 private:
  FileSystemManager_ReadDirectorySync_Params_Data();
  ~FileSystemManager_ReadDirectorySync_Params_Data() = delete;
};
static_assert(sizeof(FileSystemManager_ReadDirectorySync_Params_Data) == 16,
              "Bad sizeof(FileSystemManager_ReadDirectorySync_Params_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_ReadDirectorySync_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_ReadDirectorySync_ResponseParams_Data));
      new (data()) FileSystemManager_ReadDirectorySync_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_ReadDirectorySync_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_ReadDirectorySync_ResponseParams_Data>(index_);
    }
    FileSystemManager_ReadDirectorySync_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<::filesystem::mojom::internal::DirectoryEntry_Data>>> entries;
  int32_t error_code;
  uint8_t padfinal_[4];

 private:
  FileSystemManager_ReadDirectorySync_ResponseParams_Data();
  ~FileSystemManager_ReadDirectorySync_ResponseParams_Data() = delete;
};
static_assert(sizeof(FileSystemManager_ReadDirectorySync_ResponseParams_Data) == 24,
              "Bad sizeof(FileSystemManager_ReadDirectorySync_ResponseParams_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_Write_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_Write_Params_Data));
      new (data()) FileSystemManager_Write_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_Write_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_Write_Params_Data>(index_);
    }
    FileSystemManager_Write_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> file_path;
  mojo::internal::Pointer<mojo::internal::String_Data> blob_uuid;
  int64_t position;
  mojo::internal::Handle_Data op_request;
  mojo::internal::Interface_Data listener;
  uint8_t padfinal_[4];

 private:
  FileSystemManager_Write_Params_Data();
  ~FileSystemManager_Write_Params_Data() = delete;
};
static_assert(sizeof(FileSystemManager_Write_Params_Data) == 48,
              "Bad sizeof(FileSystemManager_Write_Params_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_WriteSync_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_WriteSync_Params_Data));
      new (data()) FileSystemManager_WriteSync_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_WriteSync_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_WriteSync_Params_Data>(index_);
    }
    FileSystemManager_WriteSync_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> file_path;
  mojo::internal::Pointer<mojo::internal::String_Data> blob_uuid;
  int64_t position;

 private:
  FileSystemManager_WriteSync_Params_Data();
  ~FileSystemManager_WriteSync_Params_Data() = delete;
};
static_assert(sizeof(FileSystemManager_WriteSync_Params_Data) == 32,
              "Bad sizeof(FileSystemManager_WriteSync_Params_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_WriteSync_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_WriteSync_ResponseParams_Data));
      new (data()) FileSystemManager_WriteSync_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_WriteSync_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_WriteSync_ResponseParams_Data>(index_);
    }
    FileSystemManager_WriteSync_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t byte_count;
  int32_t error_code;
  uint8_t padfinal_[4];

 private:
  FileSystemManager_WriteSync_ResponseParams_Data();
  ~FileSystemManager_WriteSync_ResponseParams_Data() = delete;
};
static_assert(sizeof(FileSystemManager_WriteSync_ResponseParams_Data) == 24,
              "Bad sizeof(FileSystemManager_WriteSync_ResponseParams_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_Truncate_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_Truncate_Params_Data));
      new (data()) FileSystemManager_Truncate_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_Truncate_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_Truncate_Params_Data>(index_);
    }
    FileSystemManager_Truncate_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> file_path;
  int64_t length;
  mojo::internal::Handle_Data op_request;
  uint8_t padfinal_[4];

 private:
  FileSystemManager_Truncate_Params_Data();
  ~FileSystemManager_Truncate_Params_Data() = delete;
};
static_assert(sizeof(FileSystemManager_Truncate_Params_Data) == 32,
              "Bad sizeof(FileSystemManager_Truncate_Params_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_Truncate_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_Truncate_ResponseParams_Data));
      new (data()) FileSystemManager_Truncate_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_Truncate_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_Truncate_ResponseParams_Data>(index_);
    }
    FileSystemManager_Truncate_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t error_code;
  uint8_t padfinal_[4];

 private:
  FileSystemManager_Truncate_ResponseParams_Data();
  ~FileSystemManager_Truncate_ResponseParams_Data() = delete;
};
static_assert(sizeof(FileSystemManager_Truncate_ResponseParams_Data) == 16,
              "Bad sizeof(FileSystemManager_Truncate_ResponseParams_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_TruncateSync_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_TruncateSync_Params_Data));
      new (data()) FileSystemManager_TruncateSync_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_TruncateSync_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_TruncateSync_Params_Data>(index_);
    }
    FileSystemManager_TruncateSync_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> file_path;
  int64_t length;

 private:
  FileSystemManager_TruncateSync_Params_Data();
  ~FileSystemManager_TruncateSync_Params_Data() = delete;
};
static_assert(sizeof(FileSystemManager_TruncateSync_Params_Data) == 24,
              "Bad sizeof(FileSystemManager_TruncateSync_Params_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_TruncateSync_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_TruncateSync_ResponseParams_Data));
      new (data()) FileSystemManager_TruncateSync_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_TruncateSync_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_TruncateSync_ResponseParams_Data>(index_);
    }
    FileSystemManager_TruncateSync_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t error_code;
  uint8_t padfinal_[4];

 private:
  FileSystemManager_TruncateSync_ResponseParams_Data();
  ~FileSystemManager_TruncateSync_ResponseParams_Data() = delete;
};
static_assert(sizeof(FileSystemManager_TruncateSync_ResponseParams_Data) == 16,
              "Bad sizeof(FileSystemManager_TruncateSync_ResponseParams_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_TouchFile_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_TouchFile_Params_Data));
      new (data()) FileSystemManager_TouchFile_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_TouchFile_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_TouchFile_Params_Data>(index_);
    }
    FileSystemManager_TouchFile_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> path;
  mojo::internal::Pointer<::mojo_base::mojom::internal::Time_Data> last_access_time;
  mojo::internal::Pointer<::mojo_base::mojom::internal::Time_Data> last_modified_time;

 private:
  FileSystemManager_TouchFile_Params_Data();
  ~FileSystemManager_TouchFile_Params_Data() = delete;
};
static_assert(sizeof(FileSystemManager_TouchFile_Params_Data) == 32,
              "Bad sizeof(FileSystemManager_TouchFile_Params_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_TouchFile_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_TouchFile_ResponseParams_Data));
      new (data()) FileSystemManager_TouchFile_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_TouchFile_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_TouchFile_ResponseParams_Data>(index_);
    }
    FileSystemManager_TouchFile_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t error_code;
  uint8_t padfinal_[4];

 private:
  FileSystemManager_TouchFile_ResponseParams_Data();
  ~FileSystemManager_TouchFile_ResponseParams_Data() = delete;
};
static_assert(sizeof(FileSystemManager_TouchFile_ResponseParams_Data) == 16,
              "Bad sizeof(FileSystemManager_TouchFile_ResponseParams_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_CreateSnapshotFile_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_CreateSnapshotFile_Params_Data));
      new (data()) FileSystemManager_CreateSnapshotFile_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_CreateSnapshotFile_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_CreateSnapshotFile_Params_Data>(index_);
    }
    FileSystemManager_CreateSnapshotFile_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> file_path;

 private:
  FileSystemManager_CreateSnapshotFile_Params_Data();
  ~FileSystemManager_CreateSnapshotFile_Params_Data() = delete;
};
static_assert(sizeof(FileSystemManager_CreateSnapshotFile_Params_Data) == 16,
              "Bad sizeof(FileSystemManager_CreateSnapshotFile_Params_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_CreateSnapshotFile_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_CreateSnapshotFile_ResponseParams_Data));
      new (data()) FileSystemManager_CreateSnapshotFile_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_CreateSnapshotFile_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_CreateSnapshotFile_ResponseParams_Data>(index_);
    }
    FileSystemManager_CreateSnapshotFile_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::mojo_base::mojom::internal::FileInfo_Data> file_info;
  mojo::internal::Pointer<::mojo_base::mojom::internal::FilePath_Data> platform_path;
  int32_t error_code;
  mojo::internal::Interface_Data snapshot_listener;
  uint8_t padfinal_[4];

 private:
  FileSystemManager_CreateSnapshotFile_ResponseParams_Data();
  ~FileSystemManager_CreateSnapshotFile_ResponseParams_Data() = delete;
};
static_assert(sizeof(FileSystemManager_CreateSnapshotFile_ResponseParams_Data) == 40,
              "Bad sizeof(FileSystemManager_CreateSnapshotFile_ResponseParams_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_GetPlatformPath_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_GetPlatformPath_Params_Data));
      new (data()) FileSystemManager_GetPlatformPath_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_GetPlatformPath_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_GetPlatformPath_Params_Data>(index_);
    }
    FileSystemManager_GetPlatformPath_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> file_path;

 private:
  FileSystemManager_GetPlatformPath_Params_Data();
  ~FileSystemManager_GetPlatformPath_Params_Data() = delete;
};
static_assert(sizeof(FileSystemManager_GetPlatformPath_Params_Data) == 16,
              "Bad sizeof(FileSystemManager_GetPlatformPath_Params_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_GetPlatformPath_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_GetPlatformPath_ResponseParams_Data));
      new (data()) FileSystemManager_GetPlatformPath_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_GetPlatformPath_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_GetPlatformPath_ResponseParams_Data>(index_);
    }
    FileSystemManager_GetPlatformPath_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::mojo_base::mojom::internal::FilePath_Data> platform_path;

 private:
  FileSystemManager_GetPlatformPath_ResponseParams_Data();
  ~FileSystemManager_GetPlatformPath_ResponseParams_Data() = delete;
};
static_assert(sizeof(FileSystemManager_GetPlatformPath_ResponseParams_Data) == 16,
              "Bad sizeof(FileSystemManager_GetPlatformPath_ResponseParams_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_CreateWriter_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_CreateWriter_Params_Data));
      new (data()) FileSystemManager_CreateWriter_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_CreateWriter_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_CreateWriter_Params_Data>(index_);
    }
    FileSystemManager_CreateWriter_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> file_path;

 private:
  FileSystemManager_CreateWriter_Params_Data();
  ~FileSystemManager_CreateWriter_Params_Data() = delete;
};
static_assert(sizeof(FileSystemManager_CreateWriter_Params_Data) == 16,
              "Bad sizeof(FileSystemManager_CreateWriter_Params_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_CreateWriter_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_CreateWriter_ResponseParams_Data));
      new (data()) FileSystemManager_CreateWriter_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_CreateWriter_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_CreateWriter_ResponseParams_Data>(index_);
    }
    FileSystemManager_CreateWriter_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t result;
  mojo::internal::Interface_Data writer;
  uint8_t padfinal_[4];

 private:
  FileSystemManager_CreateWriter_ResponseParams_Data();
  ~FileSystemManager_CreateWriter_ResponseParams_Data() = delete;
};
static_assert(sizeof(FileSystemManager_CreateWriter_ResponseParams_Data) == 24,
              "Bad sizeof(FileSystemManager_CreateWriter_ResponseParams_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_ChooseEntry_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_ChooseEntry_Params_Data));
      new (data()) FileSystemManager_ChooseEntry_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_ChooseEntry_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_ChooseEntry_Params_Data>(index_);
    }
    FileSystemManager_ChooseEntry_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t type;
  uint8_t include_accepts_all : 1;
  uint8_t pad1_[3];
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<internal::ChooseFileSystemEntryAcceptsOption_Data>>> accepts;

 private:
  FileSystemManager_ChooseEntry_Params_Data();
  ~FileSystemManager_ChooseEntry_Params_Data() = delete;
};
static_assert(sizeof(FileSystemManager_ChooseEntry_Params_Data) == 24,
              "Bad sizeof(FileSystemManager_ChooseEntry_Params_Data)");
class COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) FileSystemManager_ChooseEntry_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(FileSystemManager_ChooseEntry_ResponseParams_Data));
      new (data()) FileSystemManager_ChooseEntry_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    FileSystemManager_ChooseEntry_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<FileSystemManager_ChooseEntry_ResponseParams_Data>(index_);
    }
    FileSystemManager_ChooseEntry_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t error_code;
  uint8_t pad0_[4];
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<internal::FileSystemEntry_Data>>> entries;

 private:
  FileSystemManager_ChooseEntry_ResponseParams_Data();
  ~FileSystemManager_ChooseEntry_ResponseParams_Data() = delete;
};
static_assert(sizeof(FileSystemManager_ChooseEntry_ResponseParams_Data) == 24,
              "Bad sizeof(FileSystemManager_ChooseEntry_ResponseParams_Data)");

#pragma pack(pop)

}  // namespace internal
}  // namespace mojom
}  // namespace blink

#endif  // THIRD_PARTY_BLINK_PUBLIC_MOJOM_FILESYSTEM_FILE_SYSTEM_MOJOM_SHARED_INTERNAL_H_