// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_PUBLIC_MOJOM_FILESYSTEM_FILE_SYSTEM_MOJOM_SHARED_H_
#define THIRD_PARTY_BLINK_PUBLIC_MOJOM_FILESYSTEM_FILE_SYSTEM_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "third_party/blink/public/mojom/filesystem/file_system.mojom-shared-internal.h"
#include "components/services/filesystem/public/interfaces/types.mojom-shared.h"
#include "url/mojom/origin.mojom-shared.h"
#include "url/mojom/url.mojom-shared.h"
#include "mojo/public/mojom/base/file_error.mojom-shared.h"
#include "mojo/public/mojom/base/file_path.mojom-shared.h"
#include "mojo/public/mojom/base/file_info.mojom-shared.h"
#include "mojo/public/mojom/base/time.mojom-shared.h"
#include "mojo/public/mojom/base/string16.mojom-shared.h"
#include "third_party/blink/public/mojom/filesystem/file_writer.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "base/component_export.h"


namespace blink {
namespace mojom {
class FileSystemInfoDataView;

class FileSystemEntryDataView;

class ChooseFileSystemEntryAcceptsOptionDataView;



}  // namespace mojom
}  // namespace blink

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::blink::mojom::FileSystemInfoDataView> {
  using Data = ::blink::mojom::internal::FileSystemInfo_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::blink::mojom::FileSystemEntryDataView> {
  using Data = ::blink::mojom::internal::FileSystemEntry_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::blink::mojom::ChooseFileSystemEntryAcceptsOptionDataView> {
  using Data = ::blink::mojom::internal::ChooseFileSystemEntryAcceptsOption_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace blink {
namespace mojom {

enum class FileSystemType : int32_t {
  kTemporary,
  kPersistent,
  kIsolated,
  kExternal,
  kMinValue = 0,
  kMaxValue = 3,
};

COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) std::ostream& operator<<(std::ostream& os, FileSystemType value);
inline bool IsKnownEnumValue(FileSystemType value) {
  return internal::FileSystemType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class ChooseFileSystemEntryType : int32_t {
  kOpenFile,
  kOpenMultipleFiles,
  kSaveFile,
  kOpenDirectory,
  kMinValue = 0,
  kMaxValue = 3,
};

COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) std::ostream& operator<<(std::ostream& os, ChooseFileSystemEntryType value);
inline bool IsKnownEnumValue(ChooseFileSystemEntryType value) {
  return internal::ChooseFileSystemEntryType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class FileSystemCancellableOperationInterfaceBase {};

using FileSystemCancellableOperationPtrDataView =
    mojo::InterfacePtrDataView<FileSystemCancellableOperationInterfaceBase>;
using FileSystemCancellableOperationRequestDataView =
    mojo::InterfaceRequestDataView<FileSystemCancellableOperationInterfaceBase>;
using FileSystemCancellableOperationAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<FileSystemCancellableOperationInterfaceBase>;
using FileSystemCancellableOperationAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<FileSystemCancellableOperationInterfaceBase>;
class FileSystemOperationListenerInterfaceBase {};

using FileSystemOperationListenerPtrDataView =
    mojo::InterfacePtrDataView<FileSystemOperationListenerInterfaceBase>;
using FileSystemOperationListenerRequestDataView =
    mojo::InterfaceRequestDataView<FileSystemOperationListenerInterfaceBase>;
using FileSystemOperationListenerAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<FileSystemOperationListenerInterfaceBase>;
using FileSystemOperationListenerAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<FileSystemOperationListenerInterfaceBase>;
class ReceivedSnapshotListenerInterfaceBase {};

using ReceivedSnapshotListenerPtrDataView =
    mojo::InterfacePtrDataView<ReceivedSnapshotListenerInterfaceBase>;
using ReceivedSnapshotListenerRequestDataView =
    mojo::InterfaceRequestDataView<ReceivedSnapshotListenerInterfaceBase>;
using ReceivedSnapshotListenerAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<ReceivedSnapshotListenerInterfaceBase>;
using ReceivedSnapshotListenerAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<ReceivedSnapshotListenerInterfaceBase>;
class FileSystemManagerInterfaceBase {};

using FileSystemManagerPtrDataView =
    mojo::InterfacePtrDataView<FileSystemManagerInterfaceBase>;
using FileSystemManagerRequestDataView =
    mojo::InterfaceRequestDataView<FileSystemManagerInterfaceBase>;
using FileSystemManagerAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<FileSystemManagerInterfaceBase>;
using FileSystemManagerAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<FileSystemManagerInterfaceBase>;
class FileSystemInfoDataView {
 public:
  FileSystemInfoDataView() {}

  FileSystemInfoDataView(
      internal::FileSystemInfo_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetNameDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadName(UserType* output) {
    auto* pointer = data_->name.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetRootUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRootUrl(UserType* output) {
    auto* pointer = data_->root_url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadMountType(UserType* output) const {
    auto data_value = data_->mount_type;
    return mojo::internal::Deserialize<::blink::mojom::FileSystemType>(
        data_value, output);
  }

  FileSystemType mount_type() const {
    return static_cast<FileSystemType>(data_->mount_type);
  }
 private:
  internal::FileSystemInfo_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemEntryDataView {
 public:
  FileSystemEntryDataView() {}

  FileSystemEntryDataView(
      internal::FileSystemEntry_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetFileSystemIdDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFileSystemId(UserType* output) {
    auto* pointer = data_->file_system_id.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetBaseNameDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadBaseName(UserType* output) {
    auto* pointer = data_->base_name.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::FileSystemEntry_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ChooseFileSystemEntryAcceptsOptionDataView {
 public:
  ChooseFileSystemEntryAcceptsOptionDataView() {}

  ChooseFileSystemEntryAcceptsOptionDataView(
      internal::ChooseFileSystemEntryAcceptsOption_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetDescriptionDataView(
      ::mojo_base::mojom::String16DataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDescription(UserType* output) {
    auto* pointer = data_->description.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::String16DataView>(
        pointer, output, context_);
  }
  inline void GetMimeTypesDataView(
      mojo::ArrayDataView<mojo::StringDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadMimeTypes(UserType* output) {
    auto* pointer = data_->mime_types.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<mojo::StringDataView>>(
        pointer, output, context_);
  }
  inline void GetExtensionsDataView(
      mojo::ArrayDataView<mojo::StringDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadExtensions(UserType* output) {
    auto* pointer = data_->extensions.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<mojo::StringDataView>>(
        pointer, output, context_);
  }
 private:
  internal::ChooseFileSystemEntryAcceptsOption_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemCancellableOperation_Cancel_ParamsDataView {
 public:
  FileSystemCancellableOperation_Cancel_ParamsDataView() {}

  FileSystemCancellableOperation_Cancel_ParamsDataView(
      internal::FileSystemCancellableOperation_Cancel_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::FileSystemCancellableOperation_Cancel_Params_Data* data_ = nullptr;
};

class FileSystemCancellableOperation_Cancel_ResponseParamsDataView {
 public:
  FileSystemCancellableOperation_Cancel_ResponseParamsDataView() {}

  FileSystemCancellableOperation_Cancel_ResponseParamsDataView(
      internal::FileSystemCancellableOperation_Cancel_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadErrorCode(UserType* output) const {
    auto data_value = data_->error_code;
    return mojo::internal::Deserialize<::mojo_base::mojom::FileError>(
        data_value, output);
  }

  ::mojo_base::mojom::FileError error_code() const {
    return static_cast<::mojo_base::mojom::FileError>(data_->error_code);
  }
 private:
  internal::FileSystemCancellableOperation_Cancel_ResponseParams_Data* data_ = nullptr;
};

class FileSystemOperationListener_ResultsRetrieved_ParamsDataView {
 public:
  FileSystemOperationListener_ResultsRetrieved_ParamsDataView() {}

  FileSystemOperationListener_ResultsRetrieved_ParamsDataView(
      internal::FileSystemOperationListener_ResultsRetrieved_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetEntriesDataView(
      mojo::ArrayDataView<::filesystem::mojom::DirectoryEntryDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadEntries(UserType* output) {
    auto* pointer = data_->entries.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::filesystem::mojom::DirectoryEntryDataView>>(
        pointer, output, context_);
  }
  bool has_more() const {
    return data_->has_more;
  }
 private:
  internal::FileSystemOperationListener_ResultsRetrieved_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemOperationListener_DidWrite_ParamsDataView {
 public:
  FileSystemOperationListener_DidWrite_ParamsDataView() {}

  FileSystemOperationListener_DidWrite_ParamsDataView(
      internal::FileSystemOperationListener_DidWrite_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  int64_t byte_count() const {
    return data_->byte_count;
  }
  bool complete() const {
    return data_->complete;
  }
 private:
  internal::FileSystemOperationListener_DidWrite_Params_Data* data_ = nullptr;
};

class FileSystemOperationListener_ErrorOccurred_ParamsDataView {
 public:
  FileSystemOperationListener_ErrorOccurred_ParamsDataView() {}

  FileSystemOperationListener_ErrorOccurred_ParamsDataView(
      internal::FileSystemOperationListener_ErrorOccurred_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadErrorCode(UserType* output) const {
    auto data_value = data_->error_code;
    return mojo::internal::Deserialize<::mojo_base::mojom::FileError>(
        data_value, output);
  }

  ::mojo_base::mojom::FileError error_code() const {
    return static_cast<::mojo_base::mojom::FileError>(data_->error_code);
  }
 private:
  internal::FileSystemOperationListener_ErrorOccurred_Params_Data* data_ = nullptr;
};

class ReceivedSnapshotListener_DidReceiveSnapshotFile_ParamsDataView {
 public:
  ReceivedSnapshotListener_DidReceiveSnapshotFile_ParamsDataView() {}

  ReceivedSnapshotListener_DidReceiveSnapshotFile_ParamsDataView(
      internal::ReceivedSnapshotListener_DidReceiveSnapshotFile_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::ReceivedSnapshotListener_DidReceiveSnapshotFile_Params_Data* data_ = nullptr;
};

class FileSystemManager_Open_ParamsDataView {
 public:
  FileSystemManager_Open_ParamsDataView() {}

  FileSystemManager_Open_ParamsDataView(
      internal::FileSystemManager_Open_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetOriginDataView(
      ::url::mojom::OriginDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOrigin(UserType* output) {
    auto* pointer = data_->origin.Get();
    return mojo::internal::Deserialize<::url::mojom::OriginDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFileSystemType(UserType* output) const {
    auto data_value = data_->file_system_type;
    return mojo::internal::Deserialize<::blink::mojom::FileSystemType>(
        data_value, output);
  }

  FileSystemType file_system_type() const {
    return static_cast<FileSystemType>(data_->file_system_type);
  }
 private:
  internal::FileSystemManager_Open_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_Open_ResponseParamsDataView {
 public:
  FileSystemManager_Open_ResponseParamsDataView() {}

  FileSystemManager_Open_ResponseParamsDataView(
      internal::FileSystemManager_Open_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetNameDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadName(UserType* output) {
    auto* pointer = data_->name.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetRootUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRootUrl(UserType* output) {
    auto* pointer = data_->root_url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadErrorCode(UserType* output) const {
    auto data_value = data_->error_code;
    return mojo::internal::Deserialize<::mojo_base::mojom::FileError>(
        data_value, output);
  }

  ::mojo_base::mojom::FileError error_code() const {
    return static_cast<::mojo_base::mojom::FileError>(data_->error_code);
  }
 private:
  internal::FileSystemManager_Open_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_ResolveURL_ParamsDataView {
 public:
  FileSystemManager_ResolveURL_ParamsDataView() {}

  FileSystemManager_ResolveURL_ParamsDataView(
      internal::FileSystemManager_ResolveURL_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetFilesystemUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFilesystemUrl(UserType* output) {
    auto* pointer = data_->filesystem_url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
 private:
  internal::FileSystemManager_ResolveURL_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_ResolveURL_ResponseParamsDataView {
 public:
  FileSystemManager_ResolveURL_ResponseParamsDataView() {}

  FileSystemManager_ResolveURL_ResponseParamsDataView(
      internal::FileSystemManager_ResolveURL_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetInfoDataView(
      FileSystemInfoDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadInfo(UserType* output) {
    auto* pointer = data_->info.Get();
    return mojo::internal::Deserialize<::blink::mojom::FileSystemInfoDataView>(
        pointer, output, context_);
  }
  inline void GetFilePathDataView(
      ::mojo_base::mojom::FilePathDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFilePath(UserType* output) {
    auto* pointer = data_->file_path.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::FilePathDataView>(
        pointer, output, context_);
  }
  bool is_directory() const {
    return data_->is_directory;
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadErrorCode(UserType* output) const {
    auto data_value = data_->error_code;
    return mojo::internal::Deserialize<::mojo_base::mojom::FileError>(
        data_value, output);
  }

  ::mojo_base::mojom::FileError error_code() const {
    return static_cast<::mojo_base::mojom::FileError>(data_->error_code);
  }
 private:
  internal::FileSystemManager_ResolveURL_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_Move_ParamsDataView {
 public:
  FileSystemManager_Move_ParamsDataView() {}

  FileSystemManager_Move_ParamsDataView(
      internal::FileSystemManager_Move_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetSrcPathDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSrcPath(UserType* output) {
    auto* pointer = data_->src_path.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  inline void GetDestPathDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDestPath(UserType* output) {
    auto* pointer = data_->dest_path.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
 private:
  internal::FileSystemManager_Move_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_Move_ResponseParamsDataView {
 public:
  FileSystemManager_Move_ResponseParamsDataView() {}

  FileSystemManager_Move_ResponseParamsDataView(
      internal::FileSystemManager_Move_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadErrorCode(UserType* output) const {
    auto data_value = data_->error_code;
    return mojo::internal::Deserialize<::mojo_base::mojom::FileError>(
        data_value, output);
  }

  ::mojo_base::mojom::FileError error_code() const {
    return static_cast<::mojo_base::mojom::FileError>(data_->error_code);
  }
 private:
  internal::FileSystemManager_Move_ResponseParams_Data* data_ = nullptr;
};

class FileSystemManager_Copy_ParamsDataView {
 public:
  FileSystemManager_Copy_ParamsDataView() {}

  FileSystemManager_Copy_ParamsDataView(
      internal::FileSystemManager_Copy_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetSrcPathDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSrcPath(UserType* output) {
    auto* pointer = data_->src_path.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  inline void GetDestPathDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDestPath(UserType* output) {
    auto* pointer = data_->dest_path.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
 private:
  internal::FileSystemManager_Copy_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_Copy_ResponseParamsDataView {
 public:
  FileSystemManager_Copy_ResponseParamsDataView() {}

  FileSystemManager_Copy_ResponseParamsDataView(
      internal::FileSystemManager_Copy_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadErrorCode(UserType* output) const {
    auto data_value = data_->error_code;
    return mojo::internal::Deserialize<::mojo_base::mojom::FileError>(
        data_value, output);
  }

  ::mojo_base::mojom::FileError error_code() const {
    return static_cast<::mojo_base::mojom::FileError>(data_->error_code);
  }
 private:
  internal::FileSystemManager_Copy_ResponseParams_Data* data_ = nullptr;
};

class FileSystemManager_Remove_ParamsDataView {
 public:
  FileSystemManager_Remove_ParamsDataView() {}

  FileSystemManager_Remove_ParamsDataView(
      internal::FileSystemManager_Remove_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetPathDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPath(UserType* output) {
    auto* pointer = data_->path.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  bool recursive() const {
    return data_->recursive;
  }
 private:
  internal::FileSystemManager_Remove_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_Remove_ResponseParamsDataView {
 public:
  FileSystemManager_Remove_ResponseParamsDataView() {}

  FileSystemManager_Remove_ResponseParamsDataView(
      internal::FileSystemManager_Remove_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadErrorCode(UserType* output) const {
    auto data_value = data_->error_code;
    return mojo::internal::Deserialize<::mojo_base::mojom::FileError>(
        data_value, output);
  }

  ::mojo_base::mojom::FileError error_code() const {
    return static_cast<::mojo_base::mojom::FileError>(data_->error_code);
  }
 private:
  internal::FileSystemManager_Remove_ResponseParams_Data* data_ = nullptr;
};

class FileSystemManager_ReadMetadata_ParamsDataView {
 public:
  FileSystemManager_ReadMetadata_ParamsDataView() {}

  FileSystemManager_ReadMetadata_ParamsDataView(
      internal::FileSystemManager_ReadMetadata_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetPathDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPath(UserType* output) {
    auto* pointer = data_->path.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
 private:
  internal::FileSystemManager_ReadMetadata_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_ReadMetadata_ResponseParamsDataView {
 public:
  FileSystemManager_ReadMetadata_ResponseParamsDataView() {}

  FileSystemManager_ReadMetadata_ResponseParamsDataView(
      internal::FileSystemManager_ReadMetadata_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetFileInfoDataView(
      ::mojo_base::mojom::FileInfoDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFileInfo(UserType* output) {
    auto* pointer = data_->file_info.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::FileInfoDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadErrorCode(UserType* output) const {
    auto data_value = data_->error_code;
    return mojo::internal::Deserialize<::mojo_base::mojom::FileError>(
        data_value, output);
  }

  ::mojo_base::mojom::FileError error_code() const {
    return static_cast<::mojo_base::mojom::FileError>(data_->error_code);
  }
 private:
  internal::FileSystemManager_ReadMetadata_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_Create_ParamsDataView {
 public:
  FileSystemManager_Create_ParamsDataView() {}

  FileSystemManager_Create_ParamsDataView(
      internal::FileSystemManager_Create_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetPathDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPath(UserType* output) {
    auto* pointer = data_->path.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  bool exclusive() const {
    return data_->exclusive;
  }
  bool is_directory() const {
    return data_->is_directory;
  }
  bool recursive() const {
    return data_->recursive;
  }
 private:
  internal::FileSystemManager_Create_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_Create_ResponseParamsDataView {
 public:
  FileSystemManager_Create_ResponseParamsDataView() {}

  FileSystemManager_Create_ResponseParamsDataView(
      internal::FileSystemManager_Create_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadErrorCode(UserType* output) const {
    auto data_value = data_->error_code;
    return mojo::internal::Deserialize<::mojo_base::mojom::FileError>(
        data_value, output);
  }

  ::mojo_base::mojom::FileError error_code() const {
    return static_cast<::mojo_base::mojom::FileError>(data_->error_code);
  }
 private:
  internal::FileSystemManager_Create_ResponseParams_Data* data_ = nullptr;
};

class FileSystemManager_Exists_ParamsDataView {
 public:
  FileSystemManager_Exists_ParamsDataView() {}

  FileSystemManager_Exists_ParamsDataView(
      internal::FileSystemManager_Exists_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetPathDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPath(UserType* output) {
    auto* pointer = data_->path.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  bool is_directory() const {
    return data_->is_directory;
  }
 private:
  internal::FileSystemManager_Exists_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_Exists_ResponseParamsDataView {
 public:
  FileSystemManager_Exists_ResponseParamsDataView() {}

  FileSystemManager_Exists_ResponseParamsDataView(
      internal::FileSystemManager_Exists_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadErrorCode(UserType* output) const {
    auto data_value = data_->error_code;
    return mojo::internal::Deserialize<::mojo_base::mojom::FileError>(
        data_value, output);
  }

  ::mojo_base::mojom::FileError error_code() const {
    return static_cast<::mojo_base::mojom::FileError>(data_->error_code);
  }
 private:
  internal::FileSystemManager_Exists_ResponseParams_Data* data_ = nullptr;
};

class FileSystemManager_ReadDirectory_ParamsDataView {
 public:
  FileSystemManager_ReadDirectory_ParamsDataView() {}

  FileSystemManager_ReadDirectory_ParamsDataView(
      internal::FileSystemManager_ReadDirectory_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetPathDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPath(UserType* output) {
    auto* pointer = data_->path.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  UserType TakeListener() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::blink::mojom::FileSystemOperationListenerPtrDataView>(
            &data_->listener, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::FileSystemManager_ReadDirectory_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_ReadDirectorySync_ParamsDataView {
 public:
  FileSystemManager_ReadDirectorySync_ParamsDataView() {}

  FileSystemManager_ReadDirectorySync_ParamsDataView(
      internal::FileSystemManager_ReadDirectorySync_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetPathDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPath(UserType* output) {
    auto* pointer = data_->path.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
 private:
  internal::FileSystemManager_ReadDirectorySync_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_ReadDirectorySync_ResponseParamsDataView {
 public:
  FileSystemManager_ReadDirectorySync_ResponseParamsDataView() {}

  FileSystemManager_ReadDirectorySync_ResponseParamsDataView(
      internal::FileSystemManager_ReadDirectorySync_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetEntriesDataView(
      mojo::ArrayDataView<::filesystem::mojom::DirectoryEntryDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadEntries(UserType* output) {
    auto* pointer = data_->entries.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::filesystem::mojom::DirectoryEntryDataView>>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadErrorCode(UserType* output) const {
    auto data_value = data_->error_code;
    return mojo::internal::Deserialize<::mojo_base::mojom::FileError>(
        data_value, output);
  }

  ::mojo_base::mojom::FileError error_code() const {
    return static_cast<::mojo_base::mojom::FileError>(data_->error_code);
  }
 private:
  internal::FileSystemManager_ReadDirectorySync_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_Write_ParamsDataView {
 public:
  FileSystemManager_Write_ParamsDataView() {}

  FileSystemManager_Write_ParamsDataView(
      internal::FileSystemManager_Write_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetFilePathDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFilePath(UserType* output) {
    auto* pointer = data_->file_path.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  inline void GetBlobUuidDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadBlobUuid(UserType* output) {
    auto* pointer = data_->blob_uuid.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  int64_t position() const {
    return data_->position;
  }
  template <typename UserType>
  UserType TakeOpRequest() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::blink::mojom::FileSystemCancellableOperationRequestDataView>(
            &data_->op_request, &result, context_);
    DCHECK(ret);
    return result;
  }
  template <typename UserType>
  UserType TakeListener() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::blink::mojom::FileSystemOperationListenerPtrDataView>(
            &data_->listener, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::FileSystemManager_Write_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_WriteSync_ParamsDataView {
 public:
  FileSystemManager_WriteSync_ParamsDataView() {}

  FileSystemManager_WriteSync_ParamsDataView(
      internal::FileSystemManager_WriteSync_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetFilePathDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFilePath(UserType* output) {
    auto* pointer = data_->file_path.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  inline void GetBlobUuidDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadBlobUuid(UserType* output) {
    auto* pointer = data_->blob_uuid.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  int64_t position() const {
    return data_->position;
  }
 private:
  internal::FileSystemManager_WriteSync_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_WriteSync_ResponseParamsDataView {
 public:
  FileSystemManager_WriteSync_ResponseParamsDataView() {}

  FileSystemManager_WriteSync_ResponseParamsDataView(
      internal::FileSystemManager_WriteSync_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  int64_t byte_count() const {
    return data_->byte_count;
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadErrorCode(UserType* output) const {
    auto data_value = data_->error_code;
    return mojo::internal::Deserialize<::mojo_base::mojom::FileError>(
        data_value, output);
  }

  ::mojo_base::mojom::FileError error_code() const {
    return static_cast<::mojo_base::mojom::FileError>(data_->error_code);
  }
 private:
  internal::FileSystemManager_WriteSync_ResponseParams_Data* data_ = nullptr;
};

class FileSystemManager_Truncate_ParamsDataView {
 public:
  FileSystemManager_Truncate_ParamsDataView() {}

  FileSystemManager_Truncate_ParamsDataView(
      internal::FileSystemManager_Truncate_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetFilePathDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFilePath(UserType* output) {
    auto* pointer = data_->file_path.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  int64_t length() const {
    return data_->length;
  }
  template <typename UserType>
  UserType TakeOpRequest() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::blink::mojom::FileSystemCancellableOperationRequestDataView>(
            &data_->op_request, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::FileSystemManager_Truncate_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_Truncate_ResponseParamsDataView {
 public:
  FileSystemManager_Truncate_ResponseParamsDataView() {}

  FileSystemManager_Truncate_ResponseParamsDataView(
      internal::FileSystemManager_Truncate_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadErrorCode(UserType* output) const {
    auto data_value = data_->error_code;
    return mojo::internal::Deserialize<::mojo_base::mojom::FileError>(
        data_value, output);
  }

  ::mojo_base::mojom::FileError error_code() const {
    return static_cast<::mojo_base::mojom::FileError>(data_->error_code);
  }
 private:
  internal::FileSystemManager_Truncate_ResponseParams_Data* data_ = nullptr;
};

class FileSystemManager_TruncateSync_ParamsDataView {
 public:
  FileSystemManager_TruncateSync_ParamsDataView() {}

  FileSystemManager_TruncateSync_ParamsDataView(
      internal::FileSystemManager_TruncateSync_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetFilePathDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFilePath(UserType* output) {
    auto* pointer = data_->file_path.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  int64_t length() const {
    return data_->length;
  }
 private:
  internal::FileSystemManager_TruncateSync_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_TruncateSync_ResponseParamsDataView {
 public:
  FileSystemManager_TruncateSync_ResponseParamsDataView() {}

  FileSystemManager_TruncateSync_ResponseParamsDataView(
      internal::FileSystemManager_TruncateSync_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadErrorCode(UserType* output) const {
    auto data_value = data_->error_code;
    return mojo::internal::Deserialize<::mojo_base::mojom::FileError>(
        data_value, output);
  }

  ::mojo_base::mojom::FileError error_code() const {
    return static_cast<::mojo_base::mojom::FileError>(data_->error_code);
  }
 private:
  internal::FileSystemManager_TruncateSync_ResponseParams_Data* data_ = nullptr;
};

class FileSystemManager_TouchFile_ParamsDataView {
 public:
  FileSystemManager_TouchFile_ParamsDataView() {}

  FileSystemManager_TouchFile_ParamsDataView(
      internal::FileSystemManager_TouchFile_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetPathDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPath(UserType* output) {
    auto* pointer = data_->path.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  inline void GetLastAccessTimeDataView(
      ::mojo_base::mojom::TimeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadLastAccessTime(UserType* output) {
    auto* pointer = data_->last_access_time.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::TimeDataView>(
        pointer, output, context_);
  }
  inline void GetLastModifiedTimeDataView(
      ::mojo_base::mojom::TimeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadLastModifiedTime(UserType* output) {
    auto* pointer = data_->last_modified_time.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::TimeDataView>(
        pointer, output, context_);
  }
 private:
  internal::FileSystemManager_TouchFile_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_TouchFile_ResponseParamsDataView {
 public:
  FileSystemManager_TouchFile_ResponseParamsDataView() {}

  FileSystemManager_TouchFile_ResponseParamsDataView(
      internal::FileSystemManager_TouchFile_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadErrorCode(UserType* output) const {
    auto data_value = data_->error_code;
    return mojo::internal::Deserialize<::mojo_base::mojom::FileError>(
        data_value, output);
  }

  ::mojo_base::mojom::FileError error_code() const {
    return static_cast<::mojo_base::mojom::FileError>(data_->error_code);
  }
 private:
  internal::FileSystemManager_TouchFile_ResponseParams_Data* data_ = nullptr;
};

class FileSystemManager_CreateSnapshotFile_ParamsDataView {
 public:
  FileSystemManager_CreateSnapshotFile_ParamsDataView() {}

  FileSystemManager_CreateSnapshotFile_ParamsDataView(
      internal::FileSystemManager_CreateSnapshotFile_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetFilePathDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFilePath(UserType* output) {
    auto* pointer = data_->file_path.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
 private:
  internal::FileSystemManager_CreateSnapshotFile_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_CreateSnapshotFile_ResponseParamsDataView {
 public:
  FileSystemManager_CreateSnapshotFile_ResponseParamsDataView() {}

  FileSystemManager_CreateSnapshotFile_ResponseParamsDataView(
      internal::FileSystemManager_CreateSnapshotFile_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetFileInfoDataView(
      ::mojo_base::mojom::FileInfoDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFileInfo(UserType* output) {
    auto* pointer = data_->file_info.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::FileInfoDataView>(
        pointer, output, context_);
  }
  inline void GetPlatformPathDataView(
      ::mojo_base::mojom::FilePathDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPlatformPath(UserType* output) {
    auto* pointer = data_->platform_path.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::FilePathDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadErrorCode(UserType* output) const {
    auto data_value = data_->error_code;
    return mojo::internal::Deserialize<::mojo_base::mojom::FileError>(
        data_value, output);
  }

  ::mojo_base::mojom::FileError error_code() const {
    return static_cast<::mojo_base::mojom::FileError>(data_->error_code);
  }
  template <typename UserType>
  UserType TakeSnapshotListener() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::blink::mojom::ReceivedSnapshotListenerPtrDataView>(
            &data_->snapshot_listener, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::FileSystemManager_CreateSnapshotFile_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_GetPlatformPath_ParamsDataView {
 public:
  FileSystemManager_GetPlatformPath_ParamsDataView() {}

  FileSystemManager_GetPlatformPath_ParamsDataView(
      internal::FileSystemManager_GetPlatformPath_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetFilePathDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFilePath(UserType* output) {
    auto* pointer = data_->file_path.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
 private:
  internal::FileSystemManager_GetPlatformPath_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_GetPlatformPath_ResponseParamsDataView {
 public:
  FileSystemManager_GetPlatformPath_ResponseParamsDataView() {}

  FileSystemManager_GetPlatformPath_ResponseParamsDataView(
      internal::FileSystemManager_GetPlatformPath_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetPlatformPathDataView(
      ::mojo_base::mojom::FilePathDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPlatformPath(UserType* output) {
    auto* pointer = data_->platform_path.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::FilePathDataView>(
        pointer, output, context_);
  }
 private:
  internal::FileSystemManager_GetPlatformPath_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_CreateWriter_ParamsDataView {
 public:
  FileSystemManager_CreateWriter_ParamsDataView() {}

  FileSystemManager_CreateWriter_ParamsDataView(
      internal::FileSystemManager_CreateWriter_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetFilePathDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFilePath(UserType* output) {
    auto* pointer = data_->file_path.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
 private:
  internal::FileSystemManager_CreateWriter_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_CreateWriter_ResponseParamsDataView {
 public:
  FileSystemManager_CreateWriter_ResponseParamsDataView() {}

  FileSystemManager_CreateWriter_ResponseParamsDataView(
      internal::FileSystemManager_CreateWriter_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadResult(UserType* output) const {
    auto data_value = data_->result;
    return mojo::internal::Deserialize<::mojo_base::mojom::FileError>(
        data_value, output);
  }

  ::mojo_base::mojom::FileError result() const {
    return static_cast<::mojo_base::mojom::FileError>(data_->result);
  }
  template <typename UserType>
  UserType TakeWriter() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::blink::mojom::FileWriterPtrDataView>(
            &data_->writer, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::FileSystemManager_CreateWriter_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_ChooseEntry_ParamsDataView {
 public:
  FileSystemManager_ChooseEntry_ParamsDataView() {}

  FileSystemManager_ChooseEntry_ParamsDataView(
      internal::FileSystemManager_ChooseEntry_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::blink::mojom::ChooseFileSystemEntryType>(
        data_value, output);
  }

  ChooseFileSystemEntryType type() const {
    return static_cast<ChooseFileSystemEntryType>(data_->type);
  }
  inline void GetAcceptsDataView(
      mojo::ArrayDataView<ChooseFileSystemEntryAcceptsOptionDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAccepts(UserType* output) {
    auto* pointer = data_->accepts.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::blink::mojom::ChooseFileSystemEntryAcceptsOptionDataView>>(
        pointer, output, context_);
  }
  bool include_accepts_all() const {
    return data_->include_accepts_all;
  }
 private:
  internal::FileSystemManager_ChooseEntry_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileSystemManager_ChooseEntry_ResponseParamsDataView {
 public:
  FileSystemManager_ChooseEntry_ResponseParamsDataView() {}

  FileSystemManager_ChooseEntry_ResponseParamsDataView(
      internal::FileSystemManager_ChooseEntry_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadErrorCode(UserType* output) const {
    auto data_value = data_->error_code;
    return mojo::internal::Deserialize<::mojo_base::mojom::FileError>(
        data_value, output);
  }

  ::mojo_base::mojom::FileError error_code() const {
    return static_cast<::mojo_base::mojom::FileError>(data_->error_code);
  }
  inline void GetEntriesDataView(
      mojo::ArrayDataView<FileSystemEntryDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadEntries(UserType* output) {
    auto* pointer = data_->entries.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::blink::mojom::FileSystemEntryDataView>>(
        pointer, output, context_);
  }
 private:
  internal::FileSystemManager_ChooseEntry_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace blink

namespace std {

template <>
struct hash<::blink::mojom::FileSystemType>
    : public mojo::internal::EnumHashImpl<::blink::mojom::FileSystemType> {};

template <>
struct hash<::blink::mojom::ChooseFileSystemEntryType>
    : public mojo::internal::EnumHashImpl<::blink::mojom::ChooseFileSystemEntryType> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::blink::mojom::FileSystemType, ::blink::mojom::FileSystemType> {
  static ::blink::mojom::FileSystemType ToMojom(::blink::mojom::FileSystemType input) { return input; }
  static bool FromMojom(::blink::mojom::FileSystemType input, ::blink::mojom::FileSystemType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::blink::mojom::FileSystemType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::blink::mojom::FileSystemType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::blink::mojom::FileSystemType>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::blink::mojom::ChooseFileSystemEntryType, ::blink::mojom::ChooseFileSystemEntryType> {
  static ::blink::mojom::ChooseFileSystemEntryType ToMojom(::blink::mojom::ChooseFileSystemEntryType input) { return input; }
  static bool FromMojom(::blink::mojom::ChooseFileSystemEntryType input, ::blink::mojom::ChooseFileSystemEntryType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::blink::mojom::ChooseFileSystemEntryType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::blink::mojom::ChooseFileSystemEntryType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::blink::mojom::ChooseFileSystemEntryType>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::blink::mojom::FileSystemInfoDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::blink::mojom::FileSystemInfoDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::blink::mojom::internal::FileSystemInfo_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    decltype(CallWithContext(Traits::name, input, custom_context)) in_name = CallWithContext(Traits::name, input, custom_context);
    typename decltype((*output)->name)::BaseType::BufferWriter
        name_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_name, buffer, &name_writer, context);
    (*output)->name.Set(
        name_writer.is_null() ? nullptr : name_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->name.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null name in FileSystemInfo struct");
    decltype(CallWithContext(Traits::root_url, input, custom_context)) in_root_url = CallWithContext(Traits::root_url, input, custom_context);
    typename decltype((*output)->root_url)::BaseType::BufferWriter
        root_url_writer;
    mojo::internal::Serialize<::url::mojom::UrlDataView>(
        in_root_url, buffer, &root_url_writer, context);
    (*output)->root_url.Set(
        root_url_writer.is_null() ? nullptr : root_url_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->root_url.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null root_url in FileSystemInfo struct");
    mojo::internal::Serialize<::blink::mojom::FileSystemType>(
        CallWithContext(Traits::mount_type, input, custom_context), &(*output)->mount_type);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::blink::mojom::internal::FileSystemInfo_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::blink::mojom::FileSystemInfoDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::blink::mojom::FileSystemEntryDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::blink::mojom::FileSystemEntryDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::blink::mojom::internal::FileSystemEntry_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    decltype(CallWithContext(Traits::file_system_id, input, custom_context)) in_file_system_id = CallWithContext(Traits::file_system_id, input, custom_context);
    typename decltype((*output)->file_system_id)::BaseType::BufferWriter
        file_system_id_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_file_system_id, buffer, &file_system_id_writer, context);
    (*output)->file_system_id.Set(
        file_system_id_writer.is_null() ? nullptr : file_system_id_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->file_system_id.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null file_system_id in FileSystemEntry struct");
    decltype(CallWithContext(Traits::base_name, input, custom_context)) in_base_name = CallWithContext(Traits::base_name, input, custom_context);
    typename decltype((*output)->base_name)::BaseType::BufferWriter
        base_name_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_base_name, buffer, &base_name_writer, context);
    (*output)->base_name.Set(
        base_name_writer.is_null() ? nullptr : base_name_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->base_name.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null base_name in FileSystemEntry struct");
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::blink::mojom::internal::FileSystemEntry_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::blink::mojom::FileSystemEntryDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::blink::mojom::ChooseFileSystemEntryAcceptsOptionDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::blink::mojom::ChooseFileSystemEntryAcceptsOptionDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::blink::mojom::internal::ChooseFileSystemEntryAcceptsOption_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    decltype(CallWithContext(Traits::description, input, custom_context)) in_description = CallWithContext(Traits::description, input, custom_context);
    typename decltype((*output)->description)::BaseType::BufferWriter
        description_writer;
    mojo::internal::Serialize<::mojo_base::mojom::String16DataView>(
        in_description, buffer, &description_writer, context);
    (*output)->description.Set(
        description_writer.is_null() ? nullptr : description_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->description.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null description in ChooseFileSystemEntryAcceptsOption struct");
    decltype(CallWithContext(Traits::mime_types, input, custom_context)) in_mime_types = CallWithContext(Traits::mime_types, input, custom_context);
    typename decltype((*output)->mime_types)::BaseType::BufferWriter
        mime_types_writer;
    const mojo::internal::ContainerValidateParams mime_types_validate_params(
        0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
    mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
        in_mime_types, buffer, &mime_types_writer, &mime_types_validate_params,
        context);
    (*output)->mime_types.Set(
        mime_types_writer.is_null() ? nullptr : mime_types_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->mime_types.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null mime_types in ChooseFileSystemEntryAcceptsOption struct");
    decltype(CallWithContext(Traits::extensions, input, custom_context)) in_extensions = CallWithContext(Traits::extensions, input, custom_context);
    typename decltype((*output)->extensions)::BaseType::BufferWriter
        extensions_writer;
    const mojo::internal::ContainerValidateParams extensions_validate_params(
        0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
    mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
        in_extensions, buffer, &extensions_writer, &extensions_validate_params,
        context);
    (*output)->extensions.Set(
        extensions_writer.is_null() ? nullptr : extensions_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->extensions.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null extensions in ChooseFileSystemEntryAcceptsOption struct");
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::blink::mojom::internal::ChooseFileSystemEntryAcceptsOption_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::blink::mojom::ChooseFileSystemEntryAcceptsOptionDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace blink {
namespace mojom {

inline void FileSystemInfoDataView::GetNameDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->name.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void FileSystemInfoDataView::GetRootUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->root_url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}


inline void FileSystemEntryDataView::GetFileSystemIdDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->file_system_id.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void FileSystemEntryDataView::GetBaseNameDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->base_name.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void ChooseFileSystemEntryAcceptsOptionDataView::GetDescriptionDataView(
    ::mojo_base::mojom::String16DataView* output) {
  auto pointer = data_->description.Get();
  *output = ::mojo_base::mojom::String16DataView(pointer, context_);
}
inline void ChooseFileSystemEntryAcceptsOptionDataView::GetMimeTypesDataView(
    mojo::ArrayDataView<mojo::StringDataView>* output) {
  auto pointer = data_->mime_types.Get();
  *output = mojo::ArrayDataView<mojo::StringDataView>(pointer, context_);
}
inline void ChooseFileSystemEntryAcceptsOptionDataView::GetExtensionsDataView(
    mojo::ArrayDataView<mojo::StringDataView>* output) {
  auto pointer = data_->extensions.Get();
  *output = mojo::ArrayDataView<mojo::StringDataView>(pointer, context_);
}






inline void FileSystemOperationListener_ResultsRetrieved_ParamsDataView::GetEntriesDataView(
    mojo::ArrayDataView<::filesystem::mojom::DirectoryEntryDataView>* output) {
  auto pointer = data_->entries.Get();
  *output = mojo::ArrayDataView<::filesystem::mojom::DirectoryEntryDataView>(pointer, context_);
}








inline void FileSystemManager_Open_ParamsDataView::GetOriginDataView(
    ::url::mojom::OriginDataView* output) {
  auto pointer = data_->origin.Get();
  *output = ::url::mojom::OriginDataView(pointer, context_);
}


inline void FileSystemManager_Open_ResponseParamsDataView::GetNameDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->name.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void FileSystemManager_Open_ResponseParamsDataView::GetRootUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->root_url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}


inline void FileSystemManager_ResolveURL_ParamsDataView::GetFilesystemUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->filesystem_url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}


inline void FileSystemManager_ResolveURL_ResponseParamsDataView::GetInfoDataView(
    FileSystemInfoDataView* output) {
  auto pointer = data_->info.Get();
  *output = FileSystemInfoDataView(pointer, context_);
}
inline void FileSystemManager_ResolveURL_ResponseParamsDataView::GetFilePathDataView(
    ::mojo_base::mojom::FilePathDataView* output) {
  auto pointer = data_->file_path.Get();
  *output = ::mojo_base::mojom::FilePathDataView(pointer, context_);
}


inline void FileSystemManager_Move_ParamsDataView::GetSrcPathDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->src_path.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}
inline void FileSystemManager_Move_ParamsDataView::GetDestPathDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->dest_path.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}




inline void FileSystemManager_Copy_ParamsDataView::GetSrcPathDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->src_path.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}
inline void FileSystemManager_Copy_ParamsDataView::GetDestPathDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->dest_path.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}




inline void FileSystemManager_Remove_ParamsDataView::GetPathDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->path.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}




inline void FileSystemManager_ReadMetadata_ParamsDataView::GetPathDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->path.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}


inline void FileSystemManager_ReadMetadata_ResponseParamsDataView::GetFileInfoDataView(
    ::mojo_base::mojom::FileInfoDataView* output) {
  auto pointer = data_->file_info.Get();
  *output = ::mojo_base::mojom::FileInfoDataView(pointer, context_);
}


inline void FileSystemManager_Create_ParamsDataView::GetPathDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->path.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}




inline void FileSystemManager_Exists_ParamsDataView::GetPathDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->path.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}




inline void FileSystemManager_ReadDirectory_ParamsDataView::GetPathDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->path.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}


inline void FileSystemManager_ReadDirectorySync_ParamsDataView::GetPathDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->path.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}


inline void FileSystemManager_ReadDirectorySync_ResponseParamsDataView::GetEntriesDataView(
    mojo::ArrayDataView<::filesystem::mojom::DirectoryEntryDataView>* output) {
  auto pointer = data_->entries.Get();
  *output = mojo::ArrayDataView<::filesystem::mojom::DirectoryEntryDataView>(pointer, context_);
}


inline void FileSystemManager_Write_ParamsDataView::GetFilePathDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->file_path.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}
inline void FileSystemManager_Write_ParamsDataView::GetBlobUuidDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->blob_uuid.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void FileSystemManager_WriteSync_ParamsDataView::GetFilePathDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->file_path.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}
inline void FileSystemManager_WriteSync_ParamsDataView::GetBlobUuidDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->blob_uuid.Get();
  *output = mojo::StringDataView(pointer, context_);
}




inline void FileSystemManager_Truncate_ParamsDataView::GetFilePathDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->file_path.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}




inline void FileSystemManager_TruncateSync_ParamsDataView::GetFilePathDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->file_path.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}




inline void FileSystemManager_TouchFile_ParamsDataView::GetPathDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->path.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}
inline void FileSystemManager_TouchFile_ParamsDataView::GetLastAccessTimeDataView(
    ::mojo_base::mojom::TimeDataView* output) {
  auto pointer = data_->last_access_time.Get();
  *output = ::mojo_base::mojom::TimeDataView(pointer, context_);
}
inline void FileSystemManager_TouchFile_ParamsDataView::GetLastModifiedTimeDataView(
    ::mojo_base::mojom::TimeDataView* output) {
  auto pointer = data_->last_modified_time.Get();
  *output = ::mojo_base::mojom::TimeDataView(pointer, context_);
}




inline void FileSystemManager_CreateSnapshotFile_ParamsDataView::GetFilePathDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->file_path.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}


inline void FileSystemManager_CreateSnapshotFile_ResponseParamsDataView::GetFileInfoDataView(
    ::mojo_base::mojom::FileInfoDataView* output) {
  auto pointer = data_->file_info.Get();
  *output = ::mojo_base::mojom::FileInfoDataView(pointer, context_);
}
inline void FileSystemManager_CreateSnapshotFile_ResponseParamsDataView::GetPlatformPathDataView(
    ::mojo_base::mojom::FilePathDataView* output) {
  auto pointer = data_->platform_path.Get();
  *output = ::mojo_base::mojom::FilePathDataView(pointer, context_);
}


inline void FileSystemManager_GetPlatformPath_ParamsDataView::GetFilePathDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->file_path.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}


inline void FileSystemManager_GetPlatformPath_ResponseParamsDataView::GetPlatformPathDataView(
    ::mojo_base::mojom::FilePathDataView* output) {
  auto pointer = data_->platform_path.Get();
  *output = ::mojo_base::mojom::FilePathDataView(pointer, context_);
}


inline void FileSystemManager_CreateWriter_ParamsDataView::GetFilePathDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->file_path.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}




inline void FileSystemManager_ChooseEntry_ParamsDataView::GetAcceptsDataView(
    mojo::ArrayDataView<ChooseFileSystemEntryAcceptsOptionDataView>* output) {
  auto pointer = data_->accepts.Get();
  *output = mojo::ArrayDataView<ChooseFileSystemEntryAcceptsOptionDataView>(pointer, context_);
}


inline void FileSystemManager_ChooseEntry_ResponseParamsDataView::GetEntriesDataView(
    mojo::ArrayDataView<FileSystemEntryDataView>* output) {
  auto pointer = data_->entries.Get();
  *output = mojo::ArrayDataView<FileSystemEntryDataView>(pointer, context_);
}



}  // namespace mojom
}  // namespace blink

#endif  // THIRD_PARTY_BLINK_PUBLIC_MOJOM_FILESYSTEM_FILE_SYSTEM_MOJOM_SHARED_H_