// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_PUBLIC_MOJOM_FILESYSTEM_FILE_WRITER_MOJOM_SHARED_H_
#define THIRD_PARTY_BLINK_PUBLIC_MOJOM_FILESYSTEM_FILE_WRITER_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "third_party/blink/public/mojom/filesystem/file_writer.mojom-shared-internal.h"
#include "mojo/public/mojom/base/file_error.mojom-shared.h"
#include "third_party/blink/public/mojom/blob/blob.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "base/component_export.h"


namespace blink {
namespace mojom {


}  // namespace mojom
}  // namespace blink

namespace mojo {
namespace internal {

}  // namespace internal
}  // namespace mojo


namespace blink {
namespace mojom {
// Interface base classes. They are used for type safety check.
class FileWriterInterfaceBase {};

using FileWriterPtrDataView =
    mojo::InterfacePtrDataView<FileWriterInterfaceBase>;
using FileWriterRequestDataView =
    mojo::InterfaceRequestDataView<FileWriterInterfaceBase>;
using FileWriterAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<FileWriterInterfaceBase>;
using FileWriterAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<FileWriterInterfaceBase>;
class FileWriter_Write_ParamsDataView {
 public:
  FileWriter_Write_ParamsDataView() {}

  FileWriter_Write_ParamsDataView(
      internal::FileWriter_Write_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  uint64_t position() const {
    return data_->position;
  }
  template <typename UserType>
  UserType TakeBlob() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::blink::mojom::BlobPtrDataView>(
            &data_->blob, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::FileWriter_Write_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileWriter_Write_ResponseParamsDataView {
 public:
  FileWriter_Write_ResponseParamsDataView() {}

  FileWriter_Write_ResponseParamsDataView(
      internal::FileWriter_Write_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadResult(UserType* output) const {
    auto data_value = data_->result;
    return mojo::internal::Deserialize<::mojo_base::mojom::FileError>(
        data_value, output);
  }

  ::mojo_base::mojom::FileError result() const {
    return static_cast<::mojo_base::mojom::FileError>(data_->result);
  }
  uint64_t bytes_written() const {
    return data_->bytes_written;
  }
 private:
  internal::FileWriter_Write_ResponseParams_Data* data_ = nullptr;
};

class FileWriter_WriteStream_ParamsDataView {
 public:
  FileWriter_WriteStream_ParamsDataView() {}

  FileWriter_WriteStream_ParamsDataView(
      internal::FileWriter_WriteStream_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  uint64_t position() const {
    return data_->position;
  }
  mojo::ScopedDataPipeConsumerHandle TakeStream() {
    mojo::ScopedDataPipeConsumerHandle result;
    bool ret =
        mojo::internal::Deserialize<mojo::ScopedDataPipeConsumerHandle>(
            &data_->stream, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::FileWriter_WriteStream_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FileWriter_WriteStream_ResponseParamsDataView {
 public:
  FileWriter_WriteStream_ResponseParamsDataView() {}

  FileWriter_WriteStream_ResponseParamsDataView(
      internal::FileWriter_WriteStream_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadResult(UserType* output) const {
    auto data_value = data_->result;
    return mojo::internal::Deserialize<::mojo_base::mojom::FileError>(
        data_value, output);
  }

  ::mojo_base::mojom::FileError result() const {
    return static_cast<::mojo_base::mojom::FileError>(data_->result);
  }
  uint64_t bytes_written() const {
    return data_->bytes_written;
  }
 private:
  internal::FileWriter_WriteStream_ResponseParams_Data* data_ = nullptr;
};

class FileWriter_Truncate_ParamsDataView {
 public:
  FileWriter_Truncate_ParamsDataView() {}

  FileWriter_Truncate_ParamsDataView(
      internal::FileWriter_Truncate_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  uint64_t length() const {
    return data_->length;
  }
 private:
  internal::FileWriter_Truncate_Params_Data* data_ = nullptr;
};

class FileWriter_Truncate_ResponseParamsDataView {
 public:
  FileWriter_Truncate_ResponseParamsDataView() {}

  FileWriter_Truncate_ResponseParamsDataView(
      internal::FileWriter_Truncate_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadResult(UserType* output) const {
    auto data_value = data_->result;
    return mojo::internal::Deserialize<::mojo_base::mojom::FileError>(
        data_value, output);
  }

  ::mojo_base::mojom::FileError result() const {
    return static_cast<::mojo_base::mojom::FileError>(data_->result);
  }
 private:
  internal::FileWriter_Truncate_ResponseParams_Data* data_ = nullptr;
};



}  // namespace mojom
}  // namespace blink

namespace std {

}  // namespace std

namespace mojo {

}  // namespace mojo


namespace blink {
namespace mojom {














}  // namespace mojom
}  // namespace blink

#endif  // THIRD_PARTY_BLINK_PUBLIC_MOJOM_FILESYSTEM_FILE_WRITER_MOJOM_SHARED_H_