// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_PUBLIC_MOJOM_FRAME_FIND_IN_PAGE_MOJOM_SHARED_H_
#define THIRD_PARTY_BLINK_PUBLIC_MOJOM_FRAME_FIND_IN_PAGE_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "third_party/blink/public/mojom/frame/find_in_page.mojom-shared-internal.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "base/component_export.h"


namespace blink {
namespace mojom {
class FindOptionsDataView;



}  // namespace mojom
}  // namespace blink

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::blink::mojom::FindOptionsDataView> {
  using Data = ::blink::mojom::internal::FindOptions_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace blink {
namespace mojom {

enum class StopFindAction : int32_t {
  kStopFindActionClearSelection,
  kStopFindActionKeepSelection,
  kStopFindActionActivateSelection,
  kMinValue = 0,
  kMaxValue = 2,
};

COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) std::ostream& operator<<(std::ostream& os, StopFindAction value);
inline bool IsKnownEnumValue(StopFindAction value) {
  return internal::StopFindAction_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class FindMatchUpdateType : int32_t {
  kFinalUpdate,
  kMoreUpdatesComing,
  kMinValue = 0,
  kMaxValue = 1,
};

COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) std::ostream& operator<<(std::ostream& os, FindMatchUpdateType value);
inline bool IsKnownEnumValue(FindMatchUpdateType value) {
  return internal::FindMatchUpdateType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class FindInPageInterfaceBase {};

using FindInPagePtrDataView =
    mojo::InterfacePtrDataView<FindInPageInterfaceBase>;
using FindInPageRequestDataView =
    mojo::InterfaceRequestDataView<FindInPageInterfaceBase>;
using FindInPageAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<FindInPageInterfaceBase>;
using FindInPageAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<FindInPageInterfaceBase>;
class FindInPageClientInterfaceBase {};

using FindInPageClientPtrDataView =
    mojo::InterfacePtrDataView<FindInPageClientInterfaceBase>;
using FindInPageClientRequestDataView =
    mojo::InterfaceRequestDataView<FindInPageClientInterfaceBase>;
using FindInPageClientAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<FindInPageClientInterfaceBase>;
using FindInPageClientAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<FindInPageClientInterfaceBase>;
class FindOptionsDataView {
 public:
  FindOptionsDataView() {}

  FindOptionsDataView(
      internal::FindOptions_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool forward() const {
    return data_->forward;
  }
  bool match_case() const {
    return data_->match_case;
  }
  bool find_next() const {
    return data_->find_next;
  }
  bool force() const {
    return data_->force;
  }
  bool run_synchronously_for_testing() const {
    return data_->run_synchronously_for_testing;
  }
 private:
  internal::FindOptions_Data* data_ = nullptr;
};

class FindInPage_Find_ParamsDataView {
 public:
  FindInPage_Find_ParamsDataView() {}

  FindInPage_Find_ParamsDataView(
      internal::FindInPage_Find_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int32_t request_id() const {
    return data_->request_id;
  }
  inline void GetSearchTextDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSearchText(UserType* output) {
    auto* pointer = data_->search_text.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetOptionsDataView(
      FindOptionsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOptions(UserType* output) {
    auto* pointer = data_->options.Get();
    return mojo::internal::Deserialize<::blink::mojom::FindOptionsDataView>(
        pointer, output, context_);
  }
 private:
  internal::FindInPage_Find_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FindInPage_StopFinding_ParamsDataView {
 public:
  FindInPage_StopFinding_ParamsDataView() {}

  FindInPage_StopFinding_ParamsDataView(
      internal::FindInPage_StopFinding_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAction(UserType* output) const {
    auto data_value = data_->action;
    return mojo::internal::Deserialize<::blink::mojom::StopFindAction>(
        data_value, output);
  }

  StopFindAction action() const {
    return static_cast<StopFindAction>(data_->action);
  }
 private:
  internal::FindInPage_StopFinding_Params_Data* data_ = nullptr;
};

class FindInPage_ClearActiveFindMatch_ParamsDataView {
 public:
  FindInPage_ClearActiveFindMatch_ParamsDataView() {}

  FindInPage_ClearActiveFindMatch_ParamsDataView(
      internal::FindInPage_ClearActiveFindMatch_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::FindInPage_ClearActiveFindMatch_Params_Data* data_ = nullptr;
};

class FindInPage_GetNearestFindResult_ParamsDataView {
 public:
  FindInPage_GetNearestFindResult_ParamsDataView() {}

  FindInPage_GetNearestFindResult_ParamsDataView(
      internal::FindInPage_GetNearestFindResult_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetPointDataView(
      ::gfx::mojom::PointFDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPoint(UserType* output) {
    auto* pointer = data_->point.Get();
    return mojo::internal::Deserialize<::gfx::mojom::PointFDataView>(
        pointer, output, context_);
  }
 private:
  internal::FindInPage_GetNearestFindResult_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FindInPage_GetNearestFindResult_ResponseParamsDataView {
 public:
  FindInPage_GetNearestFindResult_ResponseParamsDataView() {}

  FindInPage_GetNearestFindResult_ResponseParamsDataView(
      internal::FindInPage_GetNearestFindResult_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  float distance() const {
    return data_->distance;
  }
 private:
  internal::FindInPage_GetNearestFindResult_ResponseParams_Data* data_ = nullptr;
};

class FindInPage_ActivateNearestFindResult_ParamsDataView {
 public:
  FindInPage_ActivateNearestFindResult_ParamsDataView() {}

  FindInPage_ActivateNearestFindResult_ParamsDataView(
      internal::FindInPage_ActivateNearestFindResult_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int32_t request_id() const {
    return data_->request_id;
  }
  inline void GetPointDataView(
      ::gfx::mojom::PointFDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPoint(UserType* output) {
    auto* pointer = data_->point.Get();
    return mojo::internal::Deserialize<::gfx::mojom::PointFDataView>(
        pointer, output, context_);
  }
 private:
  internal::FindInPage_ActivateNearestFindResult_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FindInPage_SetClient_ParamsDataView {
 public:
  FindInPage_SetClient_ParamsDataView() {}

  FindInPage_SetClient_ParamsDataView(
      internal::FindInPage_SetClient_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeClient() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::blink::mojom::FindInPageClientPtrDataView>(
            &data_->client, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::FindInPage_SetClient_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FindInPage_FindMatchRects_ParamsDataView {
 public:
  FindInPage_FindMatchRects_ParamsDataView() {}

  FindInPage_FindMatchRects_ParamsDataView(
      internal::FindInPage_FindMatchRects_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  int32_t current_version() const {
    return data_->current_version;
  }
 private:
  internal::FindInPage_FindMatchRects_Params_Data* data_ = nullptr;
};

class FindInPage_FindMatchRects_ResponseParamsDataView {
 public:
  FindInPage_FindMatchRects_ResponseParamsDataView() {}

  FindInPage_FindMatchRects_ResponseParamsDataView(
      internal::FindInPage_FindMatchRects_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int32_t version() const {
    return data_->version;
  }
  inline void GetRectsDataView(
      mojo::ArrayDataView<::gfx::mojom::RectFDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRects(UserType* output) {
    auto* pointer = data_->rects.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::gfx::mojom::RectFDataView>>(
        pointer, output, context_);
  }
  inline void GetActiveMatchRectDataView(
      ::gfx::mojom::RectFDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadActiveMatchRect(UserType* output) {
    auto* pointer = data_->active_match_rect.Get();
    return mojo::internal::Deserialize<::gfx::mojom::RectFDataView>(
        pointer, output, context_);
  }
 private:
  internal::FindInPage_FindMatchRects_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class FindInPageClient_SetNumberOfMatches_ParamsDataView {
 public:
  FindInPageClient_SetNumberOfMatches_ParamsDataView() {}

  FindInPageClient_SetNumberOfMatches_ParamsDataView(
      internal::FindInPageClient_SetNumberOfMatches_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  int32_t request_id() const {
    return data_->request_id;
  }
  uint32_t number_of_matches() const {
    return data_->number_of_matches;
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUpdateType(UserType* output) const {
    auto data_value = data_->update_type;
    return mojo::internal::Deserialize<::blink::mojom::FindMatchUpdateType>(
        data_value, output);
  }

  FindMatchUpdateType update_type() const {
    return static_cast<FindMatchUpdateType>(data_->update_type);
  }
 private:
  internal::FindInPageClient_SetNumberOfMatches_Params_Data* data_ = nullptr;
};

class FindInPageClient_SetActiveMatch_ParamsDataView {
 public:
  FindInPageClient_SetActiveMatch_ParamsDataView() {}

  FindInPageClient_SetActiveMatch_ParamsDataView(
      internal::FindInPageClient_SetActiveMatch_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  int32_t request_id() const {
    return data_->request_id;
  }
  inline void GetActiveMatchRectDataView(
      ::gfx::mojom::RectDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadActiveMatchRect(UserType* output) {
    auto* pointer = data_->active_match_rect.Get();
    return mojo::internal::Deserialize<::gfx::mojom::RectDataView>(
        pointer, output, context_);
  }
  int32_t active_match_ordinal() const {
    return data_->active_match_ordinal;
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUpdateType(UserType* output) const {
    auto data_value = data_->update_type;
    return mojo::internal::Deserialize<::blink::mojom::FindMatchUpdateType>(
        data_value, output);
  }

  FindMatchUpdateType update_type() const {
    return static_cast<FindMatchUpdateType>(data_->update_type);
  }
 private:
  internal::FindInPageClient_SetActiveMatch_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace blink

namespace std {

template <>
struct hash<::blink::mojom::StopFindAction>
    : public mojo::internal::EnumHashImpl<::blink::mojom::StopFindAction> {};

template <>
struct hash<::blink::mojom::FindMatchUpdateType>
    : public mojo::internal::EnumHashImpl<::blink::mojom::FindMatchUpdateType> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::blink::mojom::StopFindAction, ::blink::mojom::StopFindAction> {
  static ::blink::mojom::StopFindAction ToMojom(::blink::mojom::StopFindAction input) { return input; }
  static bool FromMojom(::blink::mojom::StopFindAction input, ::blink::mojom::StopFindAction* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::blink::mojom::StopFindAction, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::blink::mojom::StopFindAction, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::blink::mojom::StopFindAction>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::blink::mojom::FindMatchUpdateType, ::blink::mojom::FindMatchUpdateType> {
  static ::blink::mojom::FindMatchUpdateType ToMojom(::blink::mojom::FindMatchUpdateType input) { return input; }
  static bool FromMojom(::blink::mojom::FindMatchUpdateType input, ::blink::mojom::FindMatchUpdateType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::blink::mojom::FindMatchUpdateType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::blink::mojom::FindMatchUpdateType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::blink::mojom::FindMatchUpdateType>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::blink::mojom::FindOptionsDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::blink::mojom::FindOptionsDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::blink::mojom::internal::FindOptions_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    (*output)->forward = CallWithContext(Traits::forward, input, custom_context);
    (*output)->match_case = CallWithContext(Traits::match_case, input, custom_context);
    (*output)->find_next = CallWithContext(Traits::find_next, input, custom_context);
    (*output)->force = CallWithContext(Traits::force, input, custom_context);
    (*output)->run_synchronously_for_testing = CallWithContext(Traits::run_synchronously_for_testing, input, custom_context);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::blink::mojom::internal::FindOptions_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::blink::mojom::FindOptionsDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace blink {
namespace mojom {



inline void FindInPage_Find_ParamsDataView::GetSearchTextDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->search_text.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void FindInPage_Find_ParamsDataView::GetOptionsDataView(
    FindOptionsDataView* output) {
  auto pointer = data_->options.Get();
  *output = FindOptionsDataView(pointer, context_);
}






inline void FindInPage_GetNearestFindResult_ParamsDataView::GetPointDataView(
    ::gfx::mojom::PointFDataView* output) {
  auto pointer = data_->point.Get();
  *output = ::gfx::mojom::PointFDataView(pointer, context_);
}




inline void FindInPage_ActivateNearestFindResult_ParamsDataView::GetPointDataView(
    ::gfx::mojom::PointFDataView* output) {
  auto pointer = data_->point.Get();
  *output = ::gfx::mojom::PointFDataView(pointer, context_);
}






inline void FindInPage_FindMatchRects_ResponseParamsDataView::GetRectsDataView(
    mojo::ArrayDataView<::gfx::mojom::RectFDataView>* output) {
  auto pointer = data_->rects.Get();
  *output = mojo::ArrayDataView<::gfx::mojom::RectFDataView>(pointer, context_);
}
inline void FindInPage_FindMatchRects_ResponseParamsDataView::GetActiveMatchRectDataView(
    ::gfx::mojom::RectFDataView* output) {
  auto pointer = data_->active_match_rect.Get();
  *output = ::gfx::mojom::RectFDataView(pointer, context_);
}




inline void FindInPageClient_SetActiveMatch_ParamsDataView::GetActiveMatchRectDataView(
    ::gfx::mojom::RectDataView* output) {
  auto pointer = data_->active_match_rect.Get();
  *output = ::gfx::mojom::RectDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace blink

#endif  // THIRD_PARTY_BLINK_PUBLIC_MOJOM_FRAME_FIND_IN_PAGE_MOJOM_SHARED_H_