// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_PUBLIC_MOJOM_PORTAL_PORTAL_MOJOM_SHARED_H_
#define THIRD_PARTY_BLINK_PUBLIC_MOJOM_PORTAL_PORTAL_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "third_party/blink/public/mojom/portal/portal.mojom-shared-internal.h"
#include "mojo/public/mojom/base/unguessable_token.mojom-shared.h"
#include "url/mojom/url.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "base/component_export.h"


namespace blink {
namespace mojom {


}  // namespace mojom
}  // namespace blink

namespace mojo {
namespace internal {

}  // namespace internal
}  // namespace mojo


namespace blink {
namespace mojom {

enum class PortalActivationStatus : int32_t {
  kSuccess,
  kNotSupported,
  kMinValue = 0,
  kMaxValue = 1,
};

COMPONENT_EXPORT(MOJOM_SHARED_BLINK_COMMON_EXPORT) std::ostream& operator<<(std::ostream& os, PortalActivationStatus value);
inline bool IsKnownEnumValue(PortalActivationStatus value) {
  return internal::PortalActivationStatus_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class PortalInterfaceBase {};

using PortalPtrDataView =
    mojo::InterfacePtrDataView<PortalInterfaceBase>;
using PortalRequestDataView =
    mojo::InterfaceRequestDataView<PortalInterfaceBase>;
using PortalAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<PortalInterfaceBase>;
using PortalAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<PortalInterfaceBase>;
class Portal_Navigate_ParamsDataView {
 public:
  Portal_Navigate_ParamsDataView() {}

  Portal_Navigate_ParamsDataView(
      internal::Portal_Navigate_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUrl(UserType* output) {
    auto* pointer = data_->url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
 private:
  internal::Portal_Navigate_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class Portal_Activate_ParamsDataView {
 public:
  Portal_Activate_ParamsDataView() {}

  Portal_Activate_ParamsDataView(
      internal::Portal_Activate_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::Portal_Activate_Params_Data* data_ = nullptr;
};

class Portal_Activate_ResponseParamsDataView {
 public:
  Portal_Activate_ResponseParamsDataView() {}

  Portal_Activate_ResponseParamsDataView(
      internal::Portal_Activate_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadResult(UserType* output) const {
    auto data_value = data_->result;
    return mojo::internal::Deserialize<::blink::mojom::PortalActivationStatus>(
        data_value, output);
  }

  PortalActivationStatus result() const {
    return static_cast<PortalActivationStatus>(data_->result);
  }
 private:
  internal::Portal_Activate_ResponseParams_Data* data_ = nullptr;
};



}  // namespace mojom
}  // namespace blink

namespace std {

template <>
struct hash<::blink::mojom::PortalActivationStatus>
    : public mojo::internal::EnumHashImpl<::blink::mojom::PortalActivationStatus> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::blink::mojom::PortalActivationStatus, ::blink::mojom::PortalActivationStatus> {
  static ::blink::mojom::PortalActivationStatus ToMojom(::blink::mojom::PortalActivationStatus input) { return input; }
  static bool FromMojom(::blink::mojom::PortalActivationStatus input, ::blink::mojom::PortalActivationStatus* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::blink::mojom::PortalActivationStatus, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::blink::mojom::PortalActivationStatus, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::blink::mojom::PortalActivationStatus>(input), output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace blink {
namespace mojom {

inline void Portal_Navigate_ParamsDataView::GetUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}







}  // namespace mojom
}  // namespace blink

#endif  // THIRD_PARTY_BLINK_PUBLIC_MOJOM_PORTAL_PORTAL_MOJOM_SHARED_H_