// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_PUBLIC_PLATFORM_MIME_REGISTRY_MOJOM_BLINK_H_
#define THIRD_PARTY_BLINK_PUBLIC_PLATFORM_MIME_REGISTRY_MOJOM_BLINK_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "third_party/blink/public/platform/mime_registry.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/wtf_clone_equals_util.h"
#include "mojo/public/cpp/bindings/lib/wtf_hash_util.h"
#include "third_party/blink/renderer/platform/mojo/revocable_interface_ptr.h"
#include "third_party/blink/renderer/platform/wtf/hash_functions.h"
#include "third_party/blink/renderer/platform/wtf/text/wtf_string.h"

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "third_party/blink/renderer/platform/mojo/revocable_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "third_party/blink/public/platform/web_common.h"


namespace blink {
namespace mojom {
namespace blink {
class MimeRegistry;
using MimeRegistryPtr = mojo::InterfacePtr<MimeRegistry>;
using RevocableMimeRegistryPtr = ::blink::RevocableInterfacePtr<MimeRegistry>;
using MimeRegistryPtrInfo = mojo::InterfacePtrInfo<MimeRegistry>;
using ThreadSafeMimeRegistryPtr =
    mojo::ThreadSafeInterfacePtr<MimeRegistry>;
using MimeRegistryRequest = mojo::InterfaceRequest<MimeRegistry>;
using MimeRegistryAssociatedPtr =
    mojo::AssociatedInterfacePtr<MimeRegistry>;
using ThreadSafeMimeRegistryAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<MimeRegistry>;
using MimeRegistryAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<MimeRegistry>;
using MimeRegistryAssociatedRequest =
    mojo::AssociatedInterfaceRequest<MimeRegistry>;


class MimeRegistryProxy;

template <typename ImplRefTraits>
class MimeRegistryStub;

class MimeRegistryRequestValidator;
class MimeRegistryResponseValidator;

class BLINK_PLATFORM_EXPORT MimeRegistry
    : public MimeRegistryInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = true;

  using Proxy_ = MimeRegistryProxy;

  template <typename ImplRefTraits>
  using Stub_ = MimeRegistryStub<ImplRefTraits>;

  using RequestValidator_ = MimeRegistryRequestValidator;
  using ResponseValidator_ = MimeRegistryResponseValidator;
  enum MethodMinVersions : uint32_t {
    kGetMimeTypeFromExtensionMinVersion = 0,
  };
  virtual ~MimeRegistry() {}

  // Sync method. This signature is used by the client side; the service side
  // should implement the signature with callback below.
  virtual bool GetMimeTypeFromExtension(const WTF::String& extension, WTF::String* out_mime_type);

  using GetMimeTypeFromExtensionCallback = base::OnceCallback<void(const WTF::String&)>;
  virtual void GetMimeTypeFromExtension(const WTF::String& extension, GetMimeTypeFromExtensionCallback callback) = 0;
};

class BLINK_PLATFORM_EXPORT MimeRegistryProxy
    : public MimeRegistry {
 public:
  explicit MimeRegistryProxy(mojo::MessageReceiverWithResponder* receiver);
  bool GetMimeTypeFromExtension(const WTF::String& extension, WTF::String* out_mime_type) final;
  void GetMimeTypeFromExtension(const WTF::String& extension, GetMimeTypeFromExtensionCallback callback) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class BLINK_PLATFORM_EXPORT MimeRegistryStubDispatch {
 public:
  static bool Accept(MimeRegistry* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      MimeRegistry* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<MimeRegistry>>
class MimeRegistryStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  MimeRegistryStub() {}
  ~MimeRegistryStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return MimeRegistryStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return MimeRegistryStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class BLINK_PLATFORM_EXPORT MimeRegistryRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class BLINK_PLATFORM_EXPORT MimeRegistryResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace blink
}  // namespace mojom
}  // namespace blink

namespace mojo {

}  // namespace mojo

#endif  // THIRD_PARTY_BLINK_PUBLIC_PLATFORM_MIME_REGISTRY_MOJOM_BLINK_H_