// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_PUBLIC_PLATFORM_MODULES_IDLE_IDLE_MANAGER_MOJOM_SHARED_H_
#define THIRD_PARTY_BLINK_PUBLIC_PLATFORM_MODULES_IDLE_IDLE_MANAGER_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "third_party/blink/public/platform/modules/idle/idle_manager.mojom-shared-internal.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "base/component_export.h"


namespace blink {
namespace mojom {


}  // namespace mojom
}  // namespace blink

namespace mojo {
namespace internal {

}  // namespace internal
}  // namespace mojo


namespace blink {
namespace mojom {

enum class IdleState : int32_t {
  ACTIVE,
  IDLE,
  LOCKED,
  kMinValue = 0,
  kMaxValue = 2,
};

COMPONENT_EXPORT(MOJOM_SHARED_CONTENT_EXPORT) std::ostream& operator<<(std::ostream& os, IdleState value);
inline bool IsKnownEnumValue(IdleState value) {
  return internal::IdleState_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class IdleMonitorInterfaceBase {};

using IdleMonitorPtrDataView =
    mojo::InterfacePtrDataView<IdleMonitorInterfaceBase>;
using IdleMonitorRequestDataView =
    mojo::InterfaceRequestDataView<IdleMonitorInterfaceBase>;
using IdleMonitorAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<IdleMonitorInterfaceBase>;
using IdleMonitorAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<IdleMonitorInterfaceBase>;
class IdleManagerInterfaceBase {};

using IdleManagerPtrDataView =
    mojo::InterfacePtrDataView<IdleManagerInterfaceBase>;
using IdleManagerRequestDataView =
    mojo::InterfaceRequestDataView<IdleManagerInterfaceBase>;
using IdleManagerAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<IdleManagerInterfaceBase>;
using IdleManagerAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<IdleManagerInterfaceBase>;
class IdleMonitor_Update_ParamsDataView {
 public:
  IdleMonitor_Update_ParamsDataView() {}

  IdleMonitor_Update_ParamsDataView(
      internal::IdleMonitor_Update_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadState(UserType* output) const {
    auto data_value = data_->state;
    return mojo::internal::Deserialize<::blink::mojom::IdleState>(
        data_value, output);
  }

  IdleState state() const {
    return static_cast<IdleState>(data_->state);
  }
 private:
  internal::IdleMonitor_Update_Params_Data* data_ = nullptr;
};

class IdleManager_AddMonitor_ParamsDataView {
 public:
  IdleManager_AddMonitor_ParamsDataView() {}

  IdleManager_AddMonitor_ParamsDataView(
      internal::IdleManager_AddMonitor_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  uint32_t threshold() const {
    return data_->threshold;
  }
  template <typename UserType>
  UserType TakeMonitor() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::blink::mojom::IdleMonitorPtrDataView>(
            &data_->monitor, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::IdleManager_AddMonitor_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class IdleManager_AddMonitor_ResponseParamsDataView {
 public:
  IdleManager_AddMonitor_ResponseParamsDataView() {}

  IdleManager_AddMonitor_ResponseParamsDataView(
      internal::IdleManager_AddMonitor_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadState(UserType* output) const {
    auto data_value = data_->state;
    return mojo::internal::Deserialize<::blink::mojom::IdleState>(
        data_value, output);
  }

  IdleState state() const {
    return static_cast<IdleState>(data_->state);
  }
 private:
  internal::IdleManager_AddMonitor_ResponseParams_Data* data_ = nullptr;
};



}  // namespace mojom
}  // namespace blink

namespace std {

template <>
struct hash<::blink::mojom::IdleState>
    : public mojo::internal::EnumHashImpl<::blink::mojom::IdleState> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::blink::mojom::IdleState, ::blink::mojom::IdleState> {
  static ::blink::mojom::IdleState ToMojom(::blink::mojom::IdleState input) { return input; }
  static bool FromMojom(::blink::mojom::IdleState input, ::blink::mojom::IdleState* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::blink::mojom::IdleState, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::blink::mojom::IdleState, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::blink::mojom::IdleState>(input), output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace blink {
namespace mojom {








}  // namespace mojom
}  // namespace blink

#endif  // THIRD_PARTY_BLINK_PUBLIC_PLATFORM_MODULES_IDLE_IDLE_MANAGER_MOJOM_SHARED_H_