// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef UI_ACCESSIBILITY_MOJOM_AX_ACTION_DATA_MOJOM_H_
#define UI_ACCESSIBILITY_MOJOM_AX_ACTION_DATA_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "ui/accessibility/mojom/ax_action_data.mojom-shared.h"
#include "ui/accessibility/ax_enums.mojom.h"
#include "ui/accessibility/mojom/ax_tree_id.mojom.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "ui/accessibility/ax_action_data.h"


namespace ax {
namespace mojom {
class AXActionData;
using AXActionDataPtr = mojo::StructPtr<AXActionData>;






class  AXActionData {
 public:
  using DataView = AXActionDataDataView;
  using Data_ = internal::AXActionData_Data;

  template <typename... Args>
  static AXActionDataPtr New(Args&&... args) {
    return AXActionDataPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static AXActionDataPtr From(const U& u) {
    return mojo::TypeConverter<AXActionDataPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, AXActionData>::Convert(*this);
  }


  AXActionData();

  AXActionData(
      ::ax::mojom::Action action,
      const ui::AXTreeID& target_tree_id,
      const std::string& source_extension_id,
      int32_t target_node_id,
      int32_t request_id,
      int32_t flags,
      int32_t anchor_node_id,
      int32_t anchor_offset,
      int32_t focus_node_id,
      int32_t focus_offset,
      int32_t custom_action_id,
      const gfx::Rect& target_rect,
      const gfx::Point& target_point,
      const std::string& value,
      ::ax::mojom::Event hit_test_event_to_fire);

  ~AXActionData();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = AXActionDataPtr>
  AXActionDataPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, AXActionData>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        AXActionData::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        AXActionData::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::AXActionData_UnserializedMessageContext<
            UserType, AXActionData::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<AXActionData::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return AXActionData::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::AXActionData_UnserializedMessageContext<
            UserType, AXActionData::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<AXActionData::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  ::ax::mojom::Action action;
  ui::AXTreeID target_tree_id;
  std::string source_extension_id;
  int32_t target_node_id;
  int32_t request_id;
  int32_t flags;
  int32_t anchor_node_id;
  int32_t anchor_offset;
  int32_t focus_node_id;
  int32_t focus_offset;
  int32_t custom_action_id;
  gfx::Rect target_rect;
  gfx::Point target_point;
  std::string value;
  ::ax::mojom::Event hit_test_event_to_fire;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};

template <typename StructPtrType>
AXActionDataPtr AXActionData::Clone() const {
  return New(
      mojo::Clone(action),
      mojo::Clone(target_tree_id),
      mojo::Clone(source_extension_id),
      mojo::Clone(target_node_id),
      mojo::Clone(request_id),
      mojo::Clone(flags),
      mojo::Clone(anchor_node_id),
      mojo::Clone(anchor_offset),
      mojo::Clone(focus_node_id),
      mojo::Clone(focus_offset),
      mojo::Clone(custom_action_id),
      mojo::Clone(target_rect),
      mojo::Clone(target_point),
      mojo::Clone(value),
      mojo::Clone(hit_test_event_to_fire)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, AXActionData>::value>::type*>
bool AXActionData::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->action, other_struct.action))
    return false;
  if (!mojo::Equals(this->target_tree_id, other_struct.target_tree_id))
    return false;
  if (!mojo::Equals(this->source_extension_id, other_struct.source_extension_id))
    return false;
  if (!mojo::Equals(this->target_node_id, other_struct.target_node_id))
    return false;
  if (!mojo::Equals(this->request_id, other_struct.request_id))
    return false;
  if (!mojo::Equals(this->flags, other_struct.flags))
    return false;
  if (!mojo::Equals(this->anchor_node_id, other_struct.anchor_node_id))
    return false;
  if (!mojo::Equals(this->anchor_offset, other_struct.anchor_offset))
    return false;
  if (!mojo::Equals(this->focus_node_id, other_struct.focus_node_id))
    return false;
  if (!mojo::Equals(this->focus_offset, other_struct.focus_offset))
    return false;
  if (!mojo::Equals(this->custom_action_id, other_struct.custom_action_id))
    return false;
  if (!mojo::Equals(this->target_rect, other_struct.target_rect))
    return false;
  if (!mojo::Equals(this->target_point, other_struct.target_point))
    return false;
  if (!mojo::Equals(this->value, other_struct.value))
    return false;
  if (!mojo::Equals(this->hit_test_event_to_fire, other_struct.hit_test_event_to_fire))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace ax

namespace mojo {


template <>
struct  StructTraits<::ax::mojom::AXActionData::DataView,
                                         ::ax::mojom::AXActionDataPtr> {
  static bool IsNull(const ::ax::mojom::AXActionDataPtr& input) { return !input; }
  static void SetToNull(::ax::mojom::AXActionDataPtr* output) { output->reset(); }

  static decltype(::ax::mojom::AXActionData::action) action(
      const ::ax::mojom::AXActionDataPtr& input) {
    return input->action;
  }

  static const decltype(::ax::mojom::AXActionData::target_tree_id)& target_tree_id(
      const ::ax::mojom::AXActionDataPtr& input) {
    return input->target_tree_id;
  }

  static const decltype(::ax::mojom::AXActionData::source_extension_id)& source_extension_id(
      const ::ax::mojom::AXActionDataPtr& input) {
    return input->source_extension_id;
  }

  static decltype(::ax::mojom::AXActionData::target_node_id) target_node_id(
      const ::ax::mojom::AXActionDataPtr& input) {
    return input->target_node_id;
  }

  static decltype(::ax::mojom::AXActionData::request_id) request_id(
      const ::ax::mojom::AXActionDataPtr& input) {
    return input->request_id;
  }

  static decltype(::ax::mojom::AXActionData::flags) flags(
      const ::ax::mojom::AXActionDataPtr& input) {
    return input->flags;
  }

  static decltype(::ax::mojom::AXActionData::anchor_node_id) anchor_node_id(
      const ::ax::mojom::AXActionDataPtr& input) {
    return input->anchor_node_id;
  }

  static decltype(::ax::mojom::AXActionData::anchor_offset) anchor_offset(
      const ::ax::mojom::AXActionDataPtr& input) {
    return input->anchor_offset;
  }

  static decltype(::ax::mojom::AXActionData::focus_node_id) focus_node_id(
      const ::ax::mojom::AXActionDataPtr& input) {
    return input->focus_node_id;
  }

  static decltype(::ax::mojom::AXActionData::focus_offset) focus_offset(
      const ::ax::mojom::AXActionDataPtr& input) {
    return input->focus_offset;
  }

  static decltype(::ax::mojom::AXActionData::custom_action_id) custom_action_id(
      const ::ax::mojom::AXActionDataPtr& input) {
    return input->custom_action_id;
  }

  static const decltype(::ax::mojom::AXActionData::target_rect)& target_rect(
      const ::ax::mojom::AXActionDataPtr& input) {
    return input->target_rect;
  }

  static const decltype(::ax::mojom::AXActionData::target_point)& target_point(
      const ::ax::mojom::AXActionDataPtr& input) {
    return input->target_point;
  }

  static const decltype(::ax::mojom::AXActionData::value)& value(
      const ::ax::mojom::AXActionDataPtr& input) {
    return input->value;
  }

  static decltype(::ax::mojom::AXActionData::hit_test_event_to_fire) hit_test_event_to_fire(
      const ::ax::mojom::AXActionDataPtr& input) {
    return input->hit_test_event_to_fire;
  }

  static bool Read(::ax::mojom::AXActionData::DataView input, ::ax::mojom::AXActionDataPtr* output);
};

}  // namespace mojo

#endif  // UI_ACCESSIBILITY_MOJOM_AX_ACTION_DATA_MOJOM_H_