// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef UI_ACCESSIBILITY_MOJOM_AX_ASSISTANT_STRUCTURE_MOJOM_H_
#define UI_ACCESSIBILITY_MOJOM_AX_ASSISTANT_STRUCTURE_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "ui/accessibility/mojom/ax_assistant_structure.mojom-shared.h"
#include "mojo/public/mojom/base/string16.mojom.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include "ui/gfx/range/mojo/range.mojom.h"
#include "url/mojom/url.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "ui/accessibility/ax_assistant_structure.h"


namespace ax {
namespace mojom {
class AssistantTree;
using AssistantTreePtr = mojo::StructPtr<AssistantTree>;

class AssistantNode;
using AssistantNodePtr = mojo::StructPtr<AssistantNode>;

class AssistantExtra;
using AssistantExtraPtr = mojo::StructPtr<AssistantExtra>;








class  AssistantTree {
 public:
  using DataView = AssistantTreeDataView;
  using Data_ = internal::AssistantTree_Data;

  template <typename... Args>
  static AssistantTreePtr New(Args&&... args) {
    return AssistantTreePtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static AssistantTreePtr From(const U& u) {
    return mojo::TypeConverter<AssistantTreePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, AssistantTree>::Convert(*this);
  }


  AssistantTree();

  explicit AssistantTree(
      std::vector<std::unique_ptr<ui::AssistantNode>> nodes);

  ~AssistantTree();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = AssistantTreePtr>
  AssistantTreePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, AssistantTree>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        AssistantTree::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        AssistantTree::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::AssistantTree_UnserializedMessageContext<
            UserType, AssistantTree::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<AssistantTree::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return AssistantTree::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::AssistantTree_UnserializedMessageContext<
            UserType, AssistantTree::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<AssistantTree::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  std::vector<std::unique_ptr<ui::AssistantNode>> nodes;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(AssistantTree);
};


class  AssistantNode {
 public:
  using DataView = AssistantNodeDataView;
  using Data_ = internal::AssistantNode_Data;

  template <typename... Args>
  static AssistantNodePtr New(Args&&... args) {
    return AssistantNodePtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static AssistantNodePtr From(const U& u) {
    return mojo::TypeConverter<AssistantNodePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, AssistantNode>::Convert(*this);
  }


  AssistantNode();

  AssistantNode(
      const std::vector<int32_t>& children_indices,
      const gfx::Rect& rect,
      const base::string16& text,
      float text_size,
      uint32_t color,
      uint32_t bgcolor,
      bool bold,
      bool italic,
      bool underline,
      bool line_through,
      const base::Optional<gfx::Range>& selection,
      const std::string& class_name,
      const base::Optional<std::string>& role);

  ~AssistantNode();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = AssistantNodePtr>
  AssistantNodePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, AssistantNode>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        AssistantNode::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        AssistantNode::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::AssistantNode_UnserializedMessageContext<
            UserType, AssistantNode::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<AssistantNode::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return AssistantNode::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::AssistantNode_UnserializedMessageContext<
            UserType, AssistantNode::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<AssistantNode::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  std::vector<int32_t> children_indices;
  gfx::Rect rect;
  base::string16 text;
  float text_size;
  uint32_t color;
  uint32_t bgcolor;
  bool bold;
  bool italic;
  bool underline;
  bool line_through;
  base::Optional<gfx::Range> selection;
  std::string class_name;
  base::Optional<std::string> role;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


class  AssistantExtra {
 public:
  using DataView = AssistantExtraDataView;
  using Data_ = internal::AssistantExtra_Data;

  template <typename... Args>
  static AssistantExtraPtr New(Args&&... args) {
    return AssistantExtraPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static AssistantExtraPtr From(const U& u) {
    return mojo::TypeConverter<AssistantExtraPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, AssistantExtra>::Convert(*this);
  }


  AssistantExtra();

  AssistantExtra(
      const GURL& url,
      const gfx::Rect& bounds_pixel,
      const base::string16& title);

  ~AssistantExtra();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = AssistantExtraPtr>
  AssistantExtraPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, AssistantExtra>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        AssistantExtra::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        AssistantExtra::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::AssistantExtra_UnserializedMessageContext<
            UserType, AssistantExtra::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<AssistantExtra::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return AssistantExtra::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::AssistantExtra_UnserializedMessageContext<
            UserType, AssistantExtra::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<AssistantExtra::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  GURL url;
  gfx::Rect bounds_pixel;
  base::string16 title;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};

template <typename StructPtrType>
AssistantTreePtr AssistantTree::Clone() const {
  return New(
      mojo::Clone(nodes)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, AssistantTree>::value>::type*>
bool AssistantTree::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->nodes, other_struct.nodes))
    return false;
  return true;
}
template <typename StructPtrType>
AssistantNodePtr AssistantNode::Clone() const {
  return New(
      mojo::Clone(children_indices),
      mojo::Clone(rect),
      mojo::Clone(text),
      mojo::Clone(text_size),
      mojo::Clone(color),
      mojo::Clone(bgcolor),
      mojo::Clone(bold),
      mojo::Clone(italic),
      mojo::Clone(underline),
      mojo::Clone(line_through),
      mojo::Clone(selection),
      mojo::Clone(class_name),
      mojo::Clone(role)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, AssistantNode>::value>::type*>
bool AssistantNode::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->children_indices, other_struct.children_indices))
    return false;
  if (!mojo::Equals(this->rect, other_struct.rect))
    return false;
  if (!mojo::Equals(this->text, other_struct.text))
    return false;
  if (!mojo::Equals(this->text_size, other_struct.text_size))
    return false;
  if (!mojo::Equals(this->color, other_struct.color))
    return false;
  if (!mojo::Equals(this->bgcolor, other_struct.bgcolor))
    return false;
  if (!mojo::Equals(this->bold, other_struct.bold))
    return false;
  if (!mojo::Equals(this->italic, other_struct.italic))
    return false;
  if (!mojo::Equals(this->underline, other_struct.underline))
    return false;
  if (!mojo::Equals(this->line_through, other_struct.line_through))
    return false;
  if (!mojo::Equals(this->selection, other_struct.selection))
    return false;
  if (!mojo::Equals(this->class_name, other_struct.class_name))
    return false;
  if (!mojo::Equals(this->role, other_struct.role))
    return false;
  return true;
}
template <typename StructPtrType>
AssistantExtraPtr AssistantExtra::Clone() const {
  return New(
      mojo::Clone(url),
      mojo::Clone(bounds_pixel),
      mojo::Clone(title)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, AssistantExtra>::value>::type*>
bool AssistantExtra::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->url, other_struct.url))
    return false;
  if (!mojo::Equals(this->bounds_pixel, other_struct.bounds_pixel))
    return false;
  if (!mojo::Equals(this->title, other_struct.title))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace ax

namespace mojo {


template <>
struct  StructTraits<::ax::mojom::AssistantTree::DataView,
                                         ::ax::mojom::AssistantTreePtr> {
  static bool IsNull(const ::ax::mojom::AssistantTreePtr& input) { return !input; }
  static void SetToNull(::ax::mojom::AssistantTreePtr* output) { output->reset(); }

  static const decltype(::ax::mojom::AssistantTree::nodes)& nodes(
      const ::ax::mojom::AssistantTreePtr& input) {
    return input->nodes;
  }

  static bool Read(::ax::mojom::AssistantTree::DataView input, ::ax::mojom::AssistantTreePtr* output);
};


template <>
struct  StructTraits<::ax::mojom::AssistantNode::DataView,
                                         ::ax::mojom::AssistantNodePtr> {
  static bool IsNull(const ::ax::mojom::AssistantNodePtr& input) { return !input; }
  static void SetToNull(::ax::mojom::AssistantNodePtr* output) { output->reset(); }

  static const decltype(::ax::mojom::AssistantNode::children_indices)& children_indices(
      const ::ax::mojom::AssistantNodePtr& input) {
    return input->children_indices;
  }

  static const decltype(::ax::mojom::AssistantNode::rect)& rect(
      const ::ax::mojom::AssistantNodePtr& input) {
    return input->rect;
  }

  static const decltype(::ax::mojom::AssistantNode::text)& text(
      const ::ax::mojom::AssistantNodePtr& input) {
    return input->text;
  }

  static decltype(::ax::mojom::AssistantNode::text_size) text_size(
      const ::ax::mojom::AssistantNodePtr& input) {
    return input->text_size;
  }

  static decltype(::ax::mojom::AssistantNode::color) color(
      const ::ax::mojom::AssistantNodePtr& input) {
    return input->color;
  }

  static decltype(::ax::mojom::AssistantNode::bgcolor) bgcolor(
      const ::ax::mojom::AssistantNodePtr& input) {
    return input->bgcolor;
  }

  static decltype(::ax::mojom::AssistantNode::bold) bold(
      const ::ax::mojom::AssistantNodePtr& input) {
    return input->bold;
  }

  static decltype(::ax::mojom::AssistantNode::italic) italic(
      const ::ax::mojom::AssistantNodePtr& input) {
    return input->italic;
  }

  static decltype(::ax::mojom::AssistantNode::underline) underline(
      const ::ax::mojom::AssistantNodePtr& input) {
    return input->underline;
  }

  static decltype(::ax::mojom::AssistantNode::line_through) line_through(
      const ::ax::mojom::AssistantNodePtr& input) {
    return input->line_through;
  }

  static const decltype(::ax::mojom::AssistantNode::selection)& selection(
      const ::ax::mojom::AssistantNodePtr& input) {
    return input->selection;
  }

  static const decltype(::ax::mojom::AssistantNode::class_name)& class_name(
      const ::ax::mojom::AssistantNodePtr& input) {
    return input->class_name;
  }

  static const decltype(::ax::mojom::AssistantNode::role)& role(
      const ::ax::mojom::AssistantNodePtr& input) {
    return input->role;
  }

  static bool Read(::ax::mojom::AssistantNode::DataView input, ::ax::mojom::AssistantNodePtr* output);
};


template <>
struct  StructTraits<::ax::mojom::AssistantExtra::DataView,
                                         ::ax::mojom::AssistantExtraPtr> {
  static bool IsNull(const ::ax::mojom::AssistantExtraPtr& input) { return !input; }
  static void SetToNull(::ax::mojom::AssistantExtraPtr* output) { output->reset(); }

  static const decltype(::ax::mojom::AssistantExtra::url)& url(
      const ::ax::mojom::AssistantExtraPtr& input) {
    return input->url;
  }

  static const decltype(::ax::mojom::AssistantExtra::bounds_pixel)& bounds_pixel(
      const ::ax::mojom::AssistantExtraPtr& input) {
    return input->bounds_pixel;
  }

  static const decltype(::ax::mojom::AssistantExtra::title)& title(
      const ::ax::mojom::AssistantExtraPtr& input) {
    return input->title;
  }

  static bool Read(::ax::mojom::AssistantExtra::DataView input, ::ax::mojom::AssistantExtraPtr* output);
};

}  // namespace mojo

#endif  // UI_ACCESSIBILITY_MOJOM_AX_ASSISTANT_STRUCTURE_MOJOM_H_