// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef UI_ACCESSIBILITY_MOJOM_AX_HOST_MOJOM_H_
#define UI_ACCESSIBILITY_MOJOM_AX_HOST_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "ui/accessibility/mojom/ax_host.mojom-shared.h"
#include "ui/accessibility/ax_enums.mojom.h"
#include "ui/accessibility/mojom/ax_action_data.mojom.h"
#include "ui/accessibility/mojom/ax_event.mojom.h"
#include "ui/accessibility/mojom/ax_tree_id.mojom.h"
#include "ui/accessibility/mojom/ax_tree_update.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace ax {
namespace mojom {
extern const char kAXHostServiceName[];
class AXHost;
using AXHostPtr = mojo::InterfacePtr<AXHost>;
using AXHostPtrInfo = mojo::InterfacePtrInfo<AXHost>;
using ThreadSafeAXHostPtr =
    mojo::ThreadSafeInterfacePtr<AXHost>;
using AXHostRequest = mojo::InterfaceRequest<AXHost>;
using AXHostAssociatedPtr =
    mojo::AssociatedInterfacePtr<AXHost>;
using ThreadSafeAXHostAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<AXHost>;
using AXHostAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<AXHost>;
using AXHostAssociatedRequest =
    mojo::AssociatedInterfaceRequest<AXHost>;

class AXRemoteHost;
using AXRemoteHostPtr = mojo::InterfacePtr<AXRemoteHost>;
using AXRemoteHostPtrInfo = mojo::InterfacePtrInfo<AXRemoteHost>;
using ThreadSafeAXRemoteHostPtr =
    mojo::ThreadSafeInterfacePtr<AXRemoteHost>;
using AXRemoteHostRequest = mojo::InterfaceRequest<AXRemoteHost>;
using AXRemoteHostAssociatedPtr =
    mojo::AssociatedInterfacePtr<AXRemoteHost>;
using ThreadSafeAXRemoteHostAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<AXRemoteHost>;
using AXRemoteHostAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<AXRemoteHost>;
using AXRemoteHostAssociatedRequest =
    mojo::AssociatedInterfaceRequest<AXRemoteHost>;


class AXHostProxy;

template <typename ImplRefTraits>
class AXHostStub;

class AXHostRequestValidator;
class AXHostResponseValidator;

class  AXHost
    : public AXHostInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = AXHostProxy;

  template <typename ImplRefTraits>
  using Stub_ = AXHostStub<ImplRefTraits>;

  using RequestValidator_ = AXHostRequestValidator;
  using ResponseValidator_ = AXHostResponseValidator;
  enum MethodMinVersions : uint32_t {
    kRegisterRemoteHostMinVersion = 0,
    kHandleAccessibilityEventMinVersion = 0,
  };
  virtual ~AXHost() {}


  using RegisterRemoteHostCallback = base::OnceCallback<void(const ui::AXTreeID&, bool)>;
  virtual void RegisterRemoteHost(AXRemoteHostPtr remote, RegisterRemoteHostCallback callback) = 0;

  virtual void HandleAccessibilityEvent(const ui::AXTreeID& tree_id, const std::vector<ui::AXTreeUpdate>& updates, const ui::AXEvent& event) = 0;
};

class AXRemoteHostProxy;

template <typename ImplRefTraits>
class AXRemoteHostStub;

class AXRemoteHostRequestValidator;

class  AXRemoteHost
    : public AXRemoteHostInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = AXRemoteHostProxy;

  template <typename ImplRefTraits>
  using Stub_ = AXRemoteHostStub<ImplRefTraits>;

  using RequestValidator_ = AXRemoteHostRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kOnAutomationEnabledMinVersion = 0,
    kPerformActionMinVersion = 0,
  };
  virtual ~AXRemoteHost() {}

  virtual void OnAutomationEnabled(bool enabled) = 0;

  virtual void PerformAction(const ui::AXActionData& data) = 0;
};

class  AXHostProxy
    : public AXHost {
 public:
  explicit AXHostProxy(mojo::MessageReceiverWithResponder* receiver);
  void RegisterRemoteHost(AXRemoteHostPtr remote, RegisterRemoteHostCallback callback) final;
  void HandleAccessibilityEvent(const ui::AXTreeID& tree_id, const std::vector<ui::AXTreeUpdate>& updates, const ui::AXEvent& event) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  AXRemoteHostProxy
    : public AXRemoteHost {
 public:
  explicit AXRemoteHostProxy(mojo::MessageReceiverWithResponder* receiver);
  void OnAutomationEnabled(bool enabled) final;
  void PerformAction(const ui::AXActionData& data) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  AXHostStubDispatch {
 public:
  static bool Accept(AXHost* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      AXHost* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<AXHost>>
class AXHostStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  AXHostStub() {}
  ~AXHostStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AXHostStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AXHostStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  AXRemoteHostStubDispatch {
 public:
  static bool Accept(AXRemoteHost* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      AXRemoteHost* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<AXRemoteHost>>
class AXRemoteHostStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  AXRemoteHostStub() {}
  ~AXRemoteHostStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AXRemoteHostStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return AXRemoteHostStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  AXHostRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  AXRemoteHostRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  AXHostResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace ax

namespace mojo {

}  // namespace mojo

#endif  // UI_ACCESSIBILITY_MOJOM_AX_HOST_MOJOM_H_