// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef UI_BASE_MOJO_CLIPBOARD_MOJOM_SHARED_H_
#define UI_BASE_MOJO_CLIPBOARD_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "ui/base/mojo/clipboard.mojom-shared-internal.h"
#include "mojo/public/mojom/base/big_string.mojom-shared.h"
#include "mojo/public/mojom/base/string16.mojom-shared.h"
#include "mojo/public/mojom/base/time.mojom-shared.h"
#include "skia/public/interfaces/bitmap.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace ui {
namespace mojom {


}  // namespace mojom
}  // namespace ui

namespace mojo {
namespace internal {

}  // namespace internal
}  // namespace mojo


namespace ui {
namespace mojom {

enum class ClipboardType : int32_t {
  COPY_PASTE,
  SELECTION,
  DRAG,
  kMinValue = 0,
  kMaxValue = 2,
};

 std::ostream& operator<<(std::ostream& os, ClipboardType value);
inline bool IsKnownEnumValue(ClipboardType value) {
  return internal::ClipboardType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class ClipboardHostInterfaceBase {};

using ClipboardHostPtrDataView =
    mojo::InterfacePtrDataView<ClipboardHostInterfaceBase>;
using ClipboardHostRequestDataView =
    mojo::InterfaceRequestDataView<ClipboardHostInterfaceBase>;
using ClipboardHostAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<ClipboardHostInterfaceBase>;
using ClipboardHostAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<ClipboardHostInterfaceBase>;
class ClipboardHost_GetSequenceNumber_ParamsDataView {
 public:
  ClipboardHost_GetSequenceNumber_ParamsDataView() {}

  ClipboardHost_GetSequenceNumber_ParamsDataView(
      internal::ClipboardHost_GetSequenceNumber_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::ui::mojom::ClipboardType>(
        data_value, output);
  }

  ClipboardType type() const {
    return static_cast<ClipboardType>(data_->type);
  }
 private:
  internal::ClipboardHost_GetSequenceNumber_Params_Data* data_ = nullptr;
};

class ClipboardHost_GetSequenceNumber_ResponseParamsDataView {
 public:
  ClipboardHost_GetSequenceNumber_ResponseParamsDataView() {}

  ClipboardHost_GetSequenceNumber_ResponseParamsDataView(
      internal::ClipboardHost_GetSequenceNumber_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  uint64_t result() const {
    return data_->result;
  }
 private:
  internal::ClipboardHost_GetSequenceNumber_ResponseParams_Data* data_ = nullptr;
};

class ClipboardHost_IsFormatAvailable_ParamsDataView {
 public:
  ClipboardHost_IsFormatAvailable_ParamsDataView() {}

  ClipboardHost_IsFormatAvailable_ParamsDataView(
      internal::ClipboardHost_IsFormatAvailable_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetFormatDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFormat(UserType* output) {
    auto* pointer = data_->format.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::ui::mojom::ClipboardType>(
        data_value, output);
  }

  ClipboardType type() const {
    return static_cast<ClipboardType>(data_->type);
  }
 private:
  internal::ClipboardHost_IsFormatAvailable_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ClipboardHost_IsFormatAvailable_ResponseParamsDataView {
 public:
  ClipboardHost_IsFormatAvailable_ResponseParamsDataView() {}

  ClipboardHost_IsFormatAvailable_ResponseParamsDataView(
      internal::ClipboardHost_IsFormatAvailable_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool result() const {
    return data_->result;
  }
 private:
  internal::ClipboardHost_IsFormatAvailable_ResponseParams_Data* data_ = nullptr;
};

class ClipboardHost_Clear_ParamsDataView {
 public:
  ClipboardHost_Clear_ParamsDataView() {}

  ClipboardHost_Clear_ParamsDataView(
      internal::ClipboardHost_Clear_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::ui::mojom::ClipboardType>(
        data_value, output);
  }

  ClipboardType type() const {
    return static_cast<ClipboardType>(data_->type);
  }
 private:
  internal::ClipboardHost_Clear_Params_Data* data_ = nullptr;
};

class ClipboardHost_ReadAvailableTypes_ParamsDataView {
 public:
  ClipboardHost_ReadAvailableTypes_ParamsDataView() {}

  ClipboardHost_ReadAvailableTypes_ParamsDataView(
      internal::ClipboardHost_ReadAvailableTypes_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::ui::mojom::ClipboardType>(
        data_value, output);
  }

  ClipboardType type() const {
    return static_cast<ClipboardType>(data_->type);
  }
 private:
  internal::ClipboardHost_ReadAvailableTypes_Params_Data* data_ = nullptr;
};

class ClipboardHost_ReadAvailableTypes_ResponseParamsDataView {
 public:
  ClipboardHost_ReadAvailableTypes_ResponseParamsDataView() {}

  ClipboardHost_ReadAvailableTypes_ResponseParamsDataView(
      internal::ClipboardHost_ReadAvailableTypes_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetTypesDataView(
      mojo::ArrayDataView<::mojo_base::mojom::String16DataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTypes(UserType* output) {
    auto* pointer = data_->types.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::mojo_base::mojom::String16DataView>>(
        pointer, output, context_);
  }
  bool contains_filenames() const {
    return data_->contains_filenames;
  }
 private:
  internal::ClipboardHost_ReadAvailableTypes_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ClipboardHost_ReadText_ParamsDataView {
 public:
  ClipboardHost_ReadText_ParamsDataView() {}

  ClipboardHost_ReadText_ParamsDataView(
      internal::ClipboardHost_ReadText_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::ui::mojom::ClipboardType>(
        data_value, output);
  }

  ClipboardType type() const {
    return static_cast<ClipboardType>(data_->type);
  }
 private:
  internal::ClipboardHost_ReadText_Params_Data* data_ = nullptr;
};

class ClipboardHost_ReadText_ResponseParamsDataView {
 public:
  ClipboardHost_ReadText_ResponseParamsDataView() {}

  ClipboardHost_ReadText_ResponseParamsDataView(
      internal::ClipboardHost_ReadText_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetResultDataView(
      ::mojo_base::mojom::BigString16DataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadResult(UserType* output) {
    auto* pointer = data_->result.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::BigString16DataView>(
        pointer, output, context_);
  }
 private:
  internal::ClipboardHost_ReadText_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ClipboardHost_ReadAsciiText_ParamsDataView {
 public:
  ClipboardHost_ReadAsciiText_ParamsDataView() {}

  ClipboardHost_ReadAsciiText_ParamsDataView(
      internal::ClipboardHost_ReadAsciiText_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::ui::mojom::ClipboardType>(
        data_value, output);
  }

  ClipboardType type() const {
    return static_cast<ClipboardType>(data_->type);
  }
 private:
  internal::ClipboardHost_ReadAsciiText_Params_Data* data_ = nullptr;
};

class ClipboardHost_ReadAsciiText_ResponseParamsDataView {
 public:
  ClipboardHost_ReadAsciiText_ResponseParamsDataView() {}

  ClipboardHost_ReadAsciiText_ResponseParamsDataView(
      internal::ClipboardHost_ReadAsciiText_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetResultDataView(
      ::mojo_base::mojom::BigStringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadResult(UserType* output) {
    auto* pointer = data_->result.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::BigStringDataView>(
        pointer, output, context_);
  }
 private:
  internal::ClipboardHost_ReadAsciiText_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ClipboardHost_ReadHTML_ParamsDataView {
 public:
  ClipboardHost_ReadHTML_ParamsDataView() {}

  ClipboardHost_ReadHTML_ParamsDataView(
      internal::ClipboardHost_ReadHTML_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::ui::mojom::ClipboardType>(
        data_value, output);
  }

  ClipboardType type() const {
    return static_cast<ClipboardType>(data_->type);
  }
 private:
  internal::ClipboardHost_ReadHTML_Params_Data* data_ = nullptr;
};

class ClipboardHost_ReadHTML_ResponseParamsDataView {
 public:
  ClipboardHost_ReadHTML_ResponseParamsDataView() {}

  ClipboardHost_ReadHTML_ResponseParamsDataView(
      internal::ClipboardHost_ReadHTML_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetMarkupDataView(
      ::mojo_base::mojom::BigString16DataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadMarkup(UserType* output) {
    auto* pointer = data_->markup.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::BigString16DataView>(
        pointer, output, context_);
  }
  inline void GetSrcUrlDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSrcUrl(UserType* output) {
    auto* pointer = data_->src_url.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  uint32_t fragment_start() const {
    return data_->fragment_start;
  }
  uint32_t fragment_end() const {
    return data_->fragment_end;
  }
 private:
  internal::ClipboardHost_ReadHTML_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ClipboardHost_ReadRTF_ParamsDataView {
 public:
  ClipboardHost_ReadRTF_ParamsDataView() {}

  ClipboardHost_ReadRTF_ParamsDataView(
      internal::ClipboardHost_ReadRTF_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::ui::mojom::ClipboardType>(
        data_value, output);
  }

  ClipboardType type() const {
    return static_cast<ClipboardType>(data_->type);
  }
 private:
  internal::ClipboardHost_ReadRTF_Params_Data* data_ = nullptr;
};

class ClipboardHost_ReadRTF_ResponseParamsDataView {
 public:
  ClipboardHost_ReadRTF_ResponseParamsDataView() {}

  ClipboardHost_ReadRTF_ResponseParamsDataView(
      internal::ClipboardHost_ReadRTF_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetResultDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadResult(UserType* output) {
    auto* pointer = data_->result.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::ClipboardHost_ReadRTF_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ClipboardHost_ReadImage_ParamsDataView {
 public:
  ClipboardHost_ReadImage_ParamsDataView() {}

  ClipboardHost_ReadImage_ParamsDataView(
      internal::ClipboardHost_ReadImage_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::ui::mojom::ClipboardType>(
        data_value, output);
  }

  ClipboardType type() const {
    return static_cast<ClipboardType>(data_->type);
  }
 private:
  internal::ClipboardHost_ReadImage_Params_Data* data_ = nullptr;
};

class ClipboardHost_ReadImage_ResponseParamsDataView {
 public:
  ClipboardHost_ReadImage_ResponseParamsDataView() {}

  ClipboardHost_ReadImage_ResponseParamsDataView(
      internal::ClipboardHost_ReadImage_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetImageDataView(
      ::skia::mojom::BitmapDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadImage(UserType* output) {
    auto* pointer = data_->image.Get();
    return mojo::internal::Deserialize<::skia::mojom::BitmapDataView>(
        pointer, output, context_);
  }
 private:
  internal::ClipboardHost_ReadImage_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ClipboardHost_ReadCustomData_ParamsDataView {
 public:
  ClipboardHost_ReadCustomData_ParamsDataView() {}

  ClipboardHost_ReadCustomData_ParamsDataView(
      internal::ClipboardHost_ReadCustomData_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadClipboardType(UserType* output) const {
    auto data_value = data_->clipboard_type;
    return mojo::internal::Deserialize<::ui::mojom::ClipboardType>(
        data_value, output);
  }

  ClipboardType clipboard_type() const {
    return static_cast<ClipboardType>(data_->clipboard_type);
  }
  inline void GetTypeDataView(
      ::mojo_base::mojom::String16DataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) {
    auto* pointer = data_->type.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::String16DataView>(
        pointer, output, context_);
  }
 private:
  internal::ClipboardHost_ReadCustomData_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ClipboardHost_ReadCustomData_ResponseParamsDataView {
 public:
  ClipboardHost_ReadCustomData_ResponseParamsDataView() {}

  ClipboardHost_ReadCustomData_ResponseParamsDataView(
      internal::ClipboardHost_ReadCustomData_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetResultDataView(
      ::mojo_base::mojom::BigString16DataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadResult(UserType* output) {
    auto* pointer = data_->result.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::BigString16DataView>(
        pointer, output, context_);
  }
 private:
  internal::ClipboardHost_ReadCustomData_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ClipboardHost_ReadBookmark_ParamsDataView {
 public:
  ClipboardHost_ReadBookmark_ParamsDataView() {}

  ClipboardHost_ReadBookmark_ParamsDataView(
      internal::ClipboardHost_ReadBookmark_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::ClipboardHost_ReadBookmark_Params_Data* data_ = nullptr;
};

class ClipboardHost_ReadBookmark_ResponseParamsDataView {
 public:
  ClipboardHost_ReadBookmark_ResponseParamsDataView() {}

  ClipboardHost_ReadBookmark_ResponseParamsDataView(
      internal::ClipboardHost_ReadBookmark_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetTitleDataView(
      ::mojo_base::mojom::String16DataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTitle(UserType* output) {
    auto* pointer = data_->title.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::String16DataView>(
        pointer, output, context_);
  }
  inline void GetUrlDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUrl(UserType* output) {
    auto* pointer = data_->url.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::ClipboardHost_ReadBookmark_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ClipboardHost_ReadData_ParamsDataView {
 public:
  ClipboardHost_ReadData_ParamsDataView() {}

  ClipboardHost_ReadData_ParamsDataView(
      internal::ClipboardHost_ReadData_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetFormatDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFormat(UserType* output) {
    auto* pointer = data_->format.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::ClipboardHost_ReadData_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ClipboardHost_ReadData_ResponseParamsDataView {
 public:
  ClipboardHost_ReadData_ResponseParamsDataView() {}

  ClipboardHost_ReadData_ResponseParamsDataView(
      internal::ClipboardHost_ReadData_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetResultDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadResult(UserType* output) {
    auto* pointer = data_->result.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::ClipboardHost_ReadData_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ClipboardHost_GetLastModifiedTime_ParamsDataView {
 public:
  ClipboardHost_GetLastModifiedTime_ParamsDataView() {}

  ClipboardHost_GetLastModifiedTime_ParamsDataView(
      internal::ClipboardHost_GetLastModifiedTime_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::ClipboardHost_GetLastModifiedTime_Params_Data* data_ = nullptr;
};

class ClipboardHost_GetLastModifiedTime_ResponseParamsDataView {
 public:
  ClipboardHost_GetLastModifiedTime_ResponseParamsDataView() {}

  ClipboardHost_GetLastModifiedTime_ResponseParamsDataView(
      internal::ClipboardHost_GetLastModifiedTime_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetTimeDataView(
      ::mojo_base::mojom::TimeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTime(UserType* output) {
    auto* pointer = data_->time.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::TimeDataView>(
        pointer, output, context_);
  }
 private:
  internal::ClipboardHost_GetLastModifiedTime_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ClipboardHost_ClearLastModifiedTime_ParamsDataView {
 public:
  ClipboardHost_ClearLastModifiedTime_ParamsDataView() {}

  ClipboardHost_ClearLastModifiedTime_ParamsDataView(
      internal::ClipboardHost_ClearLastModifiedTime_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::ClipboardHost_ClearLastModifiedTime_Params_Data* data_ = nullptr;
};

class ClipboardHost_WriteText_ParamsDataView {
 public:
  ClipboardHost_WriteText_ParamsDataView() {}

  ClipboardHost_WriteText_ParamsDataView(
      internal::ClipboardHost_WriteText_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetTextDataView(
      ::mojo_base::mojom::BigString16DataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadText(UserType* output) {
    auto* pointer = data_->text.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::BigString16DataView>(
        pointer, output, context_);
  }
 private:
  internal::ClipboardHost_WriteText_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ClipboardHost_WriteHTML_ParamsDataView {
 public:
  ClipboardHost_WriteHTML_ParamsDataView() {}

  ClipboardHost_WriteHTML_ParamsDataView(
      internal::ClipboardHost_WriteHTML_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetMarkupDataView(
      ::mojo_base::mojom::BigString16DataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadMarkup(UserType* output) {
    auto* pointer = data_->markup.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::BigString16DataView>(
        pointer, output, context_);
  }
  inline void GetUrlDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUrl(UserType* output) {
    auto* pointer = data_->url.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::ClipboardHost_WriteHTML_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ClipboardHost_WriteRTF_ParamsDataView {
 public:
  ClipboardHost_WriteRTF_ParamsDataView() {}

  ClipboardHost_WriteRTF_ParamsDataView(
      internal::ClipboardHost_WriteRTF_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetRtfDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRtf(UserType* output) {
    auto* pointer = data_->rtf.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::ClipboardHost_WriteRTF_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ClipboardHost_WriteBookmark_ParamsDataView {
 public:
  ClipboardHost_WriteBookmark_ParamsDataView() {}

  ClipboardHost_WriteBookmark_ParamsDataView(
      internal::ClipboardHost_WriteBookmark_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetUrlDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUrl(UserType* output) {
    auto* pointer = data_->url.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetTitleDataView(
      ::mojo_base::mojom::String16DataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTitle(UserType* output) {
    auto* pointer = data_->title.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::String16DataView>(
        pointer, output, context_);
  }
 private:
  internal::ClipboardHost_WriteBookmark_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ClipboardHost_WriteWebSmartPaste_ParamsDataView {
 public:
  ClipboardHost_WriteWebSmartPaste_ParamsDataView() {}

  ClipboardHost_WriteWebSmartPaste_ParamsDataView(
      internal::ClipboardHost_WriteWebSmartPaste_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::ClipboardHost_WriteWebSmartPaste_Params_Data* data_ = nullptr;
};

class ClipboardHost_WriteBitmap_ParamsDataView {
 public:
  ClipboardHost_WriteBitmap_ParamsDataView() {}

  ClipboardHost_WriteBitmap_ParamsDataView(
      internal::ClipboardHost_WriteBitmap_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetBitmapDataView(
      ::skia::mojom::BitmapDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadBitmap(UserType* output) {
    auto* pointer = data_->bitmap.Get();
    return mojo::internal::Deserialize<::skia::mojom::BitmapDataView>(
        pointer, output, context_);
  }
 private:
  internal::ClipboardHost_WriteBitmap_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ClipboardHost_WriteData_ParamsDataView {
 public:
  ClipboardHost_WriteData_ParamsDataView() {}

  ClipboardHost_WriteData_ParamsDataView(
      internal::ClipboardHost_WriteData_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetTypeDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) {
    auto* pointer = data_->type.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetDataDataView(
      ::mojo_base::mojom::BigStringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadData(UserType* output) {
    auto* pointer = data_->data.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::BigStringDataView>(
        pointer, output, context_);
  }
 private:
  internal::ClipboardHost_WriteData_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ClipboardHost_CommitWrite_ParamsDataView {
 public:
  ClipboardHost_CommitWrite_ParamsDataView() {}

  ClipboardHost_CommitWrite_ParamsDataView(
      internal::ClipboardHost_CommitWrite_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::ui::mojom::ClipboardType>(
        data_value, output);
  }

  ClipboardType type() const {
    return static_cast<ClipboardType>(data_->type);
  }
 private:
  internal::ClipboardHost_CommitWrite_Params_Data* data_ = nullptr;
};



}  // namespace mojom
}  // namespace ui

namespace std {

template <>
struct hash<::ui::mojom::ClipboardType>
    : public mojo::internal::EnumHashImpl<::ui::mojom::ClipboardType> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::ui::mojom::ClipboardType, ::ui::mojom::ClipboardType> {
  static ::ui::mojom::ClipboardType ToMojom(::ui::mojom::ClipboardType input) { return input; }
  static bool FromMojom(::ui::mojom::ClipboardType input, ::ui::mojom::ClipboardType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::ui::mojom::ClipboardType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::ui::mojom::ClipboardType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::ui::mojom::ClipboardType>(input), output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace ui {
namespace mojom {





inline void ClipboardHost_IsFormatAvailable_ParamsDataView::GetFormatDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->format.Get();
  *output = mojo::StringDataView(pointer, context_);
}








inline void ClipboardHost_ReadAvailableTypes_ResponseParamsDataView::GetTypesDataView(
    mojo::ArrayDataView<::mojo_base::mojom::String16DataView>* output) {
  auto pointer = data_->types.Get();
  *output = mojo::ArrayDataView<::mojo_base::mojom::String16DataView>(pointer, context_);
}




inline void ClipboardHost_ReadText_ResponseParamsDataView::GetResultDataView(
    ::mojo_base::mojom::BigString16DataView* output) {
  auto pointer = data_->result.Get();
  *output = ::mojo_base::mojom::BigString16DataView(pointer, context_);
}




inline void ClipboardHost_ReadAsciiText_ResponseParamsDataView::GetResultDataView(
    ::mojo_base::mojom::BigStringDataView* output) {
  auto pointer = data_->result.Get();
  *output = ::mojo_base::mojom::BigStringDataView(pointer, context_);
}




inline void ClipboardHost_ReadHTML_ResponseParamsDataView::GetMarkupDataView(
    ::mojo_base::mojom::BigString16DataView* output) {
  auto pointer = data_->markup.Get();
  *output = ::mojo_base::mojom::BigString16DataView(pointer, context_);
}
inline void ClipboardHost_ReadHTML_ResponseParamsDataView::GetSrcUrlDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->src_url.Get();
  *output = mojo::StringDataView(pointer, context_);
}




inline void ClipboardHost_ReadRTF_ResponseParamsDataView::GetResultDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->result.Get();
  *output = mojo::StringDataView(pointer, context_);
}




inline void ClipboardHost_ReadImage_ResponseParamsDataView::GetImageDataView(
    ::skia::mojom::BitmapDataView* output) {
  auto pointer = data_->image.Get();
  *output = ::skia::mojom::BitmapDataView(pointer, context_);
}


inline void ClipboardHost_ReadCustomData_ParamsDataView::GetTypeDataView(
    ::mojo_base::mojom::String16DataView* output) {
  auto pointer = data_->type.Get();
  *output = ::mojo_base::mojom::String16DataView(pointer, context_);
}


inline void ClipboardHost_ReadCustomData_ResponseParamsDataView::GetResultDataView(
    ::mojo_base::mojom::BigString16DataView* output) {
  auto pointer = data_->result.Get();
  *output = ::mojo_base::mojom::BigString16DataView(pointer, context_);
}




inline void ClipboardHost_ReadBookmark_ResponseParamsDataView::GetTitleDataView(
    ::mojo_base::mojom::String16DataView* output) {
  auto pointer = data_->title.Get();
  *output = ::mojo_base::mojom::String16DataView(pointer, context_);
}
inline void ClipboardHost_ReadBookmark_ResponseParamsDataView::GetUrlDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->url.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void ClipboardHost_ReadData_ParamsDataView::GetFormatDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->format.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void ClipboardHost_ReadData_ResponseParamsDataView::GetResultDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->result.Get();
  *output = mojo::StringDataView(pointer, context_);
}




inline void ClipboardHost_GetLastModifiedTime_ResponseParamsDataView::GetTimeDataView(
    ::mojo_base::mojom::TimeDataView* output) {
  auto pointer = data_->time.Get();
  *output = ::mojo_base::mojom::TimeDataView(pointer, context_);
}




inline void ClipboardHost_WriteText_ParamsDataView::GetTextDataView(
    ::mojo_base::mojom::BigString16DataView* output) {
  auto pointer = data_->text.Get();
  *output = ::mojo_base::mojom::BigString16DataView(pointer, context_);
}


inline void ClipboardHost_WriteHTML_ParamsDataView::GetMarkupDataView(
    ::mojo_base::mojom::BigString16DataView* output) {
  auto pointer = data_->markup.Get();
  *output = ::mojo_base::mojom::BigString16DataView(pointer, context_);
}
inline void ClipboardHost_WriteHTML_ParamsDataView::GetUrlDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->url.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void ClipboardHost_WriteRTF_ParamsDataView::GetRtfDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->rtf.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void ClipboardHost_WriteBookmark_ParamsDataView::GetUrlDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->url.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void ClipboardHost_WriteBookmark_ParamsDataView::GetTitleDataView(
    ::mojo_base::mojom::String16DataView* output) {
  auto pointer = data_->title.Get();
  *output = ::mojo_base::mojom::String16DataView(pointer, context_);
}




inline void ClipboardHost_WriteBitmap_ParamsDataView::GetBitmapDataView(
    ::skia::mojom::BitmapDataView* output) {
  auto pointer = data_->bitmap.Get();
  *output = ::skia::mojom::BitmapDataView(pointer, context_);
}


inline void ClipboardHost_WriteData_ParamsDataView::GetTypeDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->type.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void ClipboardHost_WriteData_ParamsDataView::GetDataDataView(
    ::mojo_base::mojom::BigStringDataView* output) {
  auto pointer = data_->data.Get();
  *output = ::mojo_base::mojom::BigStringDataView(pointer, context_);
}





}  // namespace mojom
}  // namespace ui

#endif  // UI_BASE_MOJO_CLIPBOARD_MOJOM_SHARED_H_