// Copyright 2019 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef UI_OZONE_PUBLIC_INTERFACES_DRM_DEVICE_MOJOM_TEST_UTILS_H_
#define UI_OZONE_PUBLIC_INTERFACES_DRM_DEVICE_MOJOM_TEST_UTILS_H_

#include "ui/ozone/public/interfaces/drm_device.mojom.h"


namespace ui {
namespace ozone {
namespace mojom {


class  DrmDeviceInterceptorForTesting : public DrmDevice {
  virtual DrmDevice* GetForwardingInterface() = 0;
  void StartDrmDevice(StartDrmDeviceCallback callback) override;
  void CreateWindow(gfx::AcceleratedWidget widget) override;
  void DestroyWindow(gfx::AcceleratedWidget widget) override;
  void SetWindowBounds(gfx::AcceleratedWidget widget, const gfx::Rect& bounds) override;
  void TakeDisplayControl(TakeDisplayControlCallback callback) override;
  void RelinquishDisplayControl(RelinquishDisplayControlCallback callback) override;
  void RefreshNativeDisplays(RefreshNativeDisplaysCallback callback) override;
  void AddGraphicsDevice(const base::FilePath& path, base::File file) override;
  void RemoveGraphicsDevice(const base::FilePath& path) override;
  void DisableNativeDisplay(int64_t display_id, DisableNativeDisplayCallback callback) override;
  void ConfigureNativeDisplay(int64_t display_id, std::unique_ptr<display::DisplayMode> display_mode, const gfx::Point& point, ConfigureNativeDisplayCallback callback) override;
  void GetHDCPState(int64_t display_id, GetHDCPStateCallback callback) override;
  void SetHDCPState(int64_t display_id, display::HDCPState state, SetHDCPStateCallback callback) override;
  void SetColorMatrix(int64_t display_id, const std::vector<float>& color_matrix) override;
  void SetGammaCorrection(int64_t display_id, const std::vector<display::GammaRampRGBEntry>& degamma_lut, const std::vector<display::GammaRampRGBEntry>& gamma_lut) override;
  void CheckOverlayCapabilities(gfx::AcceleratedWidget widget, const std::vector<ui::OverlaySurfaceCandidate>& candidates, CheckOverlayCapabilitiesCallback callback) override;
};
class  DrmDeviceAsyncWaiter {
 public:
  explicit DrmDeviceAsyncWaiter(DrmDevice* proxy);
  ~DrmDeviceAsyncWaiter();
  void StartDrmDevice(
      bool* out_success);
  void TakeDisplayControl(
      bool* out_success);
  void RelinquishDisplayControl(
      bool* out_success);
  void RefreshNativeDisplays(
      std::vector<std::unique_ptr<display::DisplaySnapshot>>* out_display_snapshots);
  void DisableNativeDisplay(
      int64_t display_id, int64_t* out_display_id, bool* out_success);
  void ConfigureNativeDisplay(
      int64_t display_id, std::unique_ptr<display::DisplayMode> display_mode, const gfx::Point& point, int64_t* out_display_id, bool* out_success);
  void GetHDCPState(
      int64_t display_id, int64_t* out_display_id, bool* out_success, display::HDCPState* out_state);
  void SetHDCPState(
      int64_t display_id, display::HDCPState state, int64_t* out_display_id, bool* out_success);
  void CheckOverlayCapabilities(
      gfx::AcceleratedWidget widget, const std::vector<ui::OverlaySurfaceCandidate>& candidates, gfx::AcceleratedWidget* out_widget, std::vector<ui::OverlaySurfaceCandidate>* out_candidates, std::vector<ui::OverlayStatus>* out_status);

 private:
  DrmDevice* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(DrmDeviceAsyncWaiter);
};




}  // namespace mojom
}  // namespace ozone
}  // namespace ui

#endif  // UI_OZONE_PUBLIC_INTERFACES_DRM_DEVICE_MOJOM_TEST_UTILS_H_