// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef UI_OZONE_PUBLIC_INTERFACES_DRM_DEVICE_MOJOM_H_
#define UI_OZONE_PUBLIC_INTERFACES_DRM_DEVICE_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "ui/ozone/public/interfaces/drm_device.mojom-shared.h"
#include "mojo/public/mojom/base/file.mojom.h"
#include "mojo/public/mojom/base/file_path.mojom.h"
#include "ui/display/mojo/display_constants.mojom.h"
#include "ui/display/mojo/display_mode.mojom.h"
#include "ui/display/mojo/display_snapshot.mojom.h"
#include "ui/display/mojo/gamma_ramp_rgb_entry.mojom.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include "ui/gfx/mojo/accelerated_widget.mojom.h"
#include "ui/ozone/public/interfaces/overlay_surface_candidate.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace ui {
namespace ozone {
namespace mojom {
class DrmDevice;
using DrmDevicePtr = mojo::InterfacePtr<DrmDevice>;
using DrmDevicePtrInfo = mojo::InterfacePtrInfo<DrmDevice>;
using ThreadSafeDrmDevicePtr =
    mojo::ThreadSafeInterfacePtr<DrmDevice>;
using DrmDeviceRequest = mojo::InterfaceRequest<DrmDevice>;
using DrmDeviceAssociatedPtr =
    mojo::AssociatedInterfacePtr<DrmDevice>;
using ThreadSafeDrmDeviceAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<DrmDevice>;
using DrmDeviceAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<DrmDevice>;
using DrmDeviceAssociatedRequest =
    mojo::AssociatedInterfaceRequest<DrmDevice>;


class DrmDeviceProxy;

template <typename ImplRefTraits>
class DrmDeviceStub;

class DrmDeviceRequestValidator;
class DrmDeviceResponseValidator;

class  DrmDevice
    : public DrmDeviceInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = true;

  using Proxy_ = DrmDeviceProxy;

  template <typename ImplRefTraits>
  using Stub_ = DrmDeviceStub<ImplRefTraits>;

  using RequestValidator_ = DrmDeviceRequestValidator;
  using ResponseValidator_ = DrmDeviceResponseValidator;
  enum MethodMinVersions : uint32_t {
    kStartDrmDeviceMinVersion = 0,
    kCreateWindowMinVersion = 0,
    kDestroyWindowMinVersion = 0,
    kSetWindowBoundsMinVersion = 0,
    kTakeDisplayControlMinVersion = 0,
    kRelinquishDisplayControlMinVersion = 0,
    kRefreshNativeDisplaysMinVersion = 0,
    kAddGraphicsDeviceMinVersion = 0,
    kRemoveGraphicsDeviceMinVersion = 0,
    kDisableNativeDisplayMinVersion = 0,
    kConfigureNativeDisplayMinVersion = 0,
    kGetHDCPStateMinVersion = 0,
    kSetHDCPStateMinVersion = 0,
    kSetColorMatrixMinVersion = 0,
    kSetGammaCorrectionMinVersion = 0,
    kCheckOverlayCapabilitiesMinVersion = 0,
  };
  virtual ~DrmDevice() {}

  // Sync method. This signature is used by the client side; the service side
  // should implement the signature with callback below.
  virtual bool StartDrmDevice(bool* out_success);

  using StartDrmDeviceCallback = base::OnceCallback<void(bool)>;
  virtual void StartDrmDevice(StartDrmDeviceCallback callback) = 0;

  virtual void CreateWindow(gfx::AcceleratedWidget widget) = 0;

  virtual void DestroyWindow(gfx::AcceleratedWidget widget) = 0;

  virtual void SetWindowBounds(gfx::AcceleratedWidget widget, const gfx::Rect& bounds) = 0;


  using TakeDisplayControlCallback = base::OnceCallback<void(bool)>;
  virtual void TakeDisplayControl(TakeDisplayControlCallback callback) = 0;


  using RelinquishDisplayControlCallback = base::OnceCallback<void(bool)>;
  virtual void RelinquishDisplayControl(RelinquishDisplayControlCallback callback) = 0;


  using RefreshNativeDisplaysCallback = base::OnceCallback<void(std::vector<std::unique_ptr<display::DisplaySnapshot>>)>;
  virtual void RefreshNativeDisplays(RefreshNativeDisplaysCallback callback) = 0;

  virtual void AddGraphicsDevice(const base::FilePath& path, base::File file) = 0;

  virtual void RemoveGraphicsDevice(const base::FilePath& path) = 0;


  using DisableNativeDisplayCallback = base::OnceCallback<void(int64_t, bool)>;
  virtual void DisableNativeDisplay(int64_t display_id, DisableNativeDisplayCallback callback) = 0;


  using ConfigureNativeDisplayCallback = base::OnceCallback<void(int64_t, bool)>;
  virtual void ConfigureNativeDisplay(int64_t display_id, std::unique_ptr<display::DisplayMode> display_mode, const gfx::Point& point, ConfigureNativeDisplayCallback callback) = 0;


  using GetHDCPStateCallback = base::OnceCallback<void(int64_t, bool, display::HDCPState)>;
  virtual void GetHDCPState(int64_t display_id, GetHDCPStateCallback callback) = 0;


  using SetHDCPStateCallback = base::OnceCallback<void(int64_t, bool)>;
  virtual void SetHDCPState(int64_t display_id, display::HDCPState state, SetHDCPStateCallback callback) = 0;

  virtual void SetColorMatrix(int64_t display_id, const std::vector<float>& color_matrix) = 0;

  virtual void SetGammaCorrection(int64_t display_id, const std::vector<display::GammaRampRGBEntry>& degamma_lut, const std::vector<display::GammaRampRGBEntry>& gamma_lut) = 0;


  using CheckOverlayCapabilitiesCallback = base::OnceCallback<void(gfx::AcceleratedWidget, const std::vector<ui::OverlaySurfaceCandidate>&, const std::vector<ui::OverlayStatus>&)>;
  virtual void CheckOverlayCapabilities(gfx::AcceleratedWidget widget, const std::vector<ui::OverlaySurfaceCandidate>& candidates, CheckOverlayCapabilitiesCallback callback) = 0;
};

class  DrmDeviceProxy
    : public DrmDevice {
 public:
  explicit DrmDeviceProxy(mojo::MessageReceiverWithResponder* receiver);
  bool StartDrmDevice(bool* out_success) final;
  void StartDrmDevice(StartDrmDeviceCallback callback) final;
  void CreateWindow(gfx::AcceleratedWidget widget) final;
  void DestroyWindow(gfx::AcceleratedWidget widget) final;
  void SetWindowBounds(gfx::AcceleratedWidget widget, const gfx::Rect& bounds) final;
  void TakeDisplayControl(TakeDisplayControlCallback callback) final;
  void RelinquishDisplayControl(RelinquishDisplayControlCallback callback) final;
  void RefreshNativeDisplays(RefreshNativeDisplaysCallback callback) final;
  void AddGraphicsDevice(const base::FilePath& path, base::File file) final;
  void RemoveGraphicsDevice(const base::FilePath& path) final;
  void DisableNativeDisplay(int64_t display_id, DisableNativeDisplayCallback callback) final;
  void ConfigureNativeDisplay(int64_t display_id, std::unique_ptr<display::DisplayMode> display_mode, const gfx::Point& point, ConfigureNativeDisplayCallback callback) final;
  void GetHDCPState(int64_t display_id, GetHDCPStateCallback callback) final;
  void SetHDCPState(int64_t display_id, display::HDCPState state, SetHDCPStateCallback callback) final;
  void SetColorMatrix(int64_t display_id, const std::vector<float>& color_matrix) final;
  void SetGammaCorrection(int64_t display_id, const std::vector<display::GammaRampRGBEntry>& degamma_lut, const std::vector<display::GammaRampRGBEntry>& gamma_lut) final;
  void CheckOverlayCapabilities(gfx::AcceleratedWidget widget, const std::vector<ui::OverlaySurfaceCandidate>& candidates, CheckOverlayCapabilitiesCallback callback) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  DrmDeviceStubDispatch {
 public:
  static bool Accept(DrmDevice* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      DrmDevice* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<DrmDevice>>
class DrmDeviceStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  DrmDeviceStub() {}
  ~DrmDeviceStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return DrmDeviceStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return DrmDeviceStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  DrmDeviceRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  DrmDeviceResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace ozone
}  // namespace ui

namespace mojo {

}  // namespace mojo

#endif  // UI_OZONE_PUBLIC_INTERFACES_DRM_DEVICE_MOJOM_H_